/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "libpikawidgets/pikawidgets.h"

#include "actions-types.h"

#include "widgets/pikaactiongroup.h"
#include "widgets/pikahelp-ids.h"

#include "display/pikacursorview.h"

#include "cursor-info-actions.h"
#include "cursor-info-commands.h"

#include "pika-intl.h"


static const PikaToggleActionEntry cursor_info_toggle_actions[] =
{
  { "cursor-info-sample-merged", NULL,
    NC_("cursor-info-action", "_Sample Merged"), NULL, { NULL },
    NC_("cursor-info-action", "Use the composite color of all visible layers"),
    cursor_info_sample_merged_cmd_callback,
    TRUE,
    PIKA_HELP_POINTER_INFO_SAMPLE_MERGED }
};


void
cursor_info_actions_setup (PikaActionGroup *group)
{
  pika_action_group_add_toggle_actions (group, "cursor-info-action",
                                        cursor_info_toggle_actions,
                                        G_N_ELEMENTS (cursor_info_toggle_actions));
}

void
cursor_info_actions_update (PikaActionGroup *group,
                            gpointer         data)
{
  PikaCursorView *view = PIKA_CURSOR_VIEW (data);

#define SET_ACTIVE(action,condition) \
        pika_action_group_set_action_active (group, action, (condition) != 0)

  SET_ACTIVE ("cursor-info-sample-merged",
              pika_cursor_view_get_sample_merged (view));

#undef SET_ACTIVE
}
