/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "libpikawidgets/pikawidgets.h"

#include "actions-types.h"

#include "core/pika.h"

#include "widgets/pikadialogfactory.h"
#include "widgets/pikawidgets-utils.h"
#include "widgets/pikawindowstrategy.h"

#include "actions.h"
#include "dialogs-commands.h"


/*  public functions  */

void
dialogs_create_toplevel_cmd_callback (PikaAction *action,
                                      GVariant   *value,
                                      gpointer    data)
{
  GtkWidget   *widget;
  const gchar *identifier;
  return_if_no_widget (widget, data);

  identifier = g_variant_get_string (value, NULL);

  if (identifier)
    pika_dialog_factory_dialog_new (pika_dialog_factory_get_singleton (),
                                    pika_widget_get_monitor (widget),
                                    NULL /*ui_manager*/,
                                    widget,
                                    identifier, -1, TRUE);
}

void
dialogs_create_dockable_cmd_callback (PikaAction *action,
                                      GVariant   *value,
                                      gpointer    data)
{
  Pika        *pika;
  GtkWidget   *widget;
  const gchar *identifier;
  return_if_no_pika   (pika, data);
  return_if_no_widget (widget, data);

  identifier = g_variant_get_string (value, NULL);

  if (identifier)
    pika_window_strategy_show_dockable_dialog (PIKA_WINDOW_STRATEGY (pika_get_window_strategy (pika)),
                                               pika,
                                               pika_dialog_factory_get_singleton (),
                                               pika_widget_get_monitor (widget),
                                               identifier);
}
