/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "actions-types.h"

#include "core/pikacontext.h"
#include "core/pikapalette.h"

#include "widgets/pikapaletteeditor.h"
#include "widgets/pikapaletteview.h"

#include "palette-editor-commands.h"


/*  public functions  */

void
palette_editor_edit_color_cmd_callback (PikaAction *action,
                                        GVariant   *value,
                                        gpointer    data)
{
  PikaPaletteEditor *editor = PIKA_PALETTE_EDITOR (data);

  pika_palette_editor_edit_color (editor);
}

void
palette_editor_new_color_cmd_callback (PikaAction *action,
                                       GVariant   *value,
                                       gpointer    data)
{
  PikaPaletteEditor *editor      = PIKA_PALETTE_EDITOR (data);
  PikaDataEditor    *data_editor = PIKA_DATA_EDITOR (data);
  gboolean           background  = (gboolean) g_variant_get_int32 (value);

  if (data_editor->data_editable)
    {
      PikaPalette      *palette = PIKA_PALETTE (data_editor->data);
      PikaPaletteEntry *entry;
      PikaRGB           color;

      if (background)
        pika_context_get_background (data_editor->context, &color);
      else
        pika_context_get_foreground (data_editor->context, &color);

      entry = pika_palette_add_entry (palette, -1, NULL, &color);
      pika_palette_view_select_entry (PIKA_PALETTE_VIEW (editor->view), entry);
    }
}

void
palette_editor_delete_color_cmd_callback (PikaAction *action,
                                          GVariant   *value,
                                          gpointer    data)
{
  PikaPaletteEditor *editor      = PIKA_PALETTE_EDITOR (data);
  PikaDataEditor    *data_editor = PIKA_DATA_EDITOR (data);

  if (data_editor->data_editable && editor->color)
    {
      PikaPalette *palette = PIKA_PALETTE (data_editor->data);

      pika_palette_delete_entry (palette, editor->color);
    }
}

void
palette_editor_zoom_cmd_callback (PikaAction *action,
                                  GVariant   *value,
                                  gpointer    data)
{
  PikaPaletteEditor *editor    = PIKA_PALETTE_EDITOR (data);
  PikaZoomType       zoom_type = (PikaZoomType) g_variant_get_int32 (value);

  pika_palette_editor_zoom (editor, zoom_type);
}
