/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "libpikawidgets/pikawidgets.h"

#include "actions-types.h"

#include "core/pika.h"
#include "core/pikacontext.h"

#include "widgets/pikaactiongroup.h"
#include "widgets/pikadataeditor.h"
#include "widgets/pikahelp-ids.h"

#include "data-editor-commands.h"
#include "tool-preset-editor-actions.h"
#include "tool-preset-editor-commands.h"

#include "pika-intl.h"


static const PikaActionEntry tool_preset_editor_actions[] =
{
  { "tool-preset-editor-save", PIKA_ICON_DOCUMENT_SAVE,
    NC_("tool-preset-editor-action", "_Save Tool Options to Preset"), NULL, { NULL },
    NC_("tool-preset-editor-action", "Save the active tool options to this "
        "tool preset"),
    tool_preset_editor_save_cmd_callback,
    PIKA_HELP_TOOL_PRESET_SAVE },

  { "tool-preset-editor-restore", PIKA_ICON_DOCUMENT_REVERT,
    NC_("tool-preset-editor-action", "_Restore Tool Preset"), NULL, { NULL },
    NC_("tool-preset-editor-action", "Restore this tool preset"),
    tool_preset_editor_restore_cmd_callback,
    PIKA_HELP_TOOL_PRESET_RESTORE }
};


static const PikaToggleActionEntry tool_preset_editor_toggle_actions[] =
{
  { "tool-preset-editor-edit-active", PIKA_ICON_LINKED,
    NC_("tool-preset-editor-action", "Edit Active Tool Preset"), NULL, { NULL }, NULL,
    data_editor_edit_active_cmd_callback,
    FALSE,
    PIKA_HELP_TOOL_PRESET_EDITOR_EDIT_ACTIVE }
};


void
tool_preset_editor_actions_setup (PikaActionGroup *group)
{
  pika_action_group_add_actions (group, "tool-preset-editor-action",
                                 tool_preset_editor_actions,
                                 G_N_ELEMENTS (tool_preset_editor_actions));

  pika_action_group_add_toggle_actions (group, "tool-preset-editor-action",
                                        tool_preset_editor_toggle_actions,
                                        G_N_ELEMENTS (tool_preset_editor_toggle_actions));

}

void
tool_preset_editor_actions_update (PikaActionGroup *group,
                                   gpointer         user_data)
{
  PikaDataEditor *data_editor = PIKA_DATA_EDITOR (user_data);
  gboolean        edit_active = FALSE;

  edit_active = pika_data_editor_get_edit_active (data_editor);

#define SET_SENSITIVE(action,condition) \
        pika_action_group_set_action_sensitive (group, action, (condition) != 0, NULL)
#define SET_ACTIVE(action,condition) \
        pika_action_group_set_action_active (group, action, (condition) != 0)

  SET_SENSITIVE ("tool-preset-editor-save",        data_editor->data);
  SET_SENSITIVE ("tool-preset-editor-restore",     data_editor->data);
  SET_ACTIVE    ("tool-preset-editor-edit-active", edit_active);

#undef SET_SENSITIVE
#undef SET_ACTIVE
}
