/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "libpikaconfig/pikaconfig.h"
#include "libpikawidgets/pikawidgets.h"

#include "actions-types.h"

#include "core/pika.h"
#include "core/pikacontext.h"
#include "core/pikatoolinfo.h"
#include "core/pikatoolpreset.h"

#include "widgets/pikacontainereditor.h"
#include "widgets/pikacontainerview.h"

#include "tool-presets-commands.h"

#include "pika-intl.h"


/*  public functions  */

void
tool_presets_save_cmd_callback (PikaAction *action,
                                GVariant   *value,
                                gpointer    data)
{
  PikaContainerEditor *editor = PIKA_CONTAINER_EDITOR (data);
  PikaContext         *context;
  PikaToolPreset      *preset;
  PikaToolInfo        *tool_info;

  context = pika_container_view_get_context (editor->view);

  preset    = pika_context_get_tool_preset (context);
  tool_info = pika_context_get_tool (pika_get_user_context (context->pika));

  if (tool_info && preset)
    {
      PikaToolInfo *preset_tool;

      preset_tool =  pika_context_get_tool (PIKA_CONTEXT (preset->tool_options));

      if (tool_info != preset_tool)
        {
          pika_message (context->pika,
                        G_OBJECT (editor), PIKA_MESSAGE_WARNING,
                        _("Can't save '%s' tool options to an "
                          "existing '%s' tool preset."),
                        tool_info->label,
                        preset_tool->label);
          return;
        }

      pika_config_sync (G_OBJECT (tool_info->tool_options),
                        G_OBJECT (preset->tool_options), 0);
    }
}

void
tool_presets_restore_cmd_callback (PikaAction *action,
                                   GVariant   *value,
                                   gpointer    data)
{
  PikaContainerEditor *editor = PIKA_CONTAINER_EDITOR (data);
  PikaContext         *context;
  PikaToolPreset      *preset;

  context = pika_container_view_get_context (editor->view);

  preset = pika_context_get_tool_preset (context);

  if (preset)
    pika_context_tool_preset_changed (pika_get_user_context (context->pika));
}
