/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "libpikabase/pikabase.h"
#include "libpikawidgets/pikawidgets.h"

#include "actions-types.h"

#include "core/pika.h"
#include "core/pikatoolinfo.h"

#include "widgets/pikahelp-ids.h"
#include "widgets/pikauimanager.h"
#include "widgets/pikawidgets-utils.h"

#include "display/pikadisplay.h"
#include "display/pikatoolpath.h"

#include "dialogs/dialogs.h"

#include "vector-toolpath-commands.h"


/*  public functions  */

void
vector_toolpath_delete_anchor_cmd_callback (PikaAction *action,
                                            GVariant   *value,
                                            gpointer    data)
{
  PikaToolPath *tool_path = PIKA_TOOL_PATH (data);

  pika_tool_path_delete_anchor (tool_path);
}

void
vector_toolpath_shift_start_cmd_callback (PikaAction *action,
                                          GVariant   *value,
                                          gpointer    data)
{
  PikaToolPath *tool_path = PIKA_TOOL_PATH (data);

  pika_tool_path_shift_start (tool_path);
}

void
vector_toolpath_insert_anchor_cmd_callback (PikaAction *action,
                                            GVariant   *value,
                                            gpointer    data)
{
  PikaToolPath *tool_path = PIKA_TOOL_PATH (data);

  pika_tool_path_insert_anchor (tool_path);
}

void
vector_toolpath_delete_segment_cmd_callback (PikaAction *action,
                                             GVariant   *value,
                                             gpointer    data)
{
  PikaToolPath *tool_path = PIKA_TOOL_PATH (data);

  pika_tool_path_delete_segment (tool_path);
}

void
vector_toolpath_reverse_stroke_cmd_callback (PikaAction *action,
                                             GVariant   *value,
                                             gpointer    data)
{
  PikaToolPath *tool_path = PIKA_TOOL_PATH (data);

  pika_tool_path_reverse_stroke (tool_path);
}

