/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __CONFIG_ENUMS_H__
#define __CONFIG_ENUMS_H__


#define PIKA_TYPE_CANVAS_PADDING_MODE (pika_canvas_padding_mode_get_type ())

GType pika_canvas_padding_mode_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_CANVAS_PADDING_MODE_DEFAULT,      /*< desc="From theme"        >*/
  PIKA_CANVAS_PADDING_MODE_LIGHT_CHECK,  /*< desc="Light check color" >*/
  PIKA_CANVAS_PADDING_MODE_DARK_CHECK,   /*< desc="Dark check color"  >*/
  PIKA_CANVAS_PADDING_MODE_CUSTOM,       /*< desc="Custom color"      >*/
  PIKA_CANVAS_PADDING_MODE_RESET = -1    /*< skip >*/
} PikaCanvasPaddingMode;


#define PIKA_TYPE_CURSOR_FORMAT (pika_cursor_format_get_type ())

GType pika_cursor_format_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_CURSOR_FORMAT_BITMAP, /*< desc="Black & white" >*/
  PIKA_CURSOR_FORMAT_PIXBUF  /*< desc="Fancy"         >*/
} PikaCursorFormat;


#define PIKA_TYPE_CURSOR_MODE (pika_cursor_mode_get_type ())

GType pika_cursor_mode_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_CURSOR_MODE_TOOL_ICON,       /*< desc="Tool icon"                >*/
  PIKA_CURSOR_MODE_TOOL_CROSSHAIR,  /*< desc="Tool icon with crosshair" >*/
  PIKA_CURSOR_MODE_CROSSHAIR,       /*< desc="Crosshair only"           >*/
} PikaCursorMode;


#define PIKA_TYPE_EXPORT_FILE_TYPE (pika_export_file_type_get_type ())

GType pika_export_file_type_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_EXPORT_FILE_PNG,  /*< desc="PNG Image"                >*/
  PIKA_EXPORT_FILE_JPG,  /*< desc="JPEG Image"               >*/
  PIKA_EXPORT_FILE_ORA,  /*< desc="OpenRaster Image"         >*/
  PIKA_EXPORT_FILE_PSD,  /*< desc="Photoshop Image"          >*/
  PIKA_EXPORT_FILE_PDF,  /*< desc="Portable Document Format" >*/
  PIKA_EXPORT_FILE_TIF,  /*< desc="TIFF Image"               >*/
  PIKA_EXPORT_FILE_BMP,  /*< desc="Windows BMP Image"        >*/
  PIKA_EXPORT_FILE_WEBP, /*< desc="WebP Image"               >*/
} PikaExportFileType;


#define PIKA_TYPE_HANDEDNESS (pika_handedness_get_type ())

GType pika_handedness_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_HANDEDNESS_LEFT, /*< desc="Left-handed"  >*/
  PIKA_HANDEDNESS_RIGHT /*< desc="Right-handed" >*/
} PikaHandedness;


#define PIKA_TYPE_HELP_BROWSER_TYPE (pika_help_browser_type_get_type ())

GType pika_help_browser_type_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_HELP_BROWSER_PIKA,        /*< desc="PIKA help browser" >*/
  PIKA_HELP_BROWSER_WEB_BROWSER  /*< desc="Web browser"       >*/
} PikaHelpBrowserType;


#define PIKA_TYPE_ICON_SIZE (pika_icon_size_get_type ())

GType pika_icon_size_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_ICON_SIZE_SMALL,   /*< desc="Small size"       > */
  PIKA_ICON_SIZE_MEDIUM,  /*< desc="Medium size"      > */
  PIKA_ICON_SIZE_LARGE,   /*< desc="Large size"       > */
  PIKA_ICON_SIZE_HUGE     /*< desc="Huge size"        > */
} PikaIconSize;


#define PIKA_TYPE_POSITION (pika_position_get_type ())

GType pika_position_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_POSITION_TOP,    /*< desc="Top" >*/
  PIKA_POSITION_BOTTOM, /*< desc="Bottom" >*/
  PIKA_POSITION_LEFT,   /*< desc="Left"  >*/
  PIKA_POSITION_RIGHT   /*< desc="Right" >*/
} PikaPosition;

#define PIKA_TYPE_DRAG_ZOOM_MODE (pika_drag_zoom_mode_get_type ())

GType pika_drag_zoom_mode_get_type (void) G_GNUC_CONST;

typedef enum
{
  PROP_DRAG_ZOOM_MODE_DISTANCE,  /*< desc="By distance" >*/
  PROP_DRAG_ZOOM_MODE_DURATION,  /*< desc="By duration" >*/
} PikaDragZoomMode;

#define PIKA_TYPE_SPACE_BAR_ACTION (pika_space_bar_action_get_type ())

GType pika_space_bar_action_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_SPACE_BAR_ACTION_NONE,  /*< desc="No action"           >*/
  PIKA_SPACE_BAR_ACTION_PAN,   /*< desc="Pan view"            >*/
  PIKA_SPACE_BAR_ACTION_MOVE   /*< desc="Switch to Move tool" >*/
} PikaSpaceBarAction;


#define PIKA_TYPE_WINDOW_HINT (pika_window_hint_get_type ())

GType pika_window_hint_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_WINDOW_HINT_NORMAL,     /*< desc="Normal window"  >*/
  PIKA_WINDOW_HINT_UTILITY,    /*< desc="Utility window" >*/
  PIKA_WINDOW_HINT_KEEP_ABOVE  /*< desc="Keep above"     >*/
} PikaWindowHint;


#define PIKA_TYPE_ZOOM_QUALITY (pika_zoom_quality_get_type ())

GType pika_zoom_quality_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_ZOOM_QUALITY_LOW,   /*< desc="Low"  >*/
  PIKA_ZOOM_QUALITY_HIGH   /*< desc="High" >*/
} PikaZoomQuality;


#endif /* __CONFIG_ENUMS_H__ */
