/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * PikaDialogConfig class
 * Copyright (C) 2016  Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_DIALOG_CONFIG_H__
#define __PIKA_DIALOG_CONFIG_H__

#include "config/pikaguiconfig.h"


/* We don't want to include stuff from core/ here, instead do the next
 * less ugly hack...
 */
typedef struct _PikaFillOptions   PikaFillOptions;
typedef struct _PikaStrokeOptions PikaStrokeOptions;


#define PIKA_TYPE_DIALOG_CONFIG            (pika_dialog_config_get_type ())
#define PIKA_DIALOG_CONFIG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_DIALOG_CONFIG, PikaDialogConfig))
#define PIKA_DIALOG_CONFIG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_DIALOG_CONFIG, PikaDialogConfigClass))
#define PIKA_IS_DIALOG_CONFIG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_DIALOG_CONFIG))
#define PIKA_IS_DIALOG_CONFIG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_DIALOG_CONFIG))


typedef struct _PikaDialogConfigClass PikaDialogConfigClass;

struct _PikaDialogConfig
{
  PikaGuiConfig             parent_instance;

  PikaColorProfilePolicy     color_profile_policy;
  PikaMetadataRotationPolicy metadata_rotation_policy;

  gchar                    *color_profile_path;

  PikaColorRenderingIntent  image_convert_profile_intent;
  gboolean                  image_convert_profile_bpc;

  GeglDitherMethod          image_convert_precision_layer_dither_method;
  GeglDitherMethod          image_convert_precision_text_layer_dither_method;
  GeglDitherMethod          image_convert_precision_channel_dither_method;

  PikaConvertPaletteType    image_convert_indexed_palette_type;
  gint                      image_convert_indexed_max_colors;
  gboolean                  image_convert_indexed_remove_duplicates;
  PikaConvertDitherType     image_convert_indexed_dither_type;
  gboolean                  image_convert_indexed_dither_alpha;
  gboolean                  image_convert_indexed_dither_text_layers;

  PikaFillType              image_resize_fill_type;
  PikaItemSet               image_resize_layer_set;
  gboolean                  image_resize_resize_text_layers;

  gchar                    *layer_new_name;
  PikaLayerMode             layer_new_mode;
  PikaLayerColorSpace       layer_new_blend_space;
  PikaLayerColorSpace       layer_new_composite_space;
  PikaLayerCompositeMode    layer_new_composite_mode;
  gdouble                   layer_new_opacity;
  PikaFillType              layer_new_fill_type;

  PikaFillType              layer_resize_fill_type;

  PikaAddMaskType           layer_add_mask_type;
  gboolean                  layer_add_mask_invert;

  PikaMergeType             layer_merge_type;
  gboolean                  layer_merge_active_group_only;
  gboolean                  layer_merge_discard_invisible;

  gchar                    *channel_new_name;
  PikaRGB                   channel_new_color;

  gchar                    *vectors_new_name;

  gchar                    *vectors_export_path;
  gboolean                  vectors_export_active_only;

  gchar                    *vectors_import_path;
  gboolean                  vectors_import_merge;
  gboolean                  vectors_import_scale;

  gdouble                   selection_feather_radius;
  gboolean                  selection_feather_edge_lock;

  gdouble                   selection_grow_radius;

  gdouble                   selection_shrink_radius;
  gboolean                  selection_shrink_edge_lock;

  gdouble                   selection_border_radius;
  gboolean                  selection_border_edge_lock;
  PikaChannelBorderStyle    selection_border_style;

  PikaFillOptions          *fill_options;
  PikaStrokeOptions        *stroke_options;
};

struct _PikaDialogConfigClass
{
  PikaGuiConfigClass  parent_class;
};


GType  pika_dialog_config_get_type (void) G_GNUC_CONST;


#endif /* PIKA_DIALOG_CONFIG_H__ */
