/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * PikaGuiConfig class
 * Copyright (C) 2001  Sven Neumann <sven@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_GUI_CONFIG_H__
#define __PIKA_GUI_CONFIG_H__

#include "config/pikadisplayconfig.h"


#define PIKA_CONFIG_DEFAULT_THEME          "Default"
#define PIKA_CONFIG_DEFAULT_ICON_THEME     "Symbolic"


#define PIKA_TYPE_GUI_CONFIG            (pika_gui_config_get_type ())
#define PIKA_GUI_CONFIG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_GUI_CONFIG, PikaGuiConfig))
#define PIKA_GUI_CONFIG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_GUI_CONFIG, PikaGuiConfigClass))
#define PIKA_IS_GUI_CONFIG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_GUI_CONFIG))
#define PIKA_IS_GUI_CONFIG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_GUI_CONFIG))


typedef struct _PikaGuiConfigClass PikaGuiConfigClass;

struct _PikaGuiConfig
{
  PikaDisplayConfig    parent_instance;

  gboolean             edit_non_visible;
  gboolean             move_tool_changes_active;
  gint                 filter_tool_max_recent;
  gboolean             filter_tool_use_last_settings;
  gboolean             filter_tool_show_color_options;
  gboolean             trust_dirty_flag;
  gboolean             save_device_status;
  gboolean             devices_share_tool;
  gboolean             save_session_info;
  gboolean             restore_session;
  gboolean             restore_monitor;
  gboolean             save_tool_options;
  gboolean             save_accels;
  gboolean             restore_accels;
  gint                 last_opened_size;
  guint64              max_new_image_size;
  gboolean             toolbox_color_area;
  gboolean             toolbox_foo_area;
  gboolean             toolbox_image_area;
  gboolean             toolbox_mascot;
  gboolean             toolbox_groups;
  gchar               *theme_path;
  gchar               *theme;
  gboolean             prefer_dark_theme;
  gchar               *icon_theme_path;
  gchar               *icon_theme;
  gboolean             prefer_symbolic_icons;
  gboolean             override_icon_size;
  PikaIconSize         custom_icon_size;
  gboolean             use_help;
  gboolean             show_help_button;
  gchar               *help_locales;
  PikaHelpBrowserType  help_browser;
  gboolean             user_manual_online;
  gchar               *user_manual_online_uri;
  gint                 action_history_size;
  PikaWindowHint       dock_window_hint;
  PikaHandedness       cursor_handedness;
  gboolean             custom_title_bar;

  /* experimental playground */
  gboolean             playground_npd_tool;
  gboolean             playground_seamless_clone_tool;
  gboolean             playground_paint_select_tool;

  /* saved in sessionrc */
  gboolean             hide_docks;
  gboolean             single_window_mode;
  gboolean             show_tabs;
  PikaPosition         tabs_position;
  gint                 last_tip_shown;

  /* Currently ignored snce "gtk-can-change-accels" settings got deprecated in
   * GTK+ 3.10.
   */
  gboolean             can_change_accels;
};

struct _PikaGuiConfigClass
{
  PikaDisplayConfigClass  parent_class;
};


GType  pika_gui_config_get_type (void) G_GNUC_CONST;


#endif /* PIKA_GUI_CONFIG_H__ */
