/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __APP_PIKA_UTILS_H__
#define __APP_PIKA_UTILS_H__


#ifdef PIKA_RELEASE
#define PIKA_TIMER_START()
#define PIKA_TIMER_END(message)
#else
#define PIKA_TIMER_START() \
  { GTimer *_timer = g_timer_new ();

#define PIKA_TIMER_END(message) \
  g_printerr ("%s: %s took %0.4f seconds\n", \
              G_STRFUNC, message, g_timer_elapsed (_timer, NULL)); \
  g_timer_destroy (_timer); }
#endif


#define MIN4(a,b,c,d) MIN (MIN ((a), (b)), MIN ((c), (d)))
#define MAX4(a,b,c,d) MAX (MAX ((a), (b)), MAX ((c), (d)))


gint         pika_get_pid                          (void);
guint64      pika_get_physical_memory_size         (void);
gchar      * pika_get_default_language             (const gchar     *category);
PikaUnit     pika_get_default_unit                 (void);

gchar     ** pika_properties_append                (GType            object_type,
                                                    gint            *n_properties,
                                                    gchar          **names,
                                                    GValue         **values,
                                                    ...) G_GNUC_NULL_TERMINATED;
gchar     ** pika_properties_append_valist         (GType            object_type,
                                                    gint            *n_properties,
                                                    gchar          **names,
                                                    GValue         **values,
                                                    va_list          args);
void         pika_properties_free                  (gint             n_properties,
                                                    gchar          **names,
                                                    GValue          *values);

gchar      * pika_markup_extract_text              (const gchar     *markup);

const gchar* pika_enum_get_value_name              (GType            enum_type,
                                                    gint             value);

gboolean     pika_get_fill_params                  (PikaContext      *context,
                                                    PikaFillType      fill_type,
                                                    PikaRGB          *color,
                                                    PikaPattern     **pattern,
                                                    GError          **error);

/* Common values for the n_snap_lines parameter of
 * pika_constrain_line.
 */
#define PIKA_CONSTRAIN_LINE_90_DEGREES 2
#define PIKA_CONSTRAIN_LINE_45_DEGREES 4
#define PIKA_CONSTRAIN_LINE_15_DEGREES 12

void         pika_constrain_line                   (gdouble            start_x,
                                                    gdouble            start_y,
                                                    gdouble           *end_x,
                                                    gdouble           *end_y,
                                                    gint               n_snap_lines,
                                                    gdouble            offset_angle,
                                                    gdouble            xres,
                                                    gdouble            yres);

gint         pika_file_compare                     (GFile             *file1,
                                                    GFile             *file2);
gboolean     pika_file_is_executable               (GFile             *file);
gchar      * pika_file_get_extension               (GFile             *file);
GFile      * pika_file_with_new_extension          (GFile             *file,
                                                    GFile             *ext_file);
gboolean     pika_file_delete_recursive            (GFile             *file,
                                                    GError           **error);

gchar      * pika_data_input_stream_read_line_always
                                                   (GDataInputStream  *stream,
                                                    gsize             *length,
                                                    GCancellable      *cancellable,
                                                    GError           **error);

gboolean     pika_ascii_strtoi                     (const gchar       *nptr,
                                                    gchar            **endptr,
                                                    gint               base,
                                                    gint              *result);
gboolean     pika_ascii_strtod                     (const gchar       *nptr,
                                                    gchar            **endptr,
                                                    gdouble           *result);

gchar     *  pika_appstream_to_pango_markup        (const gchar       *as_text);
void         pika_appstream_to_pango_markups       (const gchar       *as_text,
                                                    gchar            **introduction,
                                                    GList            **release_items);

gint         pika_g_list_compare                   (GList             *list1,
                                                    GList             *list2);

PikaTRCType  pika_suggest_trc_for_component_type   (PikaComponentType  component_type,
                                                    PikaTRCType        old_trc);

PikaAsync  * pika_idle_run_async                   (PikaRunAsyncFunc func,
                                                    gpointer         user_data);
PikaAsync  * pika_idle_run_async_full              (gint             priority,
                                                    PikaRunAsyncFunc func,
                                                    gpointer         user_data,
                                                    GDestroyNotify   user_data_destroy_func);

PikaImage  * pika_create_image_from_buffer         (Pika              *pika,
                                                    GeglBuffer        *buffer,
                                                    const gchar       *image_name);

gint         pika_view_size_get_larger             (gint view_size);
gint         pika_view_size_get_smaller            (gint view_size);

#ifdef G_OS_WIN32

gboolean     pika_win32_have_wintab                (void);
gboolean     pika_win32_have_windows_ink           (void);

#endif

#endif /* __APP_PIKA_UTILS_H__ */
