/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikabrushcache.h
 * Copyright (C) 2011 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_BRUSH_CACHE_H__
#define __PIKA_BRUSH_CACHE_H__


#include "pikaobject.h"


#define PIKA_TYPE_BRUSH_CACHE            (pika_brush_cache_get_type ())
#define PIKA_BRUSH_CACHE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_BRUSH_CACHE, PikaBrushCache))
#define PIKA_BRUSH_CACHE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_BRUSH_CACHE, PikaBrushCacheClass))
#define PIKA_IS_BRUSH_CACHE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_BRUSH_CACHE))
#define PIKA_IS_BRUSH_CACHE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_BRUSH_CACHE))
#define PIKA_BRUSH_CACHE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_BRUSH_CACHE, PikaBrushCacheClass))


typedef struct _PikaBrushCacheClass PikaBrushCacheClass;

struct _PikaBrushCache
{
  PikaObject      parent_instance;

  GDestroyNotify  data_destroy;

  GList          *cached_units;

  gchar           debug_hit;
  gchar           debug_miss;
};

struct _PikaBrushCacheClass
{
  PikaObjectClass  parent_class;
};


GType            pika_brush_cache_get_type (void) G_GNUC_CONST;

PikaBrushCache * pika_brush_cache_new      (GDestroyNotify  data_destory,
                                            gchar           debug_hit,
                                            gchar           debug_miss);

void             pika_brush_cache_clear    (PikaBrushCache *cache);

gconstpointer    pika_brush_cache_get      (PikaBrushCache *cache,
                                            gint            width,
                                            gint            height,
                                            gdouble         scale,
                                            gdouble         aspect_ratio,
                                            gdouble         angle,
                                            gboolean        reflect,
                                            gdouble         hardness);
void             pika_brush_cache_add      (PikaBrushCache *cache,
                                            gpointer        data,
                                            gint            width,
                                            gint            height,
                                            gdouble         scale,
                                            gdouble         aspect_ratio,
                                            gdouble         angle,
                                            gboolean        reflect,
                                            gdouble         hardness);


#endif  /*  __PIKA_BRUSH_CACHE_H__  */
