/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_BUFFER_H__
#define __PIKA_BUFFER_H__


#include "pikaviewable.h"


#define PIKA_TYPE_BUFFER            (pika_buffer_get_type ())
#define PIKA_BUFFER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_BUFFER, PikaBuffer))
#define PIKA_BUFFER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_BUFFER, PikaBufferClass))
#define PIKA_IS_BUFFER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_BUFFER))
#define PIKA_IS_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_BUFFER))
#define PIKA_BUFFER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_BUFFER, PikaBufferClass))


typedef struct _PikaBufferClass PikaBufferClass;

struct _PikaBuffer
{
  PikaViewable      parent_instance;

  GeglBuffer       *buffer;
  gint              offset_x;
  gint              offset_y;

  gdouble           resolution_x;
  gdouble           resolution_y;
  PikaUnit          unit;

  PikaColorProfile *color_profile;
  PikaColorProfile *format_profile;
};

struct _PikaBufferClass
{
  PikaViewableClass  parent_class;
};


GType              pika_buffer_get_type          (void) G_GNUC_CONST;

PikaBuffer       * pika_buffer_new               (GeglBuffer       *buffer,
                                                  const gchar      *name,
                                                  gint              offset_x,
                                                  gint              offset_y,
                                                  gboolean          copy_pixels);
PikaBuffer       * pika_buffer_new_from_pixbuf   (GdkPixbuf        *pixbuf,
                                                  const gchar      *name,
                                                  gint              offset_x,
                                                  gint              offset_y);

gint               pika_buffer_get_width         (PikaBuffer       *buffer);
gint               pika_buffer_get_height        (PikaBuffer       *buffer);
const Babl       * pika_buffer_get_format        (PikaBuffer       *buffer);

GeglBuffer       * pika_buffer_get_buffer        (PikaBuffer       *buffer);

void               pika_buffer_set_resolution    (PikaBuffer       *buffer,
                                                  gdouble           resolution_x,
                                                  gdouble           resolution_y);
gboolean           pika_buffer_get_resolution    (PikaBuffer       *buffer,
                                                  gdouble          *resolution_x,
                                                  gdouble          *resolution_y);

void               pika_buffer_set_unit          (PikaBuffer       *buffer,
                                                  PikaUnit          unit);
PikaUnit           pika_buffer_get_unit          (PikaBuffer       *buffer);

void               pika_buffer_set_color_profile (PikaBuffer       *buffer,
                                                  PikaColorProfile *profile);
PikaColorProfile * pika_buffer_get_color_profile (PikaBuffer       *buffer);


#endif /* __PIKA_BUFFER_H__ */
