/* Pika - Photo and Image Kooker Application
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "libpikabase/pikabase.h"

#include "core-types.h"

#include "pikaimage.h"
#include "pikachannel.h"
#include "pikachannelpropundo.h"


static void   pika_channel_prop_undo_constructed (GObject             *object);

static void   pika_channel_prop_undo_pop         (PikaUndo            *undo,
                                                  PikaUndoMode         undo_mode,
                                                  PikaUndoAccumulator *accum);


G_DEFINE_TYPE (PikaChannelPropUndo, pika_channel_prop_undo, PIKA_TYPE_ITEM_UNDO)

#define parent_class pika_channel_prop_undo_parent_class


static void
pika_channel_prop_undo_class_init (PikaChannelPropUndoClass *klass)
{
  GObjectClass  *object_class = G_OBJECT_CLASS (klass);
  PikaUndoClass *undo_class   = PIKA_UNDO_CLASS (klass);

  object_class->constructed = pika_channel_prop_undo_constructed;

  undo_class->pop           = pika_channel_prop_undo_pop;
}

static void
pika_channel_prop_undo_init (PikaChannelPropUndo *undo)
{
}

static void
pika_channel_prop_undo_constructed (GObject *object)
{
  PikaChannelPropUndo *channel_prop_undo = PIKA_CHANNEL_PROP_UNDO (object);
  PikaChannel         *channel;

  G_OBJECT_CLASS (parent_class)->constructed (object);

  pika_assert (PIKA_IS_CHANNEL (PIKA_ITEM_UNDO (object)->item));

  channel = PIKA_CHANNEL (PIKA_ITEM_UNDO (object)->item);

  switch (PIKA_UNDO (object)->undo_type)
    {
    case PIKA_UNDO_CHANNEL_COLOR:
      pika_channel_get_color (channel, &channel_prop_undo->color);
      break;

    default:
      g_return_if_reached ();
    }
}

static void
pika_channel_prop_undo_pop (PikaUndo            *undo,
                            PikaUndoMode         undo_mode,
                            PikaUndoAccumulator *accum)
{
  PikaChannelPropUndo *channel_prop_undo = PIKA_CHANNEL_PROP_UNDO (undo);
  PikaChannel         *channel           = PIKA_CHANNEL (PIKA_ITEM_UNDO (undo)->item);

  PIKA_UNDO_CLASS (parent_class)->pop (undo, undo_mode, accum);

  switch (undo->undo_type)
    {
    case PIKA_UNDO_CHANNEL_COLOR:
      {
        PikaRGB color;

        pika_channel_get_color (channel, &color);
        pika_channel_set_color (channel, &channel_prop_undo->color, FALSE);
        channel_prop_undo->color = color;
      }
      break;

    default:
      g_return_if_reached ();
    }
}
