/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikadataloaderfactory.h
 * Copyright (C) 2001-2018 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_DATA_LOADER_FACTORY_H__
#define __PIKA_DATA_LOADER_FACTORY_H__


#include "pikadatafactory.h"


typedef GList * (* PikaDataLoadFunc) (PikaContext   *context,
                                      GFile         *file,
                                      GInputStream  *input,
                                      GError       **error);


#define PIKA_TYPE_DATA_LOADER_FACTORY            (pika_data_loader_factory_get_type ())
#define PIKA_DATA_LOADER_FACTORY(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_DATA_LOADER_FACTORY, PikaDataLoaderFactory))
#define PIKA_DATA_LOADER_FACTORY_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_DATA_LOADER_FACTORY, PikaDataLoaderFactoryClass))
#define PIKA_IS_DATA_LOADER_FACTORY(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_DATA_LOADER_FACTORY))
#define PIKA_IS_DATA_LOADER_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_DATA_LOADER_FACTORY))
#define PIKA_DATA_LOADER_FACTORY_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_DATA_LOADER_FACTORY, PikaDataLoaderFactoryClass))


typedef struct _PikaDataLoaderFactoryPrivate PikaDataLoaderFactoryPrivate;
typedef struct _PikaDataLoaderFactoryClass   PikaDataLoaderFactoryClass;

struct _PikaDataLoaderFactory
{
  PikaDataFactory               parent_instance;

  PikaDataLoaderFactoryPrivate *priv;
};

struct _PikaDataLoaderFactoryClass
{
  PikaDataFactoryClass  parent_class;
};


GType             pika_data_loader_factory_get_type     (void) G_GNUC_CONST;

PikaDataFactory * pika_data_loader_factory_new          (Pika                    *pika,
                                                         GType                    data_type,
                                                         const gchar             *path_property_name,
                                                         const gchar             *writable_property_name,
                                                         const gchar             *ext_property_name,
                                                         PikaDataNewFunc          new_func,
                                                         PikaDataGetStandardFunc  get_standard_func);

void              pika_data_loader_factory_add_loader   (PikaDataFactory         *factory,
                                                         const gchar             *name,
                                                         PikaDataLoadFunc         load_func,
                                                         const gchar             *extension,
                                                         gboolean                 writable);
void              pika_data_loader_factory_add_fallback (PikaDataFactory         *factory,
                                                         const gchar             *name,
                                                         PikaDataLoadFunc         load_func);


#endif  /*  __PIKA_DATA_LOADER_FACTORY_H__  */
