/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "core-types.h"

#include "pikadrawable.h"
#include "pikadrawable-equalize.h"
#include "pikadrawable-histogram.h"
#include "pikadrawable-operation.h"
#include "pikahistogram.h"
#include "pikaimage.h"
#include "pikaselection.h"

#include "pika-intl.h"


void
pika_drawable_equalize (PikaDrawable *drawable,
                        gboolean      mask_only)
{
  PikaImage     *image;
  PikaChannel   *selection;
  PikaHistogram *histogram;
  GeglNode      *equalize;

  g_return_if_fail (PIKA_IS_DRAWABLE (drawable));
  g_return_if_fail (pika_item_is_attached (PIKA_ITEM (drawable)));

  image = pika_item_get_image (PIKA_ITEM (drawable));
  selection = pika_image_get_mask (image);

  histogram = pika_histogram_new (FALSE);
  pika_drawable_calculate_histogram (drawable, histogram, FALSE);

  equalize = gegl_node_new_child (NULL,
                                  "operation", "pika:equalize",
                                  "histogram", histogram,
                                  NULL);

  if (! mask_only)
    pika_selection_suspend (PIKA_SELECTION (selection));

  pika_drawable_apply_operation (drawable, NULL,
                                 C_("undo-type", "Equalize"),
                                 equalize);

  if (! mask_only)
    pika_selection_resume (PIKA_SELECTION (selection));

  g_object_unref (equalize);
  g_object_unref (histogram);
}
