/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_DRAWABLE_FILL_H__
#define __PIKA_DRAWABLE_FILL_H__


/*  Lowlevel API that is used for initializing entire drawables and
 *  buffers before they are used in images, they don't push an undo.
 */

void       pika_drawable_fill              (PikaDrawable        *drawable,
                                            PikaContext         *context,
                                            PikaFillType         fill_type);
void       pika_drawable_fill_buffer       (PikaDrawable        *drawable,
                                            GeglBuffer          *buffer,
                                            const PikaRGB       *color,
                                            PikaPattern         *pattern,
                                            gint                 pattern_offset_x,
                                            gint                 pattern_offset_y);


/*  Proper API that is used for actual editing (not just initializing)
 */

void       pika_drawable_fill_boundary     (PikaDrawable        *drawable,
                                            PikaFillOptions     *options,
                                            const PikaBoundSeg  *bound_segs,
                                            gint                 n_bound_segs,
                                            gint                 offset_x,
                                            gint                 offset_y,
                                            gboolean             push_undo);

gboolean   pika_drawable_fill_vectors      (PikaDrawable        *drawable,
                                            PikaFillOptions     *options,
                                            PikaVectors         *vectors,
                                            gboolean             push_undo,
                                            GError             **error);

void       pika_drawable_fill_scan_convert (PikaDrawable        *drawable,
                                            PikaFillOptions     *options,
                                            PikaScanConvert     *scan_convert,
                                            gboolean             push_undo);


#endif /* __PIKA_DRAWABLE_FILL_H__ */
