/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "core-types.h"

#include "operations/pikalevelsconfig.h"

#include "pikadrawable.h"
#include "pikadrawable-histogram.h"
#include "pikadrawable-levels.h"
#include "pikadrawable-operation.h"
#include "pikahistogram.h"
#include "pikaprogress.h"

#include "pika-intl.h"


/*  public functions  */

void
pika_drawable_levels_stretch (PikaDrawable *drawable,
                              PikaProgress *progress)
{
  PikaLevelsConfig *config;
  PikaHistogram    *histogram;
  GeglNode         *levels;

  g_return_if_fail (PIKA_IS_DRAWABLE (drawable));
  g_return_if_fail (pika_item_is_attached (PIKA_ITEM (drawable)));
  g_return_if_fail (progress == NULL || PIKA_IS_PROGRESS (progress));

  if (! pika_item_mask_intersect (PIKA_ITEM (drawable), NULL, NULL, NULL, NULL))
    return;

  config = g_object_new (PIKA_TYPE_LEVELS_CONFIG, NULL);

  histogram = pika_histogram_new (FALSE);
  pika_drawable_calculate_histogram (drawable, histogram, FALSE);

  pika_levels_config_stretch (config, histogram,
                              pika_drawable_is_rgb (drawable));

  g_object_unref (histogram);

  levels = g_object_new (GEGL_TYPE_NODE,
                         "operation", "pika:levels",
                         NULL);

  gegl_node_set (levels,
                 "config", config,
                 NULL);

  pika_drawable_apply_operation (drawable, progress, _("Levels"),
                                 levels);

  g_object_unref (levels);
  g_object_unref (config);
}
