/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <string.h>

#include <cairo.h>
#include <gegl.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "core-types.h"

#include "pika.h"
#include "pikacontext.h"
#include "pikadrawable.h"
#include "pikadrawable-offset.h"
#include "pikadrawable-operation.h"

#include "pika-intl.h"


void
pika_drawable_offset (PikaDrawable   *drawable,
                      PikaContext    *context,
                      gboolean        wrap_around,
                      PikaOffsetType  fill_type,
                      gint            offset_x,
                      gint            offset_y)
{
  GeglNode *node;
  gint      width;
  gint      height;

  g_return_if_fail (PIKA_IS_DRAWABLE (drawable));
  g_return_if_fail (PIKA_IS_CONTEXT (context));

  if (! pika_item_mask_intersect (PIKA_ITEM (drawable),
                                  NULL, NULL, &width, &height))
    {
      return;
    }

  if (wrap_around)
    fill_type = PIKA_OFFSET_WRAP_AROUND;

  if (fill_type == PIKA_OFFSET_WRAP_AROUND)
    {
      offset_x %= width;
      offset_y %= height;
    }

  if (offset_x == 0 && offset_y == 0)
    return;

  node = gegl_node_new_child (NULL,
                              "operation", "pika:offset",
                              "context", context,
                              "type",    fill_type,
                              "x",       offset_x,
                              "y",       offset_y,
                              NULL);

  pika_drawable_apply_operation (drawable, NULL,
                                 C_("undo-type", "Offset Drawable"),
                                 node);

  g_object_unref (node);
}
