/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "core-types.h"

#include "gegl/pika-gegl-utils.h"

#include "pika-memsize.h"
#include "pikaimage.h"
#include "pikadrawable.h"
#include "pikadrawablepropundo.h"


enum
{
  PROP_0
};


static void   pika_drawable_prop_undo_constructed  (GObject             *object);
static void   pika_drawable_prop_undo_set_property (GObject             *object,
                                                    guint                property_id,
                                                    const GValue        *value,
                                                    GParamSpec          *pspec);
static void   pika_drawable_prop_undo_get_property (GObject             *object,
                                                    guint                property_id,
                                                    GValue              *value,
                                                    GParamSpec          *pspec);

static void   pika_drawable_prop_undo_pop          (PikaUndo            *undo,
                                                    PikaUndoMode         undo_mode,
                                                    PikaUndoAccumulator *accum);


G_DEFINE_TYPE (PikaDrawablePropUndo, pika_drawable_prop_undo,
               PIKA_TYPE_ITEM_UNDO)

#define parent_class pika_drawable_prop_undo_parent_class


static void
pika_drawable_prop_undo_class_init (PikaDrawablePropUndoClass *klass)
{
  GObjectClass  *object_class = G_OBJECT_CLASS (klass);
  PikaUndoClass *undo_class   = PIKA_UNDO_CLASS (klass);

  object_class->constructed  = pika_drawable_prop_undo_constructed;
  object_class->set_property = pika_drawable_prop_undo_set_property;
  object_class->get_property = pika_drawable_prop_undo_get_property;

  undo_class->pop            = pika_drawable_prop_undo_pop;
}

static void
pika_drawable_prop_undo_init (PikaDrawablePropUndo *undo)
{
}

static void
pika_drawable_prop_undo_constructed (GObject *object)
{
  PikaDrawablePropUndo *drawable_prop_undo = PIKA_DRAWABLE_PROP_UNDO (object);
  PikaDrawable         *drawable;

  G_OBJECT_CLASS (parent_class)->constructed (object);

  pika_assert (PIKA_IS_DRAWABLE (PIKA_ITEM_UNDO (object)->item));

  drawable = PIKA_DRAWABLE (PIKA_ITEM_UNDO (object)->item);

  switch (PIKA_UNDO (object)->undo_type)
    {
    case PIKA_UNDO_DRAWABLE_FORMAT:
      drawable_prop_undo->format = pika_drawable_get_format (drawable);
      break;

    default:
      g_return_if_reached ();
   }
}

static void
pika_drawable_prop_undo_set_property (GObject      *object,
                                      guint         property_id,
                                      const GValue *value,
                                      GParamSpec   *pspec)
{
  switch (property_id)
    {
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
      break;
    }
}

static void
pika_drawable_prop_undo_get_property (GObject    *object,
                                      guint       property_id,
                                      GValue     *value,
                                      GParamSpec *pspec)
{
  switch (property_id)
    {
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
      break;
    }
}

static void
pika_drawable_prop_undo_pop (PikaUndo            *undo,
                             PikaUndoMode         undo_mode,
                             PikaUndoAccumulator *accum)
{
  PikaDrawablePropUndo *drawable_prop_undo = PIKA_DRAWABLE_PROP_UNDO (undo);
  PikaDrawable         *drawable;

  drawable = PIKA_DRAWABLE (PIKA_ITEM_UNDO (undo)->item);

  PIKA_UNDO_CLASS (parent_class)->pop (undo, undo_mode, accum);

  switch (undo->undo_type)
    {
    case PIKA_UNDO_DRAWABLE_FORMAT:
      {
        const Babl *format = pika_drawable_get_format (drawable);

        pika_drawable_set_format (drawable,
                                  drawable_prop_undo->format,
                                  TRUE, FALSE);

        drawable_prop_undo->format = format;
      }
      break;

    default:
      g_return_if_reached ();
    }
}
