/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * pikaextension.h
 * Copyright (C) 2018 Jehan <jehan@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_EXTENSION_H__
#define __PIKA_EXTENSION_H__


#include "core/pikaobject.h"


#define PIKA_TYPE_EXTENSION            (pika_extension_get_type ())
#define PIKA_EXTENSION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_EXTENSION, PikaExtension))
#define PIKA_EXTENSION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_EXTENSION, PikaExtensionClass))
#define PIKA_IS_EXTENSION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_EXTENSION))
#define PIKA_IS_EXTENSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_EXTENSION))
#define PIKA_EXTENSION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_EXTENSION, PikaExtensionClass))

typedef struct _PikaExtensionPrivate  PikaExtensionPrivate;
typedef struct _PikaExtensionClass    PikaExtensionClass;

struct _PikaExtension
{
  PikaObject            parent_instance;

  PikaExtensionPrivate *p;
};

struct _PikaExtensionClass
{
  PikaObjectClass       parent_class;
};


GType           pika_extension_get_type (void) G_GNUC_CONST;

PikaExtension * pika_extension_new                     (const gchar    *dir,
                                                        gboolean        writable);
const gchar   * pika_extension_get_name                (PikaExtension  *extension);
const gchar   * pika_extension_get_comment             (PikaExtension  *extension);
const gchar   * pika_extension_get_description         (PikaExtension  *extension);
GdkPixbuf     * pika_extension_get_screenshot          (PikaExtension  *extension,
                                                        gint            width,
                                                        gint            height,
                                                        const gchar   **caption);
const gchar   * pika_extension_get_path                (PikaExtension  *extension);

gchar         * pika_extension_get_markup_description  (PikaExtension  *extension);

gboolean        pika_extension_load                    (PikaExtension  *extension,
                                                        GError        **error);
gboolean        pika_extension_run                     (PikaExtension  *extension,
                                                        GError        **error);
void            pika_extension_stop                    (PikaExtension  *extension);

GList         * pika_extension_get_brush_paths         (PikaExtension  *extension);
GList         * pika_extension_get_dynamics_paths      (PikaExtension  *extension);
GList         * pika_extension_get_mypaint_brush_paths (PikaExtension  *extension);
GList         * pika_extension_get_pattern_paths       (PikaExtension  *extension);
GList         * pika_extension_get_gradient_paths      (PikaExtension  *extension);
GList         * pika_extension_get_palette_paths       (PikaExtension  *extension);
GList         * pika_extension_get_tool_preset_paths   (PikaExtension  *extension);
GList         * pika_extension_get_splash_paths        (PikaExtension  *extension);
GList         * pika_extension_get_theme_paths         (PikaExtension  *extension);
GList         * pika_extension_get_plug_in_paths       (PikaExtension  *extension);

gint            pika_extension_cmp                     (PikaExtension  *extension1,
                                                        PikaExtension  *extension2);
gint            pika_extension_id_cmp                  (PikaExtension  *extension1,
                                                        const gchar    *id);

#endif  /* __PIKA_EXTENSION_H__ */
