/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * PikaGrid
 * Copyright (C) 2003  Henrik Brix Andersen <brix@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_GRID_H__
#define __PIKA_GRID_H__


#include "pikaobject.h"


#define PIKA_TYPE_GRID            (pika_grid_get_type ())
#define PIKA_GRID(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_GRID, PikaGrid))
#define PIKA_GRID_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_GRID, PikaGridClass))
#define PIKA_IS_GRID(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_GRID))
#define PIKA_IS_GRID_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_GRID))
#define PIKA_GRID_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_GRID, PikaGridClass))


typedef struct _PikaGridClass  PikaGridClass;

struct _PikaGrid
{
  PikaObject     parent_instance;

  PikaGridStyle  style;
  PikaRGB        fgcolor;
  PikaRGB        bgcolor;
  gdouble        xspacing;
  gdouble        yspacing;
  PikaUnit       spacing_unit;
  gdouble        xoffset;
  gdouble        yoffset;
  PikaUnit       offset_unit;
};


struct _PikaGridClass
{
  PikaObjectClass  parent_class;
};


GType          pika_grid_get_type               (void) G_GNUC_CONST;

PikaGridStyle  pika_grid_get_style              (PikaGrid           *grid);

void           pika_grid_get_fgcolor            (PikaGrid           *grid,
                                                 PikaRGB            *fgcolor);
void           pika_grid_get_bgcolor            (PikaGrid           *grid,
                                                 PikaRGB            *bgcolor);

void           pika_grid_get_spacing            (PikaGrid           *grid,
                                                 gdouble            *xspacing,
                                                 gdouble            *yspacing);
void           pika_grid_get_offset             (PikaGrid           *grid,
                                                 gdouble            *xoffset,
                                                 gdouble            *yoffset);

const gchar  * pika_grid_parasite_name          (void) G_GNUC_CONST;
PikaParasite * pika_grid_to_parasite            (PikaGrid           *grid);
PikaGrid     * pika_grid_from_parasite          (const PikaParasite *parasite);


#endif /* __PIKA_GRID_H__ */
