/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * PikaGrid
 * Copyright (C) 2003  Henrik Brix Andersen <brix@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "libpikaconfig/pikaconfig.h"

#include "core-types.h"

#include "pikagrid.h"
#include "pikaimage.h"
#include "pikaimage-grid.h"
#include "pikaimage-private.h"
#include "pikaimage-undo-push.h"

#include "pika-intl.h"


PikaGrid *
pika_image_get_grid (PikaImage *image)
{
  g_return_val_if_fail (PIKA_IS_IMAGE (image), NULL);

  return PIKA_IMAGE_GET_PRIVATE (image)->grid;
}

void
pika_image_set_grid (PikaImage *image,
                     PikaGrid  *grid,
                     gboolean   push_undo)
{
  PikaImagePrivate *private;

  g_return_if_fail (PIKA_IS_IMAGE (image));
  g_return_if_fail (PIKA_IS_GRID (grid));

  private = PIKA_IMAGE_GET_PRIVATE (image);

  if (pika_config_is_equal_to (PIKA_CONFIG (private->grid), PIKA_CONFIG (grid)))
    return;

  if (push_undo)
    pika_image_undo_push_image_grid (image,
                                     C_("undo-type", "Grid"), private->grid);

  pika_config_sync (G_OBJECT (grid), G_OBJECT (private->grid), 0);
}
