/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattisbvf
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_IMAGE_SNAP_H__
#define __PIKA_IMAGE_SNAP_H__


typedef struct _PikaImageSnapData
{
  PikaAlignmentType  snapped_side_horizontal; /* if PIKA_ARRANGE_HFILL then no snapping happen */
  PikaLayer         *snapped_layer_horizontal;

  PikaAlignmentType  snapped_side_vertical; /* if PIKA_ARRANGE_HFILL then no snapping happen */
  PikaLayer         *snapped_layer_vertical;

  PikaAlignmentType  equidistance_side_horizontal;
  PikaLayer         *near_layer_horizontal1;
  PikaLayer         *near_layer_horizontal2;

  PikaAlignmentType  equidistance_side_vertical;
  PikaLayer         *near_layer_vertical1;
  PikaLayer         *near_layer_vertical2;
} PikaImageSnapData;


gboolean    pika_image_snap_x         (PikaImage         *image,
                                       PikaImageSnapData *snapping_data,
                                       gdouble            x,
                                       gdouble           *tx,
                                       gdouble            epsilon_x,
                                       gboolean           snap_to_guides,
                                       gboolean           snap_to_grid,
                                       gboolean           snap_to_canvas,
                                       gboolean           snap_to_bbox,
                                       gboolean           snap_to_equidistance,
                                       PikaAlignmentType  alignment_side);
gboolean    pika_image_snap_y         (PikaImage         *image,
                                       PikaImageSnapData *snapping_data,
                                       gdouble            y,
                                       gdouble           *ty,
                                       gdouble            epsilon_y,
                                       gboolean           snap_to_guides,
                                       gboolean           snap_to_grid,
                                       gboolean           snap_to_canvas,
                                       gboolean           snap_to_bbox,
                                       gboolean           snap_to_equidistance,
                                       PikaAlignmentType  alignment_side);
gboolean    pika_image_snap_point     (PikaImage         *image,
                                       gdouble            x,
                                       gdouble            y,
                                       gdouble           *tx,
                                       gdouble           *ty,
                                       gdouble            epsilon_x,
                                       gdouble            epsilon_y,
                                       gboolean           snap_to_guides,
                                       gboolean           snap_to_grid,
                                       gboolean           snap_to_canvas,
                                       gboolean           snap_to_vectors,
                                       gboolean           snap_to_bbox,
                                       gboolean           show_all);
gboolean   pika_image_snap_rectangle  (PikaImage         *image,
                                       PikaImageSnapData *snapping_data,
                                       gdouble            x1,
                                       gdouble            y1,
                                       gdouble            x2,
                                       gdouble            y2,
                                       gdouble           *tx1,
                                       gdouble           *ty1,
                                       gdouble            epsilon_x,
                                       gdouble            epsilon_y,
                                       gboolean           snap_to_guides,
                                       gboolean           snap_to_grid,
                                       gboolean           snap_to_canvas,
                                       gboolean           snap_to_vectors,
                                       gboolean           snap_to_bbox,
                                       gboolean           snap_to_equidistance);


#endif /* __PIKA_IMAGE_SNAP_H__ */
