/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * pikaitemstack.h
 * Copyright (C) 2008 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_ITEM_STACK_H__
#define __PIKA_ITEM_STACK_H__

#include "pikafilterstack.h"


#define PIKA_TYPE_ITEM_STACK            (pika_item_stack_get_type ())
#define PIKA_ITEM_STACK(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_ITEM_STACK, PikaItemStack))
#define PIKA_ITEM_STACK_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_ITEM_STACK, PikaItemStackClass))
#define PIKA_IS_ITEM_STACK(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_ITEM_STACK))
#define PIKA_IS_ITEM_STACK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_ITEM_STACK))


typedef struct _PikaItemStackClass PikaItemStackClass;

struct _PikaItemStack
{
  PikaFilterStack  parent_instance;
};

struct _PikaItemStackClass
{
  PikaFilterStackClass  parent_class;
};


GType           pika_item_stack_get_type            (void) G_GNUC_CONST;
PikaContainer * pika_item_stack_new                 (GType          item_type);

gint            pika_item_stack_get_n_items         (PikaItemStack *stack);
gboolean        pika_item_stack_is_flat             (PikaItemStack *stack);
GList         * pika_item_stack_get_item_iter       (PikaItemStack *stack);
GList         * pika_item_stack_get_item_list       (PikaItemStack *stack);
PikaItem      * pika_item_stack_get_item_by_tattoo  (PikaItemStack *stack,
                                                     PikaTattoo     tattoo);
PikaItem      * pika_item_stack_get_item_by_path    (PikaItemStack *stack,
                                                     GList         *path);
PikaItem      * pika_item_stack_get_parent_by_path  (PikaItemStack *stack,
                                                     GList         *path,
                                                     gint          *index);

void            pika_item_stack_invalidate_previews (PikaItemStack *stack);
void            pika_item_stack_profile_changed     (PikaItemStack *stack);


#endif  /*  __PIKA_ITEM_STACK_H__  */
