/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * pikaitemtree.h
 * Copyright (C) 2010 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_ITEM_TREE_H__
#define __PIKA_ITEM_TREE_H__


#include "pikaobject.h"


#define PIKA_TYPE_ITEM_TREE            (pika_item_tree_get_type ())
#define PIKA_ITEM_TREE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_ITEM_TREE, PikaItemTree))
#define PIKA_ITEM_TREE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_ITEM_TREE, PikaItemTreeClass))
#define PIKA_IS_ITEM_TREE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_ITEM_TREE))
#define PIKA_IS_ITEM_TREE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_ITEM_TREE))


typedef struct _PikaItemTreeClass PikaItemTreeClass;

struct _PikaItemTree
{
  PikaObject     parent_instance;

  PikaContainer *container;
};

struct _PikaItemTreeClass
{
  PikaObjectClass  parent_class;
};


GType          pika_item_tree_get_type         (void) G_GNUC_CONST;
PikaItemTree * pika_item_tree_new              (PikaImage     *image,
                                                GType          container_type,
                                                GType          item_type);

PikaItem     * pika_item_tree_get_active_item  (PikaItemTree  *tree);
void           pika_item_tree_set_active_item  (PikaItemTree  *tree,
                                                PikaItem      *item);

GList        * pika_item_tree_get_selected_items (PikaItemTree  *tree);
void           pika_item_tree_set_selected_items (PikaItemTree  *tree,
                                                  GList         *items);

PikaItem     * pika_item_tree_get_item_by_name (PikaItemTree  *tree,
                                                const gchar   *name);

gboolean       pika_item_tree_get_insert_pos   (PikaItemTree  *tree,
                                                PikaItem      *item,
                                                PikaItem     **parent,
                                                gint          *position);

void           pika_item_tree_add_item         (PikaItemTree  *tree,
                                                PikaItem      *item,
                                                PikaItem      *parent,
                                                gint           position);
GList        * pika_item_tree_remove_item      (PikaItemTree  *tree,
                                                PikaItem      *item,
                                                GList         *new_selected);

gboolean       pika_item_tree_reorder_item     (PikaItemTree  *tree,
                                                PikaItem      *item,
                                                PikaItem      *new_parent,
                                                gint           new_index,
                                                gboolean       push_undo,
                                                const gchar   *undo_desc);

void           pika_item_tree_rename_item      (PikaItemTree  *tree,
                                                PikaItem      *item,
                                                const gchar   *new_name,
                                                gboolean       push_undo,
                                                const gchar   *undo_desc);


#endif  /*  __PIKA_ITEM_TREE_H__  */
