/* Pika - Photo and Image Kooker Application
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "core-types.h"

#include "pikalayer.h"
#include "pikalayermaskpropundo.h"


static void   pika_layer_mask_prop_undo_constructed (GObject             *object);

static void   pika_layer_mask_prop_undo_pop         (PikaUndo            *undo,
                                                     PikaUndoMode         undo_mode,
                                                     PikaUndoAccumulator *accum);


G_DEFINE_TYPE (PikaLayerMaskPropUndo, pika_layer_mask_prop_undo,
               PIKA_TYPE_ITEM_UNDO)

#define parent_class pika_layer_mask_prop_undo_parent_class


static void
pika_layer_mask_prop_undo_class_init (PikaLayerMaskPropUndoClass *klass)
{
  GObjectClass  *object_class = G_OBJECT_CLASS (klass);
  PikaUndoClass *undo_class   = PIKA_UNDO_CLASS (klass);

  object_class->constructed = pika_layer_mask_prop_undo_constructed;

  undo_class->pop           = pika_layer_mask_prop_undo_pop;
}

static void
pika_layer_mask_prop_undo_init (PikaLayerMaskPropUndo *undo)
{
}

static void
pika_layer_mask_prop_undo_constructed (GObject *object)
{
  PikaLayerMaskPropUndo *layer_mask_prop_undo;
  PikaLayer             *layer;

  layer_mask_prop_undo = PIKA_LAYER_MASK_PROP_UNDO (object);

  G_OBJECT_CLASS (parent_class)->constructed (object);

  pika_assert (PIKA_IS_LAYER (PIKA_ITEM_UNDO (object)->item));

  layer = PIKA_LAYER (PIKA_ITEM_UNDO (object)->item);

  switch (PIKA_UNDO (object)->undo_type)
    {
    case PIKA_UNDO_LAYER_MASK_APPLY:
      layer_mask_prop_undo->apply = pika_layer_get_apply_mask (layer);
      break;

    case PIKA_UNDO_LAYER_MASK_SHOW:
      layer_mask_prop_undo->show = pika_layer_get_show_mask (layer);
      break;

    default:
      g_return_if_reached ();
    }
}

static void
pika_layer_mask_prop_undo_pop (PikaUndo            *undo,
                               PikaUndoMode         undo_mode,
                               PikaUndoAccumulator *accum)
{
  PikaLayerMaskPropUndo *layer_mask_prop_undo = PIKA_LAYER_MASK_PROP_UNDO (undo);
  PikaLayer             *layer                = PIKA_LAYER (PIKA_ITEM_UNDO (undo)->item);

  PIKA_UNDO_CLASS (parent_class)->pop (undo, undo_mode, accum);

  switch (undo->undo_type)
    {
    case PIKA_UNDO_LAYER_MASK_APPLY:
      {
        gboolean apply;

        apply = pika_layer_get_apply_mask (layer);
        pika_layer_set_apply_mask (layer, layer_mask_prop_undo->apply, FALSE);
        layer_mask_prop_undo->apply = apply;
      }
      break;

    case PIKA_UNDO_LAYER_MASK_SHOW:
      {
        gboolean show;

        show = pika_layer_get_show_mask (layer);
        pika_layer_set_show_mask (layer, layer_mask_prop_undo->show, FALSE);
        layer_mask_prop_undo->show = show;
      }
      break;

    default:
      g_return_if_reached ();
    }
}
