/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * pikalist.h
 * Copyright (C) 2001-2016 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_LIST_H__
#define __PIKA_LIST_H__


#include "pikacontainer.h"


#define PIKA_TYPE_LIST            (pika_list_get_type ())
#define PIKA_LIST(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_LIST, PikaList))
#define PIKA_LIST_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_LIST, PikaListClass))
#define PIKA_IS_LIST(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_LIST))
#define PIKA_IS_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_LIST))
#define PIKA_LIST_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_LIST, PikaListClass))


typedef struct _PikaListClass PikaListClass;

struct _PikaList
{
  PikaContainer  parent_instance;

  GQueue        *queue;
  gboolean       unique_names;
  GCompareFunc   sort_func;
  gboolean       append;
};

struct _PikaListClass
{
  PikaContainerClass  parent_class;
};


GType           pika_list_get_type      (void) G_GNUC_CONST;

PikaContainer * pika_list_new           (GType         children_type,
                                         gboolean      unique_names);
PikaContainer * pika_list_new_weak      (GType         children_type,
                                         gboolean      unique_names);

void            pika_list_reverse       (PikaList     *list);
void            pika_list_set_sort_func (PikaList     *list,
                                         GCompareFunc  sort_func);
GCompareFunc    pika_list_get_sort_func (PikaList     *list);
void            pika_list_sort          (PikaList     *list,
                                         GCompareFunc  sort_func);
void            pika_list_sort_by_name  (PikaList     *list);


#endif  /* __PIKA_LIST_H__ */
