/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_OBJECT_H__
#define __PIKA_OBJECT_H__


#define PIKA_TYPE_OBJECT            (pika_object_get_type ())
#define PIKA_OBJECT(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_OBJECT, PikaObject))
#define PIKA_OBJECT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_OBJECT, PikaObjectClass))
#define PIKA_IS_OBJECT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_OBJECT))
#define PIKA_IS_OBJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_OBJECT))
#define PIKA_OBJECT_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_OBJECT, PikaObjectClass))


typedef struct _PikaObjectPrivate  PikaObjectPrivate;
typedef struct _PikaObjectClass    PikaObjectClass;

struct _PikaObject
{
  GObject            parent_instance;

  PikaObjectPrivate *p;
};

struct _PikaObjectClass
{
  GObjectClass  parent_class;

  /*  signals  */
  void    (* disconnect)   (PikaObject *object);
  void    (* name_changed) (PikaObject *object);

  /*  virtual functions  */
  gint64  (* get_memsize)  (PikaObject *object,
                            gint64     *gui_size);
};


GType         pika_object_get_type        (void) G_GNUC_CONST;

void          pika_object_set_name        (PikaObject       *object,
                                           const gchar      *name);
void          pika_object_set_name_safe   (PikaObject       *object,
                                           const gchar      *name);
void          pika_object_set_static_name (PikaObject       *object,
                                           const gchar      *name);
void          pika_object_take_name       (PikaObject       *object,
                                           gchar            *name);
const gchar * pika_object_get_name        (gconstpointer     object);
void          pika_object_name_changed    (PikaObject       *object);
void          pika_object_name_free       (PikaObject       *object);

gint          pika_object_name_collate    (PikaObject       *object1,
                                           PikaObject       *object2);
gint64        pika_object_get_memsize     (PikaObject       *object,
                                           gint64           *gui_size);

G_DEFINE_AUTOPTR_CLEANUP_FUNC (PikaObject, g_object_unref);

#endif  /* __PIKA_OBJECT_H__ */
