/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_OBJECT_QUEUE_H__
#define __PIKA_OBJECT_QUEUE_H__


#include "pikasubprogress.h"


#define PIKA_TYPE_OBJECT_QUEUE            (pika_object_queue_get_type ())
#define PIKA_OBJECT_QUEUE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_OBJECT_QUEUE, PikaObjectQueue))
#define PIKA_OBJECT_QUEUE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_OBJECT_QUEUE, PikaObjectQueueClass))
#define PIKA_IS_OBJECT_QUEUE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_OBJECT_QUEUE))
#define PIKA_IS_OBJECT_QUEUE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_OBJECT_QUEUE))
#define PIKA_OBJECT_QUEUE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_OBJECT_QUEUE, PikaObjectQueueClass))


typedef struct _PikaObjectQueueClass PikaObjectQueueClass;

struct _PikaObjectQueue
{
  PikaSubProgress  parent_instance;

  GQueue           items;
  gint64           processed_memsize;
  gint64           total_memsize;
};

struct _PikaObjectQueueClass
{
  PikaSubProgressClass  parent_class;
};


GType             pika_object_queue_get_type       (void) G_GNUC_CONST;

PikaObjectQueue * pika_object_queue_new            (PikaProgress    *progress);

void              pika_object_queue_clear          (PikaObjectQueue *queue);

void              pika_object_queue_push           (PikaObjectQueue *queue,
                                                    gpointer         object);
void              pika_object_queue_push_container (PikaObjectQueue *queue,
                                                    PikaContainer   *container);
void              pika_object_queue_push_list      (PikaObjectQueue *queue,
                                                    GList           *list);

gpointer          pika_object_queue_pop            (PikaObjectQueue *queue);


#endif /* __PIKA_OBJECT_QUEUE_H__ */
