/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __APP_PIKA_PARAM_SPECS_H__
#define __APP_PIKA_PARAM_SPECS_H__


/*
 * PIKA_TYPE_PARAM_STRING
 */

#define PIKA_TYPE_PARAM_STRING           (pika_param_string_get_type ())
#define PIKA_PARAM_SPEC_STRING(pspec)    (G_TYPE_CHECK_INSTANCE_CAST ((pspec), PIKA_TYPE_PARAM_STRING, PikaParamSpecString))
#define PIKA_IS_PARAM_SPEC_STRING(pspec) (G_TYPE_CHECK_INSTANCE_TYPE ((pspec), PIKA_TYPE_PARAM_STRING))

typedef struct _PikaParamSpecString PikaParamSpecString;

struct _PikaParamSpecString
{
  GParamSpecString parent_instance;

  guint            allow_non_utf8 : 1;
  guint            non_empty      : 1;
};

GType        pika_param_string_get_type (void) G_GNUC_CONST;

GParamSpec * pika_param_spec_string     (const gchar *name,
                                         const gchar *nick,
                                         const gchar *blurb,
                                         gboolean     allow_non_utf8,
                                         gboolean     null_ok,
                                         gboolean     non_empty,
                                         const gchar *default_value,
                                         GParamFlags  flags);


/*
 * PIKA_TYPE_PARAM_ENUM
 */

#define PIKA_TYPE_PARAM_ENUM           (pika_param_enum_get_type ())
#define PIKA_PARAM_SPEC_ENUM(pspec)    (G_TYPE_CHECK_INSTANCE_CAST ((pspec), PIKA_TYPE_PARAM_ENUM, PikaParamSpecEnum))

#define PIKA_IS_PARAM_SPEC_ENUM(pspec) (G_TYPE_CHECK_INSTANCE_TYPE ((pspec), PIKA_TYPE_PARAM_ENUM))

typedef struct _PikaParamSpecEnum PikaParamSpecEnum;

struct _PikaParamSpecEnum
{
  GParamSpecEnum  parent_instance;

  GSList         *excluded_values;
};

GType        pika_param_enum_get_type     (void) G_GNUC_CONST;

GParamSpec * pika_param_spec_enum         (const gchar       *name,
                                           const gchar       *nick,
                                           const gchar       *blurb,
                                           GType              enum_type,
                                           gint               default_value,
                                           GParamFlags        flags);

void   pika_param_spec_enum_exclude_value (PikaParamSpecEnum *espec,
                                           gint               value);


/*  include the declaration of the remaining paramspecs, they are
 *  identical app/ and libpika/.
 */
#define PIKA_COMPILATION
#include "../../libpika/pikaparamspecs.h"
#undef PIKA_COMPILATION


#endif  /*  __APP_PIKA_PARAM_SPECS_H__  */
