/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>

#include "core-types.h"

#include "pikapattern.h"
#include "pikapattern-header.h"
#include "pikapattern-save.h"
#include "pikatempbuf.h"

#include "pika-intl.h"


gboolean
pika_pattern_save (PikaData       *data,
                   GOutputStream  *output,
                   GError        **error)
{
  PikaPattern       *pattern = PIKA_PATTERN (data);
  PikaTempBuf       *mask    = pika_pattern_get_mask (pattern);
  const Babl        *format  = pika_temp_buf_get_format (mask);
  PikaPatternHeader  header;
  const gchar       *name;
  gint               width;
  gint               height;

  name   = pika_object_get_name (pattern);
  width  = pika_temp_buf_get_width  (mask);
  height = pika_temp_buf_get_height (mask);

  if (width > PIKA_PATTERN_MAX_SIZE || height > PIKA_PATTERN_MAX_SIZE)
    {
      g_set_error (error, PIKA_DATA_ERROR, PIKA_DATA_ERROR_READ,
                   _("Unsupported pattern dimensions %d x %d.\n"
                     "PIKA Patterns have a maximum size of %d x %d."),
                   width, height,
                   PIKA_PATTERN_MAX_SIZE, PIKA_PATTERN_MAX_SIZE);
      return FALSE;
    }
  header.header_size  = g_htonl (sizeof (PikaPatternHeader) +
                                 strlen (name) + 1);
  header.version      = g_htonl (1);
  header.width        = g_htonl (width);
  header.height       = g_htonl (height);
  header.bytes        = g_htonl (babl_format_get_bytes_per_pixel (format));
  header.magic_number = g_htonl (PIKA_PATTERN_MAGIC);

  if (! g_output_stream_write_all (output, &header, sizeof (header),
                                   NULL, NULL, error))
    {
      return FALSE;
    }

  if (! g_output_stream_write_all (output, name, strlen (name) + 1,
                                   NULL, NULL, error))
    {
      return FALSE;
    }

  if (! g_output_stream_write_all (output,
                                   pika_temp_buf_get_data (mask),
                                   pika_temp_buf_get_data_size (mask),
                                   NULL, NULL, error))
    {
      return FALSE;
    }

  return TRUE;
}
