/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikaprogress.h
 * Copyright (C) 2004  Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_PROGRESS_H__
#define __PIKA_PROGRESS_H__


#define PIKA_TYPE_PROGRESS (pika_progress_get_type ())
G_DECLARE_INTERFACE (PikaProgress, pika_progress, PIKA, PROGRESS, GObject)


struct _PikaProgressInterface
{
  GTypeInterface base_iface;

  /*  virtual functions  */
  PikaProgress * (* start)         (PikaProgress        *progress,
                                    gboolean             cancellable,
                                    const gchar         *message);
  void           (* end)           (PikaProgress        *progress);
  gboolean       (* is_active)     (PikaProgress        *progress);

  void           (* set_text)      (PikaProgress        *progress,
                                    const gchar         *message);
  void           (* set_value)     (PikaProgress        *progress,
                                    gdouble              percentage);
  gdouble        (* get_value)     (PikaProgress        *progress);
  void           (* pulse)         (PikaProgress        *progress);

  guint32        (* get_window_id) (PikaProgress        *progress);

  gboolean       (* message)       (PikaProgress        *progress,
                                    Pika                *pika,
                                    PikaMessageSeverity  severity,
                                    const gchar         *domain,
                                    const gchar         *message);

  /*  signals  */
  void           (* cancel)        (PikaProgress        *progress);
};


PikaProgress * pika_progress_start            (PikaProgress        *progress,
                                               gboolean             cancellable,
                                               const gchar         *format,
                                               ...) G_GNUC_PRINTF (3, 4);
void           pika_progress_end              (PikaProgress        *progress);
gboolean       pika_progress_is_active        (PikaProgress        *progress);

void           pika_progress_set_text         (PikaProgress        *progress,
                                               const gchar         *format,
                                               ...) G_GNUC_PRINTF (2, 3);
void           pika_progress_set_text_literal (PikaProgress        *progress,
                                               const gchar         *message);
void           pika_progress_set_value        (PikaProgress        *progress,
                                               gdouble              percentage);
gdouble        pika_progress_get_value        (PikaProgress        *progress);
void           pika_progress_pulse            (PikaProgress        *progress);

guint32        pika_progress_get_window_id    (PikaProgress        *progress);

gboolean       pika_progress_message          (PikaProgress        *progress,
                                               Pika                *pika,
                                               PikaMessageSeverity  severity,
                                               const gchar         *domain,
                                               const gchar         *message);

void           pika_progress_cancel           (PikaProgress        *progress);

void           pika_progress_update_and_flush (gint                 min,
                                               gint                 max,
                                               gint                 current,
                                               gpointer             data);


#endif /* __PIKA_PROGRESS_H__ */
