/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * pikaprojectable.h
 * Copyright (C) 2008  Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_PROJECTABLE_H__
#define __PIKA_PROJECTABLE_H__


#define PIKA_TYPE_PROJECTABLE (pika_projectable_get_type ())
G_DECLARE_INTERFACE (PikaProjectable, pika_projectable, PIKA, PROJECTABLE, GObject)


struct _PikaProjectableInterface
{
  GTypeInterface base_iface;

  /*  signals  */
  void         (* invalidate)         (PikaProjectable *projectable,
                                       gint             x,
                                       gint             y,
                                       gint             width,
                                       gint             height);
  void         (* flush)              (PikaProjectable *projectable,
                                       gboolean         invalidate_preview);
  void         (* structure_changed)  (PikaProjectable *projectable);
  void         (* bounds_changed)     (PikaProjectable *projectable,
                                       gint             old_x,
                                       gint             old_y);

  /*  virtual functions  */
  PikaImage  * (* get_image)          (PikaProjectable *projectable);
  const Babl * (* get_format)         (PikaProjectable *projectable);
  void         (* get_offset)         (PikaProjectable *projectable,
                                       gint            *x,
                                       gint            *y);
  GeglRectangle (* get_bounding_box)  (PikaProjectable *projectable);
  GeglNode   * (* get_graph)          (PikaProjectable *projectable);
  void         (* begin_render)       (PikaProjectable *projectable);
  void         (* end_render)         (PikaProjectable *projectable);
  void         (* invalidate_preview) (PikaProjectable *projectable);
};


void         pika_projectable_invalidate         (PikaProjectable *projectable,
                                                  gint             x,
                                                  gint             y,
                                                  gint             width,
                                                  gint             height);
void         pika_projectable_flush              (PikaProjectable *projectable,
                                                  gboolean         preview_invalidated);
void         pika_projectable_structure_changed  (PikaProjectable *projectable);
void         pika_projectable_bounds_changed     (PikaProjectable *projectable,
                                                  gint             old_x,
                                                  gint             old_y);

PikaImage  * pika_projectable_get_image          (PikaProjectable *projectable);
const Babl * pika_projectable_get_format         (PikaProjectable *projectable);
void         pika_projectable_get_offset         (PikaProjectable *projectable,
                                                  gint            *x,
                                                  gint            *y);
GeglRectangle pika_projectable_get_bounding_box  (PikaProjectable *projectable);
GeglNode   * pika_projectable_get_graph          (PikaProjectable *projectable);
void         pika_projectable_begin_render       (PikaProjectable *projectable);
void         pika_projectable_end_render         (PikaProjectable *projectable);
void         pika_projectable_invalidate_preview (PikaProjectable *projectable);


#endif  /* __PIKA_PROJECTABLE_H__ */
