/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_PROJECTION_H__
#define __PIKA_PROJECTION_H__


#include "pikaobject.h"


#define PIKA_TYPE_PROJECTION            (pika_projection_get_type ())
#define PIKA_PROJECTION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_PROJECTION, PikaProjection))
#define PIKA_PROJECTION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_PROJECTION, PikaProjectionClass))
#define PIKA_IS_PROJECTION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_PROJECTION))
#define PIKA_IS_PROJECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_PROJECTION))
#define PIKA_PROJECTION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_PROJECTION, PikaProjectionClass))


typedef struct _PikaProjectionPrivate PikaProjectionPrivate;
typedef struct _PikaProjectionClass   PikaProjectionClass;

struct _PikaProjection
{
  PikaObject             parent_instance;

  PikaProjectionPrivate *priv;
};

struct _PikaProjectionClass
{
  PikaObjectClass  parent_class;

  void (* update) (PikaProjection *proj,
                   gboolean        now,
                   gint            x,
                   gint            y,
                   gint            width,
                   gint            height);
};


GType            pika_projection_get_type          (void) G_GNUC_CONST;

PikaProjection * pika_projection_new               (PikaProjectable   *projectable);

void             pika_projection_set_priority      (PikaProjection    *projection,
                                                    gint               priority);
gint             pika_projection_get_priority      (PikaProjection    *projection);

void             pika_projection_set_priority_rect (PikaProjection    *proj,
                                                    gint               x,
                                                    gint               y,
                                                    gint               width,
                                                    gint               height);

void             pika_projection_stop_rendering    (PikaProjection    *proj);

void             pika_projection_flush             (PikaProjection    *proj);
void             pika_projection_flush_now         (PikaProjection    *proj,
                                                    gboolean           direct);
void             pika_projection_finish_draw       (PikaProjection    *proj);

gint64           pika_projection_estimate_memsize  (PikaImageBaseType  type,
                                                    PikaComponentType  component_type,
                                                    gint               width,
                                                    gint               height);


#endif /*  __PIKA_PROJECTION_H__  */
