/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikatag.h
 * Copyright (C) 2008 Aurimas Juška <aurisj@svn.gnome.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TAG_H__
#define __PIKA_TAG_H__


#include <glib-object.h>


#define PIKA_TYPE_TAG            (pika_tag_get_type ())
#define PIKA_TAG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TAG, PikaTag))
#define PIKA_TAG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_TAG, PikaTagClass))
#define PIKA_IS_TAG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TAG))
#define PIKA_IS_TAG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_TAG))
#define PIKA_TAG_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_TAG, PikaTagClass))


typedef struct _PikaTagClass    PikaTagClass;

struct _PikaTag
{
  GObject parent_instance;

  GQuark  tag;
  GQuark  collate_key;

  gboolean internal; /* Tags that are not serialized to disk */
};

struct _PikaTagClass
{
  GObjectClass parent_class;
};

GType         pika_tag_get_type            (void) G_GNUC_CONST;

PikaTag     * pika_tag_new                 (const gchar *tag_string);
PikaTag     * pika_tag_try_new             (const gchar *tag_string);

const gchar * pika_tag_get_name            (PikaTag     *tag);
guint         pika_tag_get_hash            (PikaTag     *tag);

gboolean      pika_tag_get_internal        (PikaTag     *tag);
void          pika_tag_set_internal        (PikaTag     *tag,
                                            gboolean     internal);

gboolean      pika_tag_equals              (PikaTag     *tag,
                                            PikaTag     *other);
gint          pika_tag_compare_func        (const void  *p1,
                                            const void  *p2);
gint          pika_tag_compare_with_string (PikaTag     *tag,
                                            const gchar *tag_string);
gboolean      pika_tag_has_prefix          (PikaTag     *tag,
                                            const gchar *prefix_string);
gchar       * pika_tag_string_make_valid   (const gchar *tag_string);
gboolean      pika_tag_is_tag_separator    (gunichar     c);

void          pika_tag_or_null_ref         (PikaTag     *tag_or_null);
void          pika_tag_or_null_unref       (PikaTag     *tag_or_null);


#endif /* __PIKA_TAG_H__ */
