/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * pikatagged.h
 * Copyright (C) 2008  Sven Neumann <sven@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TAGGED_H__
#define __PIKA_TAGGED_H__


#define PIKA_TYPE_TAGGED (pika_tagged_get_type ())
G_DECLARE_INTERFACE (PikaTagged, pika_tagged, PIKA, TAGGED, GObject)


struct _PikaTaggedInterface
{
  GTypeInterface base_iface;

  /*  signals            */
  void       (* tag_added)      (PikaTagged *tagged,
                                 PikaTag    *tag);
  void       (* tag_removed)    (PikaTagged *tagged,
                                 PikaTag    *tag);

  /*  virtual functions  */
  gboolean   (* add_tag)        (PikaTagged *tagged,
                                 PikaTag    *tag);
  gboolean   (* remove_tag)     (PikaTagged *tagged,
                                 PikaTag    *tag);
  GList    * (* get_tags)       (PikaTagged *tagged);
  gchar    * (* get_identifier) (PikaTagged *tagged);
  gchar    * (* get_checksum)   (PikaTagged *tagged);
};


void       pika_tagged_add_tag        (PikaTagged *tagged,
                                       PikaTag    *tag);
void       pika_tagged_remove_tag     (PikaTagged *tagged,
                                       PikaTag    *tag);

void       pika_tagged_set_tags       (PikaTagged *tagged,
                                       GList      *tags);
GList    * pika_tagged_get_tags       (PikaTagged *tagged);

gchar    * pika_tagged_get_identifier (PikaTagged *tagged);
gchar    * pika_tagged_get_checksum   (PikaTagged *tagged);

gboolean   pika_tagged_has_tag        (PikaTagged *tagged,
                                       PikaTag    *tag);


#endif  /* __PIKA_TAGGED_H__ */
