/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <cairo.h>

#include "core-types.h"

#include "pikaprojectable.h"

#include "pikatilehandlerprojectable.h"


static void   pika_tile_handler_projectable_begin_validate (PikaTileHandlerValidate *validate);
static void   pika_tile_handler_projectable_end_validate   (PikaTileHandlerValidate *validate);


G_DEFINE_TYPE (PikaTileHandlerProjectable, pika_tile_handler_projectable,
               PIKA_TYPE_TILE_HANDLER_VALIDATE)

#define parent_class pika_tile_handler_projectable_parent_class


static void
pika_tile_handler_projectable_class_init (PikaTileHandlerProjectableClass *klass)
{
  PikaTileHandlerValidateClass *validate_class;

  validate_class = PIKA_TILE_HANDLER_VALIDATE_CLASS (klass);

  validate_class->begin_validate = pika_tile_handler_projectable_begin_validate;
  validate_class->end_validate   = pika_tile_handler_projectable_end_validate;
}

static void
pika_tile_handler_projectable_init (PikaTileHandlerProjectable *projectable)
{
}

static void
pika_tile_handler_projectable_begin_validate (PikaTileHandlerValidate *validate)
{
  PikaTileHandlerProjectable *handler = PIKA_TILE_HANDLER_PROJECTABLE (validate);

  PIKA_TILE_HANDLER_VALIDATE_CLASS (parent_class)->begin_validate (validate);

  pika_projectable_begin_render (handler->projectable);
}

static void
pika_tile_handler_projectable_end_validate (PikaTileHandlerValidate *validate)
{
  PikaTileHandlerProjectable *handler = PIKA_TILE_HANDLER_PROJECTABLE (validate);

  pika_projectable_end_render (handler->projectable);

  PIKA_TILE_HANDLER_VALIDATE_CLASS (parent_class)->end_validate (validate);
}

GeglTileHandler *
pika_tile_handler_projectable_new (PikaProjectable *projectable)
{
  PikaTileHandlerProjectable *handler;

  g_return_val_if_fail (PIKA_IS_PROJECTABLE (projectable), NULL);

  handler = g_object_new (PIKA_TYPE_TILE_HANDLER_PROJECTABLE, NULL);

  PIKA_TILE_HANDLER_VALIDATE (handler)->graph =
    g_object_ref (pika_projectable_get_graph (projectable));

  handler->projectable = projectable;

  return GEGL_TILE_HANDLER (handler);
}
