/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TILE_HANDLER_PROJECTABLE_H__
#define __PIKA_TILE_HANDLER_PROJECTABLE_H__


#include "gegl/pikatilehandlervalidate.h"


/***
 * PikaTileHandlerProjectable is a GeglTileHandler that renders a
 * projectable, calling the projectable's begin_render() and end_render()
 * before/after the actual rendering.
 *
 * Note that the tile handler does not own a reference to the projectable.
 * It's the user's responsibility to manage the handler's and projectable's
 * lifetime.
 */

#define PIKA_TYPE_TILE_HANDLER_PROJECTABLE            (pika_tile_handler_projectable_get_type ())
#define PIKA_TILE_HANDLER_PROJECTABLE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TILE_HANDLER_PROJECTABLE, PikaTileHandlerProjectable))
#define PIKA_TILE_HANDLER_PROJECTABLE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  PIKA_TYPE_TILE_HANDLER_PROJECTABLE, PikaTileHandlerProjectableClass))
#define PIKA_IS_TILE_HANDLER_PROJECTABLE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TILE_HANDLER_PROJECTABLE))
#define PIKA_IS_TILE_HANDLER_PROJECTABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  PIKA_TYPE_TILE_HANDLER_PROJECTABLE))
#define PIKA_TILE_HANDLER_PROJECTABLE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  PIKA_TYPE_TILE_HANDLER_PROJECTABLE, PikaTileHandlerProjectableClass))


typedef struct _PikaTileHandlerProjectable      PikaTileHandlerProjectable;
typedef struct _PikaTileHandlerProjectableClass PikaTileHandlerProjectableClass;

struct _PikaTileHandlerProjectable
{
  PikaTileHandlerValidate  parent_instance;

  PikaProjectable         *projectable;
};

struct _PikaTileHandlerProjectableClass
{
  PikaTileHandlerValidateClass  parent_class;
};


GType             pika_tile_handler_projectable_get_type (void) G_GNUC_CONST;

GeglTileHandler * pika_tile_handler_projectable_new      (PikaProjectable *projectable);


#endif /* __PIKA_TILE_HANDLER_PROJECTABLE_H__ */
