/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TOOL_INFO_H__
#define __PIKA_TOOL_INFO_H__


#include "pikatoolitem.h"


#define PIKA_TYPE_TOOL_INFO            (pika_tool_info_get_type ())
#define PIKA_TOOL_INFO(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TOOL_INFO, PikaToolInfo))
#define PIKA_TOOL_INFO_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_TOOL_INFO, PikaToolInfoClass))
#define PIKA_IS_TOOL_INFO(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TOOL_INFO))
#define PIKA_IS_TOOL_INFO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_TOOL_INFO))
#define PIKA_TOOL_INFO_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_TOOL_INFO, PikaToolInfoClass))


typedef struct _PikaToolInfoClass PikaToolInfoClass;

struct _PikaToolInfo
{
  PikaToolItem         parent_instance;

  Pika                *pika;

  GType                tool_type;
  GType                tool_options_type;
  PikaContextPropMask  context_props;

  gchar               *label;
  gchar               *tooltip;

  gchar               *menu_label;
  gchar               *menu_accel;

  gchar               *help_domain;
  gchar               *help_id;

  gboolean             hidden;
  gboolean             experimental;

  PikaToolOptions     *tool_options;
  PikaPaintInfo       *paint_info;

  PikaContainer       *presets;
};

struct _PikaToolInfoClass
{
  PikaToolItemClass  parent_class;
};


GType          pika_tool_info_get_type         (void) G_GNUC_CONST;

PikaToolInfo * pika_tool_info_new              (Pika                *pika,
                                                GType                tool_type,
                                                GType                tool_options_type,
                                                PikaContextPropMask  context_props,
                                                const gchar         *identifier,
                                                const gchar         *label,
                                                const gchar         *tooltip,
                                                const gchar         *menu_label,
                                                const gchar         *menu_accel,
                                                const gchar         *help_domain,
                                                const gchar         *help_id,
                                                const gchar         *paint_core_name,
                                                const gchar         *icon_name);

void           pika_tool_info_set_standard     (Pika                *pika,
                                                PikaToolInfo        *tool_info);
PikaToolInfo * pika_tool_info_get_standard     (Pika                *pika);

gchar        * pika_tool_info_get_action_name (PikaToolInfo         *tool_info);

GFile        * pika_tool_info_get_options_file (PikaToolInfo        *tool_info,
                                                const gchar         *suffix);


#endif  /*  __PIKA_TOOL_INFO_H__  */
