/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-1999 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TOOL_OPTIONS_H__
#define __PIKA_TOOL_OPTIONS_H__


#include "pikacontext.h"


#define PIKA_TYPE_TOOL_OPTIONS            (pika_tool_options_get_type ())
#define PIKA_TOOL_OPTIONS(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TOOL_OPTIONS, PikaToolOptions))
#define PIKA_TOOL_OPTIONS_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_TOOL_OPTIONS, PikaToolOptionsClass))
#define PIKA_IS_TOOL_OPTIONS(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TOOL_OPTIONS))
#define PIKA_IS_TOOL_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_TOOL_OPTIONS))
#define PIKA_TOOL_OPTIONS_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_TOOL_OPTIONS, PikaToolOptionsClass))


typedef struct _PikaToolOptionsClass PikaToolOptionsClass;

struct _PikaToolOptions
{
  PikaContext   parent_instance;

  PikaToolInfo *tool_info;

  /*  if TRUE this instance is the main tool options object used for
   *  the GUI, this is not exactly clean, but there are some things
   *  (like linking brush properties to the active brush, or properly
   *  maintaining global brush, pattern etc.) that can only be done
   *  right in the object, and not by signal connections from the GUI,
   *  or upon switching tools, all of which was much more horrible.
   */
  gboolean      gui_mode;
};

struct _PikaToolOptionsClass
{
  PikaContextClass parent_class;
};


GType      pika_tool_options_get_type      (void) G_GNUC_CONST;

void       pika_tool_options_set_gui_mode  (PikaToolOptions   *tool_options,
                                            gboolean           gui_mode);
gboolean   pika_tool_options_get_gui_mode  (PikaToolOptions   *tool_options);

gboolean   pika_tool_options_serialize     (PikaToolOptions   *tool_options,
                                            GError           **error);
gboolean   pika_tool_options_deserialize   (PikaToolOptions   *tool_options,
                                            GError           **error);

gboolean   pika_tool_options_delete        (PikaToolOptions   *tool_options,
                                            GError           **error);
void       pika_tool_options_create_folder (void);


#endif  /*  __PIKA_TOOL_OPTIONS_H__  */
