/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikatriviallycancelablewaitable.c
 * Copyright (C) 2018 Ell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "core-types.h"

#include "pikacancelable.h"
#include "pikatriviallycancelablewaitable.h"
#include "pikawaitable.h"


/*  local function prototypes  */

static void   pika_trivially_cancelable_waitable_cancelable_iface_init (PikaCancelableInterface *iface);

static void   pika_trivially_cancelable_waitable_cancel                (PikaCancelable          *cancelable);


G_DEFINE_TYPE_WITH_CODE (PikaTriviallyCancelableWaitable, pika_trivially_cancelable_waitable, PIKA_TYPE_UNCANCELABLE_WAITABLE,
                         G_IMPLEMENT_INTERFACE (PIKA_TYPE_CANCELABLE,
                                                pika_trivially_cancelable_waitable_cancelable_iface_init))

#define parent_class pika_trivially_cancelable_waitable_parent_class


/*  private functions  */


static void
pika_trivially_cancelable_waitable_class_init (PikaTriviallyCancelableWaitableClass *klass)
{
}

static void
pika_trivially_cancelable_waitable_cancelable_iface_init (PikaCancelableInterface *iface)
{
  iface->cancel = pika_trivially_cancelable_waitable_cancel;
}

static void
pika_trivially_cancelable_waitable_init (PikaTriviallyCancelableWaitable *trivially_cancelable_waitable)
{
}

static void
pika_trivially_cancelable_waitable_cancel (PikaCancelable *cancelable)
{
  PikaUncancelableWaitable *uncancelable_waitable =
    PIKA_UNCANCELABLE_WAITABLE (cancelable);

  g_clear_object (&uncancelable_waitable->waitable);
}


/*  public functions  */


PikaWaitable *
pika_trivially_cancelable_waitable_new (PikaWaitable *waitable)
{
  PikaUncancelableWaitable *uncancelable_waitable;

  g_return_val_if_fail (PIKA_IS_WAITABLE (waitable), NULL);

  uncancelable_waitable = g_object_new (PIKA_TYPE_TRIVIALLY_CANCELABLE_WAITABLE,
                                        NULL);

  uncancelable_waitable->waitable = g_object_ref (waitable);

  return PIKA_WAITABLE (uncancelable_waitable);
}
