/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikauncancelablewaitable.c
 * Copyright (C) 2018 Ell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "core-types.h"

#include "pikauncancelablewaitable.h"
#include "pikawaitable.h"


/*  local function prototypes  */

static void       pika_uncancelable_waitable_waitable_iface_init (PikaWaitableInterface   *iface);

static void       pika_uncancelable_waitable_finalize            (GObject                 *object);

static void       pika_uncancelable_waitable_wait                (PikaWaitable            *waitable);
static gboolean   pika_uncancelable_waitable_try_wait            (PikaWaitable            *waitable);
static gboolean   pika_uncancelable_waitable_wait_until          (PikaWaitable            *waitable,
                                                                  gint64                   end_time);


G_DEFINE_TYPE_WITH_CODE (PikaUncancelableWaitable, pika_uncancelable_waitable, G_TYPE_OBJECT,
                         G_IMPLEMENT_INTERFACE (PIKA_TYPE_WAITABLE,
                                                pika_uncancelable_waitable_waitable_iface_init))

#define parent_class pika_uncancelable_waitable_parent_class


/*  private functions  */


static void
pika_uncancelable_waitable_class_init (PikaUncancelableWaitableClass *klass)
{
  GObjectClass *object_class = G_OBJECT_CLASS (klass);

  object_class->finalize = pika_uncancelable_waitable_finalize;
}

static void
pika_uncancelable_waitable_waitable_iface_init (PikaWaitableInterface *iface)
{
  iface->wait       = pika_uncancelable_waitable_wait;
  iface->try_wait   = pika_uncancelable_waitable_try_wait;
  iface->wait_until = pika_uncancelable_waitable_wait_until;
}

static void
pika_uncancelable_waitable_init (PikaUncancelableWaitable *uncancelable_waitable)
{
}

static void
pika_uncancelable_waitable_finalize (GObject *object)
{
  PikaUncancelableWaitable *uncancelable_waitable =
    PIKA_UNCANCELABLE_WAITABLE (object);

  g_clear_object (&uncancelable_waitable->waitable);

  G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
pika_uncancelable_waitable_wait (PikaWaitable *waitable)
{
  PikaUncancelableWaitable *uncancelable_waitable =
    PIKA_UNCANCELABLE_WAITABLE (waitable);

  if (uncancelable_waitable->waitable)
    pika_waitable_wait (uncancelable_waitable->waitable);
}

static gboolean
pika_uncancelable_waitable_try_wait (PikaWaitable *waitable)
{
  PikaUncancelableWaitable *uncancelable_waitable =
    PIKA_UNCANCELABLE_WAITABLE (waitable);

  if (uncancelable_waitable->waitable)
    return pika_waitable_try_wait (uncancelable_waitable->waitable);
  else
    return TRUE;
}

static gboolean
pika_uncancelable_waitable_wait_until (PikaWaitable *waitable,
                                gint64        end_time)
{
  PikaUncancelableWaitable *uncancelable_waitable =
    PIKA_UNCANCELABLE_WAITABLE (waitable);

  if (uncancelable_waitable->waitable)
    return pika_waitable_wait_until (uncancelable_waitable->waitable, end_time);
  else
    return TRUE;
}


/*  public functions  */


PikaWaitable *
pika_uncancelable_waitable_new (PikaWaitable *waitable)
{
  PikaUncancelableWaitable *uncancelable_waitable;

  g_return_val_if_fail (PIKA_IS_WAITABLE (waitable), NULL);

  uncancelable_waitable = g_object_new (PIKA_TYPE_UNCANCELABLE_WAITABLE, NULL);

  uncancelable_waitable->waitable = g_object_ref (waitable);

  return PIKA_WAITABLE (uncancelable_waitable);
}
