/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_UNDO_H__
#define __PIKA_UNDO_H__


#include "pikaviewable.h"


struct _PikaUndoAccumulator
{
  gboolean mode_changed;
  gboolean precision_changed;

  gboolean size_changed;
  gint     previous_origin_x;
  gint     previous_origin_y;
  gint     previous_width;
  gint     previous_height;

  gboolean resolution_changed;

  gboolean unit_changed;
};


#define PIKA_TYPE_UNDO            (pika_undo_get_type ())
#define PIKA_UNDO(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_UNDO, PikaUndo))
#define PIKA_UNDO_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_UNDO, PikaUndoClass))
#define PIKA_IS_UNDO(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_UNDO))
#define PIKA_IS_UNDO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_UNDO))
#define PIKA_UNDO_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_UNDO, PikaUndoClass))


typedef struct _PikaUndoClass PikaUndoClass;

struct _PikaUndo
{
  PikaViewable      parent_instance;

  PikaImage        *image;          /* the image this undo is part of     */
  guint             time;           /* time of undo step construction     */

  PikaUndoType      undo_type;      /* undo type                          */
  PikaDirtyMask     dirty_mask;     /* affected parts of the image        */

  PikaTempBuf      *preview;
  guint             preview_idle_id;
};

struct _PikaUndoClass
{
  PikaViewableClass  parent_class;

  void (* pop)  (PikaUndo            *undo,
                 PikaUndoMode         undo_mode,
                 PikaUndoAccumulator *accum);
  void (* free) (PikaUndo            *undo,
                 PikaUndoMode         undo_mode);
};


GType         pika_undo_get_type        (void) G_GNUC_CONST;

void          pika_undo_pop             (PikaUndo            *undo,
                                         PikaUndoMode         undo_mode,
                                         PikaUndoAccumulator *accum);
void          pika_undo_free            (PikaUndo            *undo,
                                         PikaUndoMode         undo_mode);

void          pika_undo_create_preview  (PikaUndo            *undo,
                                         PikaContext         *context,
                                         gboolean             create_now);
void          pika_undo_refresh_preview (PikaUndo            *undo,
                                         PikaContext         *context);

const gchar * pika_undo_type_to_name    (PikaUndoType         type);

gboolean      pika_undo_is_weak         (PikaUndo            *undo);
gint          pika_undo_get_age         (PikaUndo            *undo);
void          pika_undo_reset_age       (PikaUndo            *undo);


#endif /* __PIKA_UNDO_H__ */
