/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_UNDO_STACK_H__
#define __PIKA_UNDO_STACK_H__


#include "pikaundo.h"


#define PIKA_TYPE_UNDO_STACK            (pika_undo_stack_get_type ())
#define PIKA_UNDO_STACK(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_UNDO_STACK, PikaUndoStack))
#define PIKA_UNDO_STACK_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_UNDO_STACK, PikaUndoStackClass))
#define PIKA_IS_UNDO_STACK(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_UNDO_STACK))
#define PIKA_IS_UNDO_STACK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_UNDO_STACK))
#define PIKA_UNDO_STACK_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_UNDO_STACK, PikaUndoStackClass))


typedef struct _PikaUndoStackClass PikaUndoStackClass;

struct _PikaUndoStack
{
  PikaUndo       parent_instance;

  PikaContainer *undos;
};

struct _PikaUndoStackClass
{
  PikaUndoClass  parent_class;
};


GType           pika_undo_stack_get_type    (void) G_GNUC_CONST;

PikaUndoStack * pika_undo_stack_new         (PikaImage           *image);

void            pika_undo_stack_push_undo   (PikaUndoStack       *stack,
                                             PikaUndo            *undo);
PikaUndo      * pika_undo_stack_pop_undo    (PikaUndoStack       *stack,
                                             PikaUndoMode         undo_mode,
                                             PikaUndoAccumulator *accum);

PikaUndo      * pika_undo_stack_free_bottom (PikaUndoStack       *stack,
                                             PikaUndoMode         undo_mode);
PikaUndo      * pika_undo_stack_peek        (PikaUndoStack       *stack);
gint            pika_undo_stack_get_depth   (PikaUndoStack       *stack);


#endif /* __PIKA_UNDO_STACK_H__ */
