/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikawaitable.c
 * Copyright (C) 2018 Ell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */


#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "core-types.h"

#include "pikawaitable.h"


G_DEFINE_INTERFACE (PikaWaitable, pika_waitable, G_TYPE_OBJECT)


/*  private functions  */


static void
pika_waitable_default_init (PikaWaitableInterface *iface)
{
}


/*  public functions  */


void
pika_waitable_wait (PikaWaitable *waitable)
{
  PikaWaitableInterface *iface;

  g_return_if_fail (PIKA_IS_WAITABLE (waitable));

  iface = PIKA_WAITABLE_GET_IFACE (waitable);

  if (iface->wait)
    iface->wait (waitable);
}

gboolean
pika_waitable_try_wait (PikaWaitable *waitable)
{
  PikaWaitableInterface *iface;

  g_return_val_if_fail (PIKA_IS_WAITABLE (waitable), FALSE);

  iface = PIKA_WAITABLE_GET_IFACE (waitable);

  if (iface->try_wait)
    {
      return iface->try_wait (waitable);
    }
  else
    {
      pika_waitable_wait (waitable);

      return TRUE;
    }
}

gboolean
pika_waitable_wait_until (PikaWaitable *waitable,
                          gint64        end_time)
{
  PikaWaitableInterface *iface;

  g_return_val_if_fail (PIKA_IS_WAITABLE (waitable), FALSE);

  iface = PIKA_WAITABLE_GET_IFACE (waitable);

  if (iface->wait_until)
    {
      return iface->wait_until (waitable, end_time);
    }
  else
    {
      pika_waitable_wait (waitable);

      return TRUE;
    }
}

gboolean
pika_waitable_wait_for (PikaWaitable *waitable,
                        gint64        wait_duration)
{
  g_return_val_if_fail (PIKA_IS_WAITABLE (waitable), FALSE);

  if (wait_duration <= 0)
    {
      return pika_waitable_try_wait (waitable);
    }
  else
    {
      return pika_waitable_wait_until (waitable,
                                       g_get_monotonic_time () + wait_duration);
    }
}
