/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_CANVAS_H__
#define __PIKA_CANVAS_H__


#include "widgets/pikaoverlaybox.h"


#define PIKA_CANVAS_EVENT_MASK (GDK_EXPOSURE_MASK            | \
                                GDK_POINTER_MOTION_MASK      | \
                                GDK_BUTTON_PRESS_MASK        | \
                                GDK_BUTTON_RELEASE_MASK      | \
                                GDK_SCROLL_MASK              | \
                                GDK_SMOOTH_SCROLL_MASK       | \
                                GDK_TOUCHPAD_GESTURE_MASK    | \
                                GDK_STRUCTURE_MASK           | \
                                GDK_ENTER_NOTIFY_MASK        | \
                                GDK_LEAVE_NOTIFY_MASK        | \
                                GDK_FOCUS_CHANGE_MASK        | \
                                GDK_KEY_PRESS_MASK           | \
                                GDK_KEY_RELEASE_MASK         | \
                                GDK_PROXIMITY_OUT_MASK)


#define PIKA_TYPE_CANVAS            (pika_canvas_get_type ())
#define PIKA_CANVAS(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_CANVAS, PikaCanvas))
#define PIKA_CANVAS_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_CANVAS, PikaCanvasClass))
#define PIKA_IS_CANVAS(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_CANVAS))
#define PIKA_IS_CANVAS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_CANVAS))
#define PIKA_CANVAS_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_CANVAS, PikaCanvasClass))


typedef struct _PikaCanvasClass PikaCanvasClass;

struct _PikaCanvas
{
  PikaOverlayBox         parent_instance;

  PikaDisplayConfig     *config;
  PangoLayout           *layout;

  PikaCanvasPaddingMode  padding_mode;
  PikaRGB                padding_color;
};

struct _PikaCanvasClass
{
  PikaOverlayBoxClass  parent_class;
};


GType         pika_canvas_get_type     (void) G_GNUC_CONST;

GtkWidget   * pika_canvas_new          (PikaDisplayConfig     *config);

PangoLayout * pika_canvas_get_layout   (PikaCanvas            *canvas,
                                        const gchar           *format,
                                        ...) G_GNUC_PRINTF (2, 3);

void          pika_canvas_set_padding  (PikaCanvas            *canvas,
                                        PikaCanvasPaddingMode  padding_mode,
                                        const PikaRGB         *padding_color);


#endif /*  __PIKA_CANVAS_H__  */
