/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikacanvashandle.h
 * Copyright (C) 2010 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_CANVAS_HANDLE_H__
#define __PIKA_CANVAS_HANDLE_H__


#include "pikacanvasitem.h"


#define PIKA_CANVAS_HANDLE_SIZE_CIRCLE    13
#define PIKA_CANVAS_HANDLE_SIZE_CROSS     15
#define PIKA_CANVAS_HANDLE_SIZE_CROSSHAIR 43
#define PIKA_CANVAS_HANDLE_SIZE_LARGE     25
#define PIKA_CANVAS_HANDLE_SIZE_SMALL     7


#define PIKA_TYPE_CANVAS_HANDLE            (pika_canvas_handle_get_type ())
#define PIKA_CANVAS_HANDLE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_CANVAS_HANDLE, PikaCanvasHandle))
#define PIKA_CANVAS_HANDLE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_CANVAS_HANDLE, PikaCanvasHandleClass))
#define PIKA_IS_CANVAS_HANDLE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_CANVAS_HANDLE))
#define PIKA_IS_CANVAS_HANDLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_CANVAS_HANDLE))
#define PIKA_CANVAS_HANDLE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_CANVAS_HANDLE, PikaCanvasHandleClass))


typedef struct _PikaCanvasHandle      PikaCanvasHandle;
typedef struct _PikaCanvasHandleClass PikaCanvasHandleClass;

struct _PikaCanvasHandle
{
  PikaCanvasItem  parent_instance;
};

struct _PikaCanvasHandleClass
{
  PikaCanvasItemClass  parent_class;
};


GType            pika_canvas_handle_get_type     (void) G_GNUC_CONST;

PikaCanvasItem * pika_canvas_handle_new          (PikaDisplayShell *shell,
                                                  PikaHandleType    type,
                                                  PikaHandleAnchor  anchor,
                                                  gdouble           x,
                                                  gdouble           y,
                                                  gint              width,
                                                  gint              height);

void             pika_canvas_handle_get_position (PikaCanvasItem   *handle,
                                                  gdouble          *x,
                                                  gdouble          *y);
void             pika_canvas_handle_set_position (PikaCanvasItem   *handle,
                                                  gdouble           x,
                                                  gdouble           y);

gint             pika_canvas_handle_calc_size    (PikaCanvasItem   *item,
                                                  gdouble           mouse_x,
                                                  gdouble           mouse_y,
                                                  gint              normal_size,
                                                  gint              hover_size);

void             pika_canvas_handle_get_size     (PikaCanvasItem   *handle,
                                                  gint             *width,
                                                  gint             *height);
void             pika_canvas_handle_set_size     (PikaCanvasItem   *handle,
                                                  gint              width,
                                                  gint              height);

void             pika_canvas_handle_set_angles   (PikaCanvasItem   *handle,
                                                  gdouble           start_handle,
                                                  gdouble           slice_handle);


#endif /* __PIKA_CANVAS_HANDLE_H__ */
