/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_DISPLAY_IMPL_H__
#define __PIKA_DISPLAY_IMPL_H__


#include "core/pikadisplay.h"


#define PIKA_TYPE_DISPLAY_IMPL            (pika_display_impl_get_type ())
#define PIKA_DISPLAY_IMPL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_DISPLAY_IMPL, PikaDisplayImpl))
#define PIKA_DISPLAY_IMPL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_DISPLAY_IMPL, PikaDisplayImplClass))
#define PIKA_IS_DISPLAY_IMPL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_DISPLAY_IMPL))
#define PIKA_IS_DISPLAY_IMPL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_DISPLAY_IMPL))
#define PIKA_DISPLAY_IMPL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_DISPLAY_IMPL, PikaDisplayImplClass))


typedef struct _PikaDisplayImpl        PikaDisplayImpl;
typedef struct _PikaDisplayImplClass   PikaDisplayImplClass;
typedef struct _PikaDisplayImplPrivate PikaDisplayImplPrivate;


struct _PikaDisplayImpl
{
  PikaDisplay             parent_instance;

  PikaDisplayImplPrivate *priv;

};

struct _PikaDisplayImplClass
{
  PikaDisplayClass  parent_class;
};


GType              pika_display_impl_get_type   (void) G_GNUC_CONST;

PikaDisplay      * pika_display_new             (Pika              *pika,
                                                 PikaImage         *image,
                                                 PikaUnit           unit,
                                                 gdouble            scale,
                                                 PikaUIManager     *popup_manager,
                                                 PikaDialogFactory *dialog_factory,
                                                 GdkMonitor        *monitor);
void               pika_display_delete          (PikaDisplay       *display);
void               pika_display_close           (PikaDisplay       *display);

gchar            * pika_display_get_action_name (PikaDisplay       *display);

PikaImage        * pika_display_get_image       (PikaDisplay       *display);
void               pika_display_set_image       (PikaDisplay       *display,
                                                 PikaImage         *image);

gint               pika_display_get_instance    (PikaDisplay       *display);

PikaDisplayShell * pika_display_get_shell       (PikaDisplay       *display);

void               pika_display_empty           (PikaDisplay       *display);
void               pika_display_fill            (PikaDisplay       *display,
                                                 PikaImage         *image,
                                                 PikaUnit           unit,
                                                 gdouble            scale);

void               pika_display_update_bounding_box
                                                (PikaDisplay       *display);

void               pika_display_update_area     (PikaDisplay       *display,
                                                 gboolean           now,
                                                 gint               x,
                                                 gint               y,
                                                 gint               w,
                                                 gint               h);

void               pika_display_flush           (PikaDisplay       *display);
void               pika_display_flush_now       (PikaDisplay       *display);


#endif /*  __PIKA_DISPLAY_IMPL_H__  */
