/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_DISPLAY_SHELL_TRANSFORM_H__
#define __PIKA_DISPLAY_SHELL_TRANSFORM_H__


/*  zoom: functions to transform from image space to unrotated display
 *  space and back, taking into account scroll offset and scale
 */

void  pika_display_shell_zoom_coords                   (PikaDisplayShell   *shell,
                                                        const PikaCoords   *image_coords,
                                                        PikaCoords         *display_coords);
void  pika_display_shell_unzoom_coords                 (PikaDisplayShell   *shell,
                                                        const PikaCoords   *display_coords,
                                                        PikaCoords         *image_coords);

void  pika_display_shell_zoom_xy                       (PikaDisplayShell   *shell,
                                                        gdouble             x,
                                                        gdouble             y,
                                                        gint               *nx,
                                                        gint               *ny);
void  pika_display_shell_unzoom_xy                     (PikaDisplayShell   *shell,
                                                        gint                x,
                                                        gint                y,
                                                        gint               *nx,
                                                        gint               *ny,
                                                        gboolean            round);

void  pika_display_shell_zoom_xy_f                     (PikaDisplayShell   *shell,
                                                        gdouble             x,
                                                        gdouble             y,
                                                        gdouble            *nx,
                                                        gdouble            *ny);
void  pika_display_shell_unzoom_xy_f                   (PikaDisplayShell   *shell,
                                                        gdouble             x,
                                                        gdouble             y,
                                                        gdouble            *nx,
                                                        gdouble            *ny);

void  pika_display_shell_zoom_segments                 (PikaDisplayShell   *shell,
                                                        const PikaBoundSeg *src_segs,
                                                        PikaSegment        *dest_segs,
                                                        gint                n_segs,
                                                        gdouble             offset_x,
                                                        gdouble             offset_y);


/*  rotate: functions to transform from unrotated and unflipped but
 *  zoomed display space to rotated and flipped display space and back
 */

void  pika_display_shell_rotate_coords                 (PikaDisplayShell   *shell,
                                                        const PikaCoords   *image_coords,
                                                        PikaCoords         *display_coords);
void  pika_display_shell_unrotate_coords               (PikaDisplayShell   *shell,
                                                        const PikaCoords   *display_coords,
                                                        PikaCoords         *image_coords);

void  pika_display_shell_rotate_xy                     (PikaDisplayShell   *shell,
                                                        gdouble             x,
                                                        gdouble             y,
                                                        gint               *nx,
                                                        gint               *ny);
void  pika_display_shell_unrotate_xy                   (PikaDisplayShell   *shell,
                                                        gint                x,
                                                        gint                y,
                                                        gint               *nx,
                                                        gint               *ny);

void  pika_display_shell_rotate_xy_f                   (PikaDisplayShell   *shell,
                                                        gdouble             x,
                                                        gdouble             y,
                                                        gdouble            *nx,
                                                        gdouble            *ny);
void  pika_display_shell_unrotate_xy_f                 (PikaDisplayShell   *shell,
                                                        gdouble             x,
                                                        gdouble             y,
                                                        gdouble            *nx,
                                                        gdouble            *ny);

void  pika_display_shell_rotate_bounds                 (PikaDisplayShell   *shell,
                                                        gdouble             x1,
                                                        gdouble             y1,
                                                        gdouble             x2,
                                                        gdouble             y2,
                                                        gdouble            *nx1,
                                                        gdouble            *ny1,
                                                        gdouble            *nx2,
                                                        gdouble            *ny2);
void  pika_display_shell_unrotate_bounds               (PikaDisplayShell   *shell,
                                                        gdouble             x1,
                                                        gdouble             y1,
                                                        gdouble             x2,
                                                        gdouble             y2,
                                                        gdouble            *nx1,
                                                        gdouble            *ny1,
                                                        gdouble            *nx2,
                                                        gdouble            *ny2);


/*  transform: functions to transform from image space to rotated
 *  display space and back, taking into account scroll offset, scale,
 *  rotation and flipping
 */

void  pika_display_shell_transform_coords              (PikaDisplayShell   *shell,
                                                        const PikaCoords   *image_coords,
                                                        PikaCoords         *display_coords);
void  pika_display_shell_untransform_coords            (PikaDisplayShell   *shell,
                                                        const PikaCoords   *display_coords,
                                                        PikaCoords         *image_coords);

void  pika_display_shell_transform_xy                  (PikaDisplayShell   *shell,
                                                        gdouble             x,
                                                        gdouble             y,
                                                        gint               *nx,
                                                        gint               *ny);
void  pika_display_shell_untransform_xy                (PikaDisplayShell   *shell,
                                                        gint                x,
                                                        gint                y,
                                                        gint               *nx,
                                                        gint               *ny,
                                                        gboolean            round);

void  pika_display_shell_transform_xy_f                (PikaDisplayShell   *shell,
                                                        gdouble             x,
                                                        gdouble             y,
                                                        gdouble            *nx,
                                                        gdouble            *ny);
void  pika_display_shell_untransform_xy_f              (PikaDisplayShell   *shell,
                                                        gdouble             x,
                                                        gdouble             y,
                                                        gdouble            *nx,
                                                        gdouble            *ny);

void  pika_display_shell_transform_bounds              (PikaDisplayShell   *shell,
                                                        gdouble             x1,
                                                        gdouble             y1,
                                                        gdouble             x2,
                                                        gdouble             y2,
                                                        gdouble            *nx1,
                                                        gdouble            *ny1,
                                                        gdouble            *nx2,
                                                        gdouble            *ny2);
void  pika_display_shell_untransform_bounds            (PikaDisplayShell   *shell,
                                                        gdouble             x1,
                                                        gdouble             y1,
                                                        gdouble             x2,
                                                        gdouble             y2,
                                                        gdouble            *nx1,
                                                        gdouble            *ny1,
                                                        gdouble            *nx2,
                                                        gdouble            *ny2);

void  pika_display_shell_transform_bounds_with_scale   (PikaDisplayShell   *shell,
                                                        gdouble             scale,
                                                        gdouble             x1,
                                                        gdouble             y1,
                                                        gdouble             x2,
                                                        gdouble             y2,
                                                        gdouble            *nx1,
                                                        gdouble            *ny1,
                                                        gdouble            *nx2,
                                                        gdouble            *ny2);
void  pika_display_shell_untransform_bounds_with_scale (PikaDisplayShell   *shell,
                                                        gdouble             scale,
                                                        gdouble             x1,
                                                        gdouble             y1,
                                                        gdouble             x2,
                                                        gdouble             y2,
                                                        gdouble            *nx1,
                                                        gdouble            *ny1,
                                                        gdouble            *nx2,
                                                        gdouble            *ny2);

void  pika_display_shell_untransform_viewport          (PikaDisplayShell   *shell,
                                                        gboolean            clip,
                                                        gint               *x,
                                                        gint               *y,
                                                        gint               *width,
                                                        gint               *height);


#endif /* __PIKA_DISPLAY_SHELL_TRANSFORM_H__ */
