/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_DISPLAY_SHELL_H__
#define __PIKA_DISPLAY_SHELL_H__


/* Apply to a float the same rounding mode used in the renderer */
#define  PROJ_ROUND(coord)   ((gint) RINT (coord))
#define  PROJ_ROUND64(coord) ((gint64) RINT (coord))

/* scale values */
#define  SCALEX(s,x)      PROJ_ROUND ((x) * (s)->scale_x)
#define  SCALEY(s,y)      PROJ_ROUND ((y) * (s)->scale_y)

/* unscale values */
#define  UNSCALEX(s,x)    ((gint) ((x) / (s)->scale_x))
#define  UNSCALEY(s,y)    ((gint) ((y) / (s)->scale_y))
/* (and float-returning versions) */
#define  FUNSCALEX(s,x)   ((x) / (s)->scale_x)
#define  FUNSCALEY(s,y)   ((y) / (s)->scale_y)


#define PIKA_TYPE_DISPLAY_SHELL            (pika_display_shell_get_type ())
#define PIKA_DISPLAY_SHELL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_DISPLAY_SHELL, PikaDisplayShell))
#define PIKA_DISPLAY_SHELL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_DISPLAY_SHELL, PikaDisplayShellClass))
#define PIKA_IS_DISPLAY_SHELL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_DISPLAY_SHELL))
#define PIKA_IS_DISPLAY_SHELL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_DISPLAY_SHELL))
#define PIKA_DISPLAY_SHELL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_DISPLAY_SHELL, PikaDisplayShellClass))


typedef struct _PikaDisplayShellClass  PikaDisplayShellClass;

struct _PikaDisplayShell
{
  GtkEventBox        parent_instance;

  PikaDisplay       *display;

  PikaUIManager     *popup_manager;
  GdkMonitor        *initial_monitor;

  PikaDisplayOptions *options;
  PikaDisplayOptions *fullscreen_options;
  PikaDisplayOptions *no_image_options;

  PikaUnit           unit;

  gint               offset_x;         /*  offset of display image            */
  gint               offset_y;

  gdouble            scale_x;          /*  horizontal scale factor            */
  gdouble            scale_y;          /*  vertical scale factor              */

  gboolean           flip_horizontally;
  gboolean           flip_vertically;
  gdouble            rotate_angle;
  cairo_matrix_t    *rotate_transform;
  cairo_matrix_t    *rotate_untransform;

  gdouble            monitor_xres;
  gdouble            monitor_yres;
  gboolean           dot_for_dot;      /*  ignore monitor resolution          */

  PikaZoomModel     *zoom;

  gdouble            last_scale;       /*  scale used when reverting zoom     */
  guint              last_scale_time;  /*  time when last_scale was set       */
  gint               last_offset_x;    /*  offsets used when reverting zoom   */
  gint               last_offset_y;

  gdouble            other_scale;      /*  scale factor entered in Zoom->Other*/

  gint               disp_width;       /*  width of drawing area              */
  gint               disp_height;      /*  height of drawing area             */

  gboolean           proximity;        /*  is a device in proximity           */

  gboolean           show_image;       /*  whether to show the image          */

  gboolean           show_all;         /*  show the entire image              */

  Selection         *selection;        /*  Selection (marching ants)          */
  gint64             selection_update; /*  Last selection index update        */

  GList             *children;

  GtkWidget         *canvas;           /*  PikaCanvas widget                  */
  GtkGesture        *zoom_gesture;     /*  Zoom gesture handler for the canvas*/
  GtkGesture        *rotate_gesture;   /*  Rotation gesture handler           */

  GtkAdjustment     *hsbdata;          /*  adjustments                        */
  GtkAdjustment     *vsbdata;
  GtkWidget         *hsb;              /*  scroll bars                        */
  GtkWidget         *vsb;

  GtkWidget         *hrule;            /*  rulers                             */
  GtkWidget         *vrule;

  GtkWidget         *origin;           /*  NW: origin                         */
  GtkWidget         *quick_mask_button;/*  SW: quick mask button              */
  GtkWidget         *zoom_button;      /*  NE: zoom toggle button             */
  GtkWidget         *nav_ebox;         /*  SE: navigation event box           */

  GtkWidget         *statusbar;        /*  statusbar                          */

  PikaCanvasItem    *canvas_item;      /*  items drawn on the canvas          */
  PikaCanvasItem    *unrotated_item;   /*  unrotated items for e.g. cursor    */
  PikaCanvasItem    *passe_partout;    /*  item for the highlight             */
  PikaCanvasItem    *preview_items;    /*  item for previews                  */
  PikaCanvasItem    *vectors;          /*  item proxy of vectors              */
  PikaCanvasItem    *grid;             /*  item proxy of the grid             */
  PikaCanvasItem    *guides;           /*  item proxies of guides             */
  PikaCanvasItem    *sample_points;    /*  item proxies of sample points      */
  PikaCanvasItem    *canvas_boundary;  /*  item for the cabvas boundary       */
  PikaCanvasItem    *layer_boundary;   /*  item for the layer boundary        */
  PikaCanvasItem    *tool_items;       /*  tools items, below the cursor      */
  PikaCanvasItem    *cursor;           /*  item for the software cursor       */

  guint              title_idle_id;    /*  title update idle ID               */
  gchar             *title;            /*  current title                      */
  gchar             *status;           /*  current default statusbar content  */

  guint              fill_idle_id;     /*  display_shell_fill() idle ID       */

  PikaHandedness     cursor_handedness;/*  Handedness for cursor display      */
  PikaCursorType     current_cursor;   /*  Currently installed main cursor    */
  PikaToolCursorType tool_cursor;      /*  Current Tool cursor                */
  PikaCursorModifier cursor_modifier;  /*  Cursor modifier (plus, minus, ...) */

  PikaCursorType     override_cursor;  /*  Overriding cursor                  */
  gboolean           using_override_cursor;
  gboolean           draw_cursor;      /* should we draw software cursor ?    */

  GtkWidget         *close_dialog;     /*  close dialog                       */
  GtkWidget         *scale_dialog;     /*  scale (zoom) dialog                */
  GtkWidget         *rotate_dialog;    /*  rotate dialog                      */
  GtkWidget         *nav_popup;        /*  navigation popup                   */

  PikaColorConfig   *color_config;     /*  color management settings          */
  gboolean           color_config_set; /*  settings changed from defaults     */

  PikaColorTransform *profile_transform;
  GeglBuffer         *profile_buffer;  /*  buffer for profile transform       */
  guchar             *profile_data;    /*  profile_buffer's pixels            */
  gint                profile_stride;  /*  profile_buffer's stride            */

  PikaColorDisplayStack *filter_stack; /*  color display conversion stuff     */
  guint                  filter_idle_id;

  PikaColorTransform *filter_transform;
  const Babl         *filter_format;   /*  filter_buffer's format             */
  PikaColorProfile   *filter_profile;  /*  filter_format's profile            */
  GeglBuffer         *filter_buffer;   /*  buffer for display filters         */
  guchar             *filter_data;     /*  filter_buffer's pixels             */
  gint                filter_stride;   /*  filter_buffer's stride             */

  gint               render_scale;

  cairo_surface_t   *render_cache;
  cairo_region_t    *render_cache_valid;

  gint               render_buf_width;
  gint               render_buf_height;

  cairo_surface_t   *render_surface;   /*  buffer for rendering the mask      */
  cairo_surface_t   *mask_surface;     /*  buffer for rendering the mask      */
  cairo_pattern_t   *checkerboard;     /*  checkerboard pattern               */

  gint               paused_count;

  PikaTreeHandler   *vectors_freeze_handler;
  PikaTreeHandler   *vectors_thaw_handler;
  PikaTreeHandler   *vectors_visible_handler;

  gboolean           zoom_on_resize;

  gboolean           size_allocate_from_configure_event;
  gboolean           size_allocate_center_image;

  /*  the state of pika_display_shell_tool_events()  */
  GdkDevice         *grab_pointer;
  GdkDevice         *grab_pointer_source;
  guint32            grab_pointer_time;

  /*  the state of pika_display_shell_zoom_gesture_*() */
  gdouble            last_zoom_scale;
  gboolean           zoom_gesture_active;

  /*  the state of pika_display_shell_rotate_gesture_*() */
  guint              last_gesture_rotate_state;
  gdouble            initial_gesture_rotate_angle;
  gboolean           rotate_gesture_active;

  /* Two states are possible when the shell is grabbed: it can be
   * grabbed with space (or space+button1 which is the same state),
   * then if space is released but button1 was still pressed, we wait
   * for button1 to be released as well.
   */
  gboolean           space_release_pending;
  gboolean           button1_release_pending;
  const gchar       *space_shaded_tool;

  /* Modifier action currently ON. */
  PikaModifierAction mod_action;
  gchar             *mod_action_desc;

  gint               scroll_start_x;
  gint               scroll_start_y;
  gint               scroll_last_x;
  gint               scroll_last_y;
  gdouble            rotate_drag_angle;
  gpointer           scroll_info;
  PikaLayer         *picked_layer;

  GeglBuffer        *mask;
  gint               mask_offset_x;
  gint               mask_offset_y;
  PikaRGB            mask_color;
  gboolean           mask_inverted;

  PikaMotionBuffer  *motion_buffer;

  GdkPoint          *zoom_focus_point;

  gboolean           blink;
  guint              blink_timeout_id;

  PikaAlignmentType  snapped_side_horizontal; /* if PIKA_ARRANGE_HFILL then no snapping happen */
  PikaLayer         *snapped_layer_horizontal;

  PikaAlignmentType  snapped_side_vertical; /* if PIKA_ARRANGE_HFILL then no snapping happen */
  PikaLayer         *snapped_layer_vertical;

  PikaAlignmentType  equidistance_side_horizontal;
  PikaLayer         *near_layer_horizontal1;
  PikaLayer         *near_layer_horizontal2;

  PikaAlignmentType  equidistance_side_vertical;
  PikaLayer         *near_layer_vertical1;
  PikaLayer         *near_layer_vertical2;
};

struct _PikaDisplayShellClass
{
  GtkEventBoxClass  parent_class;

  void (* scaled)    (PikaDisplayShell *shell);
  void (* scrolled)  (PikaDisplayShell *shell);
  void (* rotated)   (PikaDisplayShell *shell);
  void (* reconnect) (PikaDisplayShell *shell);
};


GType             pika_display_shell_get_type      (void) G_GNUC_CONST;

GtkWidget       * pika_display_shell_new           (PikaDisplay        *display,
                                                    PikaUnit            unit,
                                                    gdouble             scale,
                                                    PikaUIManager      *popup_manager,
                                                    GdkMonitor         *monitor);

void              pika_display_shell_add_overlay   (PikaDisplayShell   *shell,
                                                    GtkWidget          *child,
                                                    gdouble             image_x,
                                                    gdouble             image_y,
                                                    PikaHandleAnchor    anchor,
                                                    gint                spacing_x,
                                                    gint                spacing_y);
void              pika_display_shell_move_overlay  (PikaDisplayShell   *shell,
                                                    GtkWidget          *child,
                                                    gdouble             image_x,
                                                    gdouble             image_y,
                                                    PikaHandleAnchor    anchor,
                                                    gint                spacing_x,
                                                    gint                spacing_y);

PikaImageWindow * pika_display_shell_get_window    (PikaDisplayShell   *shell);
PikaStatusbar   * pika_display_shell_get_statusbar (PikaDisplayShell   *shell);

PikaColorConfig * pika_display_shell_get_color_config
                                                   (PikaDisplayShell   *shell);

void              pika_display_shell_present       (PikaDisplayShell   *shell);

void              pika_display_shell_reconnect     (PikaDisplayShell   *shell);

void              pika_display_shell_empty         (PikaDisplayShell   *shell);
void              pika_display_shell_fill          (PikaDisplayShell   *shell,
                                                    PikaImage          *image,
                                                    PikaUnit            unit,
                                                    gdouble             scale);

void              pika_display_shell_scaled        (PikaDisplayShell   *shell);
void              pika_display_shell_scrolled      (PikaDisplayShell   *shell);
void              pika_display_shell_rotated       (PikaDisplayShell   *shell);

void              pika_display_shell_set_unit      (PikaDisplayShell   *shell,
                                                    PikaUnit            unit);
PikaUnit          pika_display_shell_get_unit      (PikaDisplayShell   *shell);

gboolean          pika_display_shell_snap_coords   (PikaDisplayShell   *shell,
                                                    PikaCoords         *coords,
                                                    gint                snap_offset_x,
                                                    gint                snap_offset_y,
                                                    gint                snap_width,
                                                    gint                snap_height);

gboolean          pika_display_shell_mask_bounds   (PikaDisplayShell   *shell,
                                                    gint               *x,
                                                    gint               *y,
                                                    gint               *width,
                                                    gint               *height);

void              pika_display_shell_set_show_image
                                                   (PikaDisplayShell   *shell,
                                                    gboolean            show_image);

void              pika_display_shell_set_show_all  (PikaDisplayShell   *shell,
                                                    gboolean            show_all);

PikaPickable    * pika_display_shell_get_pickable  (PikaDisplayShell   *shell);
PikaPickable    * pika_display_shell_get_canvas_pickable
                                                   (PikaDisplayShell   *shell);
GeglRectangle     pika_display_shell_get_bounding_box
                                                   (PikaDisplayShell   *shell);
gboolean          pika_display_shell_get_infinite_canvas
                                                   (PikaDisplayShell   *shell);

void              pika_display_shell_update_priority_rect
                                                   (PikaDisplayShell *shell);

void              pika_display_shell_flush         (PikaDisplayShell   *shell);

void              pika_display_shell_pause         (PikaDisplayShell   *shell);
void              pika_display_shell_resume        (PikaDisplayShell   *shell);

void              pika_display_shell_set_highlight (PikaDisplayShell   *shell,
                                                    const GdkRectangle *highlight,
                                                    double              opacity);
void              pika_display_shell_set_mask      (PikaDisplayShell   *shell,
                                                    GeglBuffer         *mask,
                                                    gint                offset_x,
                                                    gint                offset_y,
                                                    const PikaRGB      *color,
                                                    gboolean            inverted);


#endif /* __PIKA_DISPLAY_SHELL_H__ */
