/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_IMAGE_WINDOW_H__
#define __PIKA_IMAGE_WINDOW_H__


#include "widgets/pikawindow.h"


#define PIKA_TYPE_IMAGE_WINDOW            (pika_image_window_get_type ())
#define PIKA_IMAGE_WINDOW(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_IMAGE_WINDOW, PikaImageWindow))
#define PIKA_IMAGE_WINDOW_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_IMAGE_WINDOW, PikaImageWindowClass))
#define PIKA_IS_IMAGE_WINDOW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_IMAGE_WINDOW))
#define PIKA_IS_IMAGE_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_IMAGE_WINDOW))
#define PIKA_IMAGE_WINDOW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_IMAGE_WINDOW, PikaImageWindowClass))


typedef struct _PikaImageWindowClass  PikaImageWindowClass;

struct _PikaImageWindow
{
  PikaWindow  parent_instance;
};

struct _PikaImageWindowClass
{
  PikaWindowClass  parent_class;
};


GType              pika_image_window_get_type             (void) G_GNUC_CONST;

PikaImageWindow  * pika_image_window_new                  (Pika              *pika,
                                                           PikaImage         *image,
                                                           PikaDialogFactory *dialog_factory,
                                                           GdkMonitor        *monitor);
void               pika_image_window_destroy              (PikaImageWindow   *window);

PikaDockColumns  * pika_image_window_get_left_docks       (PikaImageWindow  *window);
PikaDockColumns  * pika_image_window_get_right_docks      (PikaImageWindow  *window);

void               pika_image_window_add_shell            (PikaImageWindow  *window,
                                                           PikaDisplayShell *shell);
PikaDisplayShell * pika_image_window_get_shell            (PikaImageWindow  *window,
                                                           gint              index);
void               pika_image_window_remove_shell         (PikaImageWindow  *window,
                                                           PikaDisplayShell *shell);

gint               pika_image_window_get_n_shells         (PikaImageWindow  *window);

void               pika_image_window_set_active_shell     (PikaImageWindow  *window,
                                                           PikaDisplayShell *shell);
PikaDisplayShell * pika_image_window_get_active_shell     (PikaImageWindow  *window);

PikaMenuModel    * pika_image_window_get_menubar_model    (PikaImageWindow  *window);

void               pika_image_window_set_fullscreen       (PikaImageWindow  *window,
                                                           gboolean          fullscreen);
gboolean           pika_image_window_get_fullscreen       (PikaImageWindow  *window);

void               pika_image_window_set_show_menubar     (PikaImageWindow  *window,
                                                           gboolean          show);
gboolean           pika_image_window_get_show_menubar     (PikaImageWindow  *window);

void               pika_image_window_set_show_statusbar   (PikaImageWindow  *window,
                                                           gboolean          show);
gboolean           pika_image_window_get_show_statusbar   (PikaImageWindow  *window);

gboolean           pika_image_window_is_iconified         (PikaImageWindow  *window);
gboolean           pika_image_window_is_maximized         (PikaImageWindow  *window);

gboolean           pika_image_window_has_toolbox          (PikaImageWindow  *window);

void               pika_image_window_shrink_wrap          (PikaImageWindow  *window,
                                                           gboolean          grow_only);

GtkWidget        * pika_image_window_get_default_dockbook (PikaImageWindow  *window);

void               pika_image_window_keep_canvas_pos      (PikaImageWindow  *window);
void               pika_image_window_suspend_keep_pos     (PikaImageWindow  *window);
void               pika_image_window_resume_keep_pos      (PikaImageWindow  *window);

void               pika_image_window_update_tabs          (PikaImageWindow  *window);

#endif /* __PIKA_IMAGE_WINDOW_H__ */
