/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikamotionbuffer.h
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_MOTION_BUFFER_H__
#define __PIKA_MOTION_BUFFER_H__


#include "core/pikaobject.h"


#define PIKA_TYPE_MOTION_BUFFER            (pika_motion_buffer_get_type ())
#define PIKA_MOTION_BUFFER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_MOTION_BUFFER, PikaMotionBuffer))
#define PIKA_MOTION_BUFFER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_MOTION_BUFFER, PikaMotionBufferClass))
#define PIKA_IS_MOTION_BUFFER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_MOTION_BUFFER))
#define PIKA_IS_MOTION_BUFFER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_MOTION_BUFFER))
#define PIKA_MOTION_BUFFER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_MOTION_BUFFER, PikaMotionBufferClass))


typedef struct _PikaMotionBufferClass PikaMotionBufferClass;

struct _PikaMotionBuffer
{
  PikaObject  parent_instance;

  guint32     last_read_motion_time;

  guint32     last_motion_time; /*  previous time of a forwarded motion event  */
  gdouble     last_motion_delta_time;
  gdouble     last_motion_delta_x;
  gdouble     last_motion_delta_y;
  gdouble     last_motion_distance;

  PikaCoords  last_coords;      /* last motion event                   */

  GArray     *event_history;
  GArray     *event_queue;
  gboolean    event_delay;      /* TRUE if there's an unsent event in
                                 *  the history buffer
                                 */

  gint               event_delay_timeout;
  GdkModifierType    last_active_state;
};

struct _PikaMotionBufferClass
{
  PikaObjectClass  parent_class;

  void (* stroke) (PikaMotionBuffer *buffer,
                   const PikaCoords *coords,
                   guint32           time,
                   GdkModifierType   state);
  void (* hover)  (PikaMotionBuffer *buffer,
                   const PikaCoords *coords,
                   GdkModifierType   state,
                   gboolean          proximity);
};


GType              pika_motion_buffer_get_type     (void) G_GNUC_CONST;

PikaMotionBuffer * pika_motion_buffer_new          (void);

void       pika_motion_buffer_begin_stroke         (PikaMotionBuffer *buffer,
                                                    guint32           time,
                                                    PikaCoords       *last_motion);
void       pika_motion_buffer_end_stroke           (PikaMotionBuffer *buffer);

gboolean   pika_motion_buffer_motion_event         (PikaMotionBuffer *buffer,
                                                    PikaCoords       *coords,
                                                    guint32           time,
                                                    gboolean          event_fill);
guint32    pika_motion_buffer_get_last_motion_time (PikaMotionBuffer *buffer);

void       pika_motion_buffer_request_stroke       (PikaMotionBuffer *buffer,
                                                    GdkModifierType   state,
                                                    guint32           time);
void       pika_motion_buffer_request_hover        (PikaMotionBuffer *buffer,
                                                    GdkModifierType   state,
                                                    gboolean          proximity);


#endif /* __PIKA_MOTION_BUFFER_H__ */
