/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikatoolline.h
 * Copyright (C) 2017 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TOOL_LINE_H__
#define __PIKA_TOOL_LINE_H__


#include "pikatoolwidget.h"


/* in the context of PikaToolLine, "handle" is a collective term for either an
 * endpoint or a slider.  a handle value may be either a (nonnegative) slider
 * index, or one of the values below:
 */
#define PIKA_TOOL_LINE_HANDLE_NONE  (-3)
#define PIKA_TOOL_LINE_HANDLE_START (-2)
#define PIKA_TOOL_LINE_HANDLE_END   (-1)

#define PIKA_TOOL_LINE_HANDLE_IS_SLIDER(handle) ((handle) >= 0)


#define PIKA_TYPE_TOOL_LINE            (pika_tool_line_get_type ())
#define PIKA_TOOL_LINE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TOOL_LINE, PikaToolLine))
#define PIKA_TOOL_LINE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_TOOL_LINE, PikaToolLineClass))
#define PIKA_IS_TOOL_LINE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TOOL_LINE))
#define PIKA_IS_TOOL_LINE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_TOOL_LINE))
#define PIKA_TOOL_LINE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_TOOL_LINE, PikaToolLineClass))


typedef struct _PikaToolLine        PikaToolLine;
typedef struct _PikaToolLinePrivate PikaToolLinePrivate;
typedef struct _PikaToolLineClass   PikaToolLineClass;

struct _PikaToolLine
{
  PikaToolWidget       parent_instance;

  PikaToolLinePrivate *private;
};

struct _PikaToolLineClass
{
  PikaToolWidgetClass  parent_class;

  /*  signals  */
  gboolean (* can_add_slider)           (PikaToolLine        *line,
                                         gdouble              value);
  gint     (* add_slider)               (PikaToolLine        *line,
                                         gdouble              value);
  void     (* prepare_to_remove_slider) (PikaToolLine        *line,
                                         gint                 slider,
                                         gboolean             remove);
  void     (* remove_slider)            (PikaToolLine        *line,
                                         gint                 slider);
  void     (* selection_changed)        (PikaToolLine        *line);
  gboolean (* handle_clicked)           (PikaToolLine        *line,
                                         gint                 handle,
                                         GdkModifierType      state,
                                         PikaButtonPressType  press_type);
};


GType                        pika_tool_line_get_type      (void) G_GNUC_CONST;

PikaToolWidget             * pika_tool_line_new           (PikaDisplayShell           *shell,
                                                           gdouble                     x1,
                                                           gdouble                     y1,
                                                           gdouble                     x2,
                                                           gdouble                     y2);

void                         pika_tool_line_set_sliders   (PikaToolLine               *line,
                                                           const PikaControllerSlider *sliders,
                                                           gint                        n_sliders);
const PikaControllerSlider * pika_tool_line_get_sliders   (PikaToolLine               *line,
                                                           gint                       *n_sliders);

void                         pika_tool_line_set_selection (PikaToolLine               *line,
                                                           gint                        handle);
gint                         pika_tool_line_get_selection (PikaToolLine               *line);


#endif /* __PIKA_TOOL_LINE_H__ */
