/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikatoolpath.h
 * Copyright (C) 2017 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TOOL_PATH_H__
#define __PIKA_TOOL_PATH_H__


#include "pikatoolwidget.h"


#define PIKA_TYPE_TOOL_PATH            (pika_tool_path_get_type ())
#define PIKA_TOOL_PATH(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TOOL_PATH, PikaToolPath))
#define PIKA_TOOL_PATH_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_TOOL_PATH, PikaToolPathClass))
#define PIKA_IS_TOOL_PATH(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TOOL_PATH))
#define PIKA_IS_TOOL_PATH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_TOOL_PATH))
#define PIKA_TOOL_PATH_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_TOOL_PATH, PikaToolPathClass))


typedef struct _PikaToolPath        PikaToolPath;
typedef struct _PikaToolPathPrivate PikaToolPathPrivate;
typedef struct _PikaToolPathClass   PikaToolPathClass;

struct _PikaToolPath
{
  PikaToolWidget       parent_instance;

  PikaToolPathPrivate *private;
};

struct _PikaToolPathClass
{
  PikaToolWidgetClass  parent_class;

  void (* begin_change) (PikaToolPath    *path,
                         const gchar     *desc);
  void (* end_change)   (PikaToolPath    *path,
                         gboolean         success);
  void (* activate)     (PikaToolPath    *path,
                         GdkModifierType  state);
};


GType            pika_tool_path_get_type    (void) G_GNUC_CONST;

PikaToolWidget * pika_tool_path_new         (PikaDisplayShell *shell);

void             pika_tool_path_set_vectors (PikaToolPath     *path,
                                             PikaVectors      *vectors);

void             pika_tool_path_get_popup_state (PikaToolPath *path,
                                                 gboolean     *on_handle,
                                                 gboolean     *on_curve);

void             pika_tool_path_delete_anchor  (PikaToolPath *path);
void             pika_tool_path_shift_start    (PikaToolPath *path);
void             pika_tool_path_insert_anchor  (PikaToolPath *path);
void             pika_tool_path_delete_segment (PikaToolPath *path);
void             pika_tool_path_reverse_stroke (PikaToolPath *path);

#endif /* __PIKA_TOOL_PATH_H__ */
