/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TILE_HANDLER_VALIDATE_H__
#define __PIKA_TILE_HANDLER_VALIDATE_H__

#include <gegl-buffer-backend.h>

/***
 * PikaTileHandlerValidate is a GeglTileHandler that renders the
 * projection.
 */

G_BEGIN_DECLS

#define PIKA_TYPE_TILE_HANDLER_VALIDATE            (pika_tile_handler_validate_get_type ())
#define PIKA_TILE_HANDLER_VALIDATE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TILE_HANDLER_VALIDATE, PikaTileHandlerValidate))
#define PIKA_TILE_HANDLER_VALIDATE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  PIKA_TYPE_TILE_HANDLER_VALIDATE, PikaTileHandlerValidateClass))
#define PIKA_IS_TILE_HANDLER_VALIDATE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TILE_HANDLER_VALIDATE))
#define PIKA_IS_TILE_HANDLER_VALIDATE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  PIKA_TYPE_TILE_HANDLER_VALIDATE))
#define PIKA_TILE_HANDLER_VALIDATE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  PIKA_TYPE_TILE_HANDLER_VALIDATE, PikaTileHandlerValidateClass))


typedef struct _PikaTileHandlerValidate      PikaTileHandlerValidate;
typedef struct _PikaTileHandlerValidateClass PikaTileHandlerValidateClass;

struct _PikaTileHandlerValidate
{
  GeglTileHandler  parent_instance;

  GeglNode        *graph;
  cairo_region_t  *dirty_region;
  const Babl      *format;
  gint             tile_width;
  gint             tile_height;
  gboolean         whole_tile;
  gint             validating;
  gint             suspend_validate;
};

struct _PikaTileHandlerValidateClass
{
  GeglTileHandlerClass  parent_class;

  /*  signals  */
  void (* invalidated)     (PikaTileHandlerValidate *validate,
                            const GeglRectangle     *rect);

  /*  virtual functions  */
  void (* begin_validate)  (PikaTileHandlerValidate *validate);
  void (* end_validate)    (PikaTileHandlerValidate *validate);
  void (* validate)        (PikaTileHandlerValidate *validate,
                            const GeglRectangle     *rect,
                            const Babl              *format,
                            gpointer                 dest_buf,
                            gint                     dest_stride);
  void (* validate_buffer) (PikaTileHandlerValidate *validate,
                            const GeglRectangle     *rect,
                            GeglBuffer              *buffer);
};


GType                     pika_tile_handler_validate_get_type          (void) G_GNUC_CONST;

GeglTileHandler         * pika_tile_handler_validate_new               (GeglNode                *graph);

void                      pika_tile_handler_validate_assign            (PikaTileHandlerValidate *validate,
                                                                        GeglBuffer              *buffer);
void                      pika_tile_handler_validate_unassign          (PikaTileHandlerValidate *validate,
                                                                        GeglBuffer              *buffer);
PikaTileHandlerValidate * pika_tile_handler_validate_get_assigned      (GeglBuffer              *buffer);

void                      pika_tile_handler_validate_invalidate        (PikaTileHandlerValidate *validate,
                                                                        const GeglRectangle     *rect);
void                      pika_tile_handler_validate_undo_invalidate   (PikaTileHandlerValidate *validate,
                                                                        const GeglRectangle     *rect);

void                      pika_tile_handler_validate_begin_validate    (PikaTileHandlerValidate *validate);
void                      pika_tile_handler_validate_end_validate      (PikaTileHandlerValidate *validate);

void                      pika_tile_handler_validate_validate          (PikaTileHandlerValidate *validate,
                                                                        GeglBuffer              *buffer,
                                                                        const GeglRectangle     *rect,
                                                                        gboolean                 intersect,
                                                                        gboolean                 chunked);

gboolean                  pika_tile_handler_validate_buffer_set_extent (GeglBuffer              *buffer,
                                                                        const GeglRectangle     *extent);

void                      pika_tile_handler_validate_buffer_copy       (GeglBuffer              *src_buffer,
                                                                        const GeglRectangle     *src_rect,
                                                                        GeglBuffer              *dst_buffer,
                                                                        const GeglRectangle     *dst_rect);


G_END_DECLS

#endif /* __PIKA_TILE_HANDLER_VALIDATE_H__ */
