/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "libpikathumb/pikathumb.h"

#include "menus-types.h"

#include "config/pikaguiconfig.h"

#include "core/pika.h"
#include "core/pikaviewable.h"

#include "widgets/pikaaction.h"
#include "widgets/pikaactionimpl.h"
#include "widgets/pikauimanager.h"

#include "file-menu.h"


void
file_menu_setup (PikaUIManager *manager,
                 const gchar   *ui_path)
{
  gint n_entries;
  gint i;

  g_return_if_fail (PIKA_IS_UI_MANAGER (manager));
  g_return_if_fail (ui_path != NULL);

  n_entries = PIKA_GUI_CONFIG (manager->pika->config)->last_opened_size;

  for (i = 0; i < n_entries; i++)
    {
      gchar *action_name;

      action_name = g_strdup_printf ("file-open-recent-%02d", i + 1);

      pika_ui_manager_add_ui (manager, "/File/Open Recent/[Files]",
                              action_name, FALSE);

      g_free (action_name);
    }
}
