/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pika-layer-modes.h
 * Copyright (C) 2017 Michael Natterer <mitch@gimp.org>
 *                    Øyvind Kolås <pippin@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_LAYER_MODES_H__
#define __PIKA_LAYER_MODES_H__


void                       pika_layer_modes_init                      (void);
void                       pika_layer_modes_exit                      (void);

gboolean                   pika_layer_mode_is_legacy                  (PikaLayerMode           mode);

gboolean                   pika_layer_mode_is_blend_space_mutable     (PikaLayerMode           mode);
gboolean                   pika_layer_mode_is_composite_space_mutable (PikaLayerMode           mode);
gboolean                   pika_layer_mode_is_composite_mode_mutable  (PikaLayerMode           mode);

gboolean                   pika_layer_mode_is_subtractive             (PikaLayerMode           mode);
gboolean                   pika_layer_mode_is_alpha_only              (PikaLayerMode           mode);
gboolean                   pika_layer_mode_is_trivial                 (PikaLayerMode           mode);

PikaLayerColorSpace        pika_layer_mode_get_blend_space            (PikaLayerMode           mode);
PikaLayerColorSpace        pika_layer_mode_get_composite_space        (PikaLayerMode           mode);
PikaLayerCompositeMode     pika_layer_mode_get_composite_mode         (PikaLayerMode           mode);
PikaLayerCompositeMode     pika_layer_mode_get_paint_composite_mode   (PikaLayerMode           mode);

const gchar              * pika_layer_mode_get_operation_name         (PikaLayerMode           mode);
GeglOperation            * pika_layer_mode_get_operation              (PikaLayerMode           mode);

PikaLayerModeFunc          pika_layer_mode_get_function               (PikaLayerMode           mode);
PikaLayerModeBlendFunc     pika_layer_mode_get_blend_function         (PikaLayerMode           mode);

PikaLayerModeContext       pika_layer_mode_get_context                (PikaLayerMode           mode);

PikaLayerMode            * pika_layer_mode_get_context_array          (PikaLayerMode           mode,
                                                                       PikaLayerModeContext    context,
                                                                       gint                   *n_modes);

PikaLayerModeGroup         pika_layer_mode_get_group                  (PikaLayerMode           mode);

const PikaLayerMode      * pika_layer_mode_get_group_array            (PikaLayerModeGroup      group,
                                                                       gint                   *n_modes);

gboolean                   pika_layer_mode_get_for_group              (PikaLayerMode           old_mode,
                                                                       PikaLayerModeGroup      new_group,
                                                                       PikaLayerMode          *new_mode);

const Babl               * pika_layer_mode_get_format                 (PikaLayerMode           mode,
                                                                       PikaLayerColorSpace     blend_space,
                                                                       PikaLayerColorSpace     composite_space,
                                                                       PikaLayerCompositeMode  composite_mode,
                                                                       const Babl             *preferred_format);

PikaLayerCompositeRegion   pika_layer_mode_get_included_region        (PikaLayerMode           mode,
                                                                       PikaLayerCompositeMode  composite_mode);


#endif /* __PIKA_LAYER_MODES_H__ */
