/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikaoperationlayermode.h
 * Copyright (C) 2008 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_OPERATION_LAYER_MODE_H__
#define __PIKA_OPERATION_LAYER_MODE_H__


#include <gegl-plugin.h>


#define PIKA_TYPE_OPERATION_LAYER_MODE            (pika_operation_layer_mode_get_type ())
#define PIKA_OPERATION_LAYER_MODE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_OPERATION_LAYER_MODE, PikaOperationLayerMode))
#define PIKA_OPERATION_LAYER_MODE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  PIKA_TYPE_OPERATION_LAYER_MODE, PikaOperationLayerModeClass))
#define PIKA_IS_OPERATION_LAYER_MODE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_OPERATION_LAYER_MODE))
#define PIKA_IS_OPERATION_LAYER_MODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  PIKA_TYPE_OPERATION_LAYER_MODE))
#define PIKA_OPERATION_LAYER_MODE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  PIKA_TYPE_OPERATION_LAYER_MODE, PikaOperationLayerModeClass))


typedef struct _PikaOperationLayerModeClass PikaOperationLayerModeClass;

struct _PikaOperationLayerMode
{
  GeglOperationPointComposer3  parent_instance;

  PikaLayerMode                layer_mode;
  gdouble                      opacity;
  PikaLayerColorSpace          blend_space;
  PikaLayerColorSpace          composite_space;
  PikaLayerCompositeMode       composite_mode;
  const Babl                  *cached_fish_format;
  const Babl                  *space_fish[3 /* from */][3 /* to */];

  gdouble                      prop_opacity;
  PikaLayerCompositeMode       prop_composite_mode;

  PikaLayerModeFunc            function;
  PikaLayerModeBlendFunc       blend_function;
  gboolean                     is_last_node;
  gboolean                     has_mask;
};

struct _PikaOperationLayerModeClass
{
  GeglOperationPointComposer3Class  parent_class;

  /*  virtual functions  */
  gboolean                 (* parent_process)      (GeglOperation          *operation,
                                                    GeglOperationContext   *context,
                                                    const gchar            *output_prop,
                                                    const GeglRectangle    *result,
                                                    gint                    level);
  gboolean                 (* process)             (GeglOperation          *operation,
                                                    void                   *in,
                                                    void                   *aux,
                                                    void                   *mask,
                                                    void                   *out,
                                                    glong                   samples,
                                                    const GeglRectangle    *roi,
                                                    gint                    level);

  /* Returns the composite region (any combination of the layer and the
   * backdrop) that the layer mode affects.  Most modes only affect the
   * overlapping region, and don't need to override this function.
   */
  PikaLayerCompositeRegion (* get_affected_region) (PikaOperationLayerMode *layer_mode);
};


GType                    pika_operation_layer_mode_get_type            (void) G_GNUC_CONST;

PikaLayerCompositeRegion pika_operation_layer_mode_get_affected_region (PikaOperationLayerMode *layer_mode);


#endif /* __PIKA_OPERATION_LAYER_MODE_H__ */
