/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikacurvesconfig.h
 * Copyright (C) 2007 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_CURVES_CONFIG_H__
#define __PIKA_CURVES_CONFIG_H__


#include "pikaoperationsettings.h"


#define PIKA_TYPE_CURVES_CONFIG            (pika_curves_config_get_type ())
#define PIKA_CURVES_CONFIG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_CURVES_CONFIG, PikaCurvesConfig))
#define PIKA_CURVES_CONFIG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  PIKA_TYPE_CURVES_CONFIG, PikaCurvesConfigClass))
#define PIKA_IS_CURVES_CONFIG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_CURVES_CONFIG))
#define PIKA_IS_CURVES_CONFIG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  PIKA_TYPE_CURVES_CONFIG))
#define PIKA_CURVES_CONFIG_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  PIKA_TYPE_CURVES_CONFIG, PikaCurvesConfigClass))


typedef struct _PikaCurvesConfigClass PikaCurvesConfigClass;

struct _PikaCurvesConfig
{
  PikaOperationSettings  parent_instance;

  PikaTRCType            trc;

  PikaHistogramChannel   channel;

  PikaCurve              *curve[5];
};

struct _PikaCurvesConfigClass
{
  PikaOperationSettingsClass  parent_class;
};


GType      pika_curves_config_get_type            (void) G_GNUC_CONST;

GObject  * pika_curves_config_new_spline          (gint32             channel,
                                                   const gdouble     *points,
                                                   gint               n_points);
GObject *  pika_curves_config_new_explicit        (gint32             channel,
                                                   const gdouble     *samples,
                                                   gint               n_samples);

GObject  * pika_curves_config_new_spline_cruft    (gint32             channel,
                                                   const guint8      *points,
                                                   gint               n_points);
GObject *  pika_curves_config_new_explicit_cruft  (gint32             channel,
                                                   const guint8      *samples,
                                                   gint               n_samples);

void       pika_curves_config_reset_channel       (PikaCurvesConfig  *config);

gboolean   pika_curves_config_load_cruft          (PikaCurvesConfig  *config,
                                                   GInputStream      *input,
                                                   GError           **error);
gboolean   pika_curves_config_save_cruft          (PikaCurvesConfig  *config,
                                                   GOutputStream     *output,
                                                   GError           **error);


#endif /* __PIKA_CURVES_CONFIG_H__ */
