/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikalevelsconfig.h
 * Copyright (C) 2007 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_LEVELS_CONFIG_H__
#define __PIKA_LEVELS_CONFIG_H__


#include "pikaoperationsettings.h"


#define PIKA_TYPE_LEVELS_CONFIG            (pika_levels_config_get_type ())
#define PIKA_LEVELS_CONFIG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_LEVELS_CONFIG, PikaLevelsConfig))
#define PIKA_LEVELS_CONFIG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass),  PIKA_TYPE_LEVELS_CONFIG, PikaLevelsConfigClass))
#define PIKA_IS_LEVELS_CONFIG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_LEVELS_CONFIG))
#define PIKA_IS_LEVELS_CONFIG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass),  PIKA_TYPE_LEVELS_CONFIG))
#define PIKA_LEVELS_CONFIG_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj),  PIKA_TYPE_LEVELS_CONFIG, PikaLevelsConfigClass))


typedef struct _PikaLevelsConfigClass PikaLevelsConfigClass;

struct _PikaLevelsConfig
{
  PikaOperationSettings  parent_instance;

  PikaTRCType            trc;

  PikaHistogramChannel   channel;

  gdouble                low_input[5];
  gdouble                high_input[5];

  gboolean               clamp_input;

  gdouble                gamma[5];

  gdouble                low_output[5];
  gdouble                high_output[5];

  gboolean               clamp_output;
};

struct _PikaLevelsConfigClass
{
  PikaOperationSettingsClass  parent_class;
};


GType      pika_levels_config_get_type         (void) G_GNUC_CONST;

void       pika_levels_config_reset_channel    (PikaLevelsConfig      *config);

void       pika_levels_config_stretch          (PikaLevelsConfig      *config,
                                                PikaHistogram         *histogram,
                                                gboolean               is_color);
void       pika_levels_config_stretch_channel  (PikaLevelsConfig      *config,
                                                PikaHistogram         *histogram,
                                                PikaHistogramChannel   channel);
void       pika_levels_config_adjust_by_colors (PikaLevelsConfig      *config,
                                                PikaHistogramChannel   channel,
                                                const PikaRGB         *black,
                                                const PikaRGB         *gray,
                                                const PikaRGB         *white);

PikaCurvesConfig *
           pika_levels_config_to_curves_config (PikaLevelsConfig      *config);

gboolean   pika_levels_config_load_cruft       (PikaLevelsConfig      *config,
                                                GInputStream          *input,
                                                GError               **error);
gboolean   pika_levels_config_save_cruft       (PikaLevelsConfig      *config,
                                                GOutputStream         *output,
                                                GError               **error);


#endif /* __PIKA_LEVELS_CONFIG_H__ */
