/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_PAINTBRUSH_H__
#define __PIKA_PAINTBRUSH_H__


#include "pikabrushcore.h"


#define PIKA_TYPE_PAINTBRUSH            (pika_paintbrush_get_type ())
#define PIKA_PAINTBRUSH(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_PAINTBRUSH, PikaPaintbrush))
#define PIKA_PAINTBRUSH_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_PAINTBRUSH, PikaPaintbrushClass))
#define PIKA_IS_PAINTBRUSH(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_PAINTBRUSH))
#define PIKA_IS_PAINTBRUSH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_PAINTBRUSH))
#define PIKA_PAINTBRUSH_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_PAINTBRUSH, PikaPaintbrushClass))


typedef struct _PikaPaintbrushClass PikaPaintbrushClass;

struct _PikaPaintbrush
{
  PikaBrushCore      parent_instance;

  GeglBuffer        *paint_buffer;
  const PikaTempBuf *paint_pixmap;
  PikaRGB            paint_color;
};

struct _PikaPaintbrushClass
{
  PikaBrushCoreClass  parent_class;

  /*  virtual functions  */
  gboolean   (* get_color_history_color) (PikaPaintbrush            *paintbrush,
                                          PikaDrawable              *drawable,
                                          PikaPaintOptions          *paint_options,
                                          PikaRGB                   *color);
  void       (* get_paint_params)        (PikaPaintbrush            *paintbrush,
                                          PikaDrawable              *drawable,
                                          PikaPaintOptions          *paint_options,
                                          PikaSymmetry              *sym,
                                          gdouble                    grad_point,
                                          PikaLayerMode             *paint_mode,
                                          PikaPaintApplicationMode  *paint_appl_mode,
                                          const PikaTempBuf        **paint_pixmap,
                                          PikaRGB                   *paint_color);
};


void    pika_paintbrush_register (Pika                      *pika,
                                  PikaPaintRegisterCallback  callback);

GType   pika_paintbrush_get_type (void) G_GNUC_CONST;


/*  protected  */

void    _pika_paintbrush_motion  (PikaPaintCore             *paint_core,
                                  PikaDrawable              *drawable,
                                  PikaPaintOptions          *paint_options,
                                  PikaSymmetry              *sym,
                                  gdouble                    opacity);


#endif  /*  __PIKA_PAINTBRUSH_H__  */
