/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_PERSPECTIVE_CLONE_H__
#define __PIKA_PERSPECTIVE_CLONE_H__


#include "pikaclone.h"


#define PIKA_TYPE_PERSPECTIVE_CLONE            (pika_perspective_clone_get_type ())
#define PIKA_PERSPECTIVE_CLONE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_PERSPECTIVE_CLONE, PikaPerspectiveClone))
#define PIKA_PERSPECTIVE_CLONE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_PERSPECTIVE_CLONE, PikaPerspectiveCloneClass))
#define PIKA_IS_PERSPECTIVE_CLONE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_PERSPECTIVE_CLONE))
#define PIKA_IS_PERSPECTIVE_CLONE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_PERSPECTIVE_CLONE))
#define PIKA_PERSPECTIVE_CLONE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_PERSPECTIVE_CLONE, PikaPerspectiveCloneClass))


typedef struct _PikaPerspectiveCloneClass PikaPerspectiveCloneClass;

struct _PikaPerspectiveClone
{
  PikaClone      parent_instance;

  gdouble        src_x_fv;     /* source coords in front_view perspective */
  gdouble        src_y_fv;

  gdouble        dest_x_fv;    /* destination coords in front_view perspective */
  gdouble        dest_y_fv;

  PikaMatrix3    transform;
  PikaMatrix3    transform_inv;

  GeglNode      *node;
  GeglNode      *crop;
  GeglNode      *transform_node;
  GeglNode      *src_node;
  GeglNode      *dest_node;
};

struct _PikaPerspectiveCloneClass
{
  PikaCloneClass  parent_class;
};


void    pika_perspective_clone_register      (Pika                      *pika,
                                              PikaPaintRegisterCallback  callback);

GType   pika_perspective_clone_get_type         (void) G_GNUC_CONST;

void    pika_perspective_clone_set_transform    (PikaPerspectiveClone   *clone,
                                                 PikaMatrix3            *transform);
void    pika_perspective_clone_get_source_point (PikaPerspectiveClone   *clone,
                                                 gdouble                 x,
                                                 gdouble                 y,
                                                 gdouble                *newx,
                                                 gdouble                *newy);


#endif  /*  __PIKA_PERSPECTIVE_CLONE_H__  */
