/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_SOURCE_CORE_H__
#define __PIKA_SOURCE_CORE_H__


#include "pikabrushcore.h"


#define PIKA_TYPE_SOURCE_CORE            (pika_source_core_get_type ())
#define PIKA_SOURCE_CORE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_SOURCE_CORE, PikaSourceCore))
#define PIKA_SOURCE_CORE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_SOURCE_CORE, PikaSourceCoreClass))
#define PIKA_IS_SOURCE_CORE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_SOURCE_CORE))
#define PIKA_IS_SOURCE_CORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_SOURCE_CORE))
#define PIKA_SOURCE_CORE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_SOURCE_CORE, PikaSourceCoreClass))


typedef struct _PikaSourceCoreClass PikaSourceCoreClass;

struct _PikaSourceCore
{
  PikaBrushCore  parent_instance;

  gboolean       set_source;

  gint           orig_src_x;
  gint           orig_src_y;

  gint           offset_x;
  gint           offset_y;
  gboolean       first_stroke;
};

struct _PikaSourceCoreClass
{
  PikaBrushCoreClass  parent_class;

  gboolean     (* use_source) (PikaSourceCore    *source_core,
                               PikaSourceOptions *options);

  GeglBuffer * (* get_source) (PikaSourceCore    *source_core,
                               PikaDrawable      *drawable,
                               PikaPaintOptions  *paint_options,
                               gboolean           self_drawable,
                               PikaPickable      *src_pickable,
                               gint               src_offset_x,
                               gint               src_offset_y,
                               GeglBuffer        *paint_buffer,
                               gint               paint_buffer_x,
                               gint               paint_buffer_y,
                               /* offsets *into* the paint_buffer: */
                               gint              *paint_area_offset_x,
                               gint              *paint_area_offset_y,
                               gint              *paint_area_width,
                               gint              *paint_area_height,
                               GeglRectangle     *src_rect);

  void         (*  motion)    (PikaSourceCore    *source_core,
                               PikaDrawable      *drawable,
                               PikaPaintOptions  *paint_options,
                               const PikaCoords  *coords,
                               GeglNode          *op,
                               gdouble            opacity,
                               PikaPickable      *src_pickable,
                               GeglBuffer        *src_buffer,
                               GeglRectangle     *src_rect,
                               gint               src_offset_x,
                               gint               src_offset_y,
                               GeglBuffer        *paint_buffer,
                               gint               paint_buffer_x,
                               gint               paint_buffer_y,
                               /* offsets *into* the paint_buffer: */
                               gint               paint_area_offset_x,
                               gint               paint_area_offset_y,
                               gint               paint_area_width,
                               gint               paint_area_height);
};


GType    pika_source_core_get_type   (void) G_GNUC_CONST;

gboolean pika_source_core_use_source (PikaSourceCore    *source_core,
                                      PikaSourceOptions *options);

/* TEMP HACK */
void     pika_source_core_motion     (PikaSourceCore    *source_core,
                                      PikaDrawable      *drawable,
                                      PikaPaintOptions  *paint_options,
                                      gboolean           self_drawable,
                                      PikaSymmetry      *sym);


#endif  /*  __PIKA_SOURCE_CORE_H__  */
