/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-2003 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl. */

#include "config.h"

#include "stamp-pdbgen.h"

#include <gegl.h>

#include <gdk-pixbuf/gdk-pixbuf.h>

#include "libpikabase/pikabase.h"

#include "pdb-types.h"

#include "core/pika.h"
#include "core/pikadatafactory.h"
#include "core/pikaparamspecs.h"

#include "pikapdb.h"
#include "pikaprocedure.h"
#include "internal-procs.h"


static PikaValueArray *
brushes_popup_invoker (PikaProcedure         *procedure,
                       Pika                  *pika,
                       PikaContext           *context,
                       PikaProgress          *progress,
                       const PikaValueArray  *args,
                       GError               **error)
{
  gboolean success = TRUE;
  const gchar *brush_callback;
  const gchar *popup_title;
  const gchar *initial_brush_name;

  brush_callback = g_value_get_string (pika_value_array_index (args, 0));
  popup_title = g_value_get_string (pika_value_array_index (args, 1));
  initial_brush_name = g_value_get_string (pika_value_array_index (args, 2));

  if (success)
    {
        if (pika->no_interface ||
          ! pika_pdb_lookup_procedure (pika->pdb, brush_callback) ||
          ! pika_pdb_dialog_new (pika, context, progress,
                                 pika_data_factory_get_container (pika->brush_factory),
                                 popup_title, brush_callback, initial_brush_name,
                                 NULL))
        success = FALSE;
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

static PikaValueArray *
brushes_close_popup_invoker (PikaProcedure         *procedure,
                             Pika                  *pika,
                             PikaContext           *context,
                             PikaProgress          *progress,
                             const PikaValueArray  *args,
                             GError               **error)
{
  gboolean success = TRUE;
  const gchar *brush_callback;

  brush_callback = g_value_get_string (pika_value_array_index (args, 0));

  if (success)
    {
      if (pika->no_interface ||
          ! pika_pdb_lookup_procedure (pika->pdb, brush_callback) ||
          ! pika_pdb_dialog_close (pika, pika_data_factory_get_container (pika->brush_factory),
                                   brush_callback))
        success = FALSE;
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

static PikaValueArray *
brushes_set_popup_invoker (PikaProcedure         *procedure,
                           Pika                  *pika,
                           PikaContext           *context,
                           PikaProgress          *progress,
                           const PikaValueArray  *args,
                           GError               **error)
{
  gboolean success = TRUE;
  const gchar *brush_callback;
  const gchar *brush_name;

  brush_callback = g_value_get_string (pika_value_array_index (args, 0));
  brush_name = g_value_get_string (pika_value_array_index (args, 1));

  if (success)
    {
      if (pika->no_interface ||
          ! pika_pdb_lookup_procedure (pika->pdb, brush_callback) ||
          ! pika_pdb_dialog_set (pika, pika_data_factory_get_container (pika->brush_factory),
                                 brush_callback, brush_name,
                                 NULL))
        success = FALSE;
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

void
register_brush_select_procs (PikaPDB *pdb)
{
  PikaProcedure *procedure;

  /*
   * pika-brushes-popup
   */
  procedure = pika_procedure_new (brushes_popup_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-brushes-popup");
  pika_procedure_set_static_help (procedure,
                                  "Invokes the PIKA brush selection dialog.",
                                  "Opens a dialog letting a user choose a brush.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Andy Thomas",
                                         "Andy Thomas",
                                         "1998");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("brush-callback",
                                                       "brush callback",
                                                       "The callback PDB proc to call when user chooses a brush",
                                                       FALSE, FALSE, TRUE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("popup-title",
                                                       "popup title",
                                                       "Title of the brush selection dialog",
                                                       FALSE, FALSE, FALSE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("initial-brush-name",
                                                       "initial brush name",
                                                       "The name of the brush to set as the initial choice",
                                                       FALSE, TRUE, FALSE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-brushes-close-popup
   */
  procedure = pika_procedure_new (brushes_close_popup_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-brushes-close-popup");
  pika_procedure_set_static_help (procedure,
                                  "Close the brush selection dialog.",
                                  "Closes an open brush selection dialog.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Andy Thomas",
                                         "Andy Thomas",
                                         "1998");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("brush-callback",
                                                       "brush callback",
                                                       "The name of the callback registered for this pop-up",
                                                       FALSE, FALSE, TRUE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-brushes-set-popup
   */
  procedure = pika_procedure_new (brushes_set_popup_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-brushes-set-popup");
  pika_procedure_set_static_help (procedure,
                                  "Sets the selected brush in a brush selection dialog.",
                                  "Sets the selected brush in a brush selection dialog.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Andy Thomas",
                                         "Andy Thomas",
                                         "1998");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("brush-callback",
                                                       "brush callback",
                                                       "The name of the callback registered for this pop-up",
                                                       FALSE, FALSE, TRUE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("brush-name",
                                                       "brush name",
                                                       "The name of the brush to set as selected",
                                                       FALSE, FALSE, FALSE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);
}
