/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-2003 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl. */

#include "config.h"

#include "stamp-pdbgen.h"

#include <gegl.h>

#include <gdk-pixbuf/gdk-pixbuf.h>

#include "libpikabase/pikabase.h"

#include "pdb-types.h"

#include "core/pikaparamspecs.h"

#include "pikapdb.h"
#include "pikaprocedure.h"
#include "internal-procs.h"


static GTimer *pika_debug_timer         = NULL;
static gint    pika_debug_timer_counter = 0;

static PikaValueArray *
debug_timer_start_invoker (PikaProcedure         *procedure,
                           Pika                  *pika,
                           PikaContext           *context,
                           PikaProgress          *progress,
                           const PikaValueArray  *args,
                           GError               **error)
{
  if (pika_debug_timer_counter++ == 0)
    pika_debug_timer = g_timer_new ();

  return pika_procedure_get_return_values (procedure, TRUE, NULL);
}

static PikaValueArray *
debug_timer_end_invoker (PikaProcedure         *procedure,
                         Pika                  *pika,
                         PikaContext           *context,
                         PikaProgress          *progress,
                         const PikaValueArray  *args,
                         GError               **error)
{
  gboolean success = TRUE;
  PikaValueArray *return_vals;
  gdouble elapsed = 0.0;

  elapsed = 0.0;

  if (pika_debug_timer_counter == 0)
    success = FALSE;
  else if (--pika_debug_timer_counter == 0)
    {
      elapsed = g_timer_elapsed (pika_debug_timer, NULL);

      g_printerr ("PIKA debug timer: %g seconds\n", elapsed);

      g_timer_destroy (pika_debug_timer);

      pika_debug_timer = NULL;
    }

  return_vals = pika_procedure_get_return_values (procedure, success,
                                                  error ? *error : NULL);

  if (success)
    g_value_set_double (pika_value_array_index (return_vals, 1), elapsed);

  return return_vals;
}

void
register_debug_procs (PikaPDB *pdb)
{
  PikaProcedure *procedure;

  /*
   * pika-debug-timer-start
   */
  procedure = pika_procedure_new (debug_timer_start_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-debug-timer-start");
  pika_procedure_set_static_help (procedure,
                                  "Starts measuring elapsed time.",
                                  "This procedure starts a timer, measuring the elapsed time since the call. Each call to this procedure should be matched by a call to 'pika-debug-timer-end', which returns the elapsed time.\n"
                                  "If there is already an active timer, it is not affected by the call, however, a matching 'pika-debug-timer-end' call is still required.\n"
                                  "\n"
                                  "This is a debug utility procedure. It is subject to change at any point, and should not be used in production.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Ell",
                                         "Ell",
                                         "2017");
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-debug-timer-end
   */
  procedure = pika_procedure_new (debug_timer_end_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-debug-timer-end");
  pika_procedure_set_static_help (procedure,
                                  "Finishes measuring elapsed time.",
                                  "This procedure stops the timer started by a previous 'pika-debug-timer-start' call, and prints and returns the elapsed time.\n"
                                  "If there was already an active timer at the time of corresponding call to 'pika-debug-timer-start', a dummy value is returned.\n"
                                  "\n"
                                  "This is a debug utility procedure. It is subject to change at any point, and should not be used in production.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Ell",
                                         "Ell",
                                         "2017");
  pika_procedure_add_return_value (procedure,
                                   g_param_spec_double ("elapsed",
                                                        "elapsed",
                                                        "The elapsed time, in seconds",
                                                        -G_MAXDOUBLE, G_MAXDOUBLE, 0,
                                                        PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);
}
