/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-2003 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl. */

#include "config.h"

#include "stamp-pdbgen.h"

#include <gegl.h>

#include <gdk-pixbuf/gdk-pixbuf.h>

#include "libpikabase/pikabase.h"

#include "pdb-types.h"

#include "core/pika.h"
#include "core/pikadatafactory.h"
#include "core/pikaparamspecs.h"

#include "pikapdb.h"
#include "pikaprocedure.h"
#include "internal-procs.h"


static PikaValueArray *
fonts_popup_invoker (PikaProcedure         *procedure,
                     Pika                  *pika,
                     PikaContext           *context,
                     PikaProgress          *progress,
                     const PikaValueArray  *args,
                     GError               **error)
{
  gboolean success = TRUE;
  const gchar *font_callback;
  const gchar *popup_title;
  const gchar *initial_font_name;

  font_callback = g_value_get_string (pika_value_array_index (args, 0));
  popup_title = g_value_get_string (pika_value_array_index (args, 1));
  initial_font_name = g_value_get_string (pika_value_array_index (args, 2));

  if (success)
    {
      if (pika->no_interface ||
          ! pika_pdb_lookup_procedure (pika->pdb, font_callback) ||
          ! pika_data_factory_data_wait (pika->font_factory)     ||
          ! pika_pdb_dialog_new (pika, context, progress,
                                 pika_data_factory_get_container (pika->font_factory),
                                 popup_title, font_callback, initial_font_name,
                                 NULL))
        success = FALSE;
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

static PikaValueArray *
fonts_close_popup_invoker (PikaProcedure         *procedure,
                           Pika                  *pika,
                           PikaContext           *context,
                           PikaProgress          *progress,
                           const PikaValueArray  *args,
                           GError               **error)
{
  gboolean success = TRUE;
  const gchar *font_callback;

  font_callback = g_value_get_string (pika_value_array_index (args, 0));

  if (success)
    {
      if (pika->no_interface ||
          ! pika_pdb_lookup_procedure (pika->pdb, font_callback) ||
          ! pika_pdb_dialog_close (pika,
                                   pika_data_factory_get_container (pika->font_factory),
                                   font_callback))
        success = FALSE;
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

static PikaValueArray *
fonts_set_popup_invoker (PikaProcedure         *procedure,
                         Pika                  *pika,
                         PikaContext           *context,
                         PikaProgress          *progress,
                         const PikaValueArray  *args,
                         GError               **error)
{
  gboolean success = TRUE;
  const gchar *font_callback;
  const gchar *font_name;

  font_callback = g_value_get_string (pika_value_array_index (args, 0));
  font_name = g_value_get_string (pika_value_array_index (args, 1));

  if (success)
    {
      if (pika->no_interface ||
          ! pika_pdb_lookup_procedure (pika->pdb, font_callback) ||
          ! pika_data_factory_data_wait (pika->font_factory)     ||
          ! pika_pdb_dialog_set (pika,
                                 pika_data_factory_get_container (pika->font_factory),
                                 font_callback, font_name,
                                 NULL))
        success = FALSE;
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

void
register_font_select_procs (PikaPDB *pdb)
{
  PikaProcedure *procedure;

  /*
   * pika-fonts-popup
   */
  procedure = pika_procedure_new (fonts_popup_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-fonts-popup");
  pika_procedure_set_static_help (procedure,
                                  "Invokes the Pika font selection dialog.",
                                  "Opens a dialog letting a user choose a font.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Sven Neumann <sven@gimp.org>",
                                         "Sven Neumann",
                                         "2003");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("font-callback",
                                                       "font callback",
                                                       "The callback PDB proc to call when user chooses a font",
                                                       FALSE, FALSE, TRUE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("popup-title",
                                                       "popup title",
                                                       "Title of the font selection dialog",
                                                       FALSE, FALSE, FALSE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("initial-font-name",
                                                       "initial font name",
                                                       "The name of the initial font choice.",
                                                       FALSE, TRUE, FALSE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-fonts-close-popup
   */
  procedure = pika_procedure_new (fonts_close_popup_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-fonts-close-popup");
  pika_procedure_set_static_help (procedure,
                                  "Close the font selection dialog.",
                                  "Closes an open font selection dialog.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Sven Neumann <sven@gimp.org>",
                                         "Sven Neumann",
                                         "2003");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("font-callback",
                                                       "font callback",
                                                       "The name of the callback registered in the PDB for this dialog",
                                                       FALSE, FALSE, TRUE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-fonts-set-popup
   */
  procedure = pika_procedure_new (fonts_set_popup_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-fonts-set-popup");
  pika_procedure_set_static_help (procedure,
                                  "Sets the current font in a font selection dialog.",
                                  "Sets the current font in a font selection dialog.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Sven Neumann <sven@gimp.org>",
                                         "Sven Neumann",
                                         "2003");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("font-callback",
                                                       "font callback",
                                                       "The name of the callback registered in the PDB for the dialog.",
                                                       FALSE, FALSE, TRUE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("font-name",
                                                       "font name",
                                                       "The name of the font to set as selected",
                                                       FALSE, FALSE, FALSE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);
}
