/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-2003 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl. */

#include "config.h"

#include "stamp-pdbgen.h"

#include <gegl.h>

#include <gdk-pixbuf/gdk-pixbuf.h>

#include "libpikabase/pikabase.h"

#include "pdb-types.h"

#include "core/pika.h"
#include "core/pikacontainer-filter.h"
#include "core/pikacontainer.h"
#include "core/pikadatafactory.h"
#include "core/pikaparamspecs.h"

#include "pikapdb.h"
#include "pikaprocedure.h"
#include "internal-procs.h"


static PikaValueArray *
fonts_refresh_invoker (PikaProcedure         *procedure,
                       Pika                  *pika,
                       PikaContext           *context,
                       PikaProgress          *progress,
                       const PikaValueArray  *args,
                       GError               **error)
{
  pika_data_factory_data_refresh (pika->font_factory, context);
  pika_data_factory_data_wait (pika->font_factory);

  return pika_procedure_get_return_values (procedure, TRUE, NULL);
}

static PikaValueArray *
fonts_get_list_invoker (PikaProcedure         *procedure,
                        Pika                  *pika,
                        PikaContext           *context,
                        PikaProgress          *progress,
                        const PikaValueArray  *args,
                        GError               **error)
{
  gboolean success = TRUE;
  PikaValueArray *return_vals;
  const gchar *filter;
  gchar **font_list = NULL;

  filter = g_value_get_string (pika_value_array_index (args, 0));

  if (success)
    {
      if (! pika_data_factory_data_wait (pika->font_factory))
        success = FALSE;

      if (success)
        {
          font_list = pika_container_get_filtered_name_array (pika_data_factory_get_container (pika->font_factory),
                                                              filter);
        }
    }

  return_vals = pika_procedure_get_return_values (procedure, success,
                                                  error ? *error : NULL);

  if (success)
    g_value_take_boxed (pika_value_array_index (return_vals, 1), font_list);

  return return_vals;
}

void
register_fonts_procs (PikaPDB *pdb)
{
  PikaProcedure *procedure;

  /*
   * pika-fonts-refresh
   */
  procedure = pika_procedure_new (fonts_refresh_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-fonts-refresh");
  pika_procedure_set_static_help (procedure,
                                  "Refresh current fonts. This function always succeeds.",
                                  "This procedure retrieves all fonts currently in the user's font path and updates the font dialogs accordingly. Depending on the amount of fonts on the system, this can take considerable time.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Sven Neumann <sven@gimp.org>",
                                         "Sven Neumann",
                                         "2003");
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-fonts-get-list
   */
  procedure = pika_procedure_new (fonts_get_list_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-fonts-get-list");
  pika_procedure_set_static_help (procedure,
                                  "Retrieve the list of loaded fonts.",
                                  "This procedure returns a list of the fonts that are currently available.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Sven Neumann <sven@gimp.org>",
                                         "Sven Neumann",
                                         "2003");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("filter",
                                                       "filter",
                                                       "An optional regular expression used to filter the list",
                                                       FALSE, TRUE, FALSE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_procedure_add_return_value (procedure,
                                   g_param_spec_boxed ("font-list",
                                                       "font list",
                                                       "The list of font names",
                                                       G_TYPE_STRV,
                                                       PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);
}
