/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-2003 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl. */

#include "config.h"

#include "stamp-pdbgen.h"

#include <cairo.h>

#include <gegl.h>

#include <gdk-pixbuf/gdk-pixbuf.h>

#include "libpikabase/pikabase.h"
#include "libpikacolor/pikacolor.h"

#include "libpikabase/pikabase.h"

#include "pdb-types.h"

#include "core/pikachannel-select.h"
#include "core/pikadrawable.h"
#include "core/pikaimage.h"
#include "core/pikaitem.h"
#include "core/pikaparamspecs.h"

#include "pikapdb.h"
#include "pikapdb-utils.h"
#include "pikapdbcontext.h"
#include "pikaprocedure.h"
#include "internal-procs.h"

#include "pika-intl.h"


static PikaValueArray *
image_select_color_invoker (PikaProcedure         *procedure,
                            Pika                  *pika,
                            PikaContext           *context,
                            PikaProgress          *progress,
                            const PikaValueArray  *args,
                            GError               **error)
{
  gboolean success = TRUE;
  PikaImage *image;
  gint operation;
  PikaDrawable *drawable;
  PikaRGB color;

  image = g_value_get_object (pika_value_array_index (args, 0));
  operation = g_value_get_enum (pika_value_array_index (args, 1));
  drawable = g_value_get_object (pika_value_array_index (args, 2));
  pika_value_get_rgb (pika_value_array_index (args, 3), &color);

  if (success)
    {
      PikaPDBContext *pdb_context = PIKA_PDB_CONTEXT (context);

      if (pdb_context->sample_merged ||
          pika_pdb_item_is_attached (PIKA_ITEM (drawable), image, 0, error))
        {
          GList *drawables = g_list_prepend (NULL, drawable);
          pika_channel_select_by_color (pika_image_get_mask (image), drawables,
                                        pdb_context->sample_merged,
                                        &color,
                                        pdb_context->sample_threshold,
                                        pdb_context->sample_transparent,
                                        pdb_context->sample_criterion,
                                        operation,
                                        pdb_context->antialias,
                                        pdb_context->feather,
                                        pdb_context->feather_radius_x,
                                        pdb_context->feather_radius_y);
          g_list_free (drawables);
        }
      else
        success = FALSE;
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

static PikaValueArray *
image_select_contiguous_color_invoker (PikaProcedure         *procedure,
                                       Pika                  *pika,
                                       PikaContext           *context,
                                       PikaProgress          *progress,
                                       const PikaValueArray  *args,
                                       GError               **error)
{
  gboolean success = TRUE;
  PikaImage *image;
  gint operation;
  PikaDrawable *drawable;
  gdouble x;
  gdouble y;

  image = g_value_get_object (pika_value_array_index (args, 0));
  operation = g_value_get_enum (pika_value_array_index (args, 1));
  drawable = g_value_get_object (pika_value_array_index (args, 2));
  x = g_value_get_double (pika_value_array_index (args, 3));
  y = g_value_get_double (pika_value_array_index (args, 4));

  if (success)
    {
      PikaPDBContext *pdb_context = PIKA_PDB_CONTEXT (context);

      if (pdb_context->sample_merged ||
          pika_pdb_item_is_attached (PIKA_ITEM (drawable), image, 0, error))
        {

          pika_channel_select_fuzzy (pika_image_get_mask (image),
                                     drawable,
                                     pdb_context->sample_merged,
                                     x, y,
                                     pdb_context->sample_threshold,
                                     pdb_context->sample_transparent,
                                     pdb_context->sample_criterion,
                                     pdb_context->diagonal_neighbors,
                                     operation,
                                     pdb_context->antialias,
                                     pdb_context->feather,
                                     pdb_context->feather_radius_x,
                                     pdb_context->feather_radius_y);
        }
      else
        success = FALSE;
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

static PikaValueArray *
image_select_rectangle_invoker (PikaProcedure         *procedure,
                                Pika                  *pika,
                                PikaContext           *context,
                                PikaProgress          *progress,
                                const PikaValueArray  *args,
                                GError               **error)
{
  gboolean success = TRUE;
  PikaImage *image;
  gint operation;
  gdouble x;
  gdouble y;
  gdouble width;
  gdouble height;

  image = g_value_get_object (pika_value_array_index (args, 0));
  operation = g_value_get_enum (pika_value_array_index (args, 1));
  x = g_value_get_double (pika_value_array_index (args, 2));
  y = g_value_get_double (pika_value_array_index (args, 3));
  width = g_value_get_double (pika_value_array_index (args, 4));
  height = g_value_get_double (pika_value_array_index (args, 5));

  if (success)
    {
      PikaPDBContext *pdb_context = PIKA_PDB_CONTEXT (context);

      pika_channel_select_rectangle (pika_image_get_mask (image),
                                     (gint) x, (gint) y,
                                     (gint) width, (gint) height,
                                     operation,
                                     pdb_context->feather,
                                     pdb_context->feather_radius_x,
                                     pdb_context->feather_radius_y,
                                     TRUE);
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

static PikaValueArray *
image_select_round_rectangle_invoker (PikaProcedure         *procedure,
                                      Pika                  *pika,
                                      PikaContext           *context,
                                      PikaProgress          *progress,
                                      const PikaValueArray  *args,
                                      GError               **error)
{
  gboolean success = TRUE;
  PikaImage *image;
  gint operation;
  gdouble x;
  gdouble y;
  gdouble width;
  gdouble height;
  gdouble corner_radius_x;
  gdouble corner_radius_y;

  image = g_value_get_object (pika_value_array_index (args, 0));
  operation = g_value_get_enum (pika_value_array_index (args, 1));
  x = g_value_get_double (pika_value_array_index (args, 2));
  y = g_value_get_double (pika_value_array_index (args, 3));
  width = g_value_get_double (pika_value_array_index (args, 4));
  height = g_value_get_double (pika_value_array_index (args, 5));
  corner_radius_x = g_value_get_double (pika_value_array_index (args, 6));
  corner_radius_y = g_value_get_double (pika_value_array_index (args, 7));

  if (success)
    {
      PikaPDBContext *pdb_context = PIKA_PDB_CONTEXT (context);

      pika_channel_select_round_rect (pika_image_get_mask (image),
                                      (gint) x, (gint) y,
                                      (gint) width, (gint) height,
                                      corner_radius_x,
                                      corner_radius_y,
                                      operation,
                                      pdb_context->antialias,
                                      pdb_context->feather,
                                      pdb_context->feather_radius_x,
                                      pdb_context->feather_radius_y,
                                      TRUE);
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

static PikaValueArray *
image_select_ellipse_invoker (PikaProcedure         *procedure,
                              Pika                  *pika,
                              PikaContext           *context,
                              PikaProgress          *progress,
                              const PikaValueArray  *args,
                              GError               **error)
{
  gboolean success = TRUE;
  PikaImage *image;
  gint operation;
  gdouble x;
  gdouble y;
  gdouble width;
  gdouble height;

  image = g_value_get_object (pika_value_array_index (args, 0));
  operation = g_value_get_enum (pika_value_array_index (args, 1));
  x = g_value_get_double (pika_value_array_index (args, 2));
  y = g_value_get_double (pika_value_array_index (args, 3));
  width = g_value_get_double (pika_value_array_index (args, 4));
  height = g_value_get_double (pika_value_array_index (args, 5));

  if (success)
    {
      PikaPDBContext *pdb_context = PIKA_PDB_CONTEXT (context);

      pika_channel_select_ellipse (pika_image_get_mask (image),
                                   (gint) x, (gint) y,
                                   (gint) width, (gint) height,
                                   operation,
                                   pdb_context->antialias,
                                   pdb_context->feather,
                                   pdb_context->feather_radius_x,
                                   pdb_context->feather_radius_y,
                                   TRUE);
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

static PikaValueArray *
image_select_polygon_invoker (PikaProcedure         *procedure,
                              Pika                  *pika,
                              PikaContext           *context,
                              PikaProgress          *progress,
                              const PikaValueArray  *args,
                              GError               **error)
{
  gboolean success = TRUE;
  PikaImage *image;
  gint operation;
  gint num_segs;
  const gdouble *segs;

  image = g_value_get_object (pika_value_array_index (args, 0));
  operation = g_value_get_enum (pika_value_array_index (args, 1));
  num_segs = g_value_get_int (pika_value_array_index (args, 2));
  segs = pika_value_get_float_array (pika_value_array_index (args, 3));

  if (success)
    {
      PikaPDBContext *pdb_context = PIKA_PDB_CONTEXT (context);

      pika_channel_select_polygon (pika_image_get_mask (image),
                                   _("Free Select"),
                                   num_segs / 2,
                                   (PikaVector2 *) segs,
                                   operation,
                                   pdb_context->antialias,
                                   pdb_context->feather,
                                   pdb_context->feather_radius_x,
                                   pdb_context->feather_radius_y,
                                   TRUE);
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

static PikaValueArray *
image_select_item_invoker (PikaProcedure         *procedure,
                           Pika                  *pika,
                           PikaContext           *context,
                           PikaProgress          *progress,
                           const PikaValueArray  *args,
                           GError               **error)
{
  gboolean success = TRUE;
  PikaImage *image;
  gint operation;
  PikaItem *item;

  image = g_value_get_object (pika_value_array_index (args, 0));
  operation = g_value_get_enum (pika_value_array_index (args, 1));
  item = g_value_get_object (pika_value_array_index (args, 2));

  if (success)
    {
      if (pika_pdb_item_is_attached (item, image, 0, error))
        {
          PikaPDBContext *pdb_context = PIKA_PDB_CONTEXT (context);

          pika_item_to_selection (item, operation,
                                  pdb_context->antialias,
                                  pdb_context->feather,
                                  pdb_context->feather_radius_x,
                                  pdb_context->feather_radius_y);
        }
      else
        success = FALSE;
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

void
register_image_select_procs (PikaPDB *pdb)
{
  PikaProcedure *procedure;

  /*
   * pika-image-select-color
   */
  procedure = pika_procedure_new (image_select_color_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-image-select-color");
  pika_procedure_set_static_help (procedure,
                                  "Create a selection by selecting all pixels (in the specified drawable) with the same (or similar) color to that specified.",
                                  "This tool creates a selection over the specified image. A by-color selection is determined by the supplied color under the constraints of the current context settings. Essentially, all pixels (in the drawable) that have color sufficiently close to the specified color (as determined by the threshold and criterion context values) are included in the selection. To select transparent regions, the color specified must also have minimum alpha.\n"
                                  "\n"
                                  "This procedure is affected by the following context setters: 'pika-context-set-antialias', 'pika-context-set-feather', 'pika-context-set-feather-radius', 'pika-context-set-sample-merged', 'pika-context-set-sample-criterion', 'pika-context-set-sample-threshold', 'pika-context-set-sample-transparent'.\n"
                                  "\n"
                                  "In the case of a merged sampling, the supplied drawable is ignored.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "David Gowers",
                                         "David Gowers",
                                         "2010");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_image ("image",
                                                      "image",
                                                      "The affected image",
                                                      FALSE,
                                                      PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_enum ("operation",
                                                  "operation",
                                                  "The selection operation",
                                                  PIKA_TYPE_CHANNEL_OPS,
                                                  PIKA_CHANNEL_OP_ADD,
                                                  PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_drawable ("drawable",
                                                         "drawable",
                                                         "The affected drawable",
                                                         FALSE,
                                                         PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_rgb ("color",
                                                    "color",
                                                    "The color to select",
                                                    FALSE,
                                                    NULL,
                                                    PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-image-select-contiguous-color
   */
  procedure = pika_procedure_new (image_select_contiguous_color_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-image-select-contiguous-color");
  pika_procedure_set_static_help (procedure,
                                  "Create a selection by selecting all pixels around specified coordinates with the same (or similar) color to that at the coordinates.",
                                  "This tool creates a contiguous selection over the specified image. A contiguous color selection is determined by a seed fill under the constraints of the current context settings. Essentially, the color at the specified coordinates (in the drawable) is measured and the selection expands outwards from that point to any adjacent pixels which are not significantly different (as determined by the threshold and criterion context settings). This process continues until no more expansion is possible. If antialiasing is turned on, the final selection mask will contain intermediate values based on close misses to the threshold bar at pixels along the seed fill boundary.\n"
                                  "\n"
                                  "This procedure is affected by the following context setters: 'pika-context-set-antialias', 'pika-context-set-feather', 'pika-context-set-feather-radius', 'pika-context-set-sample-merged', 'pika-context-set-sample-criterion', 'pika-context-set-sample-threshold', 'pika-context-set-sample-transparent', 'pika-context-set-diagonal-neighbors'.\n"
                                  "\n"
                                  "In the case of a merged sampling, the supplied drawable is ignored. If the sample is merged, the specified coordinates are relative to the image origin; otherwise, they are relative to the drawable's origin.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "David Gowers",
                                         "David Gowers",
                                         "2010");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_image ("image",
                                                      "image",
                                                      "The affected image",
                                                      FALSE,
                                                      PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_enum ("operation",
                                                  "operation",
                                                  "The selection operation",
                                                  PIKA_TYPE_CHANNEL_OPS,
                                                  PIKA_CHANNEL_OP_ADD,
                                                  PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_drawable ("drawable",
                                                         "drawable",
                                                         "The affected drawable",
                                                         FALSE,
                                                         PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("x",
                                                    "x",
                                                    "x coordinate of initial seed fill point: (image coordinates)",
                                                    -G_MAXDOUBLE, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("y",
                                                    "y",
                                                    "y coordinate of initial seed fill point: (image coordinates)",
                                                    -G_MAXDOUBLE, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-image-select-rectangle
   */
  procedure = pika_procedure_new (image_select_rectangle_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-image-select-rectangle");
  pika_procedure_set_static_help (procedure,
                                  "Create a rectangular selection over the specified image;",
                                  "This tool creates a rectangular selection over the specified image. The rectangular region can be either added to, subtracted from, or replace the contents of the previous selection mask.\n"
                                  "\n"
                                  "This procedure is affected by the following context setters: 'pika-context-set-feather', 'pika-context-set-feather-radius'.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Michael Natterer <mitch@gimp.org>",
                                         "Michael Natterer",
                                         "2010");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_image ("image",
                                                      "image",
                                                      "The image",
                                                      FALSE,
                                                      PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_enum ("operation",
                                                  "operation",
                                                  "The selection operation",
                                                  PIKA_TYPE_CHANNEL_OPS,
                                                  PIKA_CHANNEL_OP_ADD,
                                                  PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("x",
                                                    "x",
                                                    "x coordinate of upper-left corner of rectangle",
                                                    -G_MAXDOUBLE, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("y",
                                                    "y",
                                                    "y coordinate of upper-left corner of rectangle",
                                                    -G_MAXDOUBLE, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("width",
                                                    "width",
                                                    "The width of the rectangle",
                                                    0, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("height",
                                                    "height",
                                                    "The height of the rectangle",
                                                    0, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-image-select-round-rectangle
   */
  procedure = pika_procedure_new (image_select_round_rectangle_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-image-select-round-rectangle");
  pika_procedure_set_static_help (procedure,
                                  "Create a rectangular selection with round corners over the specified image;",
                                  "This tool creates a rectangular selection with round corners over the specified image. The rectangular region can be either added to, subtracted from, or replace the contents of the previous selection mask.\n"
                                  "\n"
                                  "This procedure is affected by the following context setters: 'pika-context-set-antialias', 'pika-context-set-feather', 'pika-context-set-feather-radius'.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Martin Nordholts",
                                         "Martin Nordholts",
                                         "2010");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_image ("image",
                                                      "image",
                                                      "The image",
                                                      FALSE,
                                                      PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_enum ("operation",
                                                  "operation",
                                                  "The selection operation",
                                                  PIKA_TYPE_CHANNEL_OPS,
                                                  PIKA_CHANNEL_OP_ADD,
                                                  PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("x",
                                                    "x",
                                                    "x coordinate of upper-left corner of rectangle",
                                                    -G_MAXDOUBLE, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("y",
                                                    "y",
                                                    "y coordinate of upper-left corner of rectangle",
                                                    -G_MAXDOUBLE, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("width",
                                                    "width",
                                                    "The width of the rectangle",
                                                    0, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("height",
                                                    "height",
                                                    "The height of the rectangle",
                                                    0, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("corner-radius-x",
                                                    "corner radius x",
                                                    "The corner radius in X direction",
                                                    0, PIKA_MAX_IMAGE_SIZE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("corner-radius-y",
                                                    "corner radius y",
                                                    "The corner radius in Y direction",
                                                    0, PIKA_MAX_IMAGE_SIZE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-image-select-ellipse
   */
  procedure = pika_procedure_new (image_select_ellipse_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-image-select-ellipse");
  pika_procedure_set_static_help (procedure,
                                  "Create an elliptical selection over the specified image.",
                                  "This tool creates an elliptical selection over the specified image. The elliptical region can be either added to, subtracted from, or replace the contents of the previous selection mask.\n"
                                  "\n"
                                  "This procedure is affected by the following context setters: 'pika-context-set-antialias', 'pika-context-set-feather', 'pika-context-set-feather-radius'.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Michael Natterer <mitch@gimp.org>",
                                         "Michael Natterer",
                                         "2010");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_image ("image",
                                                      "image",
                                                      "The image",
                                                      FALSE,
                                                      PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_enum ("operation",
                                                  "operation",
                                                  "The selection operation",
                                                  PIKA_TYPE_CHANNEL_OPS,
                                                  PIKA_CHANNEL_OP_ADD,
                                                  PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("x",
                                                    "x",
                                                    "x coordinate of upper-left corner of ellipse bounding box",
                                                    -G_MAXDOUBLE, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("y",
                                                    "y",
                                                    "y coordinate of upper-left corner of ellipse bounding box",
                                                    -G_MAXDOUBLE, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("width",
                                                    "width",
                                                    "The width of the ellipse",
                                                    0, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_double ("height",
                                                    "height",
                                                    "The height of the ellipse",
                                                    0, G_MAXDOUBLE, 0,
                                                    PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-image-select-polygon
   */
  procedure = pika_procedure_new (image_select_polygon_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-image-select-polygon");
  pika_procedure_set_static_help (procedure,
                                  "Create a polygonal selection over the specified image.",
                                  "This tool creates a polygonal selection over the specified image. The polygonal region can be either added to, subtracted from, or replace the contents of the previous selection mask. The polygon is specified through an array of floating point numbers and its length. The length of array must be 2n, where n is the number of points. Each point is defined by 2 floating point values which correspond to the x and y coordinates. If the final point does not connect to the starting point, a connecting segment is automatically added.\n"
                                  "\n"
                                  "This procedure is affected by the following context setters: 'pika-context-set-antialias', 'pika-context-set-feather', 'pika-context-set-feather-radius'.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Michael Natterer <mitch@gimp.org>",
                                         "Michael Natterer",
                                         "2010");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_image ("image",
                                                      "image",
                                                      "The image",
                                                      FALSE,
                                                      PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_enum ("operation",
                                                  "operation",
                                                  "The selection operation",
                                                  PIKA_TYPE_CHANNEL_OPS,
                                                  PIKA_CHANNEL_OP_ADD,
                                                  PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_int ("num-segs",
                                                 "num segs",
                                                 "Number of points (count 1 coordinate as two points)",
                                                 2, G_MAXINT32, 2,
                                                 PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_float_array ("segs",
                                                            "segs",
                                                            "Array of points: { p1.x, p1.y, p2.x, p2.y, ..., pn.x, pn.y}",
                                                            PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-image-select-item
   */
  procedure = pika_procedure_new (image_select_item_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-image-select-item");
  pika_procedure_set_static_help (procedure,
                                  "Transforms the specified item into a selection",
                                  "This procedure renders the item's outline into the current selection of the image the item belongs to. What exactly the item's outline is depends on the item type: for layers, it's the layer's alpha channel, for vectors the vector's shape.\n"
                                  "\n"
                                  "This procedure is affected by the following context setters: 'pika-context-set-antialias', 'pika-context-set-feather', 'pika-context-set-feather-radius'.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Michael Natterer <mitch@gimp.org>",
                                         "Michael Natterer",
                                         "2010");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_image ("image",
                                                      "image",
                                                      "The image",
                                                      FALSE,
                                                      PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               g_param_spec_enum ("operation",
                                                  "operation",
                                                  "The desired operation with current selection",
                                                  PIKA_TYPE_CHANNEL_OPS,
                                                  PIKA_CHANNEL_OP_ADD,
                                                  PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_item ("item",
                                                     "item",
                                                     "The item to render to the selection",
                                                     FALSE,
                                                     PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);
}
