/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-2003 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl. */

#include "config.h"

#include "stamp-pdbgen.h"

#include <string.h>

#include <gegl.h>

#include <gdk-pixbuf/gdk-pixbuf.h>

#include "libpikabase/pikabase.h"

#include "pdb-types.h"

#include "core/pika.h"
#include "core/pikacontainer-filter.h"
#include "core/pikacontext.h"
#include "core/pikadatafactory.h"
#include "core/pikapalette.h"
#include "core/pikaparamspecs.h"

#include "pikapdb.h"
#include "pikapdb-utils.h"
#include "pikaprocedure.h"
#include "internal-procs.h"


static PikaValueArray *
palettes_refresh_invoker (PikaProcedure         *procedure,
                          Pika                  *pika,
                          PikaContext           *context,
                          PikaProgress          *progress,
                          const PikaValueArray  *args,
                          GError               **error)
{
  pika_data_factory_data_refresh (pika->palette_factory, context);

  return pika_procedure_get_return_values (procedure, TRUE, NULL);
}

static PikaValueArray *
palettes_get_list_invoker (PikaProcedure         *procedure,
                           Pika                  *pika,
                           PikaContext           *context,
                           PikaProgress          *progress,
                           const PikaValueArray  *args,
                           GError               **error)
{
  gboolean success = TRUE;
  PikaValueArray *return_vals;
  const gchar *filter;
  gchar **palette_list = NULL;

  filter = g_value_get_string (pika_value_array_index (args, 0));

  if (success)
    {
      palette_list = pika_container_get_filtered_name_array (pika_data_factory_get_container (pika->palette_factory),
                                                             filter);
    }

  return_vals = pika_procedure_get_return_values (procedure, success,
                                                  error ? *error : NULL);

  if (success)
    g_value_take_boxed (pika_value_array_index (return_vals, 1), palette_list);

  return return_vals;
}

void
register_palettes_procs (PikaPDB *pdb)
{
  PikaProcedure *procedure;

  /*
   * pika-palettes-refresh
   */
  procedure = pika_procedure_new (palettes_refresh_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-palettes-refresh");
  pika_procedure_set_static_help (procedure,
                                  "Refreshes current palettes. This function always succeeds.",
                                  "This procedure retrieves all palettes currently in the user's palette path and updates the palette dialogs accordingly.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Adrian Likins <adrian@gimp.org>",
                                         "Adrian Likins",
                                         "1998");
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-palettes-get-list
   */
  procedure = pika_procedure_new (palettes_get_list_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-palettes-get-list");
  pika_procedure_set_static_help (procedure,
                                  "Retrieves a list of all of the available palettes",
                                  "This procedure returns a complete listing of available palettes. Each name returned can be used as input to the command 'pika-context-set-palette'.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Nathan Summers <rock@gimp.org>",
                                         "Nathan Summers",
                                         "2001");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("filter",
                                                       "filter",
                                                       "An optional regular expression used to filter the list",
                                                       FALSE, TRUE, FALSE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_procedure_add_return_value (procedure,
                                   g_param_spec_boxed ("palette-list",
                                                       "palette list",
                                                       "The list of palette names",
                                                       G_TYPE_STRV,
                                                       PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);
}
