/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-2003 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl. */

#include "config.h"

#include "stamp-pdbgen.h"

#include <gegl.h>

#include <gdk-pixbuf/gdk-pixbuf.h>

#include "libpikabase/pikabase.h"

#include "pdb-types.h"

#include "core/pika.h"
#include "core/pikadatafactory.h"
#include "core/pikaparamspecs.h"

#include "pikapdb.h"
#include "pikaprocedure.h"
#include "internal-procs.h"


static PikaValueArray *
patterns_popup_invoker (PikaProcedure         *procedure,
                        Pika                  *pika,
                        PikaContext           *context,
                        PikaProgress          *progress,
                        const PikaValueArray  *args,
                        GError               **error)
{
  gboolean success = TRUE;
  const gchar *pattern_callback;
  const gchar *popup_title;
  const gchar *initial_pattern_name;

  pattern_callback = g_value_get_string (pika_value_array_index (args, 0));
  popup_title = g_value_get_string (pika_value_array_index (args, 1));
  initial_pattern_name = g_value_get_string (pika_value_array_index (args, 2));

  if (success)
    {
      if (pika->no_interface ||
          ! pika_pdb_lookup_procedure (pika->pdb, pattern_callback) ||
          ! pika_pdb_dialog_new (pika, context, progress,
                                 pika_data_factory_get_container (pika->pattern_factory),
                                 popup_title, pattern_callback, initial_pattern_name,
                                 NULL))
        success = FALSE;
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

static PikaValueArray *
patterns_close_popup_invoker (PikaProcedure         *procedure,
                              Pika                  *pika,
                              PikaContext           *context,
                              PikaProgress          *progress,
                              const PikaValueArray  *args,
                              GError               **error)
{
  gboolean success = TRUE;
  const gchar *pattern_callback;

  pattern_callback = g_value_get_string (pika_value_array_index (args, 0));

  if (success)
    {
      if (pika->no_interface ||
          ! pika_pdb_lookup_procedure (pika->pdb, pattern_callback) ||
          ! pika_pdb_dialog_close (pika, pika_data_factory_get_container (pika->pattern_factory),
                                   pattern_callback))
        success = FALSE;
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

static PikaValueArray *
patterns_set_popup_invoker (PikaProcedure         *procedure,
                            Pika                  *pika,
                            PikaContext           *context,
                            PikaProgress          *progress,
                            const PikaValueArray  *args,
                            GError               **error)
{
  gboolean success = TRUE;
  const gchar *pattern_callback;
  const gchar *pattern_name;

  pattern_callback = g_value_get_string (pika_value_array_index (args, 0));
  pattern_name = g_value_get_string (pika_value_array_index (args, 1));

  if (success)
    {
      if (pika->no_interface ||
          ! pika_pdb_lookup_procedure (pika->pdb, pattern_callback) ||
          ! pika_pdb_dialog_set (pika, pika_data_factory_get_container (pika->pattern_factory),
                                 pattern_callback, pattern_name,
                                 NULL))
        success = FALSE;
    }

  return pika_procedure_get_return_values (procedure, success,
                                           error ? *error : NULL);
}

void
register_pattern_select_procs (PikaPDB *pdb)
{
  PikaProcedure *procedure;

  /*
   * pika-patterns-popup
   */
  procedure = pika_procedure_new (patterns_popup_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-patterns-popup");
  pika_procedure_set_static_help (procedure,
                                  "Invokes the Pika pattern selection.",
                                  "Opens the pattern selection dialog.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Andy Thomas",
                                         "Andy Thomas",
                                         "1998");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("pattern-callback",
                                                       "pattern callback",
                                                       "The callback PDB proc to call when the user chooses a pattern",
                                                       FALSE, FALSE, TRUE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("popup-title",
                                                       "popup title",
                                                       "Title of the pattern selection dialog",
                                                       FALSE, FALSE, FALSE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("initial-pattern-name",
                                                       "initial pattern name",
                                                       "The name of the pattern to set as the initial choice",
                                                       FALSE, TRUE, FALSE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-patterns-close-popup
   */
  procedure = pika_procedure_new (patterns_close_popup_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-patterns-close-popup");
  pika_procedure_set_static_help (procedure,
                                  "Close the pattern selection dialog.",
                                  "Closes an open pattern selection dialog.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Andy Thomas",
                                         "Andy Thomas",
                                         "1998");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("pattern-callback",
                                                       "pattern callback",
                                                       "The name of the callback registered for this pop-up",
                                                       FALSE, FALSE, TRUE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);

  /*
   * pika-patterns-set-popup
   */
  procedure = pika_procedure_new (patterns_set_popup_invoker);
  pika_object_set_static_name (PIKA_OBJECT (procedure),
                               "pika-patterns-set-popup");
  pika_procedure_set_static_help (procedure,
                                  "Sets the current pattern in a pattern selection dialog.",
                                  "Sets the current pattern in a pattern selection dialog.",
                                  NULL);
  pika_procedure_set_static_attribution (procedure,
                                         "Andy Thomas",
                                         "Andy Thomas",
                                         "1998");
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("pattern-callback",
                                                       "pattern callback",
                                                       "The name of the callback registered for this pop-up",
                                                       FALSE, FALSE, TRUE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_procedure_add_argument (procedure,
                               pika_param_spec_string ("pattern-name",
                                                       "pattern name",
                                                       "The name of the pattern to set as selected",
                                                       FALSE, FALSE, FALSE,
                                                       NULL,
                                                       PIKA_PARAM_READWRITE));
  pika_pdb_register_procedure (pdb, procedure);
  g_object_unref (procedure);
}
