/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_PDB_H__
#define __PIKA_PDB_H__


#include "core/pikaobject.h"


#define PIKA_TYPE_PDB            (pika_pdb_get_type ())
#define PIKA_PDB(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_PDB, PikaPDB))
#define PIKA_PDB_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_PDB, PikaPDBClass))
#define PIKA_IS_PDB(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_PDB))
#define PIKA_IS_PDB_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_PDB))
#define PIKA_PDB_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_PDB, PikaPDBClass))


typedef struct _PikaPDBClass PikaPDBClass;

struct _PikaPDB
{
  PikaObject  parent_instance;

  Pika       *pika;

  GHashTable *procedures;
  GHashTable *compat_proc_names;
};

struct _PikaPDBClass
{
  PikaObjectClass parent_class;

  void (* register_procedure)   (PikaPDB       *pdb,
                                 PikaProcedure *procedure);
  void (* unregister_procedure) (PikaPDB       *pdb,
                                 PikaProcedure *procedure);
};


GType            pika_pdb_get_type                       (void) G_GNUC_CONST;

PikaPDB        * pika_pdb_new                            (Pika           *pika);

void             pika_pdb_register_procedure             (PikaPDB        *pdb,
                                                          PikaProcedure  *procedure);
void             pika_pdb_unregister_procedure           (PikaPDB        *pdb,
                                                          PikaProcedure  *procedure);

PikaProcedure  * pika_pdb_lookup_procedure               (PikaPDB        *pdb,
                                                          const gchar    *name);

void             pika_pdb_register_compat_proc_name      (PikaPDB        *pdb,
                                                          const gchar    *old_name,
                                                          const gchar    *new_name);
const gchar    * pika_pdb_lookup_compat_proc_name        (PikaPDB        *pdb,
                                                          const gchar    *old_name);

PikaValueArray * pika_pdb_execute_procedure_by_name_args (PikaPDB        *pdb,
                                                          PikaContext    *context,
                                                          PikaProgress   *progress,
                                                          GError        **error,
                                                          const gchar    *name,
                                                          PikaValueArray *args);
PikaValueArray * pika_pdb_execute_procedure_by_name      (PikaPDB        *pdb,
                                                          PikaContext    *context,
                                                          PikaProgress   *progress,
                                                          GError        **error,
                                                          const gchar    *name,
                                                          ...);

GList          * pika_pdb_get_deprecated_procedures      (PikaPDB        *pdb);


#endif  /*  __PIKA_PDB_H__  */
