/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-1999 Spencer Kimball and Peter Mattis
 *
 * pikapdbcontext.h
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_PDB_CONTEXT_H__
#define __PIKA_PDB_CONTEXT_H__


#include "core/pikacontext.h"


#define PIKA_TYPE_PDB_CONTEXT            (pika_pdb_context_get_type ())
#define PIKA_PDB_CONTEXT(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_PDB_CONTEXT, PikaPDBContext))
#define PIKA_PDB_CONTEXT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_PDB_CONTEXT, PikaPDBContextClass))
#define PIKA_IS_PDB_CONTEXT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_PDB_CONTEXT))
#define PIKA_IS_PDB_CONTEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_PDB_CONTEXT))
#define PIKA_PDB_CONTEXT_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_PDB_CONTEXT, PikaPDBContextClass))


typedef struct _PikaPDBContext      PikaPDBContext;
typedef struct _PikaPDBContextClass PikaPDBContextClass;

struct _PikaPDBContext
{
  PikaContext             parent_instance;

  gboolean                antialias;
  gboolean                feather;
  gdouble                 feather_radius_x;
  gdouble                 feather_radius_y;
  gboolean                sample_merged;
  PikaSelectCriterion     sample_criterion;
  gdouble                 sample_threshold;
  gboolean                sample_transparent;
  gboolean                diagonal_neighbors;

  PikaInterpolationType   interpolation;
  PikaTransformDirection  transform_direction;
  PikaTransformResize     transform_resize;

  PikaContainer          *paint_options_list;
  PikaStrokeOptions      *stroke_options;

  GeglDistanceMetric      distance_metric;
};

struct _PikaPDBContextClass
{
  PikaContextClass  parent_class;
};


GType               pika_pdb_context_get_type           (void) G_GNUC_CONST;

PikaContext       * pika_pdb_context_new                (Pika           *pika,
                                                         PikaContext    *parent,
                                                         gboolean        set_parent);

PikaContainer     * pika_pdb_context_get_paint_options_list
                                                        (PikaPDBContext *context);
PikaPaintOptions  * pika_pdb_context_get_paint_options  (PikaPDBContext *context,
                                                         const gchar    *name);
GList             * pika_pdb_context_get_brush_options  (PikaPDBContext *context);

PikaStrokeOptions * pika_pdb_context_get_stroke_options (PikaPDBContext *context);


#endif  /*  __PIKA_PDB_CONTEXT_H__  */
