/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_LOG_H__
#define __PIKA_LOG_H__


typedef guint *PikaLogHandler;


typedef enum
{
  PIKA_LOG_TOOL_EVENTS        = 1 << 0,
  PIKA_LOG_TOOL_FOCUS         = 1 << 1,
  PIKA_LOG_DND                = 1 << 2,
  PIKA_LOG_HELP               = 1 << 3,
  PIKA_LOG_DIALOG_FACTORY     = 1 << 4,
  PIKA_LOG_MENUS              = 1 << 5,
  PIKA_LOG_SAVE_DIALOG        = 1 << 6,
  PIKA_LOG_IMAGE_SCALE        = 1 << 7,
  PIKA_LOG_SHADOW_TILES       = 1 << 8,
  PIKA_LOG_SCALE              = 1 << 9,
  PIKA_LOG_WM                 = 1 << 10,
  PIKA_LOG_FLOATING_SELECTION = 1 << 11,
  PIKA_LOG_SHM                = 1 << 12,
  PIKA_LOG_TEXT_EDITING       = 1 << 13,
  PIKA_LOG_KEY_EVENTS         = 1 << 14,
  PIKA_LOG_AUTO_TAB_STYLE     = 1 << 15,
  PIKA_LOG_INSTANCES          = 1 << 16,
  PIKA_LOG_RECTANGLE_TOOL     = 1 << 17,
  PIKA_LOG_BRUSH_CACHE        = 1 << 18,
  PIKA_LOG_PROJECTION         = 1 << 19,
  PIKA_LOG_XCF                = 1 << 20,
  PIKA_LOG_MAGIC_MATCH        = 1 << 21
} PikaLogFlags;


extern PikaLogFlags pika_log_flags;


void             pika_log_init           (void);
void             pika_log                (PikaLogFlags    flags,
                                          const gchar    *function,
                                          gint            line,
                                          const gchar    *format,
                                          ...) G_GNUC_PRINTF (4, 5);
void             pika_logv               (PikaLogFlags    flags,
                                          const gchar    *function,
                                          gint            line,
                                          const gchar    *format,
                                          va_list         args) G_GNUC_PRINTF (4, 0);

PikaLogHandler   pika_log_set_handler    (gboolean        global,
                                          GLogLevelFlags  log_levels,
                                          GLogFunc        log_func,
                                          gpointer        user_data);
void             pika_log_remove_handler (PikaLogHandler  handler);


#ifdef G_HAVE_ISO_VARARGS

#define PIKA_LOG(type, ...) \
        G_STMT_START { \
        if (pika_log_flags & PIKA_LOG_##type) \
          pika_log (PIKA_LOG_##type, G_STRFUNC, __LINE__, __VA_ARGS__);       \
        } G_STMT_END

#elif defined(G_HAVE_GNUC_VARARGS)

#define PIKA_LOG(type, format...) \
        G_STMT_START { \
        if (pika_log_flags & PIKA_LOG_##type) \
          pika_log (PIKA_LOG_##type, G_STRFUNC, __LINE__, format);  \
        } G_STMT_END

#else /* no varargs macros */

/* need to expand all the short forms
 * to make them known constants at compile time
 */
#define TOOL_EVENTS        PIKA_LOG_TOOL_EVENTS
#define TOOL_FOCUS         PIKA_LOG_TOOL_FOCUS
#define DND                PIKA_LOG_DND
#define HELP               PIKA_LOG_HELP
#define DIALOG_FACTORY     PIKA_LOG_DIALOG_FACTORY
#define MENUS              PIKA_LOG_MENUS
#define SAVE_DIALOG        PIKA_LOG_SAVE_DIALOG
#define IMAGE_SCALE        PIKA_LOG_IMAGE_SCALE
#define SHADOW_TILES       PIKA_LOG_SHADOW_TILES
#define SCALE              PIKA_LOG_SCALE
#define WM                 PIKA_LOG_WM
#define FLOATING_SELECTION PIKA_LOG_FLOATING_SELECTION
#define SHM                PIKA_LOG_SHM
#define TEXT_EDITING       PIKA_LOG_TEXT_EDITING
#define KEY_EVENTS         PIKA_LOG_KEY_EVENTS
#define AUTO_TAB_STYLE     PIKA_LOG_AUTO_TAB_STYLE
#define INSTANCES          PIKA_LOG_INSTANCES
#define RECTANGLE_TOOL     PIKA_LOG_RECTANGLE_TOOL
#define BRUSH_CACHE        PIKA_LOG_BRUSH_CACHE
#define PROJECTION         PIKA_LOG_PROJECTION
#define XCF                PIKA_LOG_XCF

#if 0 /* last resort */
#  define PIKA_LOG /* nothing => no varargs, no log */
#endif

static void
PIKA_LOG (PikaLogFlags flags,
          const gchar *format,
          ...)
{
  va_list args;
  va_start (args, format);
  if (pika_log_flags & flags)
    pika_logv (type, "", 0, format, args);
  va_end (args);
}

#endif  /* !__GNUC__ */

#define geimnum(vienna)  pika_l##vienna##l_dialog()
#define fnord(kosmoso)   void pika_##kosmoso##bl_dialog(void);

#endif /* __PIKA_LOG_H__ */
