/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikaplugindef.h
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_PLUG_IN_DEF_H__
#define __PIKA_PLUG_IN_DEF_H__


#include "core/pikaobject.h"


#define PIKA_TYPE_PLUG_IN_DEF            (pika_plug_in_def_get_type ())
#define PIKA_PLUG_IN_DEF(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_PLUG_IN_DEF, PikaPlugInDef))
#define PIKA_PLUG_IN_DEF_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_PLUG_IN_DEF, PikaPlugInDefClass))
#define PIKA_IS_PLUG_IN_DEF(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_PLUG_IN_DEF))
#define PIKA_IS_PLUG_IN_DEF_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_PLUG_IN_DEF))


typedef struct _PikaPlugInDefClass PikaPlugInDefClass;

struct _PikaPlugInDef
{
  PikaObject  parent_instance;

  GFile      *file;
  GSList     *procedures;
  gchar      *help_domain_name;
  gchar      *help_domain_uri;
  gint64      mtime;
  gboolean    needs_query;  /* Does the plug-in need to be queried ?     */
  gboolean    has_init;     /* Does the plug-in need to be initialized ? */
};

struct _PikaPlugInDefClass
{
  PikaObjectClass  parent_class;
};


GType           pika_plug_in_def_get_type (void) G_GNUC_CONST;

PikaPlugInDef * pika_plug_in_def_new      (GFile               *file);

void   pika_plug_in_def_add_procedure     (PikaPlugInDef       *plug_in_def,
                                           PikaPlugInProcedure *proc);
void   pika_plug_in_def_remove_procedure  (PikaPlugInDef       *plug_in_def,
                                           PikaPlugInProcedure *proc);

void   pika_plug_in_def_set_help_domain   (PikaPlugInDef       *plug_in_def,
                                           const gchar         *domain_name,
                                           const gchar         *domain_uri);

void   pika_plug_in_def_set_mtime         (PikaPlugInDef       *plug_in_def,
                                           gint64               mtime);
void   pika_plug_in_def_set_needs_query   (PikaPlugInDef       *plug_in_def,
                                           gboolean             needs_query);
void   pika_plug_in_def_set_has_init      (PikaPlugInDef       *plug_in_def,
                                           gboolean             has_init);


#endif /* __PIKA_PLUG_IN_DEF_H__ */
