/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikapluginprocframe.h
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_PLUG_IN_PROC_FRAME_H__
#define __PIKA_PLUG_IN_PROC_FRAME_H__


struct _PikaPlugInProcFrame
{
  gint                 ref_count;

  PikaContext         *main_context;
  GList               *context_stack;

  PikaProcedure       *procedure;
  GMainLoop           *main_loop;

  PikaValueArray      *return_vals;

  PikaProgress        *progress;
  gboolean             progress_created;
  gulong               progress_cancel_id;

  PikaPDBErrorHandler  error_handler;

  /*  lists of things to clean up on dispose  */
  GList               *image_cleanups;
  GList               *item_cleanups;
};


PikaPlugInProcFrame * pika_plug_in_proc_frame_new     (PikaContext         *context,
                                                       PikaProgress        *progress,
                                                       PikaPlugInProcedure *procedure);
void                  pika_plug_in_proc_frame_init    (PikaPlugInProcFrame *proc_frame,
                                                       PikaContext         *context,
                                                       PikaProgress        *progress,
                                                       PikaPlugInProcedure *procedure);

void                  pika_plug_in_proc_frame_dispose (PikaPlugInProcFrame *proc_frame,
                                                       PikaPlugIn          *plug_in);

PikaPlugInProcFrame * pika_plug_in_proc_frame_ref     (PikaPlugInProcFrame *proc_frame);
void                  pika_plug_in_proc_frame_unref   (PikaPlugInProcFrame *proc_frame,
                                                       PikaPlugIn          *plug_in);

PikaValueArray      * pika_plug_in_proc_frame_get_return_values
                                                      (PikaPlugInProcFrame *proc_frame);


#endif /* __PIKA_PLUG_IN_PROC_FRAME_H__ */
