/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * PikaText
 * Copyright (C) 2002-2003  Sven Neumann <sven@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TEXT_H__
#define __PIKA_TEXT_H__


#include "core/pikaobject.h"


#define PIKA_TYPE_TEXT            (pika_text_get_type ())
#define PIKA_TEXT(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TEXT, PikaText))
#define PIKA_TEXT_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_TEXT, PikaTextClass))
#define PIKA_IS_TEXT(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TEXT))
#define PIKA_IS_TEXT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_TEXT))
#define PIKA_TEXT_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_TEXT, PikaTextClass))


typedef struct _PikaTextClass  PikaTextClass;

struct _PikaText
{
  PikaObject             parent_instance;

  gchar                 *text;
  gchar                 *markup;
  PikaFont              *font;
  PikaUnit               unit;
  gdouble                font_size;
  gboolean               antialias;
  PikaTextHintStyle      hint_style;
  gboolean               kerning;
  gchar                 *language;
  PikaTextDirection      base_dir;
  PikaRGB                color;
  PikaCustomStyle        outline_style;
  PikaPattern           *outline_pattern;
  PikaRGB                outline_foreground;
  gdouble                outline_width;
  PikaCapStyle           outline_cap_style;
  PikaJoinStyle          outline_join_style;
  gdouble                outline_miter_limit;
  gboolean               outline_antialias;
  gdouble                outline_dash_offset;
  GArray                *outline_dash_info;
  PikaTextOutline        outline;
  PikaTextJustification  justify;
  gdouble                indent;
  gdouble                line_spacing;
  gdouble                letter_spacing;
  PikaTextBoxMode        box_mode;
  gdouble                box_width;
  gdouble                box_height;
  PikaUnit               box_unit;
  PikaMatrix2            transformation;
  gdouble                offset_x;
  gdouble                offset_y;

  gdouble                border;
  Pika                  *pika;
};

struct _PikaTextClass
{
  PikaObjectClass        parent_class;

  void (* changed) (PikaText *text);
};


GType  pika_text_get_type           (void) G_GNUC_CONST;

void   pika_text_get_transformation (PikaText    *text,
                                     PikaMatrix3 *matrix);


#endif /* __PIKA_TEXT_H__ */
