/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * PikaTextLayer
 * Copyright (C) 2002-2003  Sven Neumann <sven@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TEXT_LAYER_H__
#define __PIKA_TEXT_LAYER_H__


#include "core/pikalayer.h"


#define PIKA_TYPE_TEXT_LAYER            (pika_text_layer_get_type ())
#define PIKA_TEXT_LAYER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TEXT_LAYER, PikaTextLayer))
#define PIKA_TEXT_LAYER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_TEXT_LAYER, PikaTextLayerClass))
#define PIKA_IS_TEXT_LAYER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TEXT_LAYER))
#define PIKA_IS_TEXT_LAYER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_TEXT_LAYER))
#define PIKA_TEXT_LAYER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_TEXT_LAYER, PikaTextLayerClass))


typedef struct _PikaTextLayerClass   PikaTextLayerClass;
typedef struct _PikaTextLayerPrivate PikaTextLayerPrivate;

struct _PikaTextLayer
{
  PikaLayer     layer;

  PikaText     *text;
  const gchar  *text_parasite;        /* parasite name that this text was set from,
                                       * and that should be removed when the text
                                       * is changed.
                                       */
  gboolean      text_parasite_is_old; /* Format before XCF 19. */
  gboolean      auto_rename;
  gboolean      modified;

  const Babl   *convert_format;

  PikaTextLayerPrivate *private;
};

struct _PikaTextLayerClass
{
  PikaLayerClass  parent_class;
};


GType       pika_text_layer_get_type    (void) G_GNUC_CONST;

PikaLayer * pika_text_layer_new         (PikaImage     *image,
                                         PikaText      *text);
PikaText  * pika_text_layer_get_text    (PikaTextLayer *layer);
void        pika_text_layer_set_text    (PikaTextLayer *layer,
                                         PikaText      *text);
void        pika_text_layer_discard     (PikaTextLayer *layer);
void        pika_text_layer_set         (PikaTextLayer *layer,
                                         const gchar   *undo_desc,
                                         const gchar   *first_property_name,
                                         ...) G_GNUC_NULL_TERMINATED;

gboolean    pika_item_is_text_layer     (PikaItem      *item);


#endif /* __PIKA_TEXT_LAYER_H__ */
