/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_ALIGN_TOOL_H__
#define __PIKA_ALIGN_TOOL_H__


#include "pikadrawtool.h"


/*  tool function/operation/state/mode  */
typedef enum
{
  ALIGN_TOOL_NO_ACTION,

  ALIGN_TOOL_REF_IDLE,
  ALIGN_TOOL_REF_PICK_LAYER,
  ALIGN_TOOL_REF_PICK_GUIDE,
  ALIGN_TOOL_REF_PICK_PATH,
  ALIGN_TOOL_REF_DRAG_BOX,

  ALIGN_TOOL_ALIGN_IDLE,
  ALIGN_TOOL_ALIGN_PICK_GUIDE,
  ALIGN_TOOL_ALIGN_ADD_GUIDE,
} PikaAlignToolFunction;


#define PIKA_TYPE_ALIGN_TOOL            (pika_align_tool_get_type ())
#define PIKA_ALIGN_TOOL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_ALIGN_TOOL, PikaAlignTool))
#define PIKA_ALIGN_TOOL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_ALIGN_TOOL, PikaAlignToolClass))
#define PIKA_IS_ALIGN_TOOL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_ALIGN_TOOL))
#define PIKA_IS_ALIGN_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_ALIGN_TOOL))
#define PIKA_ALIGN_TOOL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_ALIGN_TOOL, PikaAlignToolClass))

#define PIKA_ALIGN_TOOL_GET_OPTIONS(t)  (PIKA_ALIGN_OPTIONS (pika_tool_get_options (PIKA_TOOL (t))))


typedef struct _PikaAlignTool      PikaAlignTool;
typedef struct _PikaAlignToolClass PikaAlignToolClass;

struct _PikaAlignTool
{
  PikaDrawTool           parent_instance;

  PikaAlignToolFunction  function;
  GList                 *selected_objects;

  gint                   x1, y1, x2, y2;   /* rubber-band rectangle */

  gboolean               set_reference;
};

struct _PikaAlignToolClass
{
  PikaDrawToolClass parent_class;
};


void    pika_align_tool_register (PikaToolRegisterCallback  callback,
                                 gpointer                  data);

GType   pika_align_tool_get_type (void) G_GNUC_CONST;


#endif  /*  __PIKA_ALIGN_TOOL_H__  */
