/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 *
 * pikacagetool.h
 * Copyright (C) 2010 Michael Muré <batolettre@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_CAGE_TOOL_H__
#define __PIKA_CAGE_TOOL_H__


#include "pikadrawtool.h"


#define PIKA_TYPE_CAGE_TOOL            (pika_cage_tool_get_type ())
#define PIKA_CAGE_TOOL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_CAGE_TOOL, PikaCageTool))
#define PIKA_CAGE_TOOL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_CAGE_TOOL, PikaCageToolClass))
#define PIKA_IS_CAGE_TOOL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_CAGE_TOOL))
#define PIKA_IS_CAGE_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_CAGE_TOOL))
#define PIKA_CAGE_TOOL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_CAGE_TOOL, PikaCageToolClass))

#define PIKA_CAGE_TOOL_GET_OPTIONS(t)  (PIKA_CAGE_OPTIONS (pika_tool_get_options (PIKA_TOOL (t))))


typedef struct _PikaCageTool      PikaCageTool;
typedef struct _PikaCageToolClass PikaCageToolClass;

struct _PikaCageTool
{
  PikaDrawTool    parent_instance;

  PikaCageConfig *config;

  gint            offset_x; /* used to convert the cage point coords */
  gint            offset_y; /* to drawable coords */

  gdouble         cursor_x; /* Hold the cursor x position */
  gdouble         cursor_y; /* Hold the cursor y position */

  gdouble         movement_start_x; /* Where the movement started */
  gdouble         movement_start_y; /* Where the movement started */

  gdouble         selection_start_x; /* Where the selection started */
  gdouble         selection_start_y; /* Where the selection started */

  gint            hovering_handle; /* Handle which the cursor is above */
  gint            hovering_edge; /* Edge which the cursor is above */

  GeglBuffer     *coef; /* Gegl buffer where the coefficient of the transformation are stored */
  gboolean        dirty_coef; /* Indicate if the coef are still valid */

  GeglNode       *render_node; /* Gegl node graph to render the transformation */
  GeglNode       *cage_node; /* Gegl node that compute the cage transform */
  GeglNode       *coef_node; /* Gegl node that read in the coef buffer */

  gint            tool_state; /* Current state in statemachine */

  PikaDrawableFilter *filter; /* For preview */
};

struct _PikaCageToolClass
{
  PikaDrawToolClass parent_class;
};


void    pika_cage_tool_register (PikaToolRegisterCallback  callback,
                                 gpointer                  data);

GType   pika_cage_tool_get_type (void) G_GNUC_CONST;


#endif  /*  __PIKA_CAGE_TOOL_H__  */
