/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef  __PIKA_COLOR_TOOL_H__
#define  __PIKA_COLOR_TOOL_H__


#include "pikadrawtool.h"


#define PIKA_TYPE_COLOR_TOOL            (pika_color_tool_get_type ())
#define PIKA_COLOR_TOOL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_COLOR_TOOL, PikaColorTool))
#define PIKA_COLOR_TOOL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_COLOR_TOOL, PikaColorToolClass))
#define PIKA_IS_COLOR_TOOL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_COLOR_TOOL))
#define PIKA_IS_COLOR_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_COLOR_TOOL))
#define PIKA_COLOR_TOOL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_COLOR_TOOL, PikaColorToolClass))

#define PIKA_COLOR_TOOL_GET_OPTIONS(t)  (PIKA_COLOR_OPTIONS (pika_tool_get_options (PIKA_TOOL (t))))


typedef struct _PikaColorToolClass PikaColorToolClass;

struct _PikaColorTool
{
  PikaDrawTool         parent_instance;

  gboolean             enabled;
  PikaColorOptions    *options;
  gboolean             saved_snap_to;

  PikaColorPickTarget  pick_target;

  gboolean             can_pick;
  gint                 center_x;
  gint                 center_y;
  PikaSamplePoint     *sample_point;
};

struct _PikaColorToolClass
{
  PikaDrawToolClass  parent_class;

  /*  virtual functions  */
  gboolean (* can_pick) (PikaColorTool      *tool,
                         const PikaCoords   *coords,
                         PikaDisplay        *display);
  gboolean (* pick)     (PikaColorTool      *tool,
                         const PikaCoords   *coords,
                         PikaDisplay        *display,
                         const Babl        **sample_format,
                         gpointer            pixel,
                         PikaRGB            *color);

  /*  signals  */
  void     (* picked)   (PikaColorTool      *tool,
                         const PikaCoords   *coords,
                         PikaDisplay        *display,
                         PikaColorPickState  pick_state,
                         const Babl         *sample_format,
                         gpointer            pixel,
                         const PikaRGB      *color);
};


GType      pika_color_tool_get_type   (void) G_GNUC_CONST;

void       pika_color_tool_enable     (PikaColorTool    *color_tool,
                                       PikaColorOptions *options);
void       pika_color_tool_disable    (PikaColorTool    *color_tool);
gboolean   pika_color_tool_is_enabled (PikaColorTool    *color_tool);


#endif  /*  __PIKA_COLOR_TOOL_H__  */
