/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_FOREGROUND_SELECT_TOOL_H__
#define __PIKA_FOREGROUND_SELECT_TOOL_H__


#include "pikapolygonselecttool.h"


typedef enum
{
  MATTING_STATE_FREE_SELECT = 0,
  MATTING_STATE_PAINT_TRIMAP,
  MATTING_STATE_PREVIEW_MASK,
} MattingState;


#define PIKA_TYPE_FOREGROUND_SELECT_TOOL            (pika_foreground_select_tool_get_type ())
#define PIKA_FOREGROUND_SELECT_TOOL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_FOREGROUND_SELECT_TOOL, PikaForegroundSelectTool))
#define PIKA_FOREGROUND_SELECT_TOOL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_FOREGROUND_SELECT_TOOL, PikaForegroundSelectToolClass))
#define PIKA_IS_FOREGROUND_SELECT_TOOL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_FOREGROUND_SELECT_TOOL))
#define PIKA_IS_FOREGROUND_SELECT_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_FOREGROUND_SELECT_TOOL))
#define PIKA_FOREGROUND_SELECT_TOOL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_FOREGROUND_SELECT_TOOL, PikaForegroundSelectToolClass))

#define PIKA_FOREGROUND_SELECT_TOOL_GET_OPTIONS(t)  (PIKA_FOREGROUND_SELECT_OPTIONS (pika_tool_get_options (PIKA_TOOL (t))))


typedef struct _PikaForegroundSelectTool      PikaForegroundSelectTool;
typedef struct _PikaForegroundSelectToolClass PikaForegroundSelectToolClass;

struct _PikaForegroundSelectTool
{
  PikaPolygonSelectTool  parent_instance;

  MattingState           state;

  PikaCoords             last_coords;
  GArray                *stroke;
  GeglBuffer            *trimap;
  GeglBuffer            *mask;

  GList                 *undo_stack;
  GList                 *redo_stack;

  PikaToolGui           *gui;
  GtkWidget             *preview_toggle;

  PikaCanvasItem        *grayscale_preview;
};

struct _PikaForegroundSelectToolClass
{
  PikaPolygonSelectToolClass  parent_class;
};


void    pika_foreground_select_tool_register (PikaToolRegisterCallback  callback,
                                              gpointer                  data);

GType   pika_foreground_select_tool_get_type (void) G_GNUC_CONST;


#endif  /*  __PIKA_FOREGROUND_SELECT_TOOL_H__  */
