/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "libpikawidgets/pikawidgets.h"

#include "tools-types.h"

#include "paint/pikasourceoptions.h"

#include "widgets/pikahelp-ids.h"

#include "pikahealtool.h"
#include "pikapaintoptions-gui.h"
#include "pikatoolcontrol.h"

#include "pika-intl.h"


static GtkWidget * pika_heal_options_gui (PikaToolOptions *tool_options);


G_DEFINE_TYPE (PikaHealTool, pika_heal_tool, PIKA_TYPE_SOURCE_TOOL)


void
pika_heal_tool_register (PikaToolRegisterCallback  callback,
                         gpointer                  data)
{
  (* callback) (PIKA_TYPE_HEAL_TOOL,
                PIKA_TYPE_SOURCE_OPTIONS,
                pika_heal_options_gui,
                PIKA_PAINT_OPTIONS_CONTEXT_MASK,
                "pika-heal-tool",
                _("Healing"),
                _("Healing Tool: Heal image irregularities"),
                N_("_Heal"),
                "H",
                NULL,
                PIKA_HELP_TOOL_HEAL,
                PIKA_ICON_TOOL_HEAL,
                data);
}

static void
pika_heal_tool_class_init (PikaHealToolClass *klass)
{
}

static void
pika_heal_tool_init (PikaHealTool *heal)
{
  PikaTool       *tool        = PIKA_TOOL (heal);
  PikaPaintTool  *paint_tool  = PIKA_PAINT_TOOL (tool);
  PikaSourceTool *source_tool = PIKA_SOURCE_TOOL (tool);

  pika_tool_control_set_tool_cursor (tool->control, PIKA_TOOL_CURSOR_HEAL);

  paint_tool->status      = _("Click to heal");
  paint_tool->status_ctrl = _("%s to set a new heal source");

  source_tool->status_paint           = _("Click to heal");
  /* Translators: the translation of "Click" must be the first word */
  source_tool->status_set_source      = _("Click to set a new heal source");
  source_tool->status_set_source_ctrl = _("%s to set a new heal source");
}


/*  tool options stuff  */

static GtkWidget *
pika_heal_options_gui (PikaToolOptions *tool_options)
{
  GObject   *config = G_OBJECT (tool_options);
  GtkWidget *vbox   = pika_paint_options_gui (tool_options);
  GtkWidget *button;
  GtkWidget *combo;

  /* the sample merged checkbox */
  button = pika_prop_check_button_new (config, "sample-merged",
                                       _("Sample merged"));
  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);

  /* the alignment combo */
  combo = pika_prop_enum_combo_box_new (config, "align-mode", 0, 0);
  pika_int_combo_box_set_label (PIKA_INT_COMBO_BOX (combo), _("Alignment"));
  g_object_set (combo, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
  gtk_box_pack_start (GTK_BOX (vbox), combo, TRUE, TRUE, 0);

  return vbox;
}
