/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_ISCISSORS_TOOL_H__
#define __PIKA_ISCISSORS_TOOL_H__


#include "pikaselectiontool.h"


/*  The possible states...  */
typedef enum
{
  NO_ACTION,
  SEED_PLACEMENT,
  SEED_ADJUSTMENT,
  WAITING
} IscissorsState;

/*  For oper_update & cursor_update  */
typedef enum
{
  ISCISSORS_OP_NONE,
  ISCISSORS_OP_SELECT,
  ISCISSORS_OP_MOVE_POINT,
  ISCISSORS_OP_ADD_POINT,
  ISCISSORS_OP_REMOVE_POINT,
  ISCISSORS_OP_CONNECT,
  ISCISSORS_OP_IMPOSSIBLE
} IscissorsOps;

typedef struct _ISegment ISegment;
typedef struct _ICurve   ICurve;


#define PIKA_TYPE_ISCISSORS_TOOL            (pika_iscissors_tool_get_type ())
#define PIKA_ISCISSORS_TOOL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_ISCISSORS_TOOL, PikaIscissorsTool))
#define PIKA_ISCISSORS_TOOL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_ISCISSORS_TOOL, PikaIscissorsToolClass))
#define PIKA_IS_ISCISSORS_TOOL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_ISCISSORS_TOOL))
#define PIKA_IS_ISCISSORS_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_ISCISSORS_TOOL))
#define PIKA_ISCISSORS_TOOL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_ISCISSORS_TOOL, PikaIscissorsToolClass))

#define PIKA_ISCISSORS_TOOL_GET_OPTIONS(t)  (PIKA_ISCISSORS_OPTIONS (pika_tool_get_options (PIKA_TOOL (t))))


typedef struct _PikaIscissorsTool      PikaIscissorsTool;
typedef struct _PikaIscissorsToolClass PikaIscissorsToolClass;

struct _PikaIscissorsTool
{
  PikaSelectionTool  parent_instance;

  IscissorsOps    op;

  gint            x, y;         /*  mouse coordinates                       */

  ISegment       *segment1;     /*  1st segment connected to current point  */
  ISegment       *segment2;     /*  2nd segment connected to current point  */

  ICurve         *curve;        /*  the curve                               */

  GList          *undo_stack;   /*  stack of ICurves for undo               */
  GList          *redo_stack;   /*  stack of ICurves for redo               */

  IscissorsState  state;        /*  state of iscissors                      */

  GeglBuffer     *gradient_map; /*  lazily filled gradient map              */
  PikaChannel    *mask;         /*  selection mask                          */
};

struct _PikaIscissorsToolClass
{
  PikaSelectionToolClass parent_class;
};


void    pika_iscissors_tool_register (PikaToolRegisterCallback  callback,
                                      gpointer                  data);

GType   pika_iscissors_tool_get_type (void) G_GNUC_CONST;


#endif  /*  __PIKA_ISCISSORS_TOOL_H__  */
