/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 *
 * pikanpointdeformationtool.h
 * Copyright (C) 2013 Marek Dvoroznak <dvoromar@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_N_POINT_DEFORMATION_TOOL_H__
#define __PIKA_N_POINT_DEFORMATION_TOOL_H__


#include "pikadrawtool.h"
#include "libpikamath/pikamath.h"
#include <npd/npd_common.h>


#define PIKA_TYPE_N_POINT_DEFORMATION_TOOL            (pika_n_point_deformation_tool_get_type ())
#define PIKA_N_POINT_DEFORMATION_TOOL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_N_POINT_DEFORMATION_TOOL, PikaNPointDeformationTool))
#define PIKA_N_POINT_DEFORMATION_TOOL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_N_POINT_DEFORMATION_TOOL, PikaNPointDeformationToolClass))
#define PIKA_IS_N_POINT_DEFORMATION_TOOL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_N_POINT_DEFORMATION_TOOL))
#define PIKA_IS_N_POINT_DEFORMATION_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_N_POINT_DEFORMATION_TOOL))
#define PIKA_N_POINT_DEFORMATION_TOOL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_N_POINT_DEFORMATION_TOOL, PikaNPointDeformationToolClass))

#define PIKA_N_POINT_DEFORMATION_TOOL_GET_OPTIONS(t)  (PIKA_N_POINT_DEFORMATION_OPTIONS (pika_tool_get_options (PIKA_TOOL (t))))


typedef struct _PikaNPointDeformationTool      PikaNPointDeformationTool;
typedef struct _PikaNPointDeformationToolClass PikaNPointDeformationToolClass;

struct _PikaNPointDeformationTool
{
  PikaDrawTool      parent_instance;

  guint             draw_timeout_id;
  GThread          *deform_thread;

  GeglNode         *graph;
  GeglNode         *source;
  GeglNode         *npd_node;
  GeglNode         *sink;

  GeglBuffer       *preview_buffer;

  NPDModel         *model;
  NPDControlPoint  *selected_cp;    /* last selected control point     */
  GList            *selected_cps;   /* list of selected control points */
  NPDControlPoint  *hovering_cp;

  PikaVector2      *lattice_points;

  gdouble           start_x;
  gdouble           start_y;

  gdouble           last_x;
  gdouble           last_y;

  gdouble           cursor_x;
  gdouble           cursor_y;

  gint              offset_x;
  gint              offset_y;

  gfloat            cp_scaled_radius;  /* radius of a control point scaled
                                        * according to display shell's scale
                                        */

  gboolean          active;
  volatile gboolean deformation_active;
  gboolean          rubber_band;
};

struct _PikaNPointDeformationToolClass
{
  PikaDrawToolClass parent_class;
};

void    pika_n_point_deformation_tool_register (PikaToolRegisterCallback  callback,
                                                gpointer                  data);

GType   pika_n_point_deformation_tool_get_type (void) G_GNUC_CONST;


#endif  /*  __PIKA_N_POINT_DEFORMATION_TOOL_H__  */
