/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_PAINT_TOOL_H__
#define __PIKA_PAINT_TOOL_H__


#include "pikacolortool.h"


#define PIKA_PAINT_TOOL_LINE_MASK (pika_get_extend_selection_mask ())


#define PIKA_TYPE_PAINT_TOOL            (pika_paint_tool_get_type ())
#define PIKA_PAINT_TOOL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_PAINT_TOOL, PikaPaintTool))
#define PIKA_PAINT_TOOL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_PAINT_TOOL, PikaPaintToolClass))
#define PIKA_IS_PAINT_TOOL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_PAINT_TOOL))
#define PIKA_IS_PAINT_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_PAINT_TOOL))
#define PIKA_PAINT_TOOL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_PAINT_TOOL, PikaPaintToolClass))

#define PIKA_PAINT_TOOL_GET_OPTIONS(t)  (PIKA_PAINT_OPTIONS (pika_tool_get_options (PIKA_TOOL (t))))


typedef struct _PikaPaintToolClass PikaPaintToolClass;

struct _PikaPaintTool
{
  PikaColorTool  parent_instance;

  gboolean       active;
  gboolean       pick_colors;      /*  pick color if ctrl is pressed           */
  gboolean       can_multi_paint;  /*  if paint works with multiple drawables  */
  gboolean       draw_line;

  gboolean       show_cursor;
  gboolean       draw_brush;
  gboolean       snap_brush;
  gboolean       draw_fallback;
  gint           fallback_size;
  gboolean       draw_circle;
  gint           circle_size;

  const gchar   *status;       /* status message */
  const gchar   *status_line;  /* status message when drawing a line */
  const gchar   *status_ctrl;  /* additional message for the ctrl modifier */

  PikaPaintCore *core;

  PikaDisplay   *display;
  GList         *drawables;

  gdouble        cursor_x;
  gdouble        cursor_y;

  gdouble        paint_x;
  gdouble        paint_y;
};

struct _PikaPaintToolClass
{
  PikaColorToolClass  parent_class;

  void             (* paint_prepare) (PikaPaintTool *paint_tool,
                                      PikaDisplay   *display);
  void             (* paint_start)   (PikaPaintTool *paint_tool);
  void             (* paint_end)     (PikaPaintTool *paint_tool);
  void             (* paint_flush)   (PikaPaintTool *paint_tool);

  PikaCanvasItem * (* get_outline)   (PikaPaintTool *paint_tool,
                                      PikaDisplay   *display,
                                      gdouble        x,
                                      gdouble        y);

  gboolean         (* is_alpha_only) (PikaPaintTool *paint_tool,
                                      PikaDrawable  *drawable);
};


GType   pika_paint_tool_get_type            (void) G_GNUC_CONST;

void    pika_paint_tool_set_active          (PikaPaintTool       *tool,
                                             gboolean             active);

void    pika_paint_tool_enable_color_picker (PikaPaintTool       *tool,
                                             PikaColorPickTarget  target);

void    pika_paint_tool_enable_multi_paint  (PikaPaintTool       *tool);

void    pika_paint_tool_set_draw_fallback   (PikaPaintTool       *tool,
                                             gboolean             draw_fallback,
                                             gint                 fallback_size);

void    pika_paint_tool_set_draw_circle     (PikaPaintTool       *tool,
                                             gboolean             draw_circle,
                                             gint                 circle_size);

void    pika_paint_tool_force_draw          (PikaPaintTool       *tool,
                                             gboolean             force);


#endif  /*  __PIKA_PAINT_TOOL_H__  */
