/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 *
 * pikaseamlessclonetool.h
 * Copyright (C) 2011 Barak Itkin <lightningismyname@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_SEAMLESS_CLONE_TOOL_H__
#define __PIKA_SEAMLESS_CLONE_TOOL_H__


#include "pikadrawtool.h"


#define PIKA_TYPE_SEAMLESS_CLONE_TOOL            (pika_seamless_clone_tool_get_type ())
#define PIKA_SEAMLESS_CLONE_TOOL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_SEAMLESS_CLONE_TOOL, PikaSeamlessCloneTool))
#define PIKA_SEAMLESS_CLONE_TOOL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_SEAMLESS_CLONE_TOOL, PikaSeamlessCloneToolClass))
#define PIKA_IS_SEAMLESS_CLONE_TOOL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_SEAMLESS_CLONE_TOOL))
#define PIKA_IS_SEAMLESS_CLONE_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_SEAMLESS_CLONE_TOOL))
#define PIKA_SEAMLESS_CLONE_TOOL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_SEAMLESS_CLONE_TOOL, PikaSeamlessCloneToolClass))

#define PIKA_SEAMLESS_CLONE_TOOL_GET_OPTIONS(t)  (PIKA_SEAMLESS_CLONE_OPTIONS (pika_tool_get_options (PIKA_TOOL (t))))


typedef struct _PikaSeamlessCloneTool      PikaSeamlessCloneTool;
typedef struct _PikaSeamlessCloneToolClass PikaSeamlessCloneToolClass;

struct _PikaSeamlessCloneTool
{
  PikaDrawTool    parent_instance;

  GeglBuffer     *paste;         /* A buffer containing the original
                                  * paste that will be used in the
                                  * rendering process */

  GeglNode       *render_node;    /* The parent of the Gegl graph that
                                   * renders the seamless cloning */

  GeglNode       *sc_node;        /* A Gegl node to do the seamless
                                   * cloning live with translation of
                                   * the paste */

  gint            tool_state;     /* The current state in the tool's
                                   * state machine */

  PikaDrawableFilter *filter;     /* The filter object which renders
                                   * the live preview, and commits it
                                   * when at the end */

  gint width, height;             /* The width and height of the paste.
                                   * Needed for mouse hit detection */

  gint xoff, yoff;                /* The current offset of the paste */
  gint xoff_p, yoff_p;            /* The previous offset of the paste */

  gdouble xclick, yclick;         /* The image location of the last
                                   * mouse click. To be used when the
                                   * mouse is in motion, to recalculate
                                   * the xoff and yoff values */
};

struct _PikaSeamlessCloneToolClass
{
  PikaDrawToolClass parent_class;
};


void    pika_seamless_clone_tool_register (PikaToolRegisterCallback  callback,
                                           gpointer                  data);

GType   pika_seamless_clone_tool_get_type (void) G_GNUC_CONST;


#endif  /*  __PIKA_SEAMLESS_CLONE_TOOL_H__  */
