/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_SELECTION_TOOL_H__
#define __PIKA_SELECTION_TOOL_H__


#include "pikadrawtool.h"


#define PIKA_TYPE_SELECTION_TOOL            (pika_selection_tool_get_type ())
#define PIKA_SELECTION_TOOL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_SELECTION_TOOL, PikaSelectionTool))
#define PIKA_SELECTION_TOOL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_SELECTION_TOOL, PikaSelectionToolClass))
#define PIKA_IS_SELECTION_TOOL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_SELECTION_TOOL))
#define PIKA_IS_SELECTION_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_SELECTION_TOOL))
#define PIKA_SELECTION_TOOL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_SELECTION_TOOL, PikaSelectionToolClass))

#define PIKA_SELECTION_TOOL_GET_OPTIONS(t)  (PIKA_SELECTION_OPTIONS (pika_tool_get_options (PIKA_TOOL (t))))


typedef struct _PikaSelectionTool      PikaSelectionTool;
typedef struct _PikaSelectionToolClass PikaSelectionToolClass;

struct _PikaSelectionTool
{
  PikaDrawTool    parent_instance;

  SelectFunction  function;         /*  selection function        */
  PikaChannelOps  saved_operation;  /*  saved tool options state  */

  gint            change_count;
  gboolean        saved_show_selection;
  PikaUndo       *undo;
  PikaUndo       *redo;
  gint            idle_id;

  gboolean        allow_move;
};

struct _PikaSelectionToolClass
{
  PikaDrawToolClass  parent_class;

  /*  virtual functions  */
  gboolean (* have_selection) (PikaSelectionTool *sel_tool,
                               PikaDisplay       *display);
};


GType      pika_selection_tool_get_type     (void) G_GNUC_CONST;

/*  protected function  */
gboolean   pika_selection_tool_start_edit   (PikaSelectionTool *sel_tool,
                                             PikaDisplay       *display,
                                             const PikaCoords  *coords);

void       pika_selection_tool_start_change (PikaSelectionTool *sel_tool,
                                             gboolean           create,
                                             PikaChannelOps     operation);
void       pika_selection_tool_end_change   (PikaSelectionTool *sel_tool,
                                             gboolean           cancel);


#endif  /* __PIKA_SELECTION_TOOL_H__ */
