/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "libpikawidgets/pikawidgets.h"

#include "tools-types.h"

#include "paint/pikasmudgeoptions.h"

#include "widgets/pikahelp-ids.h"
#include "widgets/pikapropwidgets.h"

#include "pikasmudgetool.h"
#include "pikapaintoptions-gui.h"
#include "pikatoolcontrol.h"

#include "pika-intl.h"


static GtkWidget * pika_smudge_options_gui (PikaToolOptions *tool_options);


G_DEFINE_TYPE (PikaSmudgeTool, pika_smudge_tool, PIKA_TYPE_BRUSH_TOOL)


void
pika_smudge_tool_register (PikaToolRegisterCallback  callback,
                           gpointer                  data)
{
  (* callback) (PIKA_TYPE_SMUDGE_TOOL,
                PIKA_TYPE_SMUDGE_OPTIONS,
                pika_smudge_options_gui,
                PIKA_PAINT_OPTIONS_CONTEXT_MASK |
                PIKA_CONTEXT_PROP_MASK_GRADIENT,
                "pika-smudge-tool",
                _("Smudge"),
                _("Smudge Tool: Smudge selectively using a brush"),
                N_("_Smudge"), "S",
                NULL, PIKA_HELP_TOOL_SMUDGE,
                PIKA_ICON_TOOL_SMUDGE,
                data);
}

static void
pika_smudge_tool_class_init (PikaSmudgeToolClass *klass)
{
}

static void
pika_smudge_tool_init (PikaSmudgeTool *smudge)
{
  PikaTool      *tool       = PIKA_TOOL (smudge);
  PikaPaintTool *paint_tool = PIKA_PAINT_TOOL (smudge);

  pika_tool_control_set_tool_cursor (tool->control, PIKA_TOOL_CURSOR_SMUDGE);

  pika_paint_tool_enable_color_picker (PIKA_PAINT_TOOL (smudge),
                                       PIKA_COLOR_PICK_TARGET_FOREGROUND);

  paint_tool->status      = _("Click to smudge");
  paint_tool->status_line = _("Click to smudge the line");
  paint_tool->status_ctrl = NULL;
}


/*  tool options stuff  */

static GtkWidget *
pika_smudge_options_gui (PikaToolOptions *tool_options)
{
  GObject   *config = G_OBJECT (tool_options);
  GtkWidget *vbox   = pika_paint_options_gui (tool_options);
  GtkWidget *scale;
  GtkWidget *button;

  button = pika_prop_check_button_new (config, "no-erasing", NULL);
  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);

  button = pika_prop_check_button_new (config, "sample-merged", NULL);
  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);

  /*  the rate scale  */
  scale = pika_prop_spin_scale_new (config, "rate",
                                    1.0, 10.0, 1);
  gtk_box_pack_start (GTK_BOX (vbox), scale, FALSE, FALSE, 0);

  scale = pika_prop_spin_scale_new (config, "flow",
                                    1.0, 10.0, 1);
  gtk_box_pack_start (GTK_BOX (vbox), scale, FALSE, FALSE, 0);

  return vbox;
}
