/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TEXT_OPTIONS_H__
#define __PIKA_TEXT_OPTIONS_H__


#include "core/pikatooloptions.h"


#define PIKA_TYPE_TEXT_OPTIONS            (pika_text_options_get_type ())
#define PIKA_TEXT_OPTIONS(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TEXT_OPTIONS, PikaTextOptions))
#define PIKA_TEXT_OPTIONS_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_TEXT_OPTIONS, PikaTextOptionsClass))
#define PIKA_IS_TEXT_OPTIONS(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TEXT_OPTIONS))
#define PIKA_IS_TEXT_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_TEXT_OPTIONS))
#define PIKA_TEXT_OPTIONS_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_TEXT_OPTIONS, PikaTextOptionsClass))


typedef struct _PikaTextOptions      PikaTextOptions;
typedef struct _PikaToolOptionsClass PikaTextOptionsClass;

struct _PikaTextOptions
{
  PikaToolOptions        tool_options;

  PikaUnit               unit;
  gdouble                font_size;
  gboolean               antialias;
  PikaTextHintStyle      hint_style;
  gchar                 *language;
  PikaTextDirection      base_dir;
  PikaTextJustification  justify;
  gdouble                indent;
  gdouble                line_spacing;
  gdouble                letter_spacing;
  PikaTextBoxMode        box_mode;

  PikaTextOutline        outline;
  PikaCustomStyle        outline_style;
  PikaRGB                outline_foreground;
  PikaPattern           *outline_pattern;
  gdouble                outline_width;
  PikaUnit               outline_unit;
  PikaCapStyle           outline_cap_style;
  PikaJoinStyle          outline_join_style;
  gdouble                outline_miter_limit;
  gboolean               outline_antialias;
  gdouble                outline_dash_offset;
  GArray                *outline_dash_info;

  PikaViewType           font_view_type;
  PikaViewSize           font_view_size;

  gboolean               use_editor;
  gboolean               show_on_canvas;

  /*  options gui  */
  GtkWidget             *size_entry;
};


GType       pika_text_options_get_type     (void) G_GNUC_CONST;

void        pika_text_options_connect_text (PikaTextOptions *options,
                                            PikaText        *text);

GtkWidget * pika_text_options_gui          (PikaToolOptions *tool_options);

GtkWidget * pika_text_options_editor_new   (GtkWindow       *parent,
                                            Pika            *pika,
                                            PikaTextOptions *options,
                                            PikaMenuFactory *menu_factory,
                                            const gchar     *title,
                                            PikaText        *text,
                                            PikaTextBuffer  *text_buffer,
                                            gdouble          xres,
                                            gdouble          yres);


#endif /* __PIKA_TEXT_OPTIONS_H__ */
