/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-2001 Spencer Kimball, Peter Mattis, and others
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TRANSFORM_TOOL_H__
#define __PIKA_TRANSFORM_TOOL_H__


#include "pikadrawtool.h"


/* This is not the number of items in the enum above, but the max size
 * of the enums at the top of each transformation tool, stored in
 * trans_info and related
 */
#define TRANS_INFO_SIZE 17

typedef gdouble TransInfo[TRANS_INFO_SIZE];


#define PIKA_TYPE_TRANSFORM_TOOL            (pika_transform_tool_get_type ())
#define PIKA_TRANSFORM_TOOL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TRANSFORM_TOOL, PikaTransformTool))
#define PIKA_TRANSFORM_TOOL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_TRANSFORM_TOOL, PikaTransformToolClass))
#define PIKA_IS_TRANSFORM_TOOL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TRANSFORM_TOOL))
#define PIKA_IS_TRANSFORM_TOOL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_TRANSFORM_TOOL))
#define PIKA_TRANSFORM_TOOL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_TRANSFORM_TOOL, PikaTransformToolClass))

#define PIKA_TRANSFORM_TOOL_GET_OPTIONS(t)  (PIKA_TRANSFORM_OPTIONS (pika_tool_get_options (PIKA_TOOL (t))))


typedef struct _PikaTransformToolClass PikaTransformToolClass;

struct _PikaTransformTool
{
  PikaDrawTool       parent_instance;

  GList             *objects;            /*  List of PikaObject initially
                                             selected and set for
                                             transform processing.        */

  gint               x1, y1;             /*  upper left hand coordinate   */
  gint               x2, y2;             /*  lower right hand coords      */

  PikaMatrix3        transform;          /*  transformation matrix        */
  gboolean           transform_valid;    /*  whether the matrix is valid  */

  gboolean           restore_type;
  PikaTransformType  saved_type;
};

struct _PikaTransformToolClass
{
  PikaDrawToolClass  parent_class;

  /*  virtual functions  */
  void                     (* recalc_matrix) (PikaTransformTool  *tr_tool);
  gchar                  * (* get_undo_desc) (PikaTransformTool  *tr_tool);
  PikaTransformDirection   (* get_direction) (PikaTransformTool  *tr_tool);
  GeglBuffer             * (* transform)     (PikaTransformTool  *tr_tool,
                                              GList              *objects,
                                              GeglBuffer         *orig_buffer,
                                              gint                orig_offset_x,
                                              gint                orig_offset_y,
                                              PikaColorProfile  **buffer_profile,
                                              gint               *new_offset_x,
                                              gint               *new_offset_y);

  const gchar *undo_desc;
  const gchar *progress_text;
};


GType        pika_transform_tool_get_type            (void) G_GNUC_CONST;

GList     * pika_transform_tool_get_selected_objects (PikaTransformTool  *tr_tool,
                                                      PikaDisplay        *display);
GList   * pika_transform_tool_check_selected_objects (PikaTransformTool  *tr_tool,
                                                      PikaDisplay        *display,
                                                      GError            **error);

gboolean     pika_transform_tool_bounds              (PikaTransformTool  *tr_tool,
                                                      PikaDisplay        *display);
void         pika_transform_tool_recalc_matrix       (PikaTransformTool  *tr_tool,
                                                      PikaDisplay        *display);

gboolean     pika_transform_tool_transform           (PikaTransformTool  *tr_tool,
                                                      PikaDisplay        *display);

void         pika_transform_tool_set_type            (PikaTransformTool  *tr_tool,
                                                      PikaTransformType   type);


#endif  /*  __PIKA_TRANSFORM_TOOL_H__  */
