/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikaanchor.c
 * Copyright (C) 2002 Simon Budig  <simon@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <glib-object.h>

#include "vectors-types.h"

#include "pikaanchor.h"


G_DEFINE_BOXED_TYPE (PikaAnchor, pika_anchor, pika_anchor_copy, pika_anchor_free)

PikaAnchor *
pika_anchor_new (PikaAnchorType    type,
                 const PikaCoords *position)
{
  PikaAnchor *anchor = g_slice_new0 (PikaAnchor);

  anchor->type = type;

  if (position)
    anchor->position = *position;

  return anchor;
}

PikaAnchor *
pika_anchor_copy (const PikaAnchor *anchor)
{
  g_return_val_if_fail (anchor != NULL, NULL);

  return g_slice_dup (PikaAnchor, anchor);
}

void
pika_anchor_free (PikaAnchor *anchor)
{
  g_return_if_fail (anchor != NULL);

  g_slice_free (PikaAnchor, anchor);
}
