/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikabezierstroke.h
 * Copyright (C) 2002 Simon Budig  <simon@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_BEZIER_STROKE_H__
#define __PIKA_BEZIER_STROKE_H__

#include "pikastroke.h"


#define PIKA_TYPE_BEZIER_STROKE            (pika_bezier_stroke_get_type ())
#define PIKA_BEZIER_STROKE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_BEZIER_STROKE, PikaBezierStroke))
#define PIKA_BEZIER_STROKE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_BEZIER_STROKE, PikaBezierStrokeClass))
#define PIKA_IS_BEZIER_STROKE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_BEZIER_STROKE))
#define PIKA_IS_BEZIER_STROKE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_BEZIER_STROKE))
#define PIKA_BEZIER_STROKE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_BEZIER_STROKE, PikaBezierStrokeClass))


typedef struct _PikaBezierStrokeClass PikaBezierStrokeClass;

struct _PikaBezierStroke
{
  PikaStroke  parent_instance;
};

struct _PikaBezierStrokeClass
{
  PikaStrokeClass  parent_class;
};


GType        pika_bezier_stroke_get_type        (void) G_GNUC_CONST;

PikaStroke * pika_bezier_stroke_new             (void);
PikaStroke * pika_bezier_stroke_new_from_coords (const PikaCoords *coords,
                                                 gint              n_coords,
                                                 gboolean          closed);

PikaStroke * pika_bezier_stroke_new_moveto      (const PikaCoords *start);
void         pika_bezier_stroke_lineto          (PikaStroke       *bez_stroke,
                                                 const PikaCoords *end);
void         pika_bezier_stroke_conicto         (PikaStroke       *bez_stroke,
                                                 const PikaCoords *control,
                                                 const PikaCoords *end);
void         pika_bezier_stroke_cubicto         (PikaStroke       *bez_stroke,
                                                 const PikaCoords *control1,
                                                 const PikaCoords *control2,
                                                 const PikaCoords *end);
void         pika_bezier_stroke_arcto           (PikaStroke       *bez_stroke,
                                                 gdouble           radius_x,
                                                 gdouble           radius_y,
                                                 gdouble           angle_rad,
                                                 gboolean          large_arc,
                                                 gboolean          sweep,
                                                 const PikaCoords *end);
PikaStroke * pika_bezier_stroke_new_ellipse     (const PikaCoords *center,
                                                 gdouble           radius_x,
                                                 gdouble           radius_y,
                                                 gdouble           angle);


PikaAnchor * pika_bezier_stroke_extend      (PikaStroke           *stroke,
                                             const PikaCoords     *coords,
                                             PikaAnchor           *neighbor,
                                             PikaVectorExtendMode  extend_mode);


#endif /* __PIKA_BEZIER_STROKE_H__ */
