/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <string.h>

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "vectors-types.h"

#include "libpikamath/pikamath.h"

#include "core/pikaimage.h"
#include "core/pikatempbuf.h"

#include "pikastroke.h"
#include "pikavectors.h"
#include "pikavectors-preview.h"


/*  public functions  */

PikaTempBuf *
pika_vectors_get_new_preview (PikaViewable *viewable,
                              PikaContext  *context,
                              gint          width,
                              gint          height)
{
  PikaVectors *vectors;
  PikaItem    *item;
  PikaStroke  *cur_stroke;
  gdouble      xscale, yscale;
  guchar      *data;
  PikaTempBuf *temp_buf;

  vectors = PIKA_VECTORS (viewable);
  item    = PIKA_ITEM (viewable);

  xscale = ((gdouble) width)  / pika_image_get_width  (pika_item_get_image (item));
  yscale = ((gdouble) height) / pika_image_get_height (pika_item_get_image (item));

  temp_buf = pika_temp_buf_new (width, height, babl_format ("Y' u8"));
  data = pika_temp_buf_get_data (temp_buf);
  memset (data, 255, width * height);

  for (cur_stroke = pika_vectors_stroke_get_next (vectors, NULL);
       cur_stroke;
       cur_stroke = pika_vectors_stroke_get_next (vectors, cur_stroke))
    {
      GArray   *coords;
      gboolean  closed;
      gint      i;

      coords = pika_stroke_interpolate (cur_stroke, 0.5, &closed);

      if (coords)
        {
          for (i = 0; i < coords->len; i++)
            {
              PikaCoords point;
              gint       x, y;

              point = g_array_index (coords, PikaCoords, i);

              x = ROUND (point.x * xscale);
              y = ROUND (point.y * yscale);

              if (x >= 0 && y >= 0 && x < width && y < height)
                data[y * width + x] = 0;
            }

          g_array_free (coords, TRUE);
        }
    }

  return temp_buf;
}
