/* Pika - Photo and Image Kooker Application
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gegl.h>

#include "vectors-types.h"

#include "core/pikaimage.h"

#include "pikavectors.h"
#include "pikavectorspropundo.h"


static void   pika_vectors_prop_undo_constructed (GObject             *object);

static void   pika_vectors_prop_undo_pop         (PikaUndo            *undo,
                                                  PikaUndoMode         undo_mode,
                                                  PikaUndoAccumulator *accum);


G_DEFINE_TYPE (PikaVectorsPropUndo, pika_vectors_prop_undo, PIKA_TYPE_ITEM_UNDO)

#define parent_class pika_vectors_prop_undo_parent_class


static void
pika_vectors_prop_undo_class_init (PikaVectorsPropUndoClass *klass)
{
  GObjectClass  *object_class = G_OBJECT_CLASS (klass);
  PikaUndoClass *undo_class   = PIKA_UNDO_CLASS (klass);

  object_class->constructed = pika_vectors_prop_undo_constructed;

  undo_class->pop           = pika_vectors_prop_undo_pop;
}

static void
pika_vectors_prop_undo_init (PikaVectorsPropUndo *undo)
{
}

static void
pika_vectors_prop_undo_constructed (GObject *object)
{
  /* PikaVectors *vectors; */

  G_OBJECT_CLASS (parent_class)->constructed (object);

  pika_assert (PIKA_IS_VECTORS (PIKA_ITEM_UNDO (object)->item));

  /* vectors = PIKA_VECTORS (PIKA_ITEM_UNDO (object)->item); */

  switch (PIKA_UNDO (object)->undo_type)
    {
    default:
      pika_assert_not_reached ();
    }
}

static void
pika_vectors_prop_undo_pop (PikaUndo            *undo,
                            PikaUndoMode         undo_mode,
                            PikaUndoAccumulator *accum)
{
#if 0
  PikaVectorsPropUndo *vectors_prop_undo = PIKA_VECTORS_PROP_UNDO (undo);
  PikaVectors         *vectors           = PIKA_VECTORS (PIKA_ITEM_UNDO (undo)->item);
#endif

  PIKA_UNDO_CLASS (parent_class)->pop (undo, undo_mode, accum);

  switch (undo->undo_type)
    {
    default:
      pika_assert_not_reached ();
    }
}
