/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikaactionfactory.h
 * Copyright (C) 2004 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_ACTION_FACTORY_H__
#define __PIKA_ACTION_FACTORY_H__


#include "core/pikaobject.h"


typedef struct _PikaActionFactoryEntry PikaActionFactoryEntry;

struct _PikaActionFactoryEntry
{
  gchar                     *identifier;
  gchar                     *label;
  gchar                     *icon_name;
  PikaActionGroupSetupFunc   setup_func;
  PikaActionGroupUpdateFunc  update_func;

  GHashTable                *groups;
};


#define PIKA_TYPE_ACTION_FACTORY            (pika_action_factory_get_type ())
#define PIKA_ACTION_FACTORY(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_ACTION_FACTORY, PikaActionFactory))
#define PIKA_ACTION_FACTORY_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_ACTION_FACTORY, PikaActionFactoryClass))
#define PIKA_IS_ACTION_FACTORY(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_ACTION_FACTORY))
#define PIKA_IS_ACTION_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_ACTION_FACTORY))
#define PIKA_ACTION_FACTORY_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_ACTION_FACTORY, PikaActionFactoryClass))


typedef struct _PikaActionFactoryClass  PikaActionFactoryClass;

struct _PikaActionFactory
{
  PikaObject  parent_instance;

  Pika       *pika;
  GList      *registered_groups;
};

struct _PikaActionFactoryClass
{
  PikaObjectClass  parent_class;
};


GType               pika_action_factory_get_type       (void) G_GNUC_CONST;

PikaActionFactory * pika_action_factory_new            (Pika                      *pika);

void                pika_action_factory_group_register (PikaActionFactory         *factory,
                                                        const gchar               *identifier,
                                                        const gchar               *label,
                                                        const gchar               *icon_name,
                                                        PikaActionGroupSetupFunc   setup_func,
                                                        PikaActionGroupUpdateFunc  update_func);

PikaActionGroup *   pika_action_factory_get_group      (PikaActionFactory         *factory,
                                                        const gchar               *identifier,
                                                        gpointer                   user_data);
void                pika_action_factory_delete_group   (PikaActionFactory         *factory,
                                                        const gchar               *identifier,
                                                        gpointer                   user_data);


#endif  /*  __PIKA_ACTION_FACTORY_H__  */
