/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikacolordialog.h
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_COLOR_DIALOG_H__
#define __PIKA_COLOR_DIALOG_H__


#include "pikaviewabledialog.h"


#define PIKA_COLOR_DIALOG_HISTORY_SIZE 12


#define PIKA_TYPE_COLOR_DIALOG            (pika_color_dialog_get_type ())
#define PIKA_COLOR_DIALOG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_COLOR_DIALOG, PikaColorDialog))
#define PIKA_COLOR_DIALOG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_COLOR_DIALOG, PikaColorDialogClass))
#define PIKA_IS_COLOR_DIALOG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_COLOR_DIALOG))
#define PIKA_IS_COLOR_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_COLOR_DIALOG))
#define PIKA_COLOR_DIALOG_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_COLOR_DIALOG, PikaColorDialogClass))


typedef struct _PikaColorDialogClass PikaColorDialogClass;

struct _PikaColorDialog
{
  PikaViewableDialog   parent_instance;

  gboolean             wants_updates;
  gboolean             user_context_aware;

  GtkWidget           *stack;
  GtkWidget           *selection;
  GtkWidget           *colormap_selection;

  PikaImage           *active_image;
  gboolean             colormap_editing;
};

struct _PikaColorDialogClass
{
  PikaViewableDialogClass  parent_class;

  void (* update) (PikaColorDialog      *dialog,
                   const PikaRGB        *color,
                   PikaColorDialogState  state);
};


GType       pika_color_dialog_get_type  (void) G_GNUC_CONST;

GtkWidget * pika_color_dialog_new       (PikaViewable      *viewable,
                                         PikaContext       *context,
                                         gboolean           context_aware,
                                         const gchar       *title,
                                         const gchar       *icon_name,
                                         const gchar       *desc,
                                         GtkWidget         *parent,
                                         PikaDialogFactory *dialog_factory,
                                         const gchar       *dialog_identifier,
                                         const PikaRGB     *color,
                                         gboolean           wants_update,
                                         gboolean           show_alpha);

void        pika_color_dialog_set_color (PikaColorDialog   *dialog,
                                         const PikaRGB     *color);
void        pika_color_dialog_get_color (PikaColorDialog   *dialog,
                                         PikaRGB           *color);


#endif /* __PIKA_COLOR_DIALOG_H__ */
