/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_COLOR_FRAME_H__
#define __PIKA_COLOR_FRAME_H__


#define PIKA_COLOR_FRAME_ROWS 6


#define PIKA_TYPE_COLOR_FRAME            (pika_color_frame_get_type ())
#define PIKA_COLOR_FRAME(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_COLOR_FRAME, PikaColorFrame))
#define PIKA_COLOR_FRAME_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_COLOR_FRAME, PikaColorFrameClass))
#define PIKA_IS_COLOR_FRAME(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_COLOR_FRAME))
#define PIKA_IS_COLOR_FRAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_COLOR_FRAME))
#define PIKA_COLOR_FRAME_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_COLOR_FRAME, PikaColorFrameClass))


typedef struct _PikaColorFrameClass PikaColorFrameClass;

struct _PikaColorFrame
{
  PikaFrame           parent_instance;

  Pika               *pika;

  gboolean            sample_valid;
  gboolean            sample_average;
  const Babl         *sample_format;
  gdouble             pixel[4];
  PikaRGB             color;
  gint                x;
  gint                y;

  PikaColorPickMode   pick_mode;

  PangoEllipsizeMode  ellipsize;

  gboolean            has_number;
  gint                number;

  gboolean            has_color_area;
  gboolean            has_coords;

  GtkWidget          *combo;
  GtkWidget          *color_area;
  GtkWidget          *coords_box_x;
  GtkWidget          *coords_box_y;
  GtkWidget          *coords_label_x;
  GtkWidget          *coords_label_y;
  GtkWidget          *name_labels[PIKA_COLOR_FRAME_ROWS];
  GtkWidget          *value_labels[PIKA_COLOR_FRAME_ROWS];

  PangoLayout        *number_layout;

  PikaImage          *image;

  PikaColorConfig    *config;
  PikaColorProfile   *view_profile;
  PikaColorRenderingIntent
                      simulation_intent;
};

struct _PikaColorFrameClass
{
  PikaFrameClass      parent_class;
};


GType       pika_color_frame_get_type           (void) G_GNUC_CONST;

GtkWidget * pika_color_frame_new                (Pika               *pika);

void        pika_color_frame_set_mode           (PikaColorFrame     *frame,
                                                 PikaColorPickMode   mode);

void        pika_color_frame_set_ellipsize      (PikaColorFrame     *frame,
                                                 PangoEllipsizeMode  ellipsize);

void        pika_color_frame_set_has_number     (PikaColorFrame     *frame,
                                                 gboolean            has_number);
void        pika_color_frame_set_number         (PikaColorFrame     *frame,
                                                 gint                number);

void        pika_color_frame_set_has_color_area (PikaColorFrame     *frame,
                                                 gboolean            has_color_area);
void        pika_color_frame_set_has_coords     (PikaColorFrame     *frame,
                                                 gboolean            has_coords);

void        pika_color_frame_set_color          (PikaColorFrame     *frame,
                                                 gboolean            sample_average,
                                                 const Babl         *format,
                                                 gpointer            pixel,
                                                 const PikaRGB      *color,
                                                 gint                x,
                                                 gint                y);
void        pika_color_frame_set_invalid        (PikaColorFrame     *frame);

void        pika_color_frame_set_color_config   (PikaColorFrame     *frame,
                                                 PikaColorConfig    *config);

void        pika_color_frame_set_image          (PikaColorFrame     *frame,
                                                 PikaImage          *image);


#endif  /*  __PIKA_COLOR_FRAME_H__  */
