/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "widgets-types.h"

#include "core/pikacontainer.h"
#include "core/pikacontext.h"

#include "pikacontainereditor.h"
#include "pikacontainerview.h"
#include "pikacontainerview-utils.h"
#include "pikadockable.h"


/*  public functions  */

PikaContainerView *
pika_container_view_get_by_dockable (PikaDockable *dockable)
{
  GtkWidget *child;

  g_return_val_if_fail (PIKA_IS_DOCKABLE (dockable), NULL);

  child = gtk_bin_get_child (GTK_BIN (dockable));

  if (child)
    {
      if (PIKA_IS_CONTAINER_EDITOR (child))
        {
          return PIKA_CONTAINER_EDITOR (child)->view;
        }
      else if (PIKA_IS_CONTAINER_VIEW (child))
        {
          return PIKA_CONTAINER_VIEW (child);
        }
    }

  return NULL;
}

void
pika_container_view_remove_active (PikaContainerView *view)
{
  PikaContext   *context;
  PikaContainer *container;

  g_return_if_fail (PIKA_IS_CONTAINER_VIEW (view));

  context   = pika_container_view_get_context (view);
  container = pika_container_view_get_container (view);

  if (context && container)
    {
      GType       children_type;
      PikaObject *active;

      children_type = pika_container_get_children_type (container);

      active = pika_context_get_by_type (context, children_type);

      if (active)
        {
          PikaObject *new;

          new = pika_container_get_neighbor_of (container, active);

          if (new)
            pika_context_set_by_type (context, children_type, new);

          pika_container_remove (container, active);
        }
    }
}
