/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikacontrollerinfo.h
 * Copyright (C) 2004-2005 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_CONTROLLER_INFO_H__
#define __PIKA_CONTROLLER_INFO_H__


#include "core/pikaviewable.h"


typedef gboolean (* PikaControllerEventSnooper) (PikaControllerInfo        *info,
                                                 PikaController            *controller,
                                                 const PikaControllerEvent *event,
                                                 gpointer                   user_data);


#define PIKA_TYPE_CONTROLLER_INFO            (pika_controller_info_get_type ())
#define PIKA_CONTROLLER_INFO(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_CONTROLLER_INFO, PikaControllerInfo))
#define PIKA_CONTROLLER_INFO_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_CONTROLLER_INFO, PikaControllerInfoClass))
#define PIKA_IS_CONTROLLER_INFO(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_CONTROLLER_INFO))
#define PIKA_IS_CONTROLLER_INFO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_CONTROLLER_INFO))
#define PIKA_CONTROLLER_INFO_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_CONTROLLER_INFO, PikaControllerInfoClass))


typedef struct _PikaControllerInfoClass PikaControllerInfoClass;

struct _PikaControllerInfo
{
  PikaViewable                parent_instance;

  gboolean                    enabled;
  gboolean                    debug_events;

  PikaController             *controller;
  GHashTable                 *mapping;

  PikaControllerEventSnooper  snooper;
  gpointer                    snooper_data;
};

struct _PikaControllerInfoClass
{
  PikaViewableClass  parent_class;

  gboolean (* event_mapped) (PikaControllerInfo        *info,
                             PikaController            *controller,
                             const PikaControllerEvent *event,
                             const gchar               *action_name);
};


GType    pika_controller_info_get_type          (void) G_GNUC_CONST;

PikaControllerInfo * pika_controller_info_new   (GType                       type);

void     pika_controller_info_set_enabled       (PikaControllerInfo         *info,
                                                 gboolean                    enabled);
gboolean pika_controller_info_get_enabled       (PikaControllerInfo         *info);

void     pika_controller_info_set_event_snooper (PikaControllerInfo         *info,
                                                 PikaControllerEventSnooper  snooper,
                                                 gpointer                    snooper_data);


#endif /* __PIKA_CONTROLLER_INFO_H__ */
