/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikadashboard.h
 * Copyright (C) 2017 Ell
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_DASHBOARD_H__
#define __PIKA_DASHBOARD_H__


#include "pikaeditor.h"


struct _PikaDashboardLogParams
{
  gint     sample_frequency;
  gboolean backtrace;
  gboolean messages;
  gboolean progressive;
};


#define PIKA_TYPE_DASHBOARD            (pika_dashboard_get_type ())
#define PIKA_DASHBOARD(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_DASHBOARD, PikaDashboard))
#define PIKA_DASHBOARD_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_DASHBOARD, PikaDashboardClass))
#define PIKA_IS_DASHBOARD(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_DASHBOARD))
#define PIKA_IS_DASHBOARD_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_DASHBOARD))
#define PIKA_DASHBOARD_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_DASHBOARD, PikaDashboardClass))


typedef struct _PikaDashboardPrivate PikaDashboardPrivate;
typedef struct _PikaDashboardClass   PikaDashboardClass;

struct _PikaDashboard
{
  PikaEditor            parent_instance;

  PikaDashboardPrivate *priv;
};

struct _PikaDashboardClass
{
  PikaEditorClass  parent_class;
};


GType                          pika_dashboard_get_type                   (void) G_GNUC_CONST;

GtkWidget                    * pika_dashboard_new                        (Pika                          *pika,
                                                                          PikaMenuFactory               *menu_factory);

gboolean                       pika_dashboard_log_start_recording        (PikaDashboard                 *dashboard,
                                                                          GFile                         *file,
                                                                          const PikaDashboardLogParams  *params,
                                                                          GError                       **error);
gboolean                       pika_dashboard_log_stop_recording         (PikaDashboard                 *dashboard,
                                                                          GError                       **error);
gboolean                       pika_dashboard_log_is_recording           (PikaDashboard                 *dashboard);
const PikaDashboardLogParams * pika_dashboard_log_get_default_params     (PikaDashboard                 *dashboard);
void                           pika_dashboard_log_add_marker             (PikaDashboard                 *dashboard,
                                                                          const gchar                   *description);

void                           pika_dashboard_reset                      (PikaDashboard                 *dashboard);

void                           pika_dashboard_set_update_interval        (PikaDashboard                 *dashboard,
                                                                          PikaDashboardUpdateInteval     update_interval);
PikaDashboardUpdateInteval     pika_dashboard_get_update_interval        (PikaDashboard                 *dashboard);

void                           pika_dashboard_set_history_duration       (PikaDashboard                 *dashboard,
                                                                          PikaDashboardHistoryDuration   history_duration);
PikaDashboardHistoryDuration   pika_dashboard_get_history_duration       (PikaDashboard                 *dashboard);

void                           pika_dashboard_set_low_swap_space_warning (PikaDashboard                 *dashboard,
                                                                          gboolean                       low_swap_space_warning);
gboolean                       pika_dashboard_get_low_swap_space_warning (PikaDashboard                 *dashboard);

void                           pika_dashboard_menu_setup                 (PikaUIManager                 *manager,
                                                                          const gchar                   *ui_path);


#endif  /*  __PIKA_DASHBOARD_H__  */
