/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikadock.h
 * Copyright (C) 2001-2005 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_DOCK_H__
#define __PIKA_DOCK_H__


#define PIKA_TYPE_DOCK            (pika_dock_get_type ())
#define PIKA_DOCK(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_DOCK, PikaDock))
#define PIKA_DOCK_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_DOCK, PikaDockClass))
#define PIKA_IS_DOCK(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_DOCK))
#define PIKA_IS_DOCK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_DOCK))
#define PIKA_DOCK_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_DOCK, PikaDockClass))


/* String used to separate dockables, e.g. "Tool Options, Layers" */
#define PIKA_DOCK_DOCKABLE_SEPARATOR C_("dock", ", ")

/* String used to separate books (GtkNotebooks) within a dock,
   e.g. "Tool Options, Layers - Brushes"
 */
#define PIKA_DOCK_BOOK_SEPARATOR C_("dock", " - ")

/* String used to separate dock columns,
   e.g. "Tool Options, Layers - Brushes | Gradients"
 */
#define PIKA_DOCK_COLUMN_SEPARATOR C_("dock", " | ")


typedef struct _PikaDockClass    PikaDockClass;
typedef struct _PikaDockPrivate  PikaDockPrivate;

/**
 * PikaDock:
 *
 * Contains a column of PikaDockbooks.
 */
struct _PikaDock
{
  GtkBox           parent_instance;

  PikaDockPrivate *p;
};

struct _PikaDockClass
{
  GtkBoxClass  parent_class;

  /*  virtual functions  */
  gchar * (* get_description)         (PikaDock       *dock,
                                       gboolean        complete);
  void    (* set_host_geometry_hints) (PikaDock       *dock,
                                       GtkWindow      *window);

  /*  signals  */
  void    (* book_added)              (PikaDock       *dock,
                                       PikaDockbook   *dockbook);
  void    (* book_removed)            (PikaDock       *dock,
                                       PikaDockbook   *dockbook);
  void    (* description_invalidated) (PikaDock       *dock);
  void    (* geometry_invalidated)    (PikaDock       *dock);
};


GType               pika_dock_get_type                (void) G_GNUC_CONST;

gchar             * pika_dock_get_description         (PikaDock       *dock,
                                                       gboolean        complete);
void                pika_dock_set_host_geometry_hints (PikaDock       *dock,
                                                       GtkWindow      *window);
void                pika_dock_invalidate_geometry     (PikaDock       *dock);
void                pika_dock_update_with_context     (PikaDock       *dock,
                                                       PikaContext    *context);
PikaContext       * pika_dock_get_context             (PikaDock       *dock);
PikaDialogFactory * pika_dock_get_dialog_factory      (PikaDock       *dock);
PikaUIManager     * pika_dock_get_ui_manager          (PikaDock       *dock);
GList             * pika_dock_get_dockbooks           (PikaDock       *dock);
gint                pika_dock_get_n_dockables         (PikaDock       *dock);
GtkWidget         * pika_dock_get_main_vbox           (PikaDock       *dock);
GtkWidget         * pika_dock_get_vbox                (PikaDock       *dock);

void                pika_dock_set_id                  (PikaDock       *dock,
                                                       gint            ID);
gint                pika_dock_get_id                  (PikaDock       *dock);

void                pika_dock_add_book                (PikaDock       *dock,
                                                       PikaDockbook   *dockbook,
                                                       gint            index);
void                pika_dock_remove_book             (PikaDock       *dock,
                                                       PikaDockbook   *dockbook);


#endif /* __PIKA_DOCK_H__ */
