/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikadockbook.h
 * Copyright (C) 2001-2007 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_DOCKBOOK_H__
#define __PIKA_DOCKBOOK_H__


#define PIKA_TYPE_DOCKBOOK            (pika_dockbook_get_type ())
#define PIKA_DOCKBOOK(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_DOCKBOOK, PikaDockbook))
#define PIKA_DOCKBOOK_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_DOCKBOOK, PikaDockbookClass))
#define PIKA_IS_DOCKBOOK(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_DOCKBOOK))
#define PIKA_IS_DOCKBOOK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_DOCKBOOK))
#define PIKA_DOCKBOOK_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_DOCKBOOK, PikaDockbookClass))


typedef void (* PikaDockbookDragCallback) (GdkDragContext *context,
                                           gboolean        begin,
                                           gpointer        data);


typedef struct _PikaDockbookClass    PikaDockbookClass;
typedef struct _PikaDockbookPrivate  PikaDockbookPrivate;

/**
 * PikaDockbook:
 *
 * Holds PikaDockables which are presented on different tabs using
 * GtkNotebook.
 */
struct _PikaDockbook
{
  GtkNotebook parent_instance;

  PikaDockbookPrivate *p;
};

struct _PikaDockbookClass
{
  GtkNotebookClass parent_class;

  void (* dockable_added)     (PikaDockbook *dockbook,
                               PikaDockable *dockable);
  void (* dockable_removed)   (PikaDockbook *dockbook,
                               PikaDockable *dockable);
  void (* dockable_reordered) (PikaDockbook *dockbook,
                               PikaDockable *dockable);
};


GType           pika_dockbook_get_type                (void) G_GNUC_CONST;
GtkWidget     * pika_dockbook_new                     (PikaMenuFactory          *menu_factory);

void            pika_dockbook_set_dock                (PikaDockbook             *dockbook,
                                                       PikaDock                 *dock);
PikaDock      * pika_dockbook_get_dock                (PikaDockbook             *dockbook);

GtkWidget     * pika_dockbook_add_from_dialog_factory (PikaDockbook             *dockbook,
                                                       const gchar              *identifiers);

void            pika_dockbook_update_with_context     (PikaDockbook             *dockbook,
                                                       PikaContext              *context);
GtkWidget    *  pika_dockbook_create_tab_widget       (PikaDockbook             *dockbook,
                                                       PikaDockable             *dockable);
void            pika_dockbook_set_drag_handler        (PikaDockbook             *dockbook,
                                                       PikaPanedBox             *drag_handler);

void            pika_dockbook_add_drag_callback       (PikaDockbookDragCallback  callback,
                                                       gpointer                  data);
void            pika_dockbook_remove_drag_callback    (PikaDockbookDragCallback  callback,
                                                       gpointer                  data);


#endif /* __PIKA_DOCKBOOK_H__ */
