/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * pikadocked.h
 * Copyright (C) 2003  Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_DOCKED_H__
#define __PIKA_DOCKED_H__


#define PIKA_TYPE_DOCKED (pika_docked_get_type ())
G_DECLARE_INTERFACE (PikaDocked, pika_docked, PIKA, DOCKED, GtkWidget)


/**
 * PikaDockedInterface:
 *
 * Interface with common methods for stuff that is docked.
 */
struct _PikaDockedInterface
{
  GTypeInterface base_iface;

  /*  signals  */
  void            (* title_changed)       (PikaDocked   *docked);

  /*  virtual functions  */
  void            (* set_aux_info)        (PikaDocked   *docked,
                                           GList        *aux_info);
  GList         * (* get_aux_info)        (PikaDocked   *docked);

  GtkWidget     * (* get_preview)         (PikaDocked   *docked,
                                           PikaContext  *context,
                                           GtkIconSize   size);
  gboolean        (* get_prefer_icon)     (PikaDocked   *docked);
  PikaUIManager * (* get_menu)            (PikaDocked   *docked,
                                           const gchar **ui_path,
                                           gpointer     *popup_data);
  gchar         * (* get_title)           (PikaDocked   *docked);

  void            (* set_context)         (PikaDocked   *docked,
                                           PikaContext  *context);

  gboolean        (* has_button_bar)      (PikaDocked   *docked);
  void            (* set_show_button_bar) (PikaDocked   *docked,
                                           gboolean      show);
  gboolean        (* get_show_button_bar) (PikaDocked   *docked);
};


void            pika_docked_title_changed       (PikaDocked   *docked);

void            pika_docked_set_aux_info        (PikaDocked   *docked,
                                                 GList        *aux_info);
GList         * pika_docked_get_aux_info        (PikaDocked   *docked);

GtkWidget     * pika_docked_get_preview         (PikaDocked   *docked,
                                                 PikaContext  *context,
                                                 GtkIconSize   size);
gboolean        pika_docked_get_prefer_icon     (PikaDocked   *docked);
PikaUIManager * pika_docked_get_menu            (PikaDocked   *docked,
                                                 const gchar **ui_path,
                                                 gpointer     *popup_data);
gchar         * pika_docked_get_title           (PikaDocked   *docked);

void            pika_docked_set_context         (PikaDocked   *docked,
                                                 PikaContext  *context);

gboolean        pika_docked_has_button_bar      (PikaDocked   *docked);
void            pika_docked_set_show_button_bar (PikaDocked   *docked,
                                                 gboolean      show);
gboolean        pika_docked_get_show_button_bar (PikaDocked   *docked);


#endif  /* __PIKA_DOCKED_H__ */
