/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikadockwindow.h
 * Copyright (C) 2001-2005 Michael Natterer <mitch@gimp.org>
 * Copyright (C)      2009 Martin Nordholts <martinn@src.gnome.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_DOCK_WINDOW_H__
#define __PIKA_DOCK_WINDOW_H__


#include "widgets/pikawindow.h"


#define PIKA_TYPE_DOCK_WINDOW            (pika_dock_window_get_type ())
#define PIKA_DOCK_WINDOW(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_DOCK_WINDOW, PikaDockWindow))
#define PIKA_DOCK_WINDOW_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_DOCK_WINDOW, PikaDockWindowClass))
#define PIKA_IS_DOCK_WINDOW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_DOCK_WINDOW))
#define PIKA_IS_DOCK_WINDOW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_DOCK_WINDOW))
#define PIKA_DOCK_WINDOW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_DOCK_WINDOW, PikaDockWindowClass))


typedef struct _PikaDockWindowClass    PikaDockWindowClass;
typedef struct _PikaDockWindowPrivate  PikaDockWindowPrivate;

/**
 * PikaDockWindow:
 *
 * A top-level window containing PikaDocks.
 */
struct _PikaDockWindow
{
  PikaWindow  parent_instance;

  PikaDockWindowPrivate *p;
};

struct _PikaDockWindowClass
{
  PikaWindowClass  parent_class;
};


GType               pika_dock_window_get_type               (void) G_GNUC_CONST;
GtkWidget         * pika_dock_window_new                    (const gchar       *role,
                                                             const gchar       *ui_manager_name,
                                                             gboolean           allow_dockbook_absence,
                                                             PikaDialogFactory *factory,
                                                             PikaContext       *context);
gint                pika_dock_window_get_id                 (PikaDockWindow    *dock_window);
void                pika_dock_window_add_dock               (PikaDockWindow    *dock_window,
                                                             PikaDock          *dock,
                                                             gint               index);
void                pika_dock_window_remove_dock            (PikaDockWindow    *dock_window,
                                                             PikaDock          *dock);
PikaContext       * pika_dock_window_get_context            (PikaDockWindow    *dock);
gboolean            pika_dock_window_get_auto_follow_active (PikaDockWindow    *menu_dock);
void                pika_dock_window_set_auto_follow_active (PikaDockWindow    *menu_dock,
                                                             gboolean           show);
gboolean            pika_dock_window_get_show_image_menu    (PikaDockWindow    *menu_dock);
void                pika_dock_window_set_show_image_menu    (PikaDockWindow    *menu_dock,
                                                             gboolean           show);
void                pika_dock_window_setup                  (PikaDockWindow    *dock_window,
                                                             PikaDockWindow    *template);
gboolean            pika_dock_window_has_toolbox            (PikaDockWindow    *dock_window);

PikaDockWindow    * pika_dock_window_from_dock              (PikaDock          *dock);



#endif /* __PIKA_DOCK_WINDOW_H__ */
