/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikafgbgeditor.h
 * Copyright (C) 2004 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_FG_BG_EDITOR_H__
#define __PIKA_FG_BG_EDITOR_H__


typedef enum
{
  PIKA_FG_BG_TARGET_INVALID,
  PIKA_FG_BG_TARGET_FOREGROUND,
  PIKA_FG_BG_TARGET_BACKGROUND,
  PIKA_FG_BG_TARGET_SWAP,
  PIKA_FG_BG_TARGET_DEFAULT
} PikaFgBgTarget;


#define PIKA_TYPE_FG_BG_EDITOR            (pika_fg_bg_editor_get_type ())
#define PIKA_FG_BG_EDITOR(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_FG_BG_EDITOR, PikaFgBgEditor))
#define PIKA_FG_BG_EDITOR_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_FG_BG_EDITOR, PikaFgBgEditorClass))
#define PIKA_IS_FG_BG_EDITOR(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_FG_BG_EDITOR))
#define PIKA_IS_FG_BG_EDITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_FG_BG_EDITOR))
#define PIKA_FG_BG_EDITOR_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_FG_BG_EDITOR, PikaFgBgEditorClass))


typedef struct _PikaFgBgEditorClass PikaFgBgEditorClass;

struct _PikaFgBgEditor
{
  GtkEventBox         parent_instance;

  PikaContext        *context;
  PikaColorConfig    *color_config;
  PikaColorTransform *transform;

  PikaActiveColor     active_color;

  PikaImage          *active_image;

  GdkPixbuf          *default_icon;
  GdkPixbuf          *swap_icon;

  gint                rect_width;
  gint                rect_height;
  gint                click_target;
};

struct _PikaFgBgEditorClass
{
  GtkEventBoxClass    parent_class;

  /*  signals  */

  void (* color_clicked)  (PikaFgBgEditor  *editor,
                           PikaActiveColor  color);
  void (* color_dropped)  (PikaFgBgEditor  *editor,
                           PikaActiveColor  color);
  void (* colors_swapped) (PikaFgBgEditor  *editor);
  void (* colors_default) (PikaFgBgEditor  *editor);

  void (* tooltip)        (PikaFgBgEditor *editor,
                           PikaFgBgTarget  target,
                           GtkTooltip      tooltip);
};


GType       pika_fg_bg_editor_get_type    (void) G_GNUC_CONST;

GtkWidget * pika_fg_bg_editor_new         (PikaContext     *context);

void        pika_fg_bg_editor_set_context (PikaFgBgEditor  *editor,
                                           PikaContext     *context);
void        pika_fg_bg_editor_set_active  (PikaFgBgEditor  *editor,
                                           PikaActiveColor  active);


#endif  /*  __PIKA_FG_BG_EDITOR_H__  */
