/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikafontfactoryview.c
 * Copyright (C) 2018 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "libpikawidgets/pikawidgets.h"

#include "widgets-types.h"

#include "core/pikacontainer.h"
#include "core/pikacontext.h"
#include "core/pikadatafactory.h"

#include "text/pikafont.h"

#include "pikafontfactoryview.h"
#include "pikacontainerview.h"
#include "pikamenufactory.h"
#include "pikaviewrenderer.h"

#include "pika-intl.h"


G_DEFINE_TYPE (PikaFontFactoryView, pika_font_factory_view,
               PIKA_TYPE_DATA_FACTORY_VIEW)

#define parent_class pika_font_factory_view_parent_class


static void
pika_font_factory_view_class_init (PikaFontFactoryViewClass *klass)
{
}

static void
pika_font_factory_view_init (PikaFontFactoryView *view)
{
}

GtkWidget *
pika_font_factory_view_new (PikaViewType     view_type,
                            PikaDataFactory *factory,
                            PikaContext     *context,
                            gint             view_size,
                            gint             view_border_width,
                            PikaMenuFactory *menu_factory)
{
  PikaFontFactoryView *factory_view;
  PikaContainerEditor *editor;

  g_return_val_if_fail (PIKA_IS_DATA_FACTORY (factory), NULL);
  g_return_val_if_fail (PIKA_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (view_size > 0 &&
                        view_size <= PIKA_VIEWABLE_MAX_PREVIEW_SIZE, NULL);
  g_return_val_if_fail (view_border_width >= 0 &&
                        view_border_width <= PIKA_VIEW_MAX_BORDER_WIDTH,
                        NULL);
  g_return_val_if_fail (menu_factory == NULL ||
                        PIKA_IS_MENU_FACTORY (menu_factory), NULL);

  factory_view = g_object_new (PIKA_TYPE_FONT_FACTORY_VIEW,
                               "view-type",         view_type,
                               "data-factory",      factory,
                               "context",           context,
                               "view-size",         view_size,
                               "view-border-width", view_border_width,
                               "menu-factory",      menu_factory,
                               "menu-identifier",   "<Fonts>",
                               "ui-path",           "/fonts-popup",
                               "action-group",      "fonts",
                               NULL);

  editor = PIKA_CONTAINER_EDITOR (factory_view);

  pika_container_editor_bind_to_async_set (editor,
                                           pika_data_factory_get_async_set (factory),
                                           _("Loading fonts (this may take "
                                             "a while...)"));

  return GTK_WIDGET (factory_view);
}
