/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_HISTOGRAM_VIEW_H__
#define __PIKA_HISTOGRAM_VIEW_H__


#define PIKA_TYPE_HISTOGRAM_VIEW            (pika_histogram_view_get_type ())
#define PIKA_HISTOGRAM_VIEW(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_HISTOGRAM_VIEW, PikaHistogramView))
#define PIKA_HISTOGRAM_VIEW_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_HISTOGRAM_VIEW, PikaHistogramViewClass))
#define PIKA_IS_HISTOGRAM_VIEW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_HISTOGRAM_VIEW))
#define PIKA_IS_HISTOGRAM_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_HISTOGRAM_VIEW))
#define PIKA_HISTOGRAM_VIEW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_HISTOGRAM_VIEW, PikaHistogramViewClass))


typedef struct _PikaHistogramViewClass  PikaHistogramViewClass;

struct _PikaHistogramView
{
  GtkDrawingArea         parent_instance;

  PikaHistogram         *histogram;
  PikaHistogram         *bg_histogram;
  PikaHistogramChannel   channel;
  PikaHistogramScale     scale;
  gint                   n_bins;
  gint                   start;
  gint                   end;

  gint                   border_width;
  gint                   subdivisions;

  GdkSeat               *grab_seat;
};

struct _PikaHistogramViewClass
{
  GtkDrawingAreaClass  parent_class;

  void (* range_changed) (PikaHistogramView *view,
                          gint               start,
                          gint               end);
};


GType           pika_histogram_view_get_type       (void) G_GNUC_CONST;

GtkWidget     * pika_histogram_view_new            (gboolean             range);

void            pika_histogram_view_set_histogram  (PikaHistogramView   *view,
                                                    PikaHistogram       *histogram);
PikaHistogram * pika_histogram_view_get_histogram  (PikaHistogramView   *view);

void            pika_histogram_view_set_background (PikaHistogramView   *view,
                                                    PikaHistogram       *histogram);
PikaHistogram * pika_histogram_view_get_background (PikaHistogramView   *view);

void            pika_histogram_view_set_channel    (PikaHistogramView   *view,
                                                    PikaHistogramChannel channel);
PikaHistogramChannel
                pika_histogram_view_get_channel    (PikaHistogramView   *view);

void            pika_histogram_view_set_scale      (PikaHistogramView   *view,
                                                    PikaHistogramScale   scale);
PikaHistogramScale
                pika_histogram_view_get_scale      (PikaHistogramView   *view);

void            pika_histogram_view_set_range      (PikaHistogramView   *view,
                                                    gint                 start,
                                                    gint                 end);
void            pika_histogram_view_get_range      (PikaHistogramView   *view,
                                                    gint                *start,
                                                    gint                *end);


#endif /* __PIKA_HISTOGRAM_VIEW_H__ */
