/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikamenufactory.h
 * Copyright (C) 2003-2004 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_MENU_FACTORY_H__
#define __PIKA_MENU_FACTORY_H__


#include "core/pikaobject.h"


typedef struct _PikaMenuFactoryEntry PikaMenuFactoryEntry;

struct _PikaMenuFactoryEntry
{
  gchar        *identifier;
  GList        *action_groups;
  GList        *managed_uis;

  GHashTable   *managers;
};


#define PIKA_TYPE_MENU_FACTORY            (pika_menu_factory_get_type ())
#define PIKA_MENU_FACTORY(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_MENU_FACTORY, PikaMenuFactory))
#define PIKA_MENU_FACTORY_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_MENU_FACTORY, PikaMenuFactoryClass))
#define PIKA_IS_MENU_FACTORY(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_MENU_FACTORY))
#define PIKA_IS_MENU_FACTORY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_MENU_FACTORY))
#define PIKA_MENU_FACTORY_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_MENU_FACTORY, PikaMenuFactoryClass))


typedef struct _PikaMenuFactoryPrivate  PikaMenuFactoryPrivate;
typedef struct _PikaMenuFactoryClass    PikaMenuFactoryClass;

struct _PikaMenuFactory
{
  PikaObject              parent_instance;

  PikaMenuFactoryPrivate *p;
};

struct _PikaMenuFactoryClass
{
  PikaObjectClass  parent_class;
};


GType             pika_menu_factory_get_type             (void) G_GNUC_CONST;
PikaMenuFactory * pika_menu_factory_new                  (Pika              *pika,
                                                          PikaActionFactory *action_factory);
void              pika_menu_factory_manager_register     (PikaMenuFactory   *factory,
                                                          const gchar       *identifier,
                                                          const gchar       *first_group,
                                                          ...)  G_GNUC_NULL_TERMINATED;
GList           * pika_menu_factory_get_registered_menus (PikaMenuFactory   *factory);
PikaUIManager   * pika_menu_factory_get_manager          (PikaMenuFactory   *factory,
                                                          const gchar       *identifier,
                                                          gpointer           callback_data);
void              pika_menu_factory_delete_manager       (PikaMenuFactory   *factory,
                                                          const gchar       *identifier,
                                                          gpointer           callback_data);


#endif  /*  __PIKA_MENU_FACTORY_H__  */
