/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * PikaNavigationView Widget
 * Copyright (C) 2001-2002 Michael Natterer <mitch@gimp.org>
 *
 * partly based on app/nav_window
 * Copyright (C) 1999 Andy Thomas <alt@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_NAVIGATION_VIEW_H__
#define __PIKA_NAVIGATION_VIEW_H__

#include "pikaview.h"


#define PIKA_TYPE_NAVIGATION_VIEW            (pika_navigation_view_get_type ())
#define PIKA_NAVIGATION_VIEW(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_NAVIGATION_VIEW, PikaNavigationView))
#define PIKA_NAVIGATION_VIEW_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_NAVIGATION_VIEW, PikaNavigationViewClass))
#define PIKA_IS_NAVIGATION_VIEW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE (obj, PIKA_TYPE_NAVIGATION_VIEW))
#define PIKA_IS_NAVIGATION_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_NAVIGATION_VIEW))
#define PIKA_NAVIGATION_VIEW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_NAVIGATION_VIEW, PikaNavigationViewClass))


typedef struct _PikaNavigationViewClass  PikaNavigationViewClass;

struct _PikaNavigationViewClass
{
  PikaViewClass  parent_class;

  void (* marker_changed) (PikaNavigationView *view,
                           gdouble             center_x,
                           gdouble             center_y,
                           gdouble             width,
                           gdouble             height);
  void (* zoom)           (PikaNavigationView *view,
                           PikaZoomType        direction,
                           gdouble             delta);
  void (* scroll)         (PikaNavigationView *view,
                           GdkEventScroll     *sevent);
};


GType   pika_navigation_view_get_type     (void) G_GNUC_CONST;

void    pika_navigation_view_set_marker   (PikaNavigationView *view,
                                           gdouble             center_x,
                                           gdouble             center_y,
                                           gdouble             width,
                                           gdouble             height,
                                           gboolean            flip_horizontally,
                                           gboolean            flip_vertically,
                                           gdouble             rotate_angle);
void    pika_navigation_view_set_canvas   (PikaNavigationView *view,
                                           gboolean            visible,
                                           gdouble             x,
                                           gdouble             y,
                                           gdouble             width,
                                           gdouble             height);
void    pika_navigation_view_set_motion_offset
                                          (PikaNavigationView *view,
                                           gint                motion_offset_x,
                                           gint                motion_offset_y);
void    pika_navigation_view_get_local_marker
                                          (PikaNavigationView *view,
                                           gint               *center_x,
                                           gint               *center_y,
                                           gint               *width,
                                           gint               *height);
void    pika_navigation_view_grab_pointer (PikaNavigationView *view,
                                           GdkEvent           *event);


#endif /* __PIKA_NAVIGATION_VIEW_H__ */
