/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikaoverlaydialog.h
 * Copyright (C) 2009-2010  Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_OVERLAY_DIALOG_H__
#define __PIKA_OVERLAY_DIALOG_H__


#include "pikaoverlayframe.h"


#define PIKA_RESPONSE_DETACH 100


#define PIKA_TYPE_OVERLAY_DIALOG            (pika_overlay_dialog_get_type ())
#define PIKA_OVERLAY_DIALOG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_OVERLAY_DIALOG, PikaOverlayDialog))
#define PIKA_OVERLAY_DIALOG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_OVERLAY_DIALOG, PikaOverlayDialogClass))
#define PIKA_IS_OVERLAY_DIALOG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_OVERLAY_DIALOG))
#define PIKA_IS_OVERLAY_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_OVERLAY_DIALOG))
#define PIKA_OVERLAY_DIALOG_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_OVERLAY_DIALOG, PikaOverlayDialogClass))


typedef struct _PikaOverlayDialog      PikaOverlayDialog;
typedef struct _PikaOverlayDialogClass PikaOverlayDialogClass;

struct _PikaOverlayDialog
{
  PikaOverlayFrame  parent_instance;

  gchar            *title;
  gchar            *icon_name;

  GtkWidget        *header;
  GtkWidget        *icon_image;
  GtkWidget        *title_label;
  GtkWidget        *detach_button;
  GtkWidget        *close_button;
  GtkWidget        *action_area;
};

struct _PikaOverlayDialogClass
{
  PikaOverlayFrameClass  parent_class;

  void (* response) (PikaOverlayDialog *overlay,
                     gint               response_id);

  void (* detach)   (PikaOverlayDialog *overlay);
  void (* close)    (PikaOverlayDialog *overlay);
};


GType       pika_overlay_dialog_get_type               (void) G_GNUC_CONST;

GtkWidget * pika_overlay_dialog_new                    (PikaToolInfo      *tool_info,
                                                        const gchar       *desc,
                                                        ...) G_GNUC_NULL_TERMINATED;

void        pika_overlay_dialog_response               (PikaOverlayDialog *overlay,
                                                        gint               response_id);
void        pika_overlay_dialog_add_buttons_valist     (PikaOverlayDialog *overlay,
                                                        va_list            args);
GtkWidget * pika_overlay_dialog_add_button             (PikaOverlayDialog *overlay,
                                                        const gchar       *button_text,
                                                        gint               response_id);
void        pika_overlay_dialog_set_alternative_button_order
                                                       (PikaOverlayDialog *overlay,
                                                        gint               n_ids,
                                                        gint              *ids);
void        pika_overlay_dialog_set_default_response   (PikaOverlayDialog *overlay,
                                                        gint               response_id);
void        pika_overlay_dialog_set_response_sensitive (PikaOverlayDialog *overlay,
                                                        gint               response_id,
                                                        gboolean           sensitive);


#endif /* __PIKA_OVERLAY_DIALOG_H__ */
