/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_PALETTE_EDITOR_H__
#define __PIKA_PALETTE_EDITOR_H__


#include "pikadataeditor.h"


#define PIKA_TYPE_PALETTE_EDITOR            (pika_palette_editor_get_type ())
#define PIKA_PALETTE_EDITOR(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_PALETTE_EDITOR, PikaPaletteEditor))
#define PIKA_PALETTE_EDITOR_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_PALETTE_EDITOR, PikaPaletteEditorClass))
#define PIKA_IS_PALETTE_EDITOR(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_PALETTE_EDITOR))
#define PIKA_IS_PALETTE_EDITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_PALETTE_EDITOR))
#define PIKA_PALETTE_EDITOR_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_PALETTE_EDITOR, PikaPaletteEditorClass))


typedef struct _PikaPaletteEditorClass PikaPaletteEditorClass;

struct _PikaPaletteEditor
{
  PikaDataEditor    parent_instance;

  GtkWidget        *view;

  GtkWidget        *index_label;
  GtkWidget        *color_name;
  GtkAdjustment    *columns_adj;

  GtkWidget        *color_dialog;

  PikaPaletteEntry *color;

  gdouble           zoom_factor;  /* range from 0.1 to 4.0 */
  gint              col_width;
  gint              last_width;
  gint              columns;
};

struct _PikaPaletteEditorClass
{
  PikaDataEditorClass  parent_class;
};


GType       pika_palette_editor_get_type   (void) G_GNUC_CONST;

GtkWidget * pika_palette_editor_new        (PikaContext        *context,
                                            PikaMenuFactory    *menu_factory);

void        pika_palette_editor_edit_color (PikaPaletteEditor  *editor);
void        pika_palette_editor_pick_color (PikaPaletteEditor  *editor,
                                            const PikaRGB      *color,
                                            PikaColorPickState  pick_state);
void        pika_palette_editor_zoom       (PikaPaletteEditor  *editor,
                                            PikaZoomType        zoom_type);

gint        pika_palette_editor_get_index  (PikaPaletteEditor *editor,
                                            const PikaRGB     *search);
gboolean    pika_palette_editor_set_index  (PikaPaletteEditor *editor,
                                            gint               index,
                                            PikaRGB           *color);

gint        pika_palette_editor_max_index  (PikaPaletteEditor *editor);


#endif /* __PIKA_PALETTE_EDITOR_H__ */
