/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikapaletteselect.c
 * Copyright (C) 2004 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "libpikabase/pikabase.h"
#include "libpikawidgets/pikawidgets.h"

#include "widgets-types.h"

#include "core/pika.h"
#include "core/pikacontext.h"
#include "core/pikapalette.h"
#include "core/pikaparamspecs.h"

#include "pdb/pikapdb.h"

#include "pikacontainerbox.h"
#include "pikadatafactoryview.h"
#include "pikapaletteselect.h"


static void             pika_palette_select_constructed  (GObject        *object);

static PikaValueArray * pika_palette_select_run_callback (PikaPdbDialog  *dialog,
                                                          PikaObject     *object,
                                                          gboolean        closing,
                                                          GError        **error);


G_DEFINE_TYPE (PikaPaletteSelect, pika_palette_select, PIKA_TYPE_PDB_DIALOG)

#define parent_class pika_palette_select_parent_class


static void
pika_palette_select_class_init (PikaPaletteSelectClass *klass)
{
  GObjectClass       *object_class = G_OBJECT_CLASS (klass);
  PikaPdbDialogClass *pdb_class    = PIKA_PDB_DIALOG_CLASS (klass);

  object_class->constructed = pika_palette_select_constructed;

  pdb_class->run_callback   = pika_palette_select_run_callback;
}

static void
pika_palette_select_init (PikaPaletteSelect *dialog)
{
}

static void
pika_palette_select_constructed (GObject *object)
{
  PikaPdbDialog *dialog = PIKA_PDB_DIALOG (object);
  GtkWidget     *content_area;

  G_OBJECT_CLASS (parent_class)->constructed (object);

  dialog->view =
    pika_data_factory_view_new (PIKA_VIEW_TYPE_LIST,
                                dialog->context->pika->palette_factory,
                                dialog->context,
                                PIKA_VIEW_SIZE_MEDIUM, 1,
                                dialog->menu_factory, "<Palettes>",
                                "/palettes-popup",
                                "palettes");

  pika_container_box_set_size_request (PIKA_CONTAINER_BOX (PIKA_CONTAINER_EDITOR (dialog->view)->view),
                                       5 * (PIKA_VIEW_SIZE_MEDIUM + 2),
                                       8 * (PIKA_VIEW_SIZE_MEDIUM + 2));

  gtk_container_set_border_width (GTK_CONTAINER (dialog->view), 12);

  content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
  gtk_box_pack_start (GTK_BOX (content_area), dialog->view, TRUE, TRUE, 0);
  gtk_widget_show (dialog->view);
}

static PikaValueArray *
pika_palette_select_run_callback (PikaPdbDialog  *dialog,
                                  PikaObject     *object,
                                  gboolean        closing,
                                  GError        **error)
{
  PikaPalette *palette = PIKA_PALETTE (object);

  return pika_pdb_execute_procedure_by_name (dialog->pdb,
                                             dialog->caller_context,
                                             NULL, error,
                                             dialog->callback_name,
                                             G_TYPE_STRING,  pika_object_get_name (object),
                                             G_TYPE_INT,     pika_palette_get_n_colors (palette),
                                             G_TYPE_BOOLEAN, closing,
                                             G_TYPE_NONE);
}
