/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikapdbdialog.h
 * Copyright (C) 2004 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_PDB_DIALOG_H__
#define __PIKA_PDB_DIALOG_H__

G_BEGIN_DECLS


#define PIKA_TYPE_PDB_DIALOG            (pika_pdb_dialog_get_type ())
#define PIKA_PDB_DIALOG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_PDB_DIALOG, PikaPdbDialog))
#define PIKA_PDB_DIALOG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_PDB_DIALOG, PikaPdbDialogClass))
#define PIKA_IS_PDB_DIALOG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_PDB_DIALOG))
#define PIKA_IS_PDB_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_PDB_DIALOG))
#define PIKA_PDB_DIALOG_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_PDB_DIALOG, PikaPdbDialogClass))


typedef struct _PikaPdbDialogClass  PikaPdbDialogClass;

struct _PikaPdbDialog
{
  PikaDialog       parent_instance;

  PikaPDB         *pdb;

  /*  The context we were created with. This is the context the plug-in
   *  exists in and must be used when calling the plug-in.
   */
  PikaContext     *caller_context;

  /*  The dialog's private context, serves just as model for the
   *  select widgets and must not be used when calling the plug-in.
   */
  PikaContext     *context;

  GType            select_type;
  PikaObject      *initial_object;
  gchar           *callback_name;
  gboolean         callback_busy;

  PikaMenuFactory *menu_factory;
  GtkWidget       *view;
};

struct _PikaPdbDialogClass
{
  PikaDialogClass  parent_class;

  GList           *dialogs;

  PikaValueArray * (* run_callback) (PikaPdbDialog  *dialog,
                                     PikaObject     *object,
                                     gboolean        closing,
                                     GError        **error);
};


GType           pika_pdb_dialog_get_type        (void) G_GNUC_CONST;

void            pika_pdb_dialog_run_callback    (PikaPdbDialog     **dialog,
                                                 gboolean            closing);

PikaPdbDialog * pika_pdb_dialog_get_by_callback (PikaPdbDialogClass *klass,
                                                 const gchar        *callback_name);


G_END_DECLS

#endif /* __PIKA_PDB_DIALOG_H__ */
