/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikasearchpopup.h
 * Copyright (C) 2015 Jehan <jehan at girinstud.io>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_SEARCH_POPUP_H__
#define __PIKA_SEARCH_POPUP_H__

#include "pikapopup.h"

#define PIKA_TYPE_SEARCH_POPUP            (pika_search_popup_get_type ())
#define PIKA_SEARCH_POPUP(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_SEARCH_POPUP, PikaSearchPopup))
#define PIKA_SEARCH_POPUP_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_SEARCH_POPUP, PikaSearchPopupClass))
#define PIKA_IS_SEARCH_POPUP(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_SEARCH_POPUP))
#define PIKA_IS_SEARCH_POPUP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_SEARCH_POPUP))
#define PIKA_SEARCH_POPUP_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_SEARCH_POPUP, PikaSearchPopupClass))

/**
 * PikaSearchPopupCallback:
 * @popup:  the #PikaSearchPopup to operate on.
 * @search: the text searched.
 *
 * Callback used by @popup to fill in its result list.
 * It should make use of pika_search_popup_add_result() to fill in
 * results.
 */
typedef struct _PikaSearchPopup           PikaSearchPopup;
typedef struct _PikaSearchPopupClass      PikaSearchPopupClass;
typedef struct _PikaSearchPopupPrivate    PikaSearchPopupPrivate;

typedef void   (*PikaSearchPopupCallback) (PikaSearchPopup  *popup,
                                           const gchar      *search,
                                           gpointer          data);

struct _PikaSearchPopup
{
  PikaPopup               parent_instance;

  PikaSearchPopupPrivate *priv;
};

struct _PikaSearchPopupClass
{
  PikaPopupClass          parent_class;
};

GType       pika_search_popup_get_type   (void);

GtkWidget * pika_search_popup_new        (Pika                    *pika,
                                          const gchar             *role,
                                          const gchar             *title,
                                          PikaSearchPopupCallback  callback,
                                          gpointer                 callback_data);

void        pika_search_popup_add_result (PikaSearchPopup *popup,
                                          PikaAction      *action,
                                          gint             section);

#endif  /*  __PIKA_SEARCH_POPUP_H__  */
