/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_SELECTION_DATA_H__
#define __PIKA_SELECTION_DATA_H__


/*  uri list  */

void            pika_selection_data_set_uri_list  (GtkSelectionData *selection,
                                                   GList            *uris);
GList         * pika_selection_data_get_uri_list  (GtkSelectionData *selection);


/*  color  */

void            pika_selection_data_set_color     (GtkSelectionData *selection,
                                                   const PikaRGB    *color);
gboolean        pika_selection_data_get_color     (GtkSelectionData *selection,
                                                   PikaRGB          *color);


/*  image (xcf)  */

void            pika_selection_data_set_xcf       (GtkSelectionData *selection,
                                                   PikaImage        *image);
PikaImage     * pika_selection_data_get_xcf       (GtkSelectionData *selection,
                                                   Pika             *pika);


/*  stream (svg/png)  */

void            pika_selection_data_set_stream    (GtkSelectionData *selection,
                                                   const guchar     *stream,
                                                   gsize             stream_length);
const guchar  * pika_selection_data_get_stream    (GtkSelectionData *selection,
                                                   gsize            *stream_length);


/*  curve  */

void            pika_selection_data_set_curve     (GtkSelectionData *selection,
                                                   PikaCurve        *curve);
PikaCurve     * pika_selection_data_get_curve     (GtkSelectionData *selection);


/*  image  */

void            pika_selection_data_set_image     (GtkSelectionData *selection,
                                                   PikaImage        *image);
PikaImage     * pika_selection_data_get_image     (GtkSelectionData *selection,
                                                   Pika             *pika);


/*  component  */

void            pika_selection_data_set_component (GtkSelectionData *selection,
                                                   PikaImage        *image,
                                                   PikaChannelType   channel);
PikaImage     * pika_selection_data_get_component (GtkSelectionData *selection,
                                                   Pika             *pika,
                                                   PikaChannelType  *channel);


/*  item  */

void            pika_selection_data_set_item      (GtkSelectionData *selection,
                                                   PikaItem         *item);
PikaItem      * pika_selection_data_get_item      (GtkSelectionData *selection,
                                                   Pika             *pika);


/*  item list  */

void            pika_selection_data_set_item_list (GtkSelectionData *selection,
                                                   GList            *items);
GList         * pika_selection_data_get_item_list (GtkSelectionData *selection,
                                                   Pika             *pika);


/*  various data  */

void            pika_selection_data_set_object    (GtkSelectionData *selection,
                                                   PikaObject       *object);

PikaBrush     * pika_selection_data_get_brush     (GtkSelectionData *selection,
                                                   Pika             *pika);
PikaPattern   * pika_selection_data_get_pattern   (GtkSelectionData *selection,
                                                   Pika             *pika);
PikaGradient  * pika_selection_data_get_gradient  (GtkSelectionData *selection,
                                                   Pika             *pika);
PikaPalette   * pika_selection_data_get_palette   (GtkSelectionData *selection,
                                                   Pika             *pika);
PikaFont      * pika_selection_data_get_font      (GtkSelectionData *selection,
                                                   Pika             *pika);
PikaBuffer    * pika_selection_data_get_buffer    (GtkSelectionData *selection,
                                                   Pika             *pika);
PikaImagefile * pika_selection_data_get_imagefile (GtkSelectionData *selection,
                                                   Pika             *pika);
PikaTemplate  * pika_selection_data_get_template  (GtkSelectionData *selection,
                                                   Pika             *pika);
PikaToolItem  * pika_selection_data_get_tool_item (GtkSelectionData *selection,
                                                   Pika             *pika);


#endif /* __PIKA_SELECTION_DATA_H__ */
