/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikasessioninfo-dock.h
 * Copyright (C) 2001-2007 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_SESSION_INFO_DOCK_H__
#define __PIKA_SESSION_INFO_DOCK_H__


/**
 * PikaSessionInfoDock:
 *
 * Contains information about a dock in the interface.
 */
struct _PikaSessionInfoDock
{
  /* Type of dock, written to/read from sessionrc. E.g. 'pika-dock' or
   * 'pika-toolbox'
   */
  gchar             *dock_type;

  /* What side this dock is in in single-window mode. Either
   * PIKA_ALIGN_LEFT, PIKA_ALIGN_RIGHT or -1.
   */
  PikaAlignmentType  side;

  /* GtkPaned position of this dock */
  gint               position;

  /*  list of PikaSessionInfoBook  */
  GList             *books;
};

PikaSessionInfoDock * pika_session_info_dock_new         (const gchar          *dock_type);
void                  pika_session_info_dock_free        (PikaSessionInfoDock  *dock_info);
void                  pika_session_info_dock_serialize   (PikaConfigWriter     *writer,
                                                          PikaSessionInfoDock  *dock);
GTokenType            pika_session_info_dock_deserialize (GScanner             *scanner,
                                                          gint                  scope,
                                                          PikaSessionInfoDock **info,
                                                          const gchar          *dock_type);
PikaSessionInfoDock * pika_session_info_dock_from_widget (PikaDock             *dock);
PikaDock            * pika_session_info_dock_restore     (PikaSessionInfoDock  *dock_info,
                                                          PikaDialogFactory    *factory,
                                                          GdkMonitor           *monitor,
                                                          PikaDockContainer    *dock_container);


#endif  /* __PIKA_SESSION_INFO_DOCK_H__ */
