/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikasessioninfo.h
 * Copyright (C) 2001-2008 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_SESSION_INFO_H__
#define __PIKA_SESSION_INFO_H__


#include "core/pikaobject.h"


#define PIKA_TYPE_SESSION_INFO            (pika_session_info_get_type ())
#define PIKA_SESSION_INFO(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_SESSION_INFO, PikaSessionInfo))
#define PIKA_SESSION_INFO_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_SESSION_INFO, PikaSessionInfoClass))
#define PIKA_IS_SESSION_INFO(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_SESSION_INFO))
#define PIKA_IS_SESSION_INFO_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_SESSION_INFO))
#define PIKA_SESSION_INFO_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_SESSION_INFO, PikaSessionInfoClass))


typedef struct _PikaSessionInfoPrivate  PikaSessionInfoPrivate;
typedef struct _PikaSessionInfoClass    PikaSessionInfoClass;

/**
 * PikaSessionInfo:
 *
 * Contains session info for one toplevel window in the interface such
 * as a dock, the empty-image-window, or the open/save dialog.
 */
struct _PikaSessionInfo
{
  PikaObject  parent_instance;

  PikaSessionInfoPrivate *p;
};

struct _PikaSessionInfoClass
{
  PikaObjectClass  parent_class;
};


GType             pika_session_info_get_type                (void) G_GNUC_CONST;

PikaSessionInfo * pika_session_info_new                     (void);

void              pika_session_info_restore                 (PikaSessionInfo        *info,
                                                             PikaDialogFactory      *factory,
                                                             GdkMonitor             *monitor);
void              pika_session_info_apply_geometry          (PikaSessionInfo        *info,
                                                             GdkMonitor             *current_monitor,
                                                             gboolean                apply_stored_monitor);
void              pika_session_info_read_geometry           (PikaSessionInfo        *info,
                                                             GdkEventConfigure      *cevent);
void              pika_session_info_get_info                (PikaSessionInfo        *info);
void              pika_session_info_get_info_with_widget    (PikaSessionInfo        *info,
                                                             GtkWidget              *widget);
void              pika_session_info_clear_info              (PikaSessionInfo        *info);
gboolean          pika_session_info_is_singleton            (PikaSessionInfo        *info);
gboolean          pika_session_info_is_session_managed      (PikaSessionInfo        *info);
gboolean          pika_session_info_get_remember_size       (PikaSessionInfo        *info);
gboolean          pika_session_info_get_remember_if_open    (PikaSessionInfo        *info);
GtkWidget       * pika_session_info_get_widget              (PikaSessionInfo        *info);
void              pika_session_info_set_widget              (PikaSessionInfo        *info,
                                                             GtkWidget              *widget);
PikaDialogFactoryEntry *
                  pika_session_info_get_factory_entry       (PikaSessionInfo        *info);
void              pika_session_info_set_factory_entry       (PikaSessionInfo        *info,
                                                             PikaDialogFactoryEntry *entry);
gboolean          pika_session_info_get_open                (PikaSessionInfo        *info);
void              pika_session_info_append_book             (PikaSessionInfo        *info,
                                                             PikaSessionInfoBook    *book);
gint              pika_session_info_get_x                   (PikaSessionInfo        *info);
gint              pika_session_info_get_y                   (PikaSessionInfo        *info);
gint              pika_session_info_get_width               (PikaSessionInfo        *info);
gint              pika_session_info_get_height              (PikaSessionInfo        *info);

void              pika_session_info_set_position_accuracy   (gint                    accuracy);
gint              pika_session_info_apply_position_accuracy (gint                    position);


#endif  /* __PIKA_SESSION_INFO_H__ */
