/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikasessionmanaged.c
 * Copyright (C) 2011 Martin Nordholts <martinn@src.gnome.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gtk/gtk.h>

#include "widgets-types.h"

#include "pikasessionmanaged.h"


G_DEFINE_INTERFACE (PikaSessionManaged, pika_session_managed, GTK_TYPE_WIDGET)


/*  private functions  */


static void
pika_session_managed_default_init (PikaSessionManagedInterface *iface)
{
}


/*  public functions  */


/**
 * pika_session_managed_get_aux_info:
 * @session_managed: A #PikaSessionManaged
 *
 * Returns: A list of #PikaSessionInfoAux created with
 *          pika_session_info_aux_new().
 **/
GList *
pika_session_managed_get_aux_info (PikaSessionManaged *session_managed)
{
  PikaSessionManagedInterface *iface;

  g_return_val_if_fail (PIKA_IS_SESSION_MANAGED (session_managed), NULL);

  iface = PIKA_SESSION_MANAGED_GET_IFACE (session_managed);

  if (iface->get_aux_info)
    return iface->get_aux_info (session_managed);

  return NULL;
}

/**
 * pika_session_managed_get_ui_manager:
 * @session_managed: A #PikaSessionManaged
 * @aux_info         A list of #PikaSessionInfoAux
 *
 * Sets aux data previously returned from
 * pika_session_managed_get_aux_info().
 **/
void
pika_session_managed_set_aux_info (PikaSessionManaged *session_managed,
                                   GList              *aux_info)
{
  PikaSessionManagedInterface *iface;

  g_return_if_fail (PIKA_IS_SESSION_MANAGED (session_managed));

  iface = PIKA_SESSION_MANAGED_GET_IFACE (session_managed);

  if (iface->set_aux_info)
    iface->set_aux_info (session_managed, aux_info);
}
