/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikatagentry.h
 * Copyright (C) 2008 Aurimas Juška <aurisj@svn.gnome.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TAG_ENTRY_H__
#define __PIKA_TAG_ENTRY_H__


#define PIKA_TYPE_TAG_ENTRY            (pika_tag_entry_get_type ())
#define PIKA_TAG_ENTRY(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TAG_ENTRY, PikaTagEntry))
#define PIKA_TAG_ENTRY_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_TAG_ENTRY, PikaTagEntryClass))
#define PIKA_IS_TAG_ENTRY(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TAG_ENTRY))
#define PIKA_IS_TAG_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_TAG_ENTRY))
#define PIKA_TAG_ENTRY_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_TAG_ENTRY, PikaTagEntryClass))


typedef struct _PikaTagEntryClass  PikaTagEntryClass;

struct _PikaTagEntry
{
  GtkEntry             parent_instance;

  PikaTaggedContainer *container;

  /* mask describes the meaning of each char in PikaTagEntry.
   * It is maintained automatically on insert-text and delete-text
   * events. If manual mask modification is desired, then
   * suppress_mask_update must be increased before calling any
   * function changing entry contents.
   * Meaning of mask chars:
   * u - undefined / unknown (just typed unparsed text)
   * t - tag
   * s - separator
   * w - whitespace.
   */
  GString             *mask;
  GList               *selected_items;
  GList               *common_tags;
  GList               *recent_list;
  gint                 tab_completion_index;
  gint                 internal_operation;
  gint                 suppress_mask_update;
  gint                 suppress_tag_query;
  PikaTagEntryMode     mode;
  gboolean             description_shown;
  gboolean             has_invalid_tags;
  guint                tag_query_idle_id;
};

struct _PikaTagEntryClass
{
  GtkEntryClass   parent_class;
};


GType          pika_tag_entry_get_type           (void) G_GNUC_CONST;

GtkWidget    * pika_tag_entry_new                (PikaTaggedContainer *container,
                                                  PikaTagEntryMode     mode);

void           pika_tag_entry_set_selected_items (PikaTagEntry        *entry,
                                                  GList               *items);
gchar       ** pika_tag_entry_parse_tags         (PikaTagEntry        *entry);
void           pika_tag_entry_set_tag_string     (PikaTagEntry        *entry,
                                                  const gchar         *tag_string);

const gchar  * pika_tag_entry_get_separator      (void);


#endif  /*  __PIKA_TAG_ENTRY_H__  */
