/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * PikaTextEditor
 * Copyright (C) 2002-2003  Sven Neumann <sven@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TEXT_EDITOR_H__
#define __PIKA_TEXT_EDITOR_H__


#define PIKA_TYPE_TEXT_EDITOR    (pika_text_editor_get_type ())
#define PIKA_TEXT_EDITOR(obj)    (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TEXT_EDITOR, PikaTextEditor))
#define PIKA_IS_TEXT_EDITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TEXT_EDITOR))


typedef struct _PikaTextEditorClass  PikaTextEditorClass;

struct _PikaTextEditor
{
  PikaDialog         parent_instance;

  /*<  private  >*/
  PikaTextDirection  base_dir;
  gchar             *font_name;

  GtkWidget         *view;
  GtkWidget         *font_toggle;
  GtkWidget         *file_dialog;
  PikaUIManager     *ui_manager;
};

struct _PikaTextEditorClass
{
  PikaDialogClass   parent_class;

  void (* text_changed) (PikaTextEditor *editor);
  void (* dir_changed)  (PikaTextEditor *editor);
};


GType               pika_text_editor_get_type      (void) G_GNUC_CONST;
GtkWidget         * pika_text_editor_new           (const gchar       *title,
                                                    GtkWindow         *parent,
                                                    Pika              *pika,
                                                    PikaMenuFactory   *menu_factory,
                                                    PikaText          *text,
                                                    PikaTextBuffer    *text_buffer,
                                                    gdouble            xres,
                                                    gdouble            yres);

void                pika_text_editor_set_text      (PikaTextEditor    *editor,
                                                    const gchar       *text,
                                                    gint               len);
gchar             * pika_text_editor_get_text      (PikaTextEditor    *editor);

void                pika_text_editor_set_direction (PikaTextEditor    *editor,
                                                    PikaTextDirection  base_dir);
PikaTextDirection   pika_text_editor_get_direction (PikaTextEditor    *editor);

void                pika_text_editor_set_font_name (PikaTextEditor    *editor,
                                                    const gchar       *font_name);
const gchar       * pika_text_editor_get_font_name (PikaTextEditor    *editor);


#endif  /* __PIKA_TEXT_EDITOR_H__ */
