/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikatoggleaction.h
 * Copyright (C) 2004 Michael Natterer <mitch@gimp.org>
 * Copyright (C) 2008 Sven Neumann <sven@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_TOGGLE_ACTION_H__
#define __PIKA_TOGGLE_ACTION_H__


#include "pikaactionimpl.h"


#define PIKA_TYPE_TOGGLE_ACTION            (pika_toggle_action_get_type ())
#define PIKA_TOGGLE_ACTION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_TOGGLE_ACTION, PikaToggleAction))
#define PIKA_TOGGLE_ACTION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_TOGGLE_ACTION, PikaToggleActionClass))
#define PIKA_IS_TOGGLE_ACTION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_TOGGLE_ACTION))
#define PIKA_IS_TOGGLE_ACTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((obj), PIKA_TYPE_TOGGLE_ACTION))
#define PIKA_TOGGLE_ACTION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), PIKA_TYPE_TOGGLE_ACTION, PikaToggleActionClass))


typedef struct _PikaToggleActionClass   PikaToggleActionClass;
typedef struct _PikaToggleActionPrivate PikaToggleActionPrivate;

struct _PikaToggleAction
{
  PikaActionImpl           parent_instance;

  PikaToggleActionPrivate *priv;
};

struct _PikaToggleActionClass
{
  PikaActionImplClass  parent_class;

  /* Signals */

  void     (* toggled) (PikaToggleAction *action);

  /* Private methods */

  gboolean (* toggle)  (PikaToggleAction *action);
};


GType             pika_toggle_action_get_type  (void) G_GNUC_CONST;

PikaAction      * pika_toggle_action_new       (const gchar *name,
                                                const gchar *label,
                                                const gchar *short_label,
                                                const gchar *tooltip,
                                                const gchar *icon_name,
                                                const gchar *help_id,
                                                PikaContext *context);

void              pika_toggle_action_set_active (PikaToggleAction *action,
                                                 gboolean          active);
gboolean          pika_toggle_action_get_active (PikaToggleAction *action);


/* Protected functions */

void             _pika_toggle_action_set_active (PikaToggleAction *action,
                                                 gboolean          active);


#endif  /* __PIKA_TOGGLE_ACTION_H__ */
