/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikatoolpresetfactoryview.c
 * Copyright (C) 2010 Alexia Death
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "libpikawidgets/pikawidgets.h"

#include "widgets-types.h"

#include "core/pikacontext.h"
#include "core/pikadatafactory.h"
#include "core/pikaviewable.h"

#include "pikaeditor.h"
#include "pikamenufactory.h"
#include "pikatoolpresetfactoryview.h"
#include "pikaviewrenderer.h"


G_DEFINE_TYPE (PikaToolPresetFactoryView, pika_tool_preset_factory_view,
               PIKA_TYPE_DATA_FACTORY_VIEW)


static void
pika_tool_preset_factory_view_class_init (PikaToolPresetFactoryViewClass *klass)
{
}

static void
pika_tool_preset_factory_view_init (PikaToolPresetFactoryView *view)
{
}

GtkWidget *
pika_tool_preset_factory_view_new (PikaViewType      view_type,
                                   PikaDataFactory  *factory,
                                   PikaContext      *context,
                                   gint              view_size,
                                   gint              view_border_width,
                                   PikaMenuFactory  *menu_factory)
{
  PikaToolPresetFactoryView *factory_view;
  PikaEditor                *editor;
  GtkWidget                 *button;

  g_return_val_if_fail (PIKA_IS_DATA_FACTORY (factory), NULL);
  g_return_val_if_fail (PIKA_IS_CONTEXT (context), NULL);
  g_return_val_if_fail (view_size > 0 &&
                        view_size <= PIKA_VIEWABLE_MAX_PREVIEW_SIZE, NULL);
  g_return_val_if_fail (view_border_width >= 0 &&
                        view_border_width <= PIKA_VIEW_MAX_BORDER_WIDTH,
                        NULL);
  g_return_val_if_fail (menu_factory == NULL ||
                        PIKA_IS_MENU_FACTORY (menu_factory), NULL);

  factory_view = g_object_new (PIKA_TYPE_TOOL_PRESET_FACTORY_VIEW,
                               "view-type",         view_type,
                               "data-factory",      factory,
                               "context",           context,
                               "view-size",         view_size,
                               "view-border-width", view_border_width,
                               "menu-factory",      menu_factory,
                               "menu-identifier",   "<ToolPresets>",
                               "ui-path",           "/tool-presets-popup",
                               "action-group",      "tool-presets",
                               NULL);

  gtk_widget_hide (pika_data_factory_view_get_duplicate_button (PIKA_DATA_FACTORY_VIEW (factory_view)));

  editor = PIKA_EDITOR (PIKA_CONTAINER_EDITOR (factory_view)->view);

  button = pika_editor_add_action_button (editor, "tool-presets",
                                          "tool-presets-save", NULL);
  gtk_box_reorder_child (pika_editor_get_button_box (editor),
                         button, 2);

  button = pika_editor_add_action_button (editor, "tool-presets",
                                          "tool-presets-restore", NULL);
  gtk_box_reorder_child (pika_editor_get_button_box (editor),
                         button, 3);

  return GTK_WIDGET (factory_view);
}
