/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikauimanager.h
 * Copyright (C) 2004 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_UI_MANAGER_H__
#define __PIKA_UI_MANAGER_H__


#include "core/pikaobject.h"


typedef void (* PikaUIMenuCallback)  (PikaUIManager *manager,
                                      const gchar   *path,
                                      const gchar   *action_name,
                                      gboolean       top,
                                      gpointer       user_data);


typedef struct _PikaUIManagerUIEntry PikaUIManagerUIEntry;

struct _PikaUIManagerUIEntry
{
  gchar                  *ui_path;
  gchar                  *basename;
  PikaUIManagerSetupFunc  setup_func;
  gboolean                setup_done;
  GtkBuilder             *builder;
};


#define PIKA_TYPE_UI_MANAGER              (pika_ui_manager_get_type ())
#define PIKA_UI_MANAGER(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_UI_MANAGER, PikaUIManager))
#define PIKA_UI_MANAGER_CLASS(vtable)     (G_TYPE_CHECK_CLASS_CAST ((vtable), PIKA_TYPE_UI_MANAGER, PikaUIManagerClass))
#define PIKA_IS_UI_MANAGER(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_UI_MANAGER))
#define PIKA_IS_UI_MANAGER_CLASS(vtable)  (G_TYPE_CHECK_CLASS_TYPE ((vtable), PIKA_TYPE_UI_MANAGER))
#define PIKA_UI_MANAGER_GET_CLASS(inst)   (G_TYPE_INSTANCE_GET_CLASS ((inst), PIKA_TYPE_UI_MANAGER, PikaUIManagerClass))


typedef struct _PikaUIManagerClass PikaUIManagerClass;

/**
 * Among other things, is responsible for updating menu bars. A more
 * appropriate name would perhaps be PikaMenubarManager.
 */
struct _PikaUIManager
{
  PikaObject    parent_instance;

  gchar        *name;
  Pika         *pika;
  GList        *registered_uis;

  GList        *ui_items;
  GList        *action_groups;

  /* Special property which should be set to TRUE only for the singleton UI
   * manager of the top menu bar.
   */
  gboolean      store_action_paths;
};

struct _PikaUIManagerClass
{
  PikaObjectClass    parent_class;

  GHashTable        *managers;

  void (* update)       (PikaUIManager *manager,
                         gpointer       update_data);
  void (* show_tooltip) (PikaUIManager *manager,
                         const gchar   *tooltip);
  void (* hide_tooltip) (PikaUIManager *manager);
  void (* ui_added)     (PikaUIManager *manager,
                         const gchar   *path,
                         const gchar   *action_name,
                         gboolean       top);
  void (* ui_removed)   (PikaUIManager *manager,
                         const gchar   *path,
                         const gchar   *action_name);
};


GType           pika_ui_manager_get_type    (void) G_GNUC_CONST;

PikaUIManager * pika_ui_manager_new         (Pika                   *pika,
                                             const gchar            *name);

GList         * pika_ui_managers_from_name  (const gchar            *name);

void            pika_ui_manager_update      (PikaUIManager          *manager,
                                             gpointer                update_data);

void            pika_ui_manager_add_action_group   (PikaUIManager   *manager,
                                                    PikaActionGroup *group);
PikaActionGroup * pika_ui_manager_get_action_group (PikaUIManager   *manager,
                                                    const gchar     *name);
GList         * pika_ui_manager_get_action_groups  (PikaUIManager   *manager);

PikaMenuModel * pika_ui_manager_get_model       (PikaUIManager      *manager,
                                                 const gchar        *path);

void            pika_ui_manager_remove_ui       (PikaUIManager      *manager,
                                                 const gchar        *action_name);
void            pika_ui_manager_remove_uis      (PikaUIManager      *manager,
                                                 const gchar        *action_name_prefix);
void            pika_ui_manager_add_ui          (PikaUIManager      *manager,
                                                 const gchar        *path,
                                                 const gchar        *action_name,
                                                 gboolean            top);
void            pika_ui_manager_foreach_ui      (PikaUIManager      *manager,
                                                 PikaUIMenuCallback  callback,
                                                 gpointer            user_data);

PikaAction    * pika_ui_manager_find_action     (PikaUIManager      *manager,
                                                 const gchar        *group_name,
                                                 const gchar        *action_name);
gboolean        pika_ui_manager_activate_action (PikaUIManager      *manager,
                                                 const gchar        *group_name,
                                                 const gchar        *action_name);
gboolean        pika_ui_manager_toggle_action   (PikaUIManager      *manager,
                                                 const gchar        *group_name,
                                                 const gchar        *action_name,
                                                 gboolean            active);

void            pika_ui_manager_ui_register (PikaUIManager          *manager,
                                             const gchar            *ui_path,
                                             const gchar            *basename,
                                             PikaUIManagerSetupFunc  setup_func);

void            pika_ui_manager_ui_popup_at_widget
                                            (PikaUIManager          *manager,
                                             const gchar            *ui_path,
                                             PikaUIManager          *child_ui_manager,
                                             const gchar            *child_ui_path,
                                             GtkWidget              *widget,
                                             GdkGravity              widget_anchor,
                                             GdkGravity              menu_anchor,
                                             const GdkEvent         *trigger_event,
                                             GDestroyNotify          popdown_func,
                                             gpointer                popdown_data);
void            pika_ui_manager_ui_popup_at_pointer
                                            (PikaUIManager          *manager,
                                             const gchar            *ui_path,
                                             GtkWidget              *attached_widget,
                                             const GdkEvent         *trigger_event,
                                             GDestroyNotify          popdown_func,
                                             gpointer                popdown_data);
void            pika_ui_manager_ui_popup_at_rect
                                            (PikaUIManager           *manager,
                                             const gchar             *ui_path,
                                             GtkWidget               *attached_widget,
                                             GdkWindow               *window,
                                             const GdkRectangle      *rect,
                                             GdkGravity               rect_anchor,
                                             GdkGravity               menu_anchor,
                                             const GdkEvent          *trigger_event,
                                             GDestroyNotify           popdown_func,
                                             gpointer                 popdown_data);


#endif  /* __PIKA_UI_MANAGER_H__ */
