/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikaview.h
 * Copyright (C) 2001-2006 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_VIEW_H__
#define __PIKA_VIEW_H__


#define PIKA_TYPE_VIEW            (pika_view_get_type ())
#define PIKA_VIEW(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_VIEW, PikaView))
#define PIKA_VIEW_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_VIEW, PikaViewClass))
#define PIKA_IS_VIEW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE (obj, PIKA_TYPE_VIEW))
#define PIKA_IS_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_VIEW))
#define PIKA_VIEW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_VIEW, PikaViewClass))


typedef struct _PikaViewClass  PikaViewClass;

struct _PikaView
{
  GtkWidget         parent_instance;

  GdkWindow        *event_window;

  PikaViewable     *viewable;
  PikaViewRenderer *renderer;

  guint             clickable : 1;
  guint             eat_button_events : 1;
  guint             show_popup : 1;
  guint             expand : 1;

  /*< private >*/
  guint             in_button : 1;
  guint             has_grab : 1;
  GdkModifierType   press_state;
};

struct _PikaViewClass
{
  GtkWidgetClass  parent_class;

  /*  signals  */
  void        (* set_viewable)   (PikaView        *view,
                                  PikaViewable    *old_viewable,
                                  PikaViewable    *new_viewable);
  void        (* clicked)        (PikaView        *view,
                                  GdkModifierType  modifier_state);
  void        (* double_clicked) (PikaView        *view);
  void        (* context)        (PikaView        *view);
};


GType          pika_view_get_type          (void) G_GNUC_CONST;

GtkWidget    * pika_view_new               (PikaContext   *context,
                                            PikaViewable  *viewable,
                                            gint           size,
                                            gint           border_width,
                                            gboolean       is_popup);
GtkWidget    * pika_view_new_full          (PikaContext   *context,
                                            PikaViewable  *viewable,
                                            gint           width,
                                            gint           height,
                                            gint           border_width,
                                            gboolean       is_popup,
                                            gboolean       clickable,
                                            gboolean       show_popup);
GtkWidget    * pika_view_new_by_types      (PikaContext   *context,
                                            GType          view_type,
                                            GType          viewable_type,
                                            gint           size,
                                            gint           border_width,
                                            gboolean       is_popup);
GtkWidget    * pika_view_new_full_by_types (PikaContext   *context,
                                            GType          view_type,
                                            GType          viewable_type,
                                            gint           width,
                                            gint           height,
                                            gint           border_width,
                                            gboolean       is_popup,
                                            gboolean       clickable,
                                            gboolean       show_popup);

PikaViewable * pika_view_get_viewable      (PikaView      *view);
void           pika_view_set_viewable      (PikaView      *view,
                                            PikaViewable  *viewable);
void           pika_view_set_expand        (PikaView      *view,
                                            gboolean       expand);


#endif /* __PIKA_VIEW_H__ */
