/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikaviewablebutton.h
 * Copyright (C) 2003-2005 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_VIEWABLE_BUTTON_H__
#define __PIKA_VIEWABLE_BUTTON_H__


#define PIKA_TYPE_VIEWABLE_BUTTON            (pika_viewable_button_get_type ())
#define PIKA_VIEWABLE_BUTTON(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_VIEWABLE_BUTTON, PikaViewableButton))
#define PIKA_VIEWABLE_BUTTON_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_VIEWABLE_BUTTON, PikaViewableButtonClass))
#define PIKA_IS_VIEWABLE_BUTTON(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_VIEWABLE_BUTTON))
#define PIKA_IS_VIEWABLE_BUTTON_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_VIEWABLE_BUTTON))
#define PIKA_VIEWABLE_BUTTON_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_VIEWABLE_BUTTON, PikaViewableButtonClass))


typedef struct _PikaViewableButtonClass PikaViewableButtonClass;

struct _PikaViewableButton
{
  PikaButton         parent_instance;

  PikaContainer     *container;
  PikaContext       *context;

  PikaViewType       popup_view_type;
  gint               popup_view_size;

  gint               button_view_size;
  gint               view_border_width;

  PikaDialogFactory *dialog_factory;
  gchar             *dialog_identifier;
  gchar             *dialog_icon_name;
  gchar             *dialog_tooltip;

  GtkWidget         *view;
};

struct _PikaViewableButtonClass
{
  PikaButtonClass  parent_class;
};


GType       pika_viewable_button_get_type (void) G_GNUC_CONST;

GtkWidget * pika_viewable_button_new      (PikaContainer      *container,
                                           PikaContext        *context,
                                           PikaViewType        view_type,
                                           gint                button_view_size,
                                           gint                view_size,
                                           gint                view_border_width,
                                           PikaDialogFactory  *dialog_factory,
                                           const gchar        *dialog_identifier,
                                           const gchar        *dialog_icon_name,
                                           const gchar        *dialog_tooltip);

PikaViewType pika_viewable_button_get_view_type (PikaViewableButton *button);
void         pika_viewable_button_set_view_type (PikaViewableButton *button,
                                                 PikaViewType        view_type);

gint         pika_viewable_button_get_view_size (PikaViewableButton *button);
void         pika_viewable_button_set_view_size (PikaViewableButton *button,
                                                 gint                view_size);


#endif /* __PIKA_VIEWABLE_BUTTON_H__ */
