/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikaviewrendererlayer.c
 * Copyright (C) 2003 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <gegl.h>
#include <gtk/gtk.h>

#include "libpikawidgets/pikawidgets.h"

#include "widgets-types.h"

#include "config/pikacoreconfig.h"

#include "core/pika.h"
#include "core/pikacontainer.h"
#include "core/pikaimage.h"

#include "text/pikatextlayer.h"

#include "pikaviewrendererlayer.h"


static void   pika_view_renderer_layer_render (PikaViewRenderer *renderer,
                                               GtkWidget        *widget);


G_DEFINE_TYPE (PikaViewRendererLayer, pika_view_renderer_layer,
               PIKA_TYPE_VIEW_RENDERER_DRAWABLE)

#define parent_class pika_view_renderer_layer_parent_class


static void
pika_view_renderer_layer_class_init (PikaViewRendererLayerClass *klass)
{
  PikaViewRendererClass *renderer_class = PIKA_VIEW_RENDERER_CLASS (klass);

  renderer_class->render = pika_view_renderer_layer_render;
}

static void
pika_view_renderer_layer_init (PikaViewRendererLayer *renderer)
{
}

static void
pika_view_renderer_layer_render (PikaViewRenderer *renderer,
                                 GtkWidget        *widget)
{
  const gchar *icon_name = NULL;

  if (pika_layer_is_floating_sel (PIKA_LAYER (renderer->viewable)))
    {
      icon_name = PIKA_ICON_LAYER_FLOATING_SELECTION;
    }
  else if (pika_item_is_text_layer (PIKA_ITEM (renderer->viewable)))
    {
      icon_name = pika_viewable_get_icon_name (renderer->viewable);
    }
  else
    {
      PikaContainer *children = pika_viewable_get_children (renderer->viewable);

      if (children)
        {
          PikaItem  *item  = PIKA_ITEM (renderer->viewable);
          PikaImage *image = pika_item_get_image (item);

          if (pika_container_get_n_children (children) == 0)
            icon_name = "folder";
          else if (image && ! image->pika->config->group_layer_previews)
            icon_name = pika_viewable_get_icon_name (renderer->viewable);
        }
    }

  if (icon_name)
    pika_view_renderer_render_icon (renderer, widget, icon_name);
  else
    PIKA_VIEW_RENDERER_CLASS (parent_class)->render (renderer, widget);
}
