/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __WIDGETS_ENUMS_H__
#define __WIDGETS_ENUMS_H__


/*
 * enums that are registered with the type system
 */

#define PIKA_TYPE_ACTIVE_COLOR (pika_active_color_get_type ())

GType pika_active_color_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_ACTIVE_COLOR_FOREGROUND, /*< desc="Foreground" >*/
  PIKA_ACTIVE_COLOR_BACKGROUND  /*< desc="Background" >*/
} PikaActiveColor;


#define PIKA_TYPE_CIRCLE_BACKGROUND (pika_circle_background_get_type ())

GType pika_circle_background_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_CIRCLE_BACKGROUND_PLAIN, /*< desc="Plain" >*/
  PIKA_CIRCLE_BACKGROUND_HSV    /*< desc="HSV"   >*/
} PikaCircleBackground;


#define PIKA_TYPE_COLOR_DIALOG_STATE (pika_color_dialog_state_get_type ())

GType pika_color_dialog_state_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_COLOR_DIALOG_OK,
  PIKA_COLOR_DIALOG_CANCEL,
  PIKA_COLOR_DIALOG_UPDATE
} PikaColorDialogState;


#define PIKA_TYPE_COLOR_PICK_TARGET (pika_color_pick_target_get_type ())

GType pika_color_pick_target_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_COLOR_PICK_TARGET_NONE,       /*< desc="Pick only"            >*/
  PIKA_COLOR_PICK_TARGET_FOREGROUND, /*< desc="Set foreground color" >*/
  PIKA_COLOR_PICK_TARGET_BACKGROUND, /*< desc="Set background color" >*/
  PIKA_COLOR_PICK_TARGET_PALETTE     /*< desc="Add to palette"       >*/
} PikaColorPickTarget;


#define PIKA_TYPE_COLOR_PICK_STATE (pika_color_pick_state_get_type ())

GType pika_color_pick_state_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_COLOR_PICK_STATE_START,
  PIKA_COLOR_PICK_STATE_UPDATE,
  PIKA_COLOR_PICK_STATE_END
} PikaColorPickState;


#define PIKA_TYPE_HISTOGRAM_SCALE (pika_histogram_scale_get_type ())

GType pika_histogram_scale_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_HISTOGRAM_SCALE_LINEAR,       /*< desc="Linear histogram"      >*/
  PIKA_HISTOGRAM_SCALE_LOGARITHMIC   /*< desc="Logarithmic histogram" >*/
} PikaHistogramScale;


#define PIKA_TYPE_TAB_STYLE (pika_tab_style_get_type ())

GType pika_tab_style_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_TAB_STYLE_ICON,          /*< desc="Icon"           >*/
  PIKA_TAB_STYLE_PREVIEW,       /*< desc="Current status" >*/
  PIKA_TAB_STYLE_NAME,          /*< desc="Text"           >*/
  PIKA_TAB_STYLE_BLURB,         /*< desc="Description"    >*/
  PIKA_TAB_STYLE_ICON_NAME,     /*< desc="Icon & text"    >*/
  PIKA_TAB_STYLE_ICON_BLURB,    /*< desc="Icon & desc"    >*/
  PIKA_TAB_STYLE_PREVIEW_NAME,  /*< desc="Status & text"  >*/
  PIKA_TAB_STYLE_PREVIEW_BLURB  /*< desc="Status & desc"  >*/
} PikaTabStyle;


#define PIKA_TYPE_TAG_ENTRY_MODE       (pika_tag_entry_mode_get_type ())

GType pika_tag_entry_mode_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_TAG_ENTRY_MODE_QUERY,
  PIKA_TAG_ENTRY_MODE_ASSIGN,
} PikaTagEntryMode;


/*
 * non-registered enums; register them if needed
 */

typedef enum  /*< skip >*/
{
  PIKA_VIEW_BG_CHECKS,
  PIKA_VIEW_BG_WHITE
} PikaViewBG;

typedef enum  /*< skip >*/
{
  PIKA_VIEW_BORDER_BLACK,
  PIKA_VIEW_BORDER_WHITE,
  PIKA_VIEW_BORDER_RED,
  PIKA_VIEW_BORDER_GREEN
} PikaViewBorderType;

typedef enum  /*< skip >*/
{
  PIKA_DND_TYPE_NONE         = 0,
  PIKA_DND_TYPE_URI_LIST     = 1,
  PIKA_DND_TYPE_TEXT_PLAIN   = 2,
  PIKA_DND_TYPE_NETSCAPE_URL = 3,
  PIKA_DND_TYPE_XDS          = 4,
  PIKA_DND_TYPE_COLOR        = 5,
  PIKA_DND_TYPE_SVG          = 6,
  PIKA_DND_TYPE_SVG_XML      = 7,
  PIKA_DND_TYPE_PIXBUF       = 8,
  PIKA_DND_TYPE_IMAGE        = 9,
  PIKA_DND_TYPE_COMPONENT    = 10,
  PIKA_DND_TYPE_LAYER        = 11,
  PIKA_DND_TYPE_CHANNEL      = 12,
  PIKA_DND_TYPE_LAYER_MASK   = 13,
  PIKA_DND_TYPE_VECTORS      = 14,
  PIKA_DND_TYPE_BRUSH        = 15,
  PIKA_DND_TYPE_PATTERN      = 16,
  PIKA_DND_TYPE_GRADIENT     = 17,
  PIKA_DND_TYPE_PALETTE      = 18,
  PIKA_DND_TYPE_FONT         = 19,
  PIKA_DND_TYPE_BUFFER       = 20,
  PIKA_DND_TYPE_IMAGEFILE    = 21,
  PIKA_DND_TYPE_TEMPLATE     = 22,
  PIKA_DND_TYPE_TOOL_ITEM    = 23,
  PIKA_DND_TYPE_NOTEBOOK_TAB = 24,

  PIKA_DND_TYPE_LAYER_LIST   = 25,
  PIKA_DND_TYPE_CHANNEL_LIST = 26,
  PIKA_DND_TYPE_VECTORS_LIST = 27,

  PIKA_DND_TYPE_LAST         = PIKA_DND_TYPE_VECTORS_LIST
} PikaDndType;

typedef enum  /*< skip >*/
{
  PIKA_DROP_NONE,
  PIKA_DROP_ABOVE,
  PIKA_DROP_BELOW
} PikaDropType;

typedef enum  /*< skip >*/
{
  PIKA_CURSOR_NONE = 1024,  /* (GDK_LAST_CURSOR + 2) yes, this is insane */
  PIKA_CURSOR_MOUSE,
  PIKA_CURSOR_CROSSHAIR,
  PIKA_CURSOR_CROSSHAIR_SMALL,
  PIKA_CURSOR_BAD,
  PIKA_CURSOR_MOVE,
  PIKA_CURSOR_ZOOM,
  PIKA_CURSOR_COLOR_PICKER,
  PIKA_CURSOR_CORNER_TOP,
  PIKA_CURSOR_CORNER_TOP_RIGHT,
  PIKA_CURSOR_CORNER_RIGHT,
  PIKA_CURSOR_CORNER_BOTTOM_RIGHT,
  PIKA_CURSOR_CORNER_BOTTOM,
  PIKA_CURSOR_CORNER_BOTTOM_LEFT,
  PIKA_CURSOR_CORNER_LEFT,
  PIKA_CURSOR_CORNER_TOP_LEFT,
  PIKA_CURSOR_SIDE_TOP,
  PIKA_CURSOR_SIDE_TOP_RIGHT,
  PIKA_CURSOR_SIDE_RIGHT,
  PIKA_CURSOR_SIDE_BOTTOM_RIGHT,
  PIKA_CURSOR_SIDE_BOTTOM,
  PIKA_CURSOR_SIDE_BOTTOM_LEFT,
  PIKA_CURSOR_SIDE_LEFT,
  PIKA_CURSOR_SIDE_TOP_LEFT,
  PIKA_CURSOR_SINGLE_DOT,
  PIKA_CURSOR_LAST
} PikaCursorType;

typedef enum  /*< skip >*/
{
  PIKA_TOOL_CURSOR_NONE,
  PIKA_TOOL_CURSOR_RECT_SELECT,
  PIKA_TOOL_CURSOR_ELLIPSE_SELECT,
  PIKA_TOOL_CURSOR_FREE_SELECT,
  PIKA_TOOL_CURSOR_POLYGON_SELECT,
  PIKA_TOOL_CURSOR_FUZZY_SELECT,
  PIKA_TOOL_CURSOR_PATHS,
  PIKA_TOOL_CURSOR_PATHS_ANCHOR,
  PIKA_TOOL_CURSOR_PATHS_CONTROL,
  PIKA_TOOL_CURSOR_PATHS_SEGMENT,
  PIKA_TOOL_CURSOR_ISCISSORS,
  PIKA_TOOL_CURSOR_MOVE,
  PIKA_TOOL_CURSOR_ZOOM,
  PIKA_TOOL_CURSOR_CROP,
  PIKA_TOOL_CURSOR_RESIZE,
  PIKA_TOOL_CURSOR_ROTATE,
  PIKA_TOOL_CURSOR_SHEAR,
  PIKA_TOOL_CURSOR_PERSPECTIVE,
  PIKA_TOOL_CURSOR_TRANSFORM_3D_CAMERA,
  PIKA_TOOL_CURSOR_FLIP_HORIZONTAL,
  PIKA_TOOL_CURSOR_FLIP_VERTICAL,
  PIKA_TOOL_CURSOR_TEXT,
  PIKA_TOOL_CURSOR_COLOR_PICKER,
  PIKA_TOOL_CURSOR_BUCKET_FILL,
  PIKA_TOOL_CURSOR_GRADIENT,
  PIKA_TOOL_CURSOR_PENCIL,
  PIKA_TOOL_CURSOR_PAINTBRUSH,
  PIKA_TOOL_CURSOR_AIRBRUSH,
  PIKA_TOOL_CURSOR_INK,
  PIKA_TOOL_CURSOR_CLONE,
  PIKA_TOOL_CURSOR_HEAL,
  PIKA_TOOL_CURSOR_ERASER,
  PIKA_TOOL_CURSOR_SMUDGE,
  PIKA_TOOL_CURSOR_BLUR,
  PIKA_TOOL_CURSOR_DODGE,
  PIKA_TOOL_CURSOR_BURN,
  PIKA_TOOL_CURSOR_MEASURE,
  PIKA_TOOL_CURSOR_WARP,
  PIKA_TOOL_CURSOR_HAND,
  PIKA_TOOL_CURSOR_LAST
} PikaToolCursorType;

typedef enum  /*< skip >*/
{
  PIKA_CURSOR_MODIFIER_NONE,
  PIKA_CURSOR_MODIFIER_BAD,
  PIKA_CURSOR_MODIFIER_PLUS,
  PIKA_CURSOR_MODIFIER_MINUS,
  PIKA_CURSOR_MODIFIER_INTERSECT,
  PIKA_CURSOR_MODIFIER_MOVE,
  PIKA_CURSOR_MODIFIER_RESIZE,
  PIKA_CURSOR_MODIFIER_ROTATE,
  PIKA_CURSOR_MODIFIER_ZOOM,
  PIKA_CURSOR_MODIFIER_CONTROL,
  PIKA_CURSOR_MODIFIER_ANCHOR,
  PIKA_CURSOR_MODIFIER_FOREGROUND,
  PIKA_CURSOR_MODIFIER_BACKGROUND,
  PIKA_CURSOR_MODIFIER_PATTERN,
  PIKA_CURSOR_MODIFIER_JOIN,
  PIKA_CURSOR_MODIFIER_SELECT,
  PIKA_CURSOR_MODIFIER_LAST
} PikaCursorModifier;

typedef enum  /*< skip >*/
{
  PIKA_DEVICE_VALUE_MODE       = 1 << 0,
  PIKA_DEVICE_VALUE_AXES       = 1 << 1,
  PIKA_DEVICE_VALUE_KEYS       = 1 << 2,
  PIKA_DEVICE_VALUE_TOOL       = 1 << 3,
  PIKA_DEVICE_VALUE_FOREGROUND = 1 << 4,
  PIKA_DEVICE_VALUE_BACKGROUND = 1 << 5,
  PIKA_DEVICE_VALUE_BRUSH      = 1 << 6,
  PIKA_DEVICE_VALUE_PATTERN    = 1 << 7,
  PIKA_DEVICE_VALUE_GRADIENT   = 1 << 8
} PikaDeviceValues;

typedef enum  /*< skip >*/
{
  PIKA_DIALOGS_SHOWN,
  PIKA_DIALOGS_HIDDEN_EXPLICITLY,  /* user used the Tab key to hide dialogs */
  PIKA_DIALOGS_HIDDEN_WITH_DISPLAY /* dialogs are hidden with the display   */
} PikaDialogsState;

typedef enum  /*< skip >*/
{
  PIKA_DASHBOARD_UPDATE_INTERVAL_0_25_SEC =    250,
  PIKA_DASHBOARD_UPDATE_INTERVAL_0_5_SEC  =    500,
  PIKA_DASHBOARD_UPDATE_INTERVAL_1_SEC    =   1000,
  PIKA_DASHBOARD_UPDATE_INTERVAL_2_SEC    =   2000,
  PIKA_DASHBOARD_UPDATE_INTERVAL_4_SEC    =   4000
} PikaDashboardUpdateInteval;

typedef enum  /*< skip >*/
{
  PIKA_DASHBOARD_HISTORY_DURATION_15_SEC  =  15000,
  PIKA_DASHBOARD_HISTORY_DURATION_30_SEC  =  30000,
  PIKA_DASHBOARD_HISTORY_DURATION_60_SEC  =  60000,
  PIKA_DASHBOARD_HISTORY_DURATION_120_SEC = 120000,
  PIKA_DASHBOARD_HISTORY_DURATION_240_SEC = 240000
} PikaDashboardHistoryDuration;


#endif /* __WIDGETS_ENUMS_H__ */
