/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-2003 Peter Mattis and Spencer Kimball
 *
 * pikabrushselect_pdb.c
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl */

#include "config.h"

#include "stamp-pdbgen.h"

#include "pika.h"


/**
 * SECTION: pikabrushselect
 * @title: pikabrushselect
 * @short_description: Methods of a font chooser dialog
 *
 * A dialog letting a user choose a brush.  Read more at
 * pikafontselect.
 **/


/**
 * pika_brushes_popup:
 * @brush_callback: The callback PDB proc to call when user chooses a brush.
 * @popup_title: Title of the brush selection dialog.
 * @initial_brush_name: The name of the brush to set as the initial choice.
 *
 * Invokes the PIKA brush selection dialog.
 *
 * Opens a dialog letting a user choose a brush.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_brushes_popup (const gchar *brush_callback,
                    const gchar *popup_title,
                    const gchar *initial_brush_name)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, brush_callback,
                                          G_TYPE_STRING, popup_title,
                                          G_TYPE_STRING, initial_brush_name,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-brushes-popup",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}

/**
 * pika_brushes_close_popup:
 * @brush_callback: The name of the callback registered for this pop-up.
 *
 * Close the brush selection dialog.
 *
 * Closes an open brush selection dialog.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_brushes_close_popup (const gchar *brush_callback)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, brush_callback,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-brushes-close-popup",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}

/**
 * pika_brushes_set_popup:
 * @brush_callback: The name of the callback registered for this pop-up.
 * @brush_name: The name of the brush to set as selected.
 *
 * Sets the selected brush in a brush selection dialog.
 *
 * Sets the selected brush in a brush selection dialog.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_brushes_set_popup (const gchar *brush_callback,
                        const gchar *brush_name)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, brush_callback,
                                          G_TYPE_STRING, brush_name,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-brushes-set-popup",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}
