/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikadrawable.h
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_H_INSIDE__) && !defined (PIKA_COMPILATION)
#error "Only <libpika/pika.h> can be included directly."
#endif

#ifndef __PIKA_DRAWABLE_H__
#define __PIKA_DRAWABLE_H__

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


#include <libpika/pikaitem.h>


#define PIKA_TYPE_DRAWABLE (pika_drawable_get_type ())
G_DECLARE_DERIVABLE_TYPE (PikaDrawable, pika_drawable, PIKA, DRAWABLE, PikaItem)


struct _PikaDrawableClass
{
  PikaItemClass parent_class;

  /* Padding for future expansion */
  void (*_pika_reserved1) (void);
  void (*_pika_reserved2) (void);
  void (*_pika_reserved3) (void);
  void (*_pika_reserved4) (void);
  void (*_pika_reserved5) (void);
  void (*_pika_reserved6) (void);
  void (*_pika_reserved7) (void);
  void (*_pika_reserved8) (void);
  void (*_pika_reserved9) (void);
};


PikaDrawable * pika_drawable_get_by_id              (gint32        drawable_id);

GeglBuffer   * pika_drawable_get_buffer             (PikaDrawable  *drawable);
GeglBuffer   * pika_drawable_get_shadow_buffer      (PikaDrawable  *drawable);

const Babl   * pika_drawable_get_format             (PikaDrawable  *drawable);
const Babl   * pika_drawable_get_thumbnail_format   (PikaDrawable  *drawable);

GBytes       * pika_drawable_get_thumbnail_data     (PikaDrawable  *drawable,
                                                     gint           width,
                                                     gint           height,
                                                     gint          *actual_width,
                                                     gint          *actual_height,
                                                     gint          *bpp);
GdkPixbuf    * pika_drawable_get_thumbnail          (PikaDrawable  *drawable,
                                                     gint           width,
                                                     gint           height,
                                                     PikaPixbufTransparency alpha);

GBytes       * pika_drawable_get_sub_thumbnail_data (PikaDrawable  *drawable,
                                                     gint           src_x,
                                                     gint           src_y,
                                                     gint           src_width,
                                                     gint           src_height,
                                                     gint           dest_width,
                                                     gint           dest_height,
                                                     gint          *actual_width,
                                                     gint          *actual_height,
                                                     gint          *bpp);
GdkPixbuf    * pika_drawable_get_sub_thumbnail      (PikaDrawable  *drawable,
                                                     gint           src_x,
                                                     gint           src_y,
                                                     gint           src_width,
                                                     gint           src_height,
                                                     gint           dest_width,
                                                     gint           dest_height,
                                                     PikaPixbufTransparency alpha);


G_END_DECLS

#endif /* __PIKA_DRAWABLE_H__ */
