/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikafileprocedure.h
 * Copyright (C) 2019 Michael Natterer <mitch@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_FILE_PROCEDURE_H__
#define __PIKA_FILE_PROCEDURE_H__

#include <libpika/pikaprocedure.h>

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


#define PIKA_TYPE_FILE_PROCEDURE            (pika_file_procedure_get_type ())
#define PIKA_FILE_PROCEDURE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_FILE_PROCEDURE, PikaFileProcedure))
#define PIKA_FILE_PROCEDURE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_FILE_PROCEDURE, PikaFileProcedureClass))
#define PIKA_IS_FILE_PROCEDURE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_FILE_PROCEDURE))
#define PIKA_IS_FILE_PROCEDURE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_FILE_PROCEDURE))
#define PIKA_FILE_PROCEDURE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_FILE_PROCEDURE, PikaFileProcedureClass))


typedef struct _PikaFileProcedure        PikaFileProcedure;
typedef struct _PikaFileProcedureClass   PikaFileProcedureClass;
typedef struct _PikaFileProcedurePrivate PikaFileProcedurePrivate;

struct _PikaFileProcedure
{
  PikaProcedure             parent_instance;

  PikaFileProcedurePrivate *priv;
};

struct _PikaFileProcedureClass
{
  PikaProcedureClass parent_class;
};


GType           pika_file_procedure_get_type        (void) G_GNUC_CONST;

void            pika_file_procedure_set_format_name (PikaFileProcedure *procedure,
                                                     const gchar       *format_name);
const gchar   * pika_file_procedure_get_format_name (PikaFileProcedure *procedure);

void            pika_file_procedure_set_mime_types  (PikaFileProcedure *procedure,
                                                     const gchar       *mime_types);
const gchar   * pika_file_procedure_get_mime_types  (PikaFileProcedure *procedure);

void            pika_file_procedure_set_extensions  (PikaFileProcedure *procedure,
                                                     const gchar       *extensions);
const gchar   * pika_file_procedure_get_extensions  (PikaFileProcedure *procedure);

void            pika_file_procedure_set_prefixes    (PikaFileProcedure *procedure,
                                                     const gchar       *prefixes);
const gchar   * pika_file_procedure_get_prefixes    (PikaFileProcedure *procedure);

void            pika_file_procedure_set_magics      (PikaFileProcedure *procedure,
                                                     const gchar       *magics);
const gchar   * pika_file_procedure_get_magics      (PikaFileProcedure *procedure);

void            pika_file_procedure_set_priority    (PikaFileProcedure *procedure,
                                                     gint               priority);
gint            pika_file_procedure_get_priority    (PikaFileProcedure *procedure);

void            pika_file_procedure_set_handles_remote
                                                    (PikaFileProcedure *procedure,
                                                     gboolean           handles_remote);
gboolean        pika_file_procedure_get_handles_remote
                                                    (PikaFileProcedure *procedure);


G_END_DECLS

#endif  /*  __PIKA_FILE_PROCEDURE_H__  */
