/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-2003 Peter Mattis and Spencer Kimball
 *
 * pikafonts_pdb.c
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl */

#include "config.h"

#include "stamp-pdbgen.h"

#include "pika.h"


/**
 * SECTION: pikafonts
 * @title: pikafonts
 * @short_description: Operations related to fonts.
 *
 * Operations related to fonts.
 **/


/**
 * pika_fonts_refresh:
 *
 * Refresh current fonts. This function always succeeds.
 *
 * This procedure retrieves all fonts currently in the user's font path
 * and updates the font dialogs accordingly. Depending on the amount of
 * fonts on the system, this can take considerable time.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_fonts_refresh (void)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-fonts-refresh",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}

/**
 * pika_fonts_get_list:
 * @filter: An optional regular expression used to filter the list.
 *
 * Retrieve the list of loaded fonts.
 *
 * This procedure returns a list of the fonts that are currently
 * available.
 *
 * Returns: (array zero-terminated=1) (transfer full): The list of font names.
 *          The returned value must be freed with g_strfreev().
 **/
gchar **
pika_fonts_get_list (const gchar *filter)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gchar **font_list = NULL;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, filter,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-fonts-get-list",
                                              args);
  pika_value_array_unref (args);

  if (PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS)
    font_list = PIKA_VALUES_DUP_STRV (return_vals, 1);

  pika_value_array_unref (return_vals);

  return font_list;
}
