/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-2003 Peter Mattis and Spencer Kimball
 *
 * pikagradientselect_pdb.c
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl */

#include "config.h"

#include "stamp-pdbgen.h"

#include "pika.h"


/**
 * SECTION: pikagradientselect
 * @title: pikagradientselect
 * @short_description: Methods of a gradient chooser dialog
 *
 * A dialog letting a user choose a gradient.  Read more at
 * pikafontselect.
 **/


/**
 * pika_gradients_popup:
 * @gradient_callback: The callback PDB proc to call when user chooses a gradient.
 * @popup_title: Title of the gradient selection dialog.
 * @initial_gradient_name: The name of the initial gradient choice.
 *
 * Invokes the Pika gradients selection dialog.
 *
 * Opens a dialog letting a user choose a gradient.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_gradients_popup (const gchar *gradient_callback,
                      const gchar *popup_title,
                      const gchar *initial_gradient_name)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, gradient_callback,
                                          G_TYPE_STRING, popup_title,
                                          G_TYPE_STRING, initial_gradient_name,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-gradients-popup",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}

/**
 * pika_gradients_close_popup:
 * @gradient_callback: The name of the callback registered for this pop-up.
 *
 * Close the gradient selection dialog.
 *
 * Closes an open gradient selection dialog.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_gradients_close_popup (const gchar *gradient_callback)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, gradient_callback,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-gradients-close-popup",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}

/**
 * pika_gradients_set_popup:
 * @gradient_callback: The name of the callback registered for this pop-up.
 * @gradient_name: The name of the gradient to set as selected.
 *
 * Sets the current gradient in a gradient selection dialog.
 *
 * Sets the current gradient in a gradient selection dialog.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_gradients_set_popup (const gchar *gradient_callback,
                          const gchar *gradient_name)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, gradient_callback,
                                          G_TYPE_STRING, gradient_name,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-gradients-set-popup",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}
