/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-2000 Peter Mattis and Spencer Kimball
 *
 * pikaimage.h
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_H_INSIDE__) && !defined (PIKA_COMPILATION)
#error "Only <libpika/pika.h> can be included directly."
#endif

#ifndef __PIKA_IMAGE_H__
#define __PIKA_IMAGE_H__

G_BEGIN_DECLS


/* For information look into the C source or the html documentation */


#define PIKA_TYPE_IMAGE (pika_image_get_type ())
G_DECLARE_FINAL_TYPE (PikaImage, pika_image, PIKA, IMAGE, GObject)


gint32         pika_image_get_id             (PikaImage    *image);
PikaImage    * pika_image_get_by_id          (gint32        image_id);

gboolean       pika_image_is_valid           (PikaImage    *image);

GList        * pika_list_images              (void);

GList        * pika_image_list_layers        (PikaImage    *image);
GList        * pika_image_list_channels      (PikaImage    *image);
GList        * pika_image_list_vectors       (PikaImage    *image);

GList    * pika_image_list_selected_layers   (PikaImage    *image);
gboolean   pika_image_take_selected_layers   (PikaImage    *image,
                                              GList        *layers);
GList    * pika_image_list_selected_channels (PikaImage    *image);
gboolean   pika_image_take_selected_channels (PikaImage    *image,
                                              GList        *channels);
GList    * pika_image_list_selected_vectors  (PikaImage    *image);
gboolean   pika_image_take_selected_vectors  (PikaImage    *image,
                                              GList        *vectors);

GList    * pika_image_list_selected_drawables(PikaImage    *image);

guchar       * pika_image_get_colormap       (PikaImage    *image,
                                              gint         *colormap_len,
                                              gint         *num_colors);
gboolean       pika_image_set_colormap       (PikaImage    *image,
                                              const guchar *colormap,
                                              gint          num_colors);

guchar       * pika_image_get_thumbnail_data (PikaImage    *image,
                                              gint         *width,
                                              gint         *height,
                                              gint         *bpp);
GdkPixbuf    * pika_image_get_thumbnail      (PikaImage    *image,
                                              gint          width,
                                              gint          height,
                                              PikaPixbufTransparency  alpha);

PikaMetadata * pika_image_get_metadata       (PikaImage    *image);
gboolean       pika_image_set_metadata       (PikaImage    *image,
                                              PikaMetadata *metadata);


G_END_DECLS

#endif /* __PIKA_IMAGE_H__ */
