/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-2000 Peter Mattis and Spencer Kimball
 *
 * pikalayer.h
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_H_INSIDE__) && !defined (PIKA_COMPILATION)
#error "Only <libpika/pika.h> can be included directly."
#endif

#ifndef __PIKA_LAYER_H__
#define __PIKA_LAYER_H__

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


#define PIKA_TYPE_LAYER (pika_layer_get_type ())
G_DECLARE_DERIVABLE_TYPE (PikaLayer, pika_layer, PIKA, LAYER, PikaDrawable)

struct _PikaLayerClass
{
  PikaDrawableClass parent_class;

  /*  virtual functions  */
  PikaLayer * (* copy) (PikaLayer *layer);

  /* Padding for future expansion */
  void (*_pika_reserved1) (void);
  void (*_pika_reserved2) (void);
  void (*_pika_reserved3) (void);
  void (*_pika_reserved4) (void);
  void (*_pika_reserved5) (void);
  void (*_pika_reserved6) (void);
  void (*_pika_reserved7) (void);
  void (*_pika_reserved8) (void);
  void (*_pika_reserved9) (void);
};

PikaLayer * pika_layer_get_by_id          (gint32           layer_id);

PikaLayer * pika_layer_new                (PikaImage       *image,
                                           const gchar     *name,
                                           gint             width,
                                           gint             height,
                                           PikaImageType    type,
                                           gdouble          opacity,
                                           PikaLayerMode    mode);

PikaLayer * pika_layer_new_from_pixbuf    (PikaImage       *image,
                                           const gchar     *name,
                                           GdkPixbuf       *pixbuf,
                                           gdouble          opacity,
                                           PikaLayerMode    mode,
                                           gdouble          progress_start,
                                           gdouble          progress_end);
PikaLayer * pika_layer_new_from_surface   (PikaImage       *image,
                                           const gchar     *name,
                                           cairo_surface_t *surface,
                                           gdouble          progress_start,
                                           gdouble          progress_end);

PikaLayer * pika_layer_copy               (PikaLayer       *layer);


G_END_DECLS

#endif /* __PIKA_LAYER_H__ */
