/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikaloadprocedure.h
 * Copyright (C) 2019 Michael Natterer <mitch@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_LOAD_PROCEDURE_H__
#define __PIKA_LOAD_PROCEDURE_H__

#include <libpika/pikafileprocedure.h>

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


/**
 * PikaRunLoadFunc:
 * @procedure:   the #PikaProcedure that runs.
 * @run_mode:    the #PikaRunMode.
 * @file:        the #GFile to load from.
 * @args:        the @procedure's remaining arguments.
 * @run_data: (closure): the run_data given in pika_load_procedure_new().
 *
 * The load function is run during the lifetime of the PIKA session,
 * each time a plug-in load procedure is called.
 *
 * Returns: (transfer full): the @procedure's return values.
 *
 * Since: 3.0
 **/
typedef PikaValueArray * (* PikaRunLoadFunc) (PikaProcedure        *procedure,
                                              PikaRunMode           run_mode,
                                              GFile                *file,
                                              const PikaValueArray *args,
                                              gpointer              run_data);


#define PIKA_TYPE_LOAD_PROCEDURE            (pika_load_procedure_get_type ())
#define PIKA_LOAD_PROCEDURE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_LOAD_PROCEDURE, PikaLoadProcedure))
#define PIKA_LOAD_PROCEDURE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_LOAD_PROCEDURE, PikaLoadProcedureClass))
#define PIKA_IS_LOAD_PROCEDURE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_LOAD_PROCEDURE))
#define PIKA_IS_LOAD_PROCEDURE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_LOAD_PROCEDURE))
#define PIKA_LOAD_PROCEDURE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_LOAD_PROCEDURE, PikaLoadProcedureClass))


typedef struct _PikaLoadProcedure        PikaLoadProcedure;
typedef struct _PikaLoadProcedureClass   PikaLoadProcedureClass;
typedef struct _PikaLoadProcedurePrivate PikaLoadProcedurePrivate;

struct _PikaLoadProcedure
{
  PikaFileProcedure         parent_instance;

  PikaLoadProcedurePrivate *priv;
};

struct _PikaLoadProcedureClass
{
  PikaFileProcedureClass parent_class;
};


GType           pika_load_procedure_get_type             (void) G_GNUC_CONST;

PikaProcedure * pika_load_procedure_new                  (PikaPlugIn        *plug_in,
                                                          const gchar       *name,
                                                          PikaPDBProcType    proc_type,
                                                          PikaRunLoadFunc    run_func,
                                                          gpointer           run_data,
                                                          GDestroyNotify     run_data_destroy);

void            pika_load_procedure_set_handles_raw      (PikaLoadProcedure *procedure,
                                                          gboolean           handles_raw);
gboolean        pika_load_procedure_get_handles_raw      (PikaLoadProcedure *procedure);

void            pika_load_procedure_set_thumbnail_loader (PikaLoadProcedure *procedure,
                                                          const gchar       *thumbnail_proc);
const gchar   * pika_load_procedure_get_thumbnail_loader (PikaLoadProcedure *procedure);


G_END_DECLS

#endif  /*  __PIKA_LOAD_PROCEDURE_H__  */
