/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-2003 Peter Mattis and Spencer Kimball
 *
 * pikamessage_pdb.c
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl */

#include "config.h"

#include "stamp-pdbgen.h"

#include "pika.h"


/**
 * SECTION: pikamessage
 * @title: pikamessage
 * @short_description: Display a dialog box with a message.
 *
 * Display a dialog box with a message.
 **/


/**
 * pika_message:
 * @message: Message to display in the dialog.
 *
 * Displays a dialog box with a message.
 *
 * Displays a dialog box with a message. Useful for status or error
 * reporting. The message must be in UTF-8 encoding.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_message (const gchar *message)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, message,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-message",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}

/**
 * pika_message_get_handler:
 *
 * Returns the current state of where warning messages are displayed.
 *
 * This procedure returns the way g_message warnings are displayed.
 * They can be shown in a dialog box or printed on the console where
 * pika was started.
 *
 * Returns: The current handler type.
 **/
PikaMessageHandlerType
pika_message_get_handler (void)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  PikaMessageHandlerType handler = 0;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-message-get-handler",
                                              args);
  pika_value_array_unref (args);

  if (PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS)
    handler = PIKA_VALUES_GET_ENUM (return_vals, 1);

  pika_value_array_unref (return_vals);

  return handler;
}

/**
 * pika_message_set_handler:
 * @handler: The new handler type.
 *
 * Controls where warning messages are displayed.
 *
 * This procedure controls how g_message warnings are displayed. They
 * can be shown in a dialog box or printed on the console where pika
 * was started.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_message_set_handler (PikaMessageHandlerType handler)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          PIKA_TYPE_MESSAGE_HANDLER_TYPE, handler,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-message-set-handler",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}
