/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-2003 Peter Mattis and Spencer Kimball
 *
 * pikapaletteselect_pdb.c
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl */

#include "config.h"

#include "stamp-pdbgen.h"

#include "pika.h"


/**
 * SECTION: pikapaletteselect
 * @title: pikapaletteselect
 * @short_description: Methods of a palette chooser dialog
 *
 * A dialog letting a user choose a palette.  Read more at
 * pikafontselect.
 **/


/**
 * pika_palettes_popup:
 * @palette_callback: The callback PDB proc to call when user chooses a palette.
 * @popup_title: Title of the palette selection dialog.
 * @initial_palette_name: The name of the palette to set as the initial choice.
 *
 * Invokes the Pika palette selection dialog.
 *
 * Opens a dialog letting a user choose a palette.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_palettes_popup (const gchar *palette_callback,
                     const gchar *popup_title,
                     const gchar *initial_palette_name)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, palette_callback,
                                          G_TYPE_STRING, popup_title,
                                          G_TYPE_STRING, initial_palette_name,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-palettes-popup",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}

/**
 * pika_palettes_close_popup:
 * @palette_callback: The name of the callback registered for this pop-up.
 *
 * Close the palette selection dialog.
 *
 * Closes an open palette selection dialog.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_palettes_close_popup (const gchar *palette_callback)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, palette_callback,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-palettes-close-popup",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}

/**
 * pika_palettes_set_popup:
 * @palette_callback: The name of the callback registered for this pop-up.
 * @palette_name: The name of the palette to set as selected.
 *
 * Sets the current palette in a palette selection dialog.
 *
 * Sets the current palette in a palette selection dialog.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_palettes_set_popup (const gchar *palette_callback,
                         const gchar *palette_name)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, palette_callback,
                                          G_TYPE_STRING, palette_name,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-palettes-set-popup",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}
