/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-2003 Peter Mattis and Spencer Kimball
 *
 * pikapatternselect_pdb.c
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

/* NOTE: This file is auto-generated by pdbgen.pl */

#include "config.h"

#include "stamp-pdbgen.h"

#include "pika.h"


/**
 * SECTION: pikapatternselect
 * @title: pikapatternselect
 * @short_description: Methods of a pattern chooser dialog
 *
 * A dialog letting a user choose a pattern.  Read more at
 * pikafontselect.
 **/


/**
 * pika_patterns_popup:
 * @pattern_callback: The callback PDB proc to call when the user chooses a pattern.
 * @popup_title: Title of the pattern selection dialog.
 * @initial_pattern_name: The name of the pattern to set as the initial choice.
 *
 * Invokes the Pika pattern selection.
 *
 * Opens the pattern selection dialog.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_patterns_popup (const gchar *pattern_callback,
                     const gchar *popup_title,
                     const gchar *initial_pattern_name)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, pattern_callback,
                                          G_TYPE_STRING, popup_title,
                                          G_TYPE_STRING, initial_pattern_name,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-patterns-popup",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}

/**
 * pika_patterns_close_popup:
 * @pattern_callback: The name of the callback registered for this pop-up.
 *
 * Close the pattern selection dialog.
 *
 * Closes an open pattern selection dialog.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_patterns_close_popup (const gchar *pattern_callback)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, pattern_callback,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-patterns-close-popup",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}

/**
 * pika_patterns_set_popup:
 * @pattern_callback: The name of the callback registered for this pop-up.
 * @pattern_name: The name of the pattern to set as selected.
 *
 * Sets the current pattern in a pattern selection dialog.
 *
 * Sets the current pattern in a pattern selection dialog.
 *
 * Returns: TRUE on success.
 **/
gboolean
pika_patterns_set_popup (const gchar *pattern_callback,
                         const gchar *pattern_name)
{
  PikaValueArray *args;
  PikaValueArray *return_vals;
  gboolean success = TRUE;

  args = pika_value_array_new_from_types (NULL,
                                          G_TYPE_STRING, pattern_callback,
                                          G_TYPE_STRING, pattern_name,
                                          G_TYPE_NONE);

  return_vals = pika_pdb_run_procedure_array (pika_get_pdb (),
                                              "pika-patterns-set-popup",
                                              args);
  pika_value_array_unref (args);

  success = PIKA_VALUES_GET_ENUM (return_vals, 0) == PIKA_PDB_SUCCESS;

  pika_value_array_unref (return_vals);

  return success;
}
