/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikapdb.h
 * Copyright (C) 2019 Michael Natterer <mitch@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_H_INSIDE__) && !defined (PIKA_COMPILATION)
#error "Only <libpika/pika.h> can be included directly."
#endif

#ifndef __PIKA_PDB_H__
#define __PIKA_PDB_H__

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


#define PIKA_TYPE_PDB            (pika_pdb_get_type ())
#define PIKA_PDB(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_PDB, PikaPDB))
#define PIKA_PDB_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_PDB, PikaPDBClass))
#define PIKA_IS_PDB(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_PDB))
#define PIKA_IS_PDB_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_PDB))
#define PIKA_PDB_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_PDB, PikaPDBClass))


typedef struct _PikaPDBClass   PikaPDBClass;
typedef struct _PikaPDBPrivate PikaPDBPrivate;

struct _PikaPDB
{
  GObject         parent_instance;

  PikaPDBPrivate *priv;
};

struct _PikaPDBClass
{
  GObjectClass parent_class;

  /* Padding for future expansion */
  void (*_pika_reserved1) (void);
  void (*_pika_reserved2) (void);
  void (*_pika_reserved3) (void);
  void (*_pika_reserved4) (void);
  void (*_pika_reserved5) (void);
  void (*_pika_reserved6) (void);
  void (*_pika_reserved7) (void);
  void (*_pika_reserved8) (void);
};


GType            pika_pdb_get_type             (void) G_GNUC_CONST;

gboolean         pika_pdb_procedure_exists     (PikaPDB              *pdb,
                                                const gchar          *procedure_name);

PikaProcedure  * pika_pdb_lookup_procedure     (PikaPDB              *pdb,
                                                const gchar          *procedure_name);

PikaValueArray * pika_pdb_run_procedure        (PikaPDB              *pdb,
                                                const gchar          *procedure_name,
                                                GType                 first_type,
                                                ...);
PikaValueArray * pika_pdb_run_procedure_valist (PikaPDB              *pdb,
                                                const gchar          *procedure_name,
                                                GType                 first_type,
                                                va_list               args);
PikaValueArray * pika_pdb_run_procedure_argv   (PikaPDB              *pdb,
                                                const gchar          *procedure_name,
                                                const GValue         *arguments,
                                                gint                  n_arguments);
PikaValueArray * pika_pdb_run_procedure_array  (PikaPDB              *pdb,
                                                const gchar          *procedure_name,
                                                const PikaValueArray *arguments);
PikaValueArray * pika_pdb_run_procedure_config (PikaPDB              *pdb,
                                                const gchar          *procedure_name,
                                                PikaProcedureConfig  *config);

gchar          * pika_pdb_temp_procedure_name  (PikaPDB              *pdb);

gboolean         pika_pdb_dump_to_file         (PikaPDB              *pdb,
                                                GFile                *file);
gchar         ** pika_pdb_query_procedures     (PikaPDB              *pdb,
                                                const gchar          *name,
                                                const gchar          *blurb,
                                                const gchar          *help,
                                                const gchar          *help_id,
                                                const gchar          *authors,
                                                const gchar          *copyright,
                                                const gchar          *date,
                                                const gchar          *proc_type);

const gchar       * pika_pdb_get_last_error    (PikaPDB              *pdb);
PikaPDBStatusType   pika_pdb_get_last_status   (PikaPDB              *pdb);


/*  Cruft API  */

gboolean   pika_pdb_get_data      (const gchar      *identifier,
                                   gpointer          data);
gint       pika_pdb_get_data_size (const gchar      *identifier);
gboolean   pika_pdb_set_data      (const gchar      *identifier,
                                   gconstpointer     data,
                                   guint32           data_len);


G_END_DECLS

#endif  /*  __PIKA_PDB_H__  */
