/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-2003 Peter Mattis and Spencer Kimball
 *
 * pikaprocedureconfig.h
 * Copyright (C) 2019 Michael Natterer <mitch@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_H_INSIDE__) && !defined (PIKA_COMPILATION)
#error "Only <libpika/pika.h> can be included directly."
#endif

#ifndef __PIKA_PROCEDURE_CONFIG_H__
#define __PIKA_PROCEDURE_CONFIG_H__

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


#define PIKA_TYPE_PROCEDURE_CONFIG            (pika_procedure_config_get_type ())
#define PIKA_PROCEDURE_CONFIG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_PROCEDURE_CONFIG, PikaProcedureConfig))
#define PIKA_PROCEDURE_CONFIG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_PROCEDURE_CONFIG, PikaProcedureConfigClass))
#define PIKA_IS_PROCEDURE_CONFIG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_PROCEDURE_CONFIG))
#define PIKA_IS_PROCEDURE_CONFIG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_PROCEDURE_CONFIG))
#define PIKA_PROCEDURE_CONFIG_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_PROCEDURE_CONFIG, PikaProcedureConfigClass))


typedef struct _PikaProcedureConfigClass   PikaProcedureConfigClass;
typedef struct _PikaProcedureConfigPrivate PikaProcedureConfigPrivate;

struct _PikaProcedureConfig
{
  GObject                     parent_instance;

  PikaProcedureConfigPrivate *priv;
};

struct _PikaProcedureConfigClass
{
  GObjectClass  parent_class;

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType   pika_procedure_config_get_type      (void) G_GNUC_CONST;

PikaProcedure *
        pika_procedure_config_get_procedure (PikaProcedureConfig *config);

void    pika_procedure_config_set_values    (PikaProcedureConfig  *config,
                                             const PikaValueArray *values);
void    pika_procedure_config_get_values    (PikaProcedureConfig  *config,
                                             PikaValueArray       *values);

void    pika_procedure_config_begin_run     (PikaProcedureConfig  *config,
                                             PikaImage            *image,
                                             PikaRunMode           run_mode,
                                             const PikaValueArray *args);
void    pika_procedure_config_end_run       (PikaProcedureConfig  *config,
                                             PikaPDBStatusType     status);

PikaMetadata *
        pika_procedure_config_begin_export  (PikaProcedureConfig  *config,
                                             PikaImage            *original_image,
                                             PikaRunMode           run_mode,
                                             const PikaValueArray *args,
                                             const gchar          *mime_type);
void    pika_procedure_config_end_export    (PikaProcedureConfig  *config,
                                             PikaImage            *exported_image,
                                             GFile                *file,
                                             PikaPDBStatusType     status);

void    pika_procedure_config_save_metadata (PikaProcedureConfig  *config,
                                             PikaImage            *exported_image,
                                             GFile                *file);


G_END_DECLS

#endif /* __PIKA_PROCEDURE_CONFIG_H__ */
