/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_H_INSIDE__) && !defined (PIKA_COMPILATION)
#error "Only <libpika/pika.h> can be included directly."
#endif

#ifndef __PIKA_RESOURCE_H__
#define __PIKA_RESOURCE_H__

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


#define PIKA_TYPE_RESOURCE (pika_resource_get_type ())
G_DECLARE_DERIVABLE_TYPE (PikaResource, pika_resource, PIKA, RESOURCE, GObject)


struct _PikaResourceClass
{
  GObjectClass parent_class;

  /* Padding for future expansion */
  void (*_pika_reserved1) (void);
  void (*_pika_reserved2) (void);
  void (*_pika_reserved3) (void);
  void (*_pika_reserved4) (void);
  void (*_pika_reserved5) (void);
  void (*_pika_reserved6) (void);
  void (*_pika_reserved7) (void);
  void (*_pika_reserved8) (void);
};

gint32         pika_resource_get_id      (PikaResource *resource);
PikaResource * pika_resource_get_by_id   (gint32        resource_id);
PikaResource * pika_resource_get_by_name (GType         resource_type,
                                          const gchar  *resource_name);

gboolean       pika_resource_is_valid    (PikaResource *resource);
gboolean       pika_resource_is_brush    (PikaResource *resource);
gboolean       pika_resource_is_pattern  (PikaResource *resource);
gboolean       pika_resource_is_gradient (PikaResource *resource);
gboolean       pika_resource_is_palette  (PikaResource *resource);
gboolean       pika_resource_is_font     (PikaResource *resource);


#define PIKA_TYPE_BRUSH (pika_brush_get_type ())
G_DECLARE_FINAL_TYPE (PikaBrush, pika_brush, PIKA, BRUSH, PikaResource)

#define PIKA_TYPE_PATTERN (pika_pattern_get_type ())
G_DECLARE_FINAL_TYPE (PikaPattern, pika_pattern, PIKA, PATTERN, PikaResource)

#define PIKA_TYPE_GRADIENT (pika_gradient_get_type ())
G_DECLARE_FINAL_TYPE (PikaGradient, pika_gradient, PIKA, GRADIENT, PikaResource)

#define PIKA_TYPE_PALETTE (pika_palette_get_type ())
G_DECLARE_FINAL_TYPE (PikaPalette, pika_palette, PIKA, PALETTE, PikaResource)

#define PIKA_TYPE_FONT (pika_font_get_type ())
G_DECLARE_FINAL_TYPE (PikaFont, pika_font, PIKA, FONT, PikaResource)


G_END_DECLS

#endif /* __PIKA_RESOURCE_H__ */
