/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikathumbnailprocedure.h
 * Copyright (C) 2019 Michael Natterer <mitch@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_THUMBNAIL_PROCEDURE_H__
#define __PIKA_THUMBNAIL_PROCEDURE_H__

#include <libpika/pikaprocedure.h>

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


/**
 * PikaRunThumbnailFunc:
 * @procedure:   the #PikaProcedure that runs.
 * @file:        the #GFile to load the thumbnail from.
 * @size:        the requested thumbnail size.
 * @args:        the @procedure's remaining arguments.
 * @run_data: (closure): the run_data given in pika_thumbnail_procedure_new().
 *
 * The thumbnail function is run during the lifetime of the PIKA session,
 * each time a plug-in thumbnail procedure is called.
 *
 * Returns: (transfer full): the @procedure's return values.
 *
 * Since: 3.0
 **/
typedef PikaValueArray * (* PikaRunThumbnailFunc) (PikaProcedure        *procedure,
                                                   GFile                *file,
                                                   gint                  size,
                                                   const PikaValueArray *args,
                                                   gpointer              run_data);


#define PIKA_TYPE_THUMBNAIL_PROCEDURE            (pika_thumbnail_procedure_get_type ())
#define PIKA_THUMBNAIL_PROCEDURE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_THUMBNAIL_PROCEDURE, PikaThumbnailProcedure))
#define PIKA_THUMBNAIL_PROCEDURE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_THUMBNAIL_PROCEDURE, PikaThumbnailProcedureClass))
#define PIKA_IS_THUMBNAIL_PROCEDURE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_THUMBNAIL_PROCEDURE))
#define PIKA_IS_THUMBNAIL_PROCEDURE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_THUMBNAIL_PROCEDURE))
#define PIKA_THUMBNAIL_PROCEDURE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_THUMBNAIL_PROCEDURE, PikaThumbnailProcedureClass))


typedef struct _PikaThumbnailProcedure        PikaThumbnailProcedure;
typedef struct _PikaThumbnailProcedureClass   PikaThumbnailProcedureClass;
typedef struct _PikaThumbnailProcedurePrivate PikaThumbnailProcedurePrivate;

struct _PikaThumbnailProcedure
{
  PikaProcedure                  parent_instance;

  PikaThumbnailProcedurePrivate *priv;
};

struct _PikaThumbnailProcedureClass
{
  PikaProcedureClass parent_class;
};


GType           pika_thumbnail_procedure_get_type (void) G_GNUC_CONST;

PikaProcedure * pika_thumbnail_procedure_new      (PikaPlugIn           *plug_in,
                                                   const gchar          *name,
                                                   PikaPDBProcType       proc_type,
                                                   PikaRunThumbnailFunc  run_func,
                                                   gpointer              run_data,
                                                   GDestroyNotify        run_data_destroy);


G_END_DECLS

#endif  /*  __PIKA_THUMBNAIL_PROCEDURE_H__  */
