/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikaenv.h
 * Copyright (C) 1999 Tor Lillqvist <tml@iki.fi>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_BASE_H_INSIDE__) && !defined (PIKA_BASE_COMPILATION)
#error "Only <libpikabase/pikabase.h> can be included directly."
#endif

#ifndef __PIKA_ENV_H__
#define __PIKA_ENV_H__

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


#ifdef G_OS_WIN32
#  ifdef __PIKA_ENV_C__
#    define PIKAVAR extern __declspec(dllexport)
#  else  /* !__PIKA_ENV_C__ */
#    define PIKAVAR extern __declspec(dllimport)
#  endif /* !__PIKA_ENV_C__ */
#else  /* !G_OS_WIN32 */
#  define PIKAVAR extern
#endif

PIKAVAR const guint pika_major_version;
PIKAVAR const guint pika_minor_version;
PIKAVAR const guint pika_micro_version;


const gchar * pika_directory                   (void) G_GNUC_CONST;
const gchar * pika_installation_directory      (void) G_GNUC_CONST;
const gchar * pika_data_directory              (void) G_GNUC_CONST;
const gchar * pika_locale_directory            (void) G_GNUC_CONST;
const gchar * pika_sysconf_directory           (void) G_GNUC_CONST;
const gchar * pika_plug_in_directory           (void) G_GNUC_CONST;
const gchar * pika_cache_directory             (void) G_GNUC_CONST;
const gchar * pika_temp_directory              (void) G_GNUC_CONST;

GFile       * pika_directory_file              (const gchar *first_element,
                                                ...) G_GNUC_MALLOC;
GFile       * pika_installation_directory_file (const gchar *first_element,
                                                ...) G_GNUC_MALLOC;
GFile       * pika_data_directory_file         (const gchar *first_element,
                                                ...) G_GNUC_MALLOC;
GFile       * pika_locale_directory_file       (const gchar *first_element,
                                                ...) G_GNUC_MALLOC;
GFile       * pika_sysconf_directory_file      (const gchar *first_element,
                                                ...) G_GNUC_MALLOC;
GFile       * pika_plug_in_directory_file      (const gchar *first_element,
                                                ...) G_GNUC_MALLOC;

GList       * pika_path_parse                  (const gchar  *path,
                                                gint          max_paths,
                                                gboolean      check,
                                                GList       **check_failed);
gchar       * pika_path_to_str                 (GList        *path) G_GNUC_MALLOC;
void          pika_path_free                   (GList        *path);

gchar       * pika_path_get_user_writable_dir  (GList        *path) G_GNUC_MALLOC;


/*  should be considered private, don't use!  */
void          pika_env_init                    (gboolean      plug_in);


G_END_DECLS

#endif  /*  __PIKA_ENV_H__  */
