/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * PikaColorManaged interface
 * Copyright (C) 2007  Sven Neumann <sven@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_COLOR_H_INSIDE__) && !defined (PIKA_COLOR_COMPILATION)
#error "Only <libpikacolor/pikacolor.h> can be included directly."
#endif

#ifndef __PIKA_COLOR_MANAGED_H__
#define __PIKA_COLOR_MANAGED_H__

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


#define PIKA_TYPE_COLOR_MANAGED (pika_color_managed_get_type ())
G_DECLARE_INTERFACE (PikaColorManaged, pika_color_managed, PIKA, COLOR_MANAGED, GObject)

/**
 * PikaColorManagedInterface:
 * @base_iface: The parent interface
 * @get_icc_profile: Returns the ICC profile of the pixels managed by
 *                   the object
 * @profile_changed: This signal is emitted when the object's color profile
 *                   has changed
 * @get_color_profile: Returns the #PikaColorProfile of the pixels managed
 *                     by the object
 * @get_simulation_profile: Returns the simulation #PikaColorProfile of the
 *                          pixels managed by the object
 * @get_simulation_rendering_intent: Returns the simulation #PikaColorRenderingIntent
 *                                   of the pixels managed by the object
 * @get_simulation_bpc: Returns whether black point compensation is enabled for the
 *                      simulation of the pixels managed by the object
 **/
struct _PikaColorManagedInterface
{
  GTypeInterface  base_iface;

  /**
   * PikaColorManagedInterface::get_icc_profile:
   * @managed: an object the implements the #PikaColorManaged interface
   * @len: (out): return location for the number of bytes in the profile data
   *
   * Returns: (array length=len): A blob of data that represents an ICC color
   *                              profile.
   *
   * Since: 2.4
   */
  const guint8     * (* get_icc_profile)                     (PikaColorManaged *managed,
                                                              gsize            *len);

  /*  signals  */
  void               (* profile_changed)                     (PikaColorManaged *managed);

  void               (* simulation_profile_changed)          (PikaColorManaged *managed);

  void               (* simulation_intent_changed)           (PikaColorManaged *managed);

  void               (* simulation_bpc_changed)              (PikaColorManaged *managed);

  /*  virtual functions  */
  PikaColorProfile * (* get_color_profile)               (PikaColorManaged *managed);
  PikaColorProfile * (* get_simulation_profile)          (PikaColorManaged *managed);
  PikaColorRenderingIntent
                     (* get_simulation_intent)           (PikaColorManaged *managed);
  gboolean           (* get_simulation_bpc)              (PikaColorManaged *managed);
};


const guint8     *       pika_color_managed_get_icc_profile            (PikaColorManaged *managed,
                                                                        gsize            *len);
PikaColorProfile *       pika_color_managed_get_color_profile          (PikaColorManaged *managed);

PikaColorProfile *       pika_color_managed_get_simulation_profile     (PikaColorManaged *managed);

PikaColorRenderingIntent pika_color_managed_get_simulation_intent      (PikaColorManaged *managed);

gboolean                 pika_color_managed_get_simulation_bpc         (PikaColorManaged *managed);

void                     pika_color_managed_profile_changed            (PikaColorManaged *managed);

void                     pika_color_managed_simulation_profile_changed (PikaColorManaged *managed);

void                     pika_color_managed_simulation_intent_changed  (PikaColorManaged *managed);

void                     pika_color_managed_simulation_bpc_changed     (PikaColorManaged *managed);


G_END_DECLS

#endif  /* __PIKA_COLOR_MANAGED_IFACE_H__ */
