/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Spencer Kimball and Peter Mattis
 *
 * pikacolorprofile.h
 * Copyright (C) 2014  Michael Natterer <mitch@gimp.org>
 *                     Elle Stone <ellestone@ninedegreesbelow.com>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_COLOR_H_INSIDE__) && !defined (PIKA_COLOR_COMPILATION)
#error "Only <libpikacolor/pikacolor.h> can be included directly."
#endif

#ifndef __PIKA_COLOR_PROFILE_H__
#define __PIKA_COLOR_PROFILE_H__

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


#define PIKA_TYPE_COLOR_PROFILE            (pika_color_profile_get_type ())
#define PIKA_COLOR_PROFILE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_COLOR_PROFILE, PikaColorProfile))
#define PIKA_COLOR_PROFILE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_COLOR_PROFILE, PikaColorProfileClass))
#define PIKA_IS_COLOR_PROFILE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_COLOR_PROFILE))
#define PIKA_IS_COLOR_PROFILE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_COLOR_PROFILE))
#define PIKA_COLOR_PROFILE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_COLOR_PROFILE, PikaColorProfileClass))


typedef struct _PikaColorProfilePrivate PikaColorProfilePrivate;
typedef struct _PikaColorProfileClass   PikaColorProfileClass;

struct _PikaColorProfile
{
  GObject                  parent_instance;

  PikaColorProfilePrivate *priv;
};

struct _PikaColorProfileClass
{
  GObjectClass  parent_class;

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType              pika_color_profile_get_type              (void) G_GNUC_CONST;

PikaColorProfile * pika_color_profile_new_rgb_srgb          (void);
PikaColorProfile * pika_color_profile_new_rgb_srgb_linear   (void);
PikaColorProfile * pika_color_profile_new_rgb_adobe         (void);

PikaColorProfile * pika_color_profile_new_d65_gray_srgb_trc (void);
PikaColorProfile * pika_color_profile_new_d65_gray_linear   (void);
PikaColorProfile * pika_color_profile_new_d50_gray_lab_trc  (void);

PikaColorProfile *
       pika_color_profile_new_srgb_trc_from_color_profile   (PikaColorProfile  *profile);
PikaColorProfile *
       pika_color_profile_new_linear_from_color_profile     (PikaColorProfile  *profile);

PikaColorProfile * pika_color_profile_new_from_file         (GFile             *file,
                                                             GError           **error);

PikaColorProfile * pika_color_profile_new_from_icc_profile  (const guint8      *data,
                                                             gsize              length,
                                                             GError           **error);
PikaColorProfile * pika_color_profile_new_from_lcms_profile (gpointer           lcms_profile,
                                                             GError           **error);

gboolean           pika_color_profile_save_to_file          (PikaColorProfile  *profile,
                                                             GFile             *file,
                                                             GError           **error);

const guint8     * pika_color_profile_get_icc_profile       (PikaColorProfile  *profile,
                                                             gsize             *length);
gpointer           pika_color_profile_get_lcms_profile      (PikaColorProfile  *profile);

const gchar      * pika_color_profile_get_description       (PikaColorProfile  *profile);
const gchar      * pika_color_profile_get_manufacturer      (PikaColorProfile  *profile);
const gchar      * pika_color_profile_get_model             (PikaColorProfile  *profile);
const gchar      * pika_color_profile_get_copyright         (PikaColorProfile  *profile);

const gchar      * pika_color_profile_get_label             (PikaColorProfile  *profile);
const gchar      * pika_color_profile_get_summary           (PikaColorProfile  *profile);

gboolean           pika_color_profile_is_equal              (PikaColorProfile  *profile1,
                                                             PikaColorProfile  *profile2);

gboolean           pika_color_profile_is_rgb                (PikaColorProfile  *profile);
gboolean           pika_color_profile_is_gray               (PikaColorProfile  *profile);
gboolean           pika_color_profile_is_cmyk               (PikaColorProfile  *profile);

gboolean           pika_color_profile_is_linear             (PikaColorProfile  *profile);

const Babl       * pika_color_profile_get_space             (PikaColorProfile  *profile,
                                                             PikaColorRenderingIntent intent,
                                                             GError           **error);
const Babl       * pika_color_profile_get_format            (PikaColorProfile  *profile,
                                                             const Babl        *format,
                                                             PikaColorRenderingIntent intent,
                                                             GError           **error);

const Babl       * pika_color_profile_get_lcms_format       (const Babl        *format,
                                                             guint32           *lcms_format);


G_END_DECLS

#endif  /* __PIKA_COLOR_PROFILE_H__ */
