/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * PikaColorConfig class
 * Copyright (C) 2004  Stefan Döhla <stefan@doehla.de>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_CONFIG_H_INSIDE__) && !defined (PIKA_CONFIG_COMPILATION)
#error "Only <libpikaconfig/pikaconfig.h> can be included directly."
#endif

#ifndef __PIKA_COLOR_CONFIG_H__
#define __PIKA_COLOR_CONFIG_H__


#define PIKA_TYPE_COLOR_CONFIG            (pika_color_config_get_type ())
#define PIKA_COLOR_CONFIG(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_COLOR_CONFIG, PikaColorConfig))
#define PIKA_COLOR_CONFIG_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_COLOR_CONFIG, PikaColorConfigClass))
#define PIKA_IS_COLOR_CONFIG(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_COLOR_CONFIG))
#define PIKA_IS_COLOR_CONFIG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_COLOR_CONFIG))


typedef struct _PikaColorConfigPrivate PikaColorConfigPrivate;
typedef struct _PikaColorConfigClass   PikaColorConfigClass;

struct _PikaColorConfig
{
  GObject                 parent_instance;

  PikaColorConfigPrivate *priv;
};

struct _PikaColorConfigClass
{
  GObjectClass  parent_class;

  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType              pika_color_config_get_type                     (void) G_GNUC_CONST;

PikaColorManagementMode
                   pika_color_config_get_mode                     (PikaColorConfig  *config);

PikaColorRenderingIntent
                   pika_color_config_get_display_intent           (PikaColorConfig  *config);
gboolean           pika_color_config_get_display_bpc              (PikaColorConfig  *config);
gboolean           pika_color_config_get_display_optimize         (PikaColorConfig  *config);
gboolean           pika_color_config_get_display_profile_from_gdk (PikaColorConfig  *config);

PikaColorRenderingIntent
                   pika_color_config_get_simulation_intent        (PikaColorConfig  *config);
gboolean           pika_color_config_get_simulation_bpc           (PikaColorConfig  *config);
gboolean           pika_color_config_get_simulation_optimize      (PikaColorConfig  *config);
gboolean           pika_color_config_get_simulation_gamut_check   (PikaColorConfig  *config);
void               pika_color_config_get_out_of_gamut_color       (PikaColorConfig  *config,
                                                                   PikaRGB          *color);

PikaColorProfile * pika_color_config_get_rgb_color_profile        (PikaColorConfig  *config,
                                                                   GError          **error);
PikaColorProfile * pika_color_config_get_gray_color_profile       (PikaColorConfig  *config,
                                                                   GError          **error);
PikaColorProfile * pika_color_config_get_cmyk_color_profile       (PikaColorConfig  *config,
                                                                   GError          **error);
PikaColorProfile * pika_color_config_get_display_color_profile    (PikaColorConfig  *config,
                                                                   GError          **error);
PikaColorProfile * pika_color_config_get_simulation_color_profile (PikaColorConfig  *config,
                                                                   GError          **error);


#endif /* PIKA_COLOR_CONFIG_H__ */
