/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * Thumbnail handling according to the Thumbnail Managing Standard.
 * https://specifications.freedesktop.org/thumbnail-spec/
 *
 * Copyright (C) 2001-2003  Sven Neumann <sven@gimp.org>
 *                          Michael Natterer <mitch@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_THUMB_ENUMS_H__
#define __PIKA_THUMB_ENUMS_H__

G_BEGIN_DECLS


/**
 * SECTION: pikathumb-enums
 * @title: PikaThumb-enums
 * @short_description: Enumerations used by libpikathumb
 *
 * Enumerations used by libpikathumb
 **/


/**
 * PikaThumbFileType:
 * @PIKA_THUMB_FILE_TYPE_NONE:    file does not exist
 * @PIKA_THUMB_FILE_TYPE_REGULAR: a regular file
 * @PIKA_THUMB_FILE_TYPE_FOLDER:  a directory
 * @PIKA_THUMB_FILE_TYPE_SPECIAL: a special file (device node, fifo, socket, ...)
 *
 * File types as returned by pika_thumb_file_test().
 **/
#define PIKA_TYPE_THUMB_FILE_TYPE (pika_thumb_file_type_get_type ())

GType pika_thumb_file_type_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_THUMB_FILE_TYPE_NONE,
  PIKA_THUMB_FILE_TYPE_REGULAR,
  PIKA_THUMB_FILE_TYPE_FOLDER,
  PIKA_THUMB_FILE_TYPE_SPECIAL
} PikaThumbFileType;


/**
 * PikaThumbSize:
 * @PIKA_THUMB_SIZE_FAIL:   special size used to indicate a thumbnail
 *                          creation failure
 * @PIKA_THUMB_SIZE_NORMAL: normal thumbnail size (128 pixels)
 * @PIKA_THUMB_SIZE_LARGE:  large thumbnail size (256 pixels)
 *
 * Possible thumbnail sizes as defined by the Thumbnail Managing
 * Standard.
 **/
#define PIKA_TYPE_THUMB_SIZE (pika_thumb_size_get_type ())

GType pika_thumb_size_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_THUMB_SIZE_FAIL   = 0,
  PIKA_THUMB_SIZE_NORMAL = 128,
  PIKA_THUMB_SIZE_LARGE  = 256
} PikaThumbSize;


/**
 * PikaThumbState:
 * @PIKA_THUMB_STATE_UNKNOWN:   nothing is known about the file/thumbnail
 * @PIKA_THUMB_STATE_REMOTE:    the file is on a remote file system
 * @PIKA_THUMB_STATE_FOLDER:    the file is a directory
 * @PIKA_THUMB_STATE_SPECIAL:   the file is a special file
 * @PIKA_THUMB_STATE_NOT_FOUND: the file/thumbnail doesn't exist
 * @PIKA_THUMB_STATE_EXISTS:    the file/thumbnail exists
 * @PIKA_THUMB_STATE_OLD:       the thumbnail may be outdated
 * @PIKA_THUMB_STATE_FAILED:    the thumbnail couldn't be created
 * @PIKA_THUMB_STATE_OK:        the thumbnail exists and matches the image
 *
 * Possible image and thumbnail file states used by libpikathumb.
 **/
#define PIKA_TYPE_THUMB_STATE (pika_thumb_state_get_type ())

GType pika_thumb_state_get_type (void) G_GNUC_CONST;

typedef enum
{
  PIKA_THUMB_STATE_UNKNOWN,
  PIKA_THUMB_STATE_REMOTE,
  PIKA_THUMB_STATE_FOLDER,
  PIKA_THUMB_STATE_SPECIAL,
  PIKA_THUMB_STATE_NOT_FOUND,
  PIKA_THUMB_STATE_EXISTS,
  PIKA_THUMB_STATE_OLD,
  PIKA_THUMB_STATE_FAILED,
  PIKA_THUMB_STATE_OK
} PikaThumbState;


G_END_DECLS

#endif  /* __PIKA_THUMB_ENUMS_H__ */
