/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikabrowser.h
 * Copyright (C) 2005 Michael Natterer <mitch@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_BROWSER_H__
#define __PIKA_BROWSER_H__

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


#define PIKA_TYPE_BROWSER            (pika_browser_get_type ())
#define PIKA_BROWSER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_BROWSER, PikaBrowser))
#define PIKA_BROWSER_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_BROWSER, PikaBrowserClass))
#define PIKA_IS_BROWSER(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_BROWSER))
#define PIKA_IS_BROWSER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_BROWSER))
#define PIKA_BROWSER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_BROWSER, PikaBrowserClass))


typedef struct _PikaBrowserPrivate PikaBrowserPrivate;
typedef struct _PikaBrowserClass   PikaBrowserClass;

struct _PikaBrowser
{
  GtkPaned            parent_instance;

  PikaBrowserPrivate *priv;
};

struct _PikaBrowserClass
{
  GtkPanedClass  parent_class;

  void (* search) (PikaBrowser *browser,
                   const gchar *search_string,
                   gint         search_type);

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType       pika_browser_get_type         (void) G_GNUC_CONST;

GtkWidget * pika_browser_new                (void);

void        pika_browser_add_search_types   (PikaBrowser *browser,
                                             const gchar *first_type_label,
                                             gint         first_type_id,
                                             ...) G_GNUC_NULL_TERMINATED;

GtkWidget * pika_browser_get_left_vbox      (PikaBrowser *browser);
GtkWidget * pika_browser_get_right_vbox     (PikaBrowser *browser);

void        pika_browser_set_search_summary (PikaBrowser *browser,
                                             const gchar *summary);
void        pika_browser_set_widget         (PikaBrowser *browser,
                                             GtkWidget   *widget);
void        pika_browser_show_message       (PikaBrowser *browser,
                                             const gchar *message);


G_END_DECLS

#endif  /*  __PIKA_BROWSER_H__  */
