/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikacolorarea.h
 * Copyright (C) 2001-2002  Sven Neumann <sven@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

/* This provides a color preview area. The preview
 * can handle transparency by showing the checkerboard and
 * handles drag'n'drop.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_COLOR_AREA_H__
#define __PIKA_COLOR_AREA_H__

G_BEGIN_DECLS


#define PIKA_TYPE_COLOR_AREA            (pika_color_area_get_type ())
#define PIKA_COLOR_AREA(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_COLOR_AREA, PikaColorArea))
#define PIKA_COLOR_AREA_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_COLOR_AREA, PikaColorAreaClass))
#define PIKA_IS_COLOR_AREA(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_COLOR_AREA))
#define PIKA_IS_COLOR_AREA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_COLOR_AREA))
#define PIKA_COLOR_AREA_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_COLOR_AREA, PikaColorAreaClass))


typedef struct _PikaColorAreaPrivate PikaColorAreaPrivate;
typedef struct _PikaColorAreaClass   PikaColorAreaClass;

struct _PikaColorArea
{
  GtkDrawingArea        parent_instance;

  PikaColorAreaPrivate *priv;
};

struct _PikaColorAreaClass
{
  GtkDrawingAreaClass  parent_class;

  void (* color_changed) (PikaColorArea *area);

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType       pika_color_area_get_type         (void) G_GNUC_CONST;

GtkWidget * pika_color_area_new              (const PikaRGB     *color,
                                              PikaColorAreaType  type,
                                              GdkModifierType    drag_mask);

void        pika_color_area_set_color        (PikaColorArea     *area,
                                              const PikaRGB     *color);
void        pika_color_area_get_color        (PikaColorArea     *area,
                                              PikaRGB           *color);

gboolean    pika_color_area_has_alpha        (PikaColorArea     *area);
void        pika_color_area_set_type         (PikaColorArea     *area,
                                              PikaColorAreaType  type);
void        pika_color_area_enable_drag      (PikaColorArea     *area,
                                              GdkModifierType    drag_mask);
void        pika_color_area_set_draw_border  (PikaColorArea     *area,
                                              gboolean           draw_border);
void        pika_color_area_set_out_of_gamut (PikaColorArea     *area,
                                              gboolean           out_of_gamut);

void        pika_color_area_set_color_config (PikaColorArea     *area,
                                              PikaColorConfig   *config);


G_END_DECLS

#endif /* __PIKA_COLOR_AREA_H__ */
