/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikacolordisplay.c
 * Copyright (C) 1999 Manish Singh <yosh@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_COLOR_DISPLAY_H__
#define __PIKA_COLOR_DISPLAY_H__

G_BEGIN_DECLS

/* For information look at the html documentation */


#define PIKA_TYPE_COLOR_DISPLAY            (pika_color_display_get_type ())
#define PIKA_COLOR_DISPLAY(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_COLOR_DISPLAY, PikaColorDisplay))
#define PIKA_COLOR_DISPLAY_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_COLOR_DISPLAY, PikaColorDisplayClass))
#define PIKA_IS_COLOR_DISPLAY(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_COLOR_DISPLAY))
#define PIKA_IS_COLOR_DISPLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_COLOR_DISPLAY))
#define PIKA_COLOR_DISPLAY_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_COLOR_DISPLAY, PikaColorDisplayClass))


typedef struct _PikaColorDisplayPrivate PikaColorDisplayPrivate;
typedef struct _PikaColorDisplayClass   PikaColorDisplayClass;

struct _PikaColorDisplay
{
  GObject                 parent_instance;

  PikaColorDisplayPrivate *priv;
};

struct _PikaColorDisplayClass
{
  GObjectClass  parent_class;

  const gchar  *name;
  const gchar  *help_id;
  const gchar  *icon_name;

  /*  virtual functions  */
  void        (* convert_buffer) (PikaColorDisplay *display,
                                  GeglBuffer       *buffer,
                                  GeglRectangle    *area);
  GtkWidget * (* configure)      (PikaColorDisplay *display);

  /*  signals  */
  void        (* changed)        (PikaColorDisplay *display);

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType              pika_color_display_get_type        (void) G_GNUC_CONST;

PikaColorDisplay * pika_color_display_clone           (PikaColorDisplay *display);

void               pika_color_display_convert_buffer  (PikaColorDisplay *display,
                                                       GeglBuffer       *buffer,
                                                       GeglRectangle    *area);
void               pika_color_display_load_state      (PikaColorDisplay *display,
                                                       PikaParasite     *state);
PikaParasite     * pika_color_display_save_state      (PikaColorDisplay *display);
GtkWidget        * pika_color_display_configure       (PikaColorDisplay *display);
void               pika_color_display_configure_reset (PikaColorDisplay *display);

void               pika_color_display_changed         (PikaColorDisplay *display);

void               pika_color_display_set_enabled     (PikaColorDisplay *display,
                                                       gboolean          enabled);
gboolean           pika_color_display_get_enabled     (PikaColorDisplay *display);

PikaColorConfig  * pika_color_display_get_config      (PikaColorDisplay *display);
PikaColorManaged * pika_color_display_get_managed     (PikaColorDisplay *display);


G_END_DECLS

#endif /* __PIKA_COLOR_DISPLAY_H__ */
