/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikacolordisplaystack.h
 * Copyright (C) 2003 Michael Natterer <mitch@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_COLOR_DISPLAY_STACK_H__
#define __PIKA_COLOR_DISPLAY_STACK_H__

G_BEGIN_DECLS

/* For information look at the html documentation */


#define PIKA_TYPE_COLOR_DISPLAY_STACK            (pika_color_display_stack_get_type ())
#define PIKA_COLOR_DISPLAY_STACK(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_COLOR_DISPLAY_STACK, PikaColorDisplayStack))
#define PIKA_COLOR_DISPLAY_STACK_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_COLOR_DISPLAY_STACK, PikaColorDisplayStackClass))
#define PIKA_IS_COLOR_DISPLAY_STACK(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_COLOR_DISPLAY_STACK))
#define PIKA_IS_COLOR_DISPLAY_STACK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_COLOR_DISPLAY_STACK))
#define PIKA_COLOR_DISPLAY_STACK_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_COLOR_DISPLAY_STACK, PikaColorDisplayStackClass))


typedef struct _PikaColorDisplayStackPrivate PikaColorDisplayStackPrivate;
typedef struct _PikaColorDisplayStackClass   PikaColorDisplayStackClass;

struct _PikaColorDisplayStack
{
  GObject                       parent_instance;

  PikaColorDisplayStackPrivate *priv;
};

struct _PikaColorDisplayStackClass
{
  GObjectClass  parent_class;

  void (* changed)   (PikaColorDisplayStack *stack);

  void (* added)     (PikaColorDisplayStack *stack,
                      PikaColorDisplay      *display,
                      gint                   position);
  void (* removed)   (PikaColorDisplayStack *stack,
                      PikaColorDisplay      *display);
  void (* reordered) (PikaColorDisplayStack *stack,
                      PikaColorDisplay      *display,
                      gint                   position);

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType                   pika_color_display_stack_get_type (void) G_GNUC_CONST;
PikaColorDisplayStack * pika_color_display_stack_new      (void);
PikaColorDisplayStack * pika_color_display_stack_clone    (PikaColorDisplayStack *stack);

void    pika_color_display_stack_changed         (PikaColorDisplayStack *stack);

GList * pika_color_display_stack_get_filters     (PikaColorDisplayStack *stack);

void    pika_color_display_stack_add             (PikaColorDisplayStack *stack,
                                                  PikaColorDisplay      *display);
void    pika_color_display_stack_remove          (PikaColorDisplayStack *stack,
                                                  PikaColorDisplay      *display);
void    pika_color_display_stack_reorder_up      (PikaColorDisplayStack *stack,
                                                  PikaColorDisplay      *display);
void    pika_color_display_stack_reorder_down    (PikaColorDisplayStack *stack,
                                                  PikaColorDisplay      *display);
void    pika_color_display_stack_convert_buffer  (PikaColorDisplayStack *stack,
                                                  GeglBuffer            *buffer,
                                                  GeglRectangle         *area);


G_END_DECLS

#endif /* __PIKA_COLOR_DISPLAY_STACK_H__ */
