/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikacolorselection.h
 * Copyright (C) 2003 Michael Natterer <mitch@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_COLOR_SELECTION_H__
#define __PIKA_COLOR_SELECTION_H__

G_BEGIN_DECLS

/* For information look at the html documentation */


#define PIKA_TYPE_COLOR_SELECTION            (pika_color_selection_get_type ())
#define PIKA_COLOR_SELECTION(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_COLOR_SELECTION, PikaColorSelection))
#define PIKA_COLOR_SELECTION_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_COLOR_SELECTION, PikaColorSelectionClass))
#define PIKA_IS_COLOR_SELECTION(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_COLOR_SELECTION))
#define PIKA_IS_COLOR_SELECTION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_COLOR_SELECTION))
#define PIKA_COLOR_SELECTION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_COLOR_SELECTION, PikaColorSelectionClass))


typedef struct _PikaColorSelectionPrivate PikaColorSelectionPrivate;
typedef struct _PikaColorSelectionClass   PikaColorSelectionClass;

struct _PikaColorSelection
{
  GtkBox                     parent_instance;

  PikaColorSelectionPrivate *priv;
};

struct _PikaColorSelectionClass
{
  GtkBoxClass  parent_class;

  void (* color_changed) (PikaColorSelection *selection);

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType       pika_color_selection_get_type       (void) G_GNUC_CONST;

GtkWidget * pika_color_selection_new            (void);

void        pika_color_selection_set_show_alpha (PikaColorSelection *selection,
                                                 gboolean           show_alpha);
gboolean    pika_color_selection_get_show_alpha (PikaColorSelection *selection);

void        pika_color_selection_set_color      (PikaColorSelection *selection,
                                                 const PikaRGB      *color);
void        pika_color_selection_get_color      (PikaColorSelection *selection,
                                                 PikaRGB            *color);

void        pika_color_selection_set_old_color  (PikaColorSelection *selection,
                                                 const PikaRGB      *color);
void        pika_color_selection_get_old_color  (PikaColorSelection *selection,
                                                 PikaRGB            *color);

void        pika_color_selection_reset          (PikaColorSelection *selection);

void        pika_color_selection_color_changed  (PikaColorSelection *selection);

void        pika_color_selection_set_simulation (PikaColorSelection *selection,
                                                 PikaColorProfile   *profile,
                                                 PikaColorRenderingIntent intent,
                                                 gboolean            bpc);

void        pika_color_selection_set_config     (PikaColorSelection *selection,
                                                 PikaColorConfig    *config);

GtkWidget * pika_color_selection_get_notebook   (PikaColorSelection *selection);
GtkWidget * pika_color_selection_get_right_vbox (PikaColorSelection *selection);


G_END_DECLS

#endif /* __PIKA_COLOR_SELECTION_H__ */
