/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikaenumstore.h
 * Copyright (C) 2004  Sven Neumann <sven@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_ENUM_STORE_H__
#define __PIKA_ENUM_STORE_H__

#include <libpikawidgets/pikaintstore.h>


G_BEGIN_DECLS

#define PIKA_TYPE_ENUM_STORE            (pika_enum_store_get_type ())
#define PIKA_ENUM_STORE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_ENUM_STORE, PikaEnumStore))
#define PIKA_ENUM_STORE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_ENUM_STORE, PikaEnumStoreClass))
#define PIKA_IS_ENUM_STORE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_ENUM_STORE))
#define PIKA_IS_ENUM_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_ENUM_STORE))
#define PIKA_ENUM_STORE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_ENUM_STORE, PikaEnumStoreClass))


typedef struct _PikaEnumStorePrivate PikaEnumStorePrivate;
typedef struct _PikaEnumStoreClass   PikaEnumStoreClass;

struct _PikaEnumStore
{
  PikaIntStore          parent_instance;

  PikaEnumStorePrivate *priv;
};

struct _PikaEnumStoreClass
{
  PikaIntStoreClass  parent_class;

  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType          pika_enum_store_get_type               (void) G_GNUC_CONST;

GtkListStore * pika_enum_store_new                    (GType    enum_type);
GtkListStore * pika_enum_store_new_with_range         (GType    enum_type,
                                                       gint     minimum,
                                                       gint     maximum);
GtkListStore * pika_enum_store_new_with_values        (GType    enum_type,
                                                       gint     n_values,
                                                       ...);
GtkListStore * pika_enum_store_new_with_values_valist (GType    enum_type,
                                                       gint     n_values,
                                                       va_list  args);

void           pika_enum_store_set_icon_prefix  (PikaEnumStore *store,
                                                 const gchar   *icon_prefix);


G_END_DECLS

#endif  /* __PIKA_ENUM_STORE_H__ */
