/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikaicons.h
 * Copyright (C) 2001-2015 Michael Natterer <mitch@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_ICONS_H__
#define __PIKA_ICONS_H__

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


/*  random actions that don't fit in any category  */

#define PIKA_ICON_ATTACH                    "pika-attach"
#define PIKA_ICON_DETACH                    "pika-detach"
#define PIKA_ICON_INVERT                    "pika-invert"
#define PIKA_ICON_RECORD                    "media-record"
#define PIKA_ICON_RESET                     "pika-reset"
#define PIKA_ICON_SHRED                     "pika-shred"


/*  random states/things that don't fit in any category  */

#define PIKA_ICON_BUSINESS_CARD             "pika-business-card"
#define PIKA_ICON_CHAR_PICKER               "pika-char-picker"
#define PIKA_ICON_CURSOR                    "pika-cursor"
#define PIKA_ICON_DISPLAY                   "pika-display"
#define PIKA_ICON_GEGL                      "pika-gegl"
#define PIKA_ICON_LINKED                    "pika-linked"
#define PIKA_ICON_MARKER                    "pika-marker"
#define PIKA_ICON_SMARTPHONE                "pika-smartphone"
#define PIKA_ICON_TRANSPARENCY              "pika-transparency"
#define PIKA_ICON_VIDEO                     "pika-video"
#define PIKA_ICON_VISIBLE                   "pika-visible"
#define PIKA_ICON_WEB                       "pika-web"


/*  random objects/entities that don't fit in any category  */

#define PIKA_ICON_BRUSH                     PIKA_ICON_TOOL_PAINTBRUSH
#define PIKA_ICON_BUFFER                    PIKA_ICON_EDIT_PASTE
#define PIKA_ICON_COLORMAP                  "pika-colormap"
#define PIKA_ICON_DYNAMICS                  "pika-dynamics"
#define PIKA_ICON_FILE_MANAGER              "pika-file-manager"
#define PIKA_ICON_FONT                      "gtk-select-font"
#define PIKA_ICON_GRADIENT                  PIKA_ICON_TOOL_GRADIENT
#define PIKA_ICON_GRID                      "pika-grid"
#define PIKA_ICON_INPUT_DEVICE              "pika-input-device"
#define PIKA_ICON_MYPAINT_BRUSH             PIKA_ICON_TOOL_MYPAINT_BRUSH
#define PIKA_ICON_PALETTE                   "gtk-select-color"
#define PIKA_ICON_PATTERN                   "pika-pattern"
#define PIKA_ICON_PLUGIN                    "pika-plugin"
#define PIKA_ICON_SAMPLE_POINT              "pika-sample-point"
#define PIKA_ICON_SYMMETRY                  "pika-symmetry"
#define PIKA_ICON_TEMPLATE                  "pika-template"
#define PIKA_ICON_TOOL_PRESET               "pika-tool-preset"


/*  not really icons  */

#define PIKA_ICON_FRAME                     "pika-frame"
#define PIKA_ICON_TEXTURE                   "pika-texture"


/*  icons that follow, or at least try to follow the FDO naming and
 *  category conventions; and groups of icons with a common prefix;
 *  all sorted alphabetically
 *
 *  see also:
 *  https://specifications.freedesktop.org/icon-naming-spec/latest/ar01s04.html
 *
 *  When icons are available as standard Freedesktop icons, we use these
 *  in priority. As a fallback, we use standard GTK+ icons. As last
 *  fallback, we create our own icons under the "pika-" namespace.
 */

#define PIKA_ICON_APPLICATION_EXIT          "application-exit"

#define PIKA_ICON_ASPECT_PORTRAIT           "pika-portrait"
#define PIKA_ICON_ASPECT_LANDSCAPE          "pika-landscape"

#define PIKA_ICON_CAP_BUTT                  "pika-cap-butt"
#define PIKA_ICON_CAP_ROUND                 "pika-cap-round"
#define PIKA_ICON_CAP_SQUARE                "pika-cap-square"

#define PIKA_ICON_CENTER                    "pika-center"
#define PIKA_ICON_CENTER_HORIZONTAL         "pika-hcenter"
#define PIKA_ICON_CENTER_VERTICAL           "pika-vcenter"

#define PIKA_ICON_CHAIN_HORIZONTAL          "pika-hchain"
#define PIKA_ICON_CHAIN_HORIZONTAL_BROKEN   "pika-hchain-broken"
#define PIKA_ICON_CHAIN_VERTICAL            "pika-vchain"
#define PIKA_ICON_CHAIN_VERTICAL_BROKEN     "pika-vchain-broken"

#define PIKA_ICON_CHANNEL                   "pika-channel"
#define PIKA_ICON_CHANNEL_ALPHA             "pika-channel-alpha"
#define PIKA_ICON_CHANNEL_BLUE              "pika-channel-blue"
#define PIKA_ICON_CHANNEL_GRAY              "pika-channel-gray"
#define PIKA_ICON_CHANNEL_GREEN             "pika-channel-green"
#define PIKA_ICON_CHANNEL_INDEXED           "pika-channel-indexed"
#define PIKA_ICON_CHANNEL_RED               "pika-channel-red"

#define PIKA_ICON_CLOSE                     "pika-close"
#define PIKA_ICON_CLOSE_ALL                 "pika-close-all"

#define PIKA_ICON_COLOR_PICKER_BLACK        "pika-color-picker-black"
#define PIKA_ICON_COLOR_PICKER_GRAY         "pika-color-picker-gray"
#define PIKA_ICON_COLOR_PICKER_WHITE        "pika-color-picker-white"
#define PIKA_ICON_COLOR_PICK_FROM_SCREEN    "pika-color-pick-from-screen"

#define PIKA_ICON_COLOR_SELECTOR_CMYK       "pika-color-cmyk"
#define PIKA_ICON_COLOR_SELECTOR_TRIANGLE   "pika-color-triangle"
#define PIKA_ICON_COLOR_SELECTOR_WATER      "pika-color-water"

#define PIKA_ICON_COLOR_SPACE_LINEAR        "pika-color-space-linear"
#define PIKA_ICON_COLOR_SPACE_NON_LINEAR    "pika-color-space-non-linear"
#define PIKA_ICON_COLOR_SPACE_PERCEPTUAL    "pika-color-space-perceptual"

#define PIKA_ICON_COLORS_DEFAULT            "pika-default-colors"
#define PIKA_ICON_COLORS_SWAP               "pika-swap-colors"

#define PIKA_ICON_CONTROLLER                "pika-controller"
#define PIKA_ICON_CONTROLLER_KEYBOARD       "pika-controller-keyboard"
#define PIKA_ICON_CONTROLLER_LINUX_INPUT    "pika-controller-linux-input"
#define PIKA_ICON_CONTROLLER_MIDI           "pika-controller-midi"
#define PIKA_ICON_CONTROLLER_MOUSE          PIKA_ICON_CURSOR
#define PIKA_ICON_CONTROLLER_WHEEL          "pika-controller-wheel"

#define PIKA_ICON_CONVERT_RGB               "pika-convert-rgb"
#define PIKA_ICON_CONVERT_GRAYSCALE         "pika-convert-grayscale"
#define PIKA_ICON_CONVERT_INDEXED           "pika-convert-indexed"
#define PIKA_ICON_CONVERT_PRECISION         PIKA_ICON_CONVERT_RGB

#define PIKA_ICON_CURVE_FREE                "pika-curve-free"
#define PIKA_ICON_CURVE_SMOOTH              "pika-curve-smooth"

#define PIKA_ICON_DIALOG_CHANNELS           "pika-channels"
#define PIKA_ICON_DIALOG_DASHBOARD          "pika-dashboard"
#define PIKA_ICON_DIALOG_DEVICE_STATUS      "pika-device-status"
#define PIKA_ICON_DIALOG_ERROR              "dialog-error"
#define PIKA_ICON_DIALOG_IMAGES             "pika-images"
#define PIKA_ICON_DIALOG_INFORMATION        "dialog-information"
#define PIKA_ICON_DIALOG_LAYERS             "pika-layers"
#define PIKA_ICON_DIALOG_NAVIGATION         "pika-navigation"
#define PIKA_ICON_DIALOG_PATHS              "pika-paths"
#define PIKA_ICON_DIALOG_QUESTION           "dialog-question"
#define PIKA_ICON_DIALOG_RESHOW_FILTER      "pika-reshow-filter"
#define PIKA_ICON_DIALOG_TOOLS              "pika-tools"
#define PIKA_ICON_DIALOG_TOOL_OPTIONS       "pika-tool-options"
#define PIKA_ICON_DIALOG_UNDO_HISTORY       "pika-undo-history"
#define PIKA_ICON_DIALOG_WARNING            "dialog-warning"

#define PIKA_ICON_DISPLAY_FILTER              "pika-display-filter"
#define PIKA_ICON_DISPLAY_FILTER_CLIP_WARNING "pika-display-filter-clip-warning"
#define PIKA_ICON_DISPLAY_FILTER_COLORBLIND   "pika-display-filter-colorblind"
#define PIKA_ICON_DISPLAY_FILTER_CONTRAST     "pika-display-filter-contrast"
#define PIKA_ICON_DISPLAY_FILTER_GAMMA        "pika-display-filter-gamma"
#define PIKA_ICON_DISPLAY_FILTER_LCMS         "pika-display-filter-lcms"
#define PIKA_ICON_DISPLAY_FILTER_PROOF        "pika-display-filter-proof"

#define PIKA_ICON_LOCK                      "pika-lock"
#define PIKA_ICON_LOCK_ALPHA                "pika-lock-alpha"
#define PIKA_ICON_LOCK_CONTENT              "pika-lock-content"
#define PIKA_ICON_LOCK_POSITION             "pika-lock-position"
#define PIKA_ICON_LOCK_VISIBILITY           "pika-lock-visibility"
#define PIKA_ICON_LOCK_MULTI                "pika-lock-multi"

#define PIKA_ICON_DOCUMENT_NEW              "document-new"
#define PIKA_ICON_DOCUMENT_OPEN             "document-open"
#define PIKA_ICON_DOCUMENT_OPEN_RECENT      "document-open-recent"
#define PIKA_ICON_DOCUMENT_PAGE_SETUP       "document-page-setup"
#define PIKA_ICON_DOCUMENT_PRINT            "document-print"
#define PIKA_ICON_DOCUMENT_PRINT_RESOLUTION "document-print"
#define PIKA_ICON_DOCUMENT_PROPERTIES       "document-properties"
#define PIKA_ICON_DOCUMENT_REVERT           "document-revert"
#define PIKA_ICON_DOCUMENT_SAVE             "document-save"
#define PIKA_ICON_DOCUMENT_SAVE_AS          "document-save-as"

#define PIKA_ICON_EDIT                      "gtk-edit"
#define PIKA_ICON_EDIT_CLEAR                "edit-clear"
#define PIKA_ICON_EDIT_COPY                 "edit-copy"
#define PIKA_ICON_EDIT_CUT                  "edit-cut"
#define PIKA_ICON_EDIT_DELETE               "edit-delete"
#define PIKA_ICON_EDIT_FIND                 "edit-find"
#define PIKA_ICON_EDIT_PASTE                "edit-paste"
#define PIKA_ICON_EDIT_PASTE_AS_NEW         "pika-paste-as-new"
#define PIKA_ICON_EDIT_PASTE_INTO           "pika-paste-into"
#define PIKA_ICON_EDIT_REDO                 "edit-redo"
#define PIKA_ICON_EDIT_UNDO                 "edit-undo"

#define PIKA_ICON_EVEN_HORIZONTAL_GAP       "pika-even-horizontal-gap"
#define PIKA_ICON_EVEN_VERTICAL_GAP         "pika-even-vertical-gap"

#define PIKA_ICON_FILL_HORIZONTAL           "pika-hfill"
#define PIKA_ICON_FILL_VERTICAL             "pika-vfill"

#define PIKA_ICON_FOLDER_NEW                "folder-new"

#define PIKA_ICON_FORMAT_INDENT_MORE         "format-indent-more"
#define PIKA_ICON_FORMAT_INDENT_LESS         "format-indent-less"
#define PIKA_ICON_FORMAT_JUSTIFY_CENTER      "format-justify-center"
#define PIKA_ICON_FORMAT_JUSTIFY_FILL        "format-justify-fill"
#define PIKA_ICON_FORMAT_JUSTIFY_LEFT        "format-justify-left"
#define PIKA_ICON_FORMAT_JUSTIFY_RIGHT       "format-justify-right"
#define PIKA_ICON_FORMAT_TEXT_BOLD           "format-text-bold"
#define PIKA_ICON_FORMAT_TEXT_ITALIC         "format-text-italic"
#define PIKA_ICON_FORMAT_TEXT_STRIKETHROUGH  "format-text-strikethrough"
#define PIKA_ICON_FORMAT_TEXT_UNDERLINE      "format-text-underline"
#define PIKA_ICON_FORMAT_TEXT_DIRECTION_LTR  "format-text-direction-ltr"
#define PIKA_ICON_FORMAT_TEXT_DIRECTION_RTL  "format-text-direction-rtl"
#define PIKA_ICON_FORMAT_TEXT_DIRECTION_TTB_RTL           "pika-text-dir-ttb-rtl" /* use FDO */
#define PIKA_ICON_FORMAT_TEXT_DIRECTION_TTB_RTL_UPRIGHT   "pika-text-dir-ttb-rtl-upright" /* use FDO */
#define PIKA_ICON_FORMAT_TEXT_DIRECTION_TTB_LTR           "pika-text-dir-ttb-ltr" /* use FDO */
#define PIKA_ICON_FORMAT_TEXT_DIRECTION_TTB_LTR_UPRIGHT   "pika-text-dir-ttb-ltr-upright" /* use FDO */
#define PIKA_ICON_FORMAT_TEXT_SPACING_LETTER "pika-letter-spacing"
#define PIKA_ICON_FORMAT_TEXT_SPACING_LINE   "pika-line-spacing"

#define PIKA_ICON_GRADIENT_LINEAR               "pika-gradient-linear"
#define PIKA_ICON_GRADIENT_BILINEAR             "pika-gradient-bilinear"
#define PIKA_ICON_GRADIENT_RADIAL               "pika-gradient-radial"
#define PIKA_ICON_GRADIENT_SQUARE               "pika-gradient-square"
#define PIKA_ICON_GRADIENT_CONICAL_SYMMETRIC    "pika-gradient-conical-symmetric"
#define PIKA_ICON_GRADIENT_CONICAL_ASYMMETRIC   "pika-gradient-conical-asymmetric"
#define PIKA_ICON_GRADIENT_SHAPEBURST_ANGULAR   "pika-gradient-shapeburst-angular"
#define PIKA_ICON_GRADIENT_SHAPEBURST_SPHERICAL "pika-gradient-shapeburst-spherical"
#define PIKA_ICON_GRADIENT_SHAPEBURST_DIMPLED   "pika-gradient-shapeburst-dimpled"
#define PIKA_ICON_GRADIENT_SPIRAL_CLOCKWISE     "pika-gradient-spiral-clockwise"
#define PIKA_ICON_GRADIENT_SPIRAL_ANTICLOCKWISE "pika-gradient-spiral-anticlockwise"

#define PIKA_ICON_GRAVITY_EAST              "pika-gravity-east"
#define PIKA_ICON_GRAVITY_NORTH             "pika-gravity-north"
#define PIKA_ICON_GRAVITY_NORTH_EAST        "pika-gravity-north-east"
#define PIKA_ICON_GRAVITY_NORTH_WEST        "pika-gravity-north-west"
#define PIKA_ICON_GRAVITY_SOUTH             "pika-gravity-south"
#define PIKA_ICON_GRAVITY_SOUTH_EAST        "pika-gravity-south-east"
#define PIKA_ICON_GRAVITY_SOUTH_WEST        "pika-gravity-south-west"
#define PIKA_ICON_GRAVITY_WEST              "pika-gravity-west"

#define PIKA_ICON_GO_BOTTOM                 "go-bottom"
#define PIKA_ICON_GO_DOWN                   "go-down"
#define PIKA_ICON_GO_FIRST                  "go-first"
#define PIKA_ICON_GO_HOME                   "go-home"
#define PIKA_ICON_GO_LAST                   "go-last"
#define PIKA_ICON_GO_TOP                    "go-top"
#define PIKA_ICON_GO_UP                     "go-up"
#define PIKA_ICON_GO_PREVIOUS               "go-previous"
#define PIKA_ICON_GO_NEXT                   "go-next"

#define PIKA_ICON_HELP                      "system-help"
#define PIKA_ICON_HELP_ABOUT                "help-about"
#define PIKA_ICON_HELP_USER_MANUAL          "pika-user-manual"

#define PIKA_ICON_HISTOGRAM                 "pika-histogram"
#define PIKA_ICON_HISTOGRAM_LINEAR          "pika-histogram-linear"
#define PIKA_ICON_HISTOGRAM_LOGARITHMIC     "pika-histogram-logarithmic"

#define PIKA_ICON_IMAGE                     "pika-image"
#define PIKA_ICON_IMAGE_OPEN                "pika-image-open"
#define PIKA_ICON_IMAGE_RELOAD              "pika-image-reload"

#define PIKA_ICON_JOIN_MITER                "pika-join-miter"
#define PIKA_ICON_JOIN_ROUND                "pika-join-round"
#define PIKA_ICON_JOIN_BEVEL                "pika-join-bevel"

#define PIKA_ICON_LAYER                     "pika-layer"
#define PIKA_ICON_LAYER_ANCHOR              "pika-anchor"
#define PIKA_ICON_LAYER_FLOATING_SELECTION  "pika-floating-selection"
#define PIKA_ICON_LAYER_MASK                "pika-layer-mask"
#define PIKA_ICON_LAYER_MERGE_DOWN          "pika-merge-down"
#define PIKA_ICON_LAYER_TEXT_LAYER          "pika-text-layer"
#define PIKA_ICON_LAYER_TO_IMAGESIZE        "pika-layer-to-imagesize"

#define PIKA_ICON_LIST                      "pika-list"
#define PIKA_ICON_LIST_ADD                  "list-add"
#define PIKA_ICON_LIST_REMOVE               "list-remove"

#define PIKA_ICON_MENU_LEFT                 "pika-menu-left"
#define PIKA_ICON_MENU_RIGHT                "pika-menu-right"

#define PIKA_ICON_OBJECT_DUPLICATE          "pika-duplicate"
#define PIKA_ICON_OBJECT_FLIP_HORIZONTAL    "object-flip-horizontal"
#define PIKA_ICON_OBJECT_FLIP_VERTICAL      "object-flip-vertical"
#define PIKA_ICON_OBJECT_RESIZE             "pika-resize"
#define PIKA_ICON_OBJECT_ROTATE_180         "pika-rotate-180"
#define PIKA_ICON_OBJECT_ROTATE_270         "object-rotate-left"
#define PIKA_ICON_OBJECT_ROTATE_90          "object-rotate-right"
#define PIKA_ICON_OBJECT_SCALE              "pika-scale"

#define PIKA_ICON_PATH                      "pika-path"
#define PIKA_ICON_PATH_STROKE               "pika-path-stroke"

#define PIKA_ICON_PIVOT_CENTER              "pika-pivot-center"
#define PIKA_ICON_PIVOT_EAST                "pika-pivot-east"
#define PIKA_ICON_PIVOT_NORTH               "pika-pivot-north"
#define PIKA_ICON_PIVOT_NORTH_EAST          "pika-pivot-north-east"
#define PIKA_ICON_PIVOT_NORTH_WEST          "pika-pivot-north-west"
#define PIKA_ICON_PIVOT_SOUTH               "pika-pivot-south"
#define PIKA_ICON_PIVOT_SOUTH_EAST          "pika-pivot-south-east"
#define PIKA_ICON_PIVOT_SOUTH_WEST          "pika-pivot-south-west"
#define PIKA_ICON_PIVOT_WEST                "pika-pivot-west"

#define PIKA_ICON_PREFERENCES_SYSTEM        "preferences-system"

#define PIKA_ICON_PROCESS_STOP              "process-stop"

#define PIKA_ICON_QUICK_MASK_OFF            "pika-quick-mask-off"
#define PIKA_ICON_QUICK_MASK_ON             "pika-quick-mask-on"

#define PIKA_ICON_SELECTION                 "pika-selection"
#define PIKA_ICON_SELECTION_ADD             "pika-selection-add"
#define PIKA_ICON_SELECTION_ALL             "pika-selection-all"
#define PIKA_ICON_SELECTION_BORDER          "pika-selection-border"
#define PIKA_ICON_SELECTION_GROW            "pika-selection-grow"
#define PIKA_ICON_SELECTION_INTERSECT       "pika-selection-intersect"
#define PIKA_ICON_SELECTION_NONE            "pika-selection-none"
#define PIKA_ICON_SELECTION_REPLACE         "pika-selection-replace"
#define PIKA_ICON_SELECTION_SHRINK          "pika-selection-shrink"
#define PIKA_ICON_SELECTION_STROKE          "pika-selection-stroke"
#define PIKA_ICON_SELECTION_SUBTRACT        "pika-selection-subtract"
#define PIKA_ICON_SELECTION_TO_CHANNEL      "pika-selection-to-channel"
#define PIKA_ICON_SELECTION_TO_PATH         "pika-selection-to-path"

#define PIKA_ICON_SHAPE_CIRCLE              "pika-shape-circle"
#define PIKA_ICON_SHAPE_DIAMOND             "pika-shape-diamond"
#define PIKA_ICON_SHAPE_SQUARE              "pika-shape-square"

#define PIKA_ICON_SYSTEM_RUN                "system-run"

#define PIKA_ICON_TOOL_AIRBRUSH             "pika-tool-airbrush"
#define PIKA_ICON_TOOL_ALIGN                "pika-tool-align"
#define PIKA_ICON_TOOL_BLUR                 "pika-tool-blur"
#define PIKA_ICON_TOOL_BRIGHTNESS_CONTRAST  "pika-tool-brightness-contrast"
#define PIKA_ICON_TOOL_BUCKET_FILL          "pika-tool-bucket-fill"
#define PIKA_ICON_TOOL_BY_COLOR_SELECT      "pika-tool-by-color-select"
#define PIKA_ICON_TOOL_CAGE                 "pika-tool-cage"
#define PIKA_ICON_TOOL_CLONE                "pika-tool-clone"
#define PIKA_ICON_TOOL_COLORIZE             "pika-tool-colorize"
#define PIKA_ICON_TOOL_COLOR_BALANCE        "pika-tool-color-balance"
#define PIKA_ICON_TOOL_COLOR_PICKER         "pika-tool-color-picker"
#define PIKA_ICON_TOOL_COLOR_TEMPERATURE    "pika-tool-color-temperature"
#define PIKA_ICON_TOOL_CROP                 "pika-tool-crop"
#define PIKA_ICON_TOOL_CURVES               "pika-tool-curves"
#define PIKA_ICON_TOOL_DESATURATE           "pika-tool-desaturate"
#define PIKA_ICON_TOOL_DODGE                "pika-tool-dodge"
#define PIKA_ICON_TOOL_ELLIPSE_SELECT       "pika-tool-ellipse-select"
#define PIKA_ICON_TOOL_ERASER               "pika-tool-eraser"
#define PIKA_ICON_TOOL_EXPOSURE             "pika-tool-exposure"
#define PIKA_ICON_TOOL_FLIP                 "pika-tool-flip"
#define PIKA_ICON_TOOL_FOREGROUND_SELECT    "pika-tool-foreground-select"
#define PIKA_ICON_TOOL_FREE_SELECT          "pika-tool-free-select"
#define PIKA_ICON_TOOL_FUZZY_SELECT         "pika-tool-fuzzy-select"
#define PIKA_ICON_TOOL_GRADIENT             "pika-tool-gradient"
#define PIKA_ICON_TOOL_HANDLE_TRANSFORM     "pika-tool-handle-transform"
#define PIKA_ICON_TOOL_HEAL                 "pika-tool-heal"
#define PIKA_ICON_TOOL_HUE_SATURATION       "pika-tool-hue-saturation"
#define PIKA_ICON_TOOL_INK                  "pika-tool-ink"
#define PIKA_ICON_TOOL_ISCISSORS            "pika-tool-iscissors"
#define PIKA_ICON_TOOL_LEVELS               "pika-tool-levels"
#define PIKA_ICON_TOOL_MEASURE              "pika-tool-measure"
#define PIKA_ICON_TOOL_MOVE                 "pika-tool-move"
#define PIKA_ICON_TOOL_MYPAINT_BRUSH        "pika-tool-mypaint-brush"
#define PIKA_ICON_TOOL_N_POINT_DEFORMATION  "pika-tool-n-point-deformation"
#define PIKA_ICON_TOOL_OFFSET               "pika-tool-offset"
#define PIKA_ICON_TOOL_PAINTBRUSH           "pika-tool-paintbrush"
#define PIKA_ICON_TOOL_PAINT_SELECT         "pika-tool-paint-select"
#define PIKA_ICON_TOOL_PATH                 "pika-tool-path"
#define PIKA_ICON_TOOL_PENCIL               "pika-tool-pencil"
#define PIKA_ICON_TOOL_PERSPECTIVE          "pika-tool-perspective"
#define PIKA_ICON_TOOL_PERSPECTIVE_CLONE    "pika-tool-perspective-clone"
#define PIKA_ICON_TOOL_POSTERIZE            "pika-tool-posterize"
#define PIKA_ICON_TOOL_RECT_SELECT          "pika-tool-rect-select"
#define PIKA_ICON_TOOL_ROTATE               "pika-tool-rotate"
#define PIKA_ICON_TOOL_SCALE                "pika-tool-scale"
#define PIKA_ICON_TOOL_SEAMLESS_CLONE       "pika-tool-seamless-clone"
#define PIKA_ICON_TOOL_SHADOWS_HIGHLIGHTS   "pika-tool-shadows-highlights"
#define PIKA_ICON_TOOL_SHEAR                "pika-tool-shear"
#define PIKA_ICON_TOOL_SMUDGE               "pika-tool-smudge"
#define PIKA_ICON_TOOL_TEXT                 "pika-tool-text"
#define PIKA_ICON_TOOL_THRESHOLD            "pika-tool-threshold"
#define PIKA_ICON_TOOL_TRANSFORM_3D         "pika-tool-transform-3d"
#define PIKA_ICON_TOOL_UNIFIED_TRANSFORM    "pika-tool-unified-transform"
#define PIKA_ICON_TOOL_WARP                 "pika-tool-warp"
#define PIKA_ICON_TOOL_ZOOM                 "pika-tool-zoom"

#define PIKA_ICON_TRANSFORM_3D_CAMERA       "pika-transform-3d-camera"
#define PIKA_ICON_TRANSFORM_3D_MOVE         "pika-transform-3d-move"
#define PIKA_ICON_TRANSFORM_3D_ROTATE       "pika-transform-3d-rotate"

#define PIKA_ICON_VIEW_REFRESH              "view-refresh"
#define PIKA_ICON_VIEW_FULLSCREEN           "view-fullscreen"

#define PIKA_ICON_MASCOT                    "pika-mascot"
#define PIKA_ICON_MASCOT_EEK                "pika-mascot-eek"

#define PIKA_ICON_WINDOW_CLOSE              "window-close"
#define PIKA_ICON_WINDOW_MOVE_TO_SCREEN     "pika-move-to-screen"
#define PIKA_ICON_WINDOW_NEW                "window-new"

#define PIKA_ICON_ZOOM_IN                   "zoom-in"
#define PIKA_ICON_ZOOM_ORIGINAL             "zoom-original"
#define PIKA_ICON_ZOOM_OUT                  "zoom-out"
#define PIKA_ICON_ZOOM_FIT_BEST             "zoom-fit-best"
#define PIKA_ICON_ZOOM_FOLLOW_WINDOW        "pika-zoom-follow-window"


void   pika_icons_init           (void);

void   pika_icons_set_icon_theme (GFile *path);


G_END_DECLS

#endif /* __PIKA_ICONS_H__ */
