/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikaintcombobox.h
 * Copyright (C) 2004  Sven Neumann <sven@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_INT_COMBO_BOX_H__
#define __PIKA_INT_COMBO_BOX_H__

G_BEGIN_DECLS


#define PIKA_TYPE_INT_COMBO_BOX            (pika_int_combo_box_get_type ())
#define PIKA_INT_COMBO_BOX(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_INT_COMBO_BOX, PikaIntComboBox))
#define PIKA_INT_COMBO_BOX_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_INT_COMBO_BOX, PikaIntComboBoxClass))
#define PIKA_IS_INT_COMBO_BOX(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_INT_COMBO_BOX))
#define PIKA_IS_INT_COMBO_BOX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_INT_COMBO_BOX))
#define PIKA_INT_COMBO_BOX_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_INT_COMBO_BOX, PikaIntComboBoxClass))


typedef struct _PikaIntComboBoxPrivate PikaIntComboBoxPrivate;
typedef struct _PikaIntComboBoxClass   PikaIntComboBoxClass;

struct _PikaIntComboBox
{
  GtkComboBox             parent_instance;

  PikaIntComboBoxPrivate *priv;
};

struct _PikaIntComboBoxClass
{
  GtkComboBoxClass  parent_class;

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


/**
 * PikaIntSensitivityFunc:
 * @value:
 * @data: (closure):
 */
typedef  gboolean (* PikaIntSensitivityFunc) (gint      value,
                                              gpointer  data);



GType         pika_int_combo_box_get_type        (void) G_GNUC_CONST;

GtkWidget   * pika_int_combo_box_new             (const gchar     *first_label,
                                                  gint             first_value,
                                                  ...) G_GNUC_NULL_TERMINATED;
GtkWidget   * pika_int_combo_box_new_valist      (const gchar     *first_label,
                                                  gint             first_value,
                                                  va_list          values);

GtkWidget   * pika_int_combo_box_new_array       (gint             n_values,
                                                  const gchar     *labels[]);

void          pika_int_combo_box_prepend         (PikaIntComboBox *combo_box,
                                                  ...);
void          pika_int_combo_box_append          (PikaIntComboBox *combo_box,
                                                  ...);

gboolean      pika_int_combo_box_set_active      (PikaIntComboBox *combo_box,
                                                  gint             value);
gboolean      pika_int_combo_box_get_active      (PikaIntComboBox *combo_box,
                                                  gint            *value);

gboolean
      pika_int_combo_box_set_active_by_user_data (PikaIntComboBox *combo_box,
                                                  gpointer         user_data);
gboolean
      pika_int_combo_box_get_active_user_data    (PikaIntComboBox *combo_box,
                                                  gpointer        *user_data);

gulong        pika_int_combo_box_connect         (PikaIntComboBox *combo_box,
                                                  gint             value,
                                                  GCallback        callback,
                                                  gpointer         data,
                                                  GDestroyNotify   data_destroy);

void          pika_int_combo_box_set_label       (PikaIntComboBox *combo_box,
                                                  const gchar     *label);
const gchar * pika_int_combo_box_get_label       (PikaIntComboBox *combo_box);

void          pika_int_combo_box_set_layout      (PikaIntComboBox       *combo_box,
                                                  PikaIntComboBoxLayout  layout);
PikaIntComboBoxLayout
              pika_int_combo_box_get_layout      (PikaIntComboBox       *combo_box);

void          pika_int_combo_box_set_sensitivity (PikaIntComboBox        *combo_box,
                                                  PikaIntSensitivityFunc  func,
                                                  gpointer                data,
                                                  GDestroyNotify          destroy);


G_END_DECLS

#endif  /* __PIKA_INT_COMBO_BOX_H__ */
