/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikaintradioframe.h
 * Copyright (C) 2022 Jehan
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#include <libpikawidgets/pikaframe.h>

#ifndef __PIKA_INT_RADIO_FRAME_H__
#define __PIKA_INT_RADIO_FRAME_H__

G_BEGIN_DECLS


#define PIKA_TYPE_INT_RADIO_FRAME            (pika_int_radio_frame_get_type ())
#define PIKA_INT_RADIO_FRAME(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_INT_RADIO_FRAME, PikaIntRadioFrame))
#define PIKA_INT_RADIO_FRAME_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_INT_RADIO_FRAME, PikaIntRadioFrameClass))
#define PIKA_IS_INT_RADIO_FRAME(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_INT_RADIO_FRAME))
#define PIKA_IS_INT_RADIO_FRAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_INT_RADIO_FRAME))
#define PIKA_INT_RADIO_FRAME_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_INT_RADIO_FRAME, PikaIntRadioFrameClass))


typedef struct _PikaIntRadioFrameClass   PikaIntRadioFrameClass;

struct _PikaIntRadioFrame
{
  PikaFrame       parent_instance;
};

struct _PikaIntRadioFrameClass
{
  PikaFrameClass  parent_class;

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


/**
 * PikaIntRadioFrameSensitivityFunc:
 * @value: the value associated with a radio button.
 * @user_data: the data associated with a radio button.
 * @new_value: the value to check instead if the function returns %FALSE.
 * @data: (closure): the data set in pika_int_radio_frame_set_sensitivity()
 *
 * Signature for a function called on each radio button value and data,
 * each time the %PikaIntRadioFrame is drawn, to make some radio button
 * insensitive.
 * If the function returns %FALSE, it usually means that the value is
 * not a valid choice in current situation. In this case, you might want
 * to toggle instead another value automatically. Set @new_value to the
 * value to toggle. If you leave this untouched, the radio button will
 * stay toggled despite being insensitive. This is up to you to decide
 * whether this is meaningful.
 *
 * Returns: %TRUE if the button stays sensitive, %FALSE otherwise.
 */
typedef  gboolean (* PikaIntRadioFrameSensitivityFunc) (gint      value,
                                                        gpointer  user_data,
                                                        gint     *new_value,
                                                        gpointer  data);



GType         pika_int_radio_frame_get_type        (void) G_GNUC_CONST;

GtkWidget   * pika_int_radio_frame_new_from_store  (const gchar       *title,
                                                    PikaIntStore      *store);
GtkWidget   * pika_int_radio_frame_new             (const gchar       *first_label,
                                                    gint               first_value,
                                                    ...) G_GNUC_NULL_TERMINATED;
GtkWidget   * pika_int_radio_frame_new_valist      (const gchar       *first_label,
                                                    gint               first_value,
                                                    va_list            values);

GtkWidget   * pika_int_radio_frame_new_array       (const gchar       *labels[]);

void          pika_int_radio_frame_prepend         (PikaIntRadioFrame *radio_frame,
                                                    ...);
void          pika_int_radio_frame_append          (PikaIntRadioFrame *radio_frame,
                                                    ...);

gboolean      pika_int_radio_frame_set_active      (PikaIntRadioFrame *radio_frame,
                                                    gint               value);
gint          pika_int_radio_frame_get_active      (PikaIntRadioFrame *radio_frame);

gboolean
      pika_int_radio_frame_set_active_by_user_data (PikaIntRadioFrame *radio_frame,
                                                    gpointer           user_data);
gboolean
      pika_int_radio_frame_get_active_user_data    (PikaIntRadioFrame *radio_frame,
                                                    gpointer          *user_data);

void          pika_int_radio_frame_set_sensitivity (PikaIntRadioFrame *radio_frame,
                                                    PikaIntRadioFrameSensitivityFunc  func,
                                                    gpointer           data,
                                                    GDestroyNotify     destroy);


G_END_DECLS

#endif  /* __PIKA_INT_RADIO_FRAME_H__ */
