/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikaintstore.c
 * Copyright (C) 2004  Sven Neumann <sven@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_INT_STORE_H__
#define __PIKA_INT_STORE_H__

G_BEGIN_DECLS


/**
 * PikaIntStoreColumns:
 * @PIKA_INT_STORE_VALUE:       the integer value
 * @PIKA_INT_STORE_LABEL:       a human-readable label
 * @PIKA_INT_STORE_ABBREV:      an abbreviated label
 * @PIKA_INT_STORE_ICON_NAME:   an icon name
 * @PIKA_INT_STORE_PIXBUF:      a #GdkPixbuf
 * @PIKA_INT_STORE_USER_DATA:   arbitrary user data
 * @PIKA_INT_STORE_NUM_COLUMNS: the number of columns
 *
 * The column types of #PikaIntStore.
 **/
typedef enum
{
  PIKA_INT_STORE_VALUE,
  PIKA_INT_STORE_LABEL,
  PIKA_INT_STORE_ABBREV,
  PIKA_INT_STORE_ICON_NAME,
  PIKA_INT_STORE_PIXBUF,
  PIKA_INT_STORE_USER_DATA,
  PIKA_INT_STORE_NUM_COLUMNS
} PikaIntStoreColumns;


#define PIKA_TYPE_INT_STORE            (pika_int_store_get_type ())
#define PIKA_INT_STORE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_INT_STORE, PikaIntStore))
#define PIKA_INT_STORE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_INT_STORE, PikaIntStoreClass))
#define PIKA_IS_INT_STORE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_INT_STORE))
#define PIKA_IS_INT_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_INT_STORE))
#define PIKA_INT_STORE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_INT_STORE, PikaIntStoreClass))


typedef struct _PikaIntStorePrivate PikaIntStorePrivate;
typedef struct _PikaIntStoreClass   PikaIntStoreClass;

struct _PikaIntStore
{
  GtkListStore         parent_instance;

  PikaIntStorePrivate *priv;
};

struct _PikaIntStoreClass
{
  GtkListStoreClass  parent_class;

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType          pika_int_store_get_type        (void) G_GNUC_CONST;

GtkListStore * pika_int_store_new             (const gchar   *first_label,
                                               gint           first_value,
                                               ...) G_GNUC_NULL_TERMINATED;
GtkListStore * pika_int_store_new_valist      (const gchar   *first_label,
                                               gint           first_value,
                                               va_list        values);
GtkListStore * pika_int_store_new_array       (gint           n_values,
                                               const gchar   *labels[]);

gboolean       pika_int_store_lookup_by_value (GtkTreeModel  *model,
                                               gint           value,
                                               GtkTreeIter   *iter);
gboolean   pika_int_store_lookup_by_user_data (GtkTreeModel  *model,
                                               gpointer       user_data,
                                               GtkTreeIter   *iter);


G_END_DECLS

#endif  /* __PIKA_INT_STORE_H__ */
