/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikalabelcolor.h
 * Copyright (C) 2022 Jehan
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_LABEL_COLOR_H__
#define __PIKA_LABEL_COLOR_H__

#include <libpikawidgets/pikalabeled.h>

G_BEGIN_DECLS

#define PIKA_TYPE_LABEL_COLOR (pika_label_color_get_type ())
G_DECLARE_DERIVABLE_TYPE (PikaLabelColor, pika_label_color, PIKA, LABEL_COLOR, PikaLabeled)

struct _PikaLabelColorClass
{
  PikaLabeledClass   parent_class;

  /*  Signals */
  void (* value_changed)   (PikaLabelColor *color);

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};

GtkWidget * pika_label_color_new              (const gchar       *label,
                                               const PikaRGB     *color,
                                               gboolean           editable);

/* TODO: it would be interesting for such a widget to have an API to
 * customize the label being also above or below, left or right. I could
 * imagine wanting to pretty-list several colors with specific layouts.
 */

void        pika_label_color_set_value        (PikaLabelColor     *color,
                                               const PikaRGB      *value);
void        pika_label_color_get_value        (PikaLabelColor     *color,
                                               PikaRGB            *value);

void        pika_label_color_set_editable     (PikaLabelColor     *color,
                                               gboolean            editable);
gboolean    pika_label_color_is_editable      (PikaLabelColor     *color);

GtkWidget * pika_label_color_get_color_widget (PikaLabelColor     *color);

G_END_DECLS

#endif /* __PIKA_LABEL_COLOR_H__ */
