/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikaratioentry.h
 * Copyright (C) 2006  Simon Budig       <simon@gimp.org>
 * Copyright (C) 2007  Sven Neumann      <sven@gimp.org>
 * Copyright (C) 2007  Martin Nordholts  <martin@svn.gnome.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_NUMBER_PAIR_ENTRY_H__
#define __PIKA_NUMBER_PAIR_ENTRY_H__

G_BEGIN_DECLS


#define PIKA_TYPE_NUMBER_PAIR_ENTRY            (pika_number_pair_entry_get_type ())
#define PIKA_NUMBER_PAIR_ENTRY(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_NUMBER_PAIR_ENTRY, PikaNumberPairEntry))
#define PIKA_NUMBER_PAIR_ENTRY_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_NUMBER_PAIR_ENTRY, PikaNumberPairEntryClass))
#define PIKA_IS_NUMBER_PAIR_ENTRY(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_NUMBER_PAIR_ENTRY))
#define PIKA_IS_NUMBER_PAIR_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_NUMBER_PAIR_ENTRY))
#define PIKA_NUMBER_PAIR_ENTRY_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_NUMBER_PAIR_AREA, PikaNumberPairEntryClass))


typedef struct _PikaNumberPairEntryPrivate PikaNumberPairEntryPrivate;
typedef struct _PikaNumberPairEntryClass   PikaNumberPairEntryClass;


struct _PikaNumberPairEntry
{
  GtkEntry                    parent_instance;

  PikaNumberPairEntryPrivate *priv;
};

struct _PikaNumberPairEntryClass
{
  GtkEntryClass  parent_class;

  void (* numbers_changed) (PikaNumberPairEntry *entry);
  void (* ratio_changed)   (PikaNumberPairEntry *entry);

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType          pika_number_pair_entry_get_type           (void) G_GNUC_CONST;
GtkWidget *    pika_number_pair_entry_new                (const gchar         *separators,
                                                          gboolean             allow_simplification,
                                                          gdouble              min_valid_value,
                                                          gdouble              max_valid_value);
void           pika_number_pair_entry_set_default_values (PikaNumberPairEntry *entry,
                                                          gdouble              left,
                                                          gdouble              right);
void           pika_number_pair_entry_get_default_values (PikaNumberPairEntry *entry,
                                                          gdouble             *left,
                                                          gdouble             *right);
void           pika_number_pair_entry_set_values         (PikaNumberPairEntry *entry,
                                                          gdouble              left,
                                                          gdouble              right);
void           pika_number_pair_entry_get_values         (PikaNumberPairEntry *entry,
                                                          gdouble             *left,
                                                          gdouble             *right);

void           pika_number_pair_entry_set_default_text   (PikaNumberPairEntry *entry,
                                                          const gchar         *string);
const gchar *  pika_number_pair_entry_get_default_text   (PikaNumberPairEntry *entry);

void           pika_number_pair_entry_set_ratio          (PikaNumberPairEntry *entry,
                                                          gdouble              ratio);
gdouble        pika_number_pair_entry_get_ratio          (PikaNumberPairEntry *entry);

void           pika_number_pair_entry_set_aspect         (PikaNumberPairEntry *entry,
                                                          PikaAspectType       aspect);
PikaAspectType pika_number_pair_entry_get_aspect         (PikaNumberPairEntry *entry);

void           pika_number_pair_entry_set_user_override  (PikaNumberPairEntry *entry,
                                                          gboolean             user_override);
gboolean       pika_number_pair_entry_get_user_override  (PikaNumberPairEntry *entry);


G_END_DECLS

#endif /* __PIKA_NUMBER_PAIR_ENTRY_H__ */
