/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikapatheditor.h
 * Copyright (C) 1999-2004 Michael Natterer <mitch@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_PATH_EDITOR_H__
#define __PIKA_PATH_EDITOR_H__

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


#define PIKA_TYPE_PATH_EDITOR            (pika_path_editor_get_type ())
#define PIKA_PATH_EDITOR(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_PATH_EDITOR, PikaPathEditor))
#define PIKA_PATH_EDITOR_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_PATH_EDITOR, PikaPathEditorClass))
#define PIKA_IS_PATH_EDITOR(obj)         (G_TYPE_CHECK_INSTANCE_TYPE (obj, PIKA_TYPE_PATH_EDITOR))
#define PIKA_IS_PATH_EDITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_PATH_EDITOR))
#define PIKA_PATH_EDITOR_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_PATH_EDITOR, PikaPathEditorClass))


typedef struct _PikaPathEditorPrivate PikaPathEditorPrivate;
typedef struct _PikaPathEditorClass   PikaPathEditorClass;

struct _PikaPathEditor
{
  GtkBox                 parent_instance;

  PikaPathEditorPrivate *priv;

  /* FIXME MOVE TO PRIVATE */
  GtkWidget         *upper_hbox;

  GtkWidget         *new_button;
  GtkWidget         *up_button;
  GtkWidget         *down_button;
  GtkWidget         *delete_button;

  GtkWidget         *file_entry;

  GtkListStore      *dir_list;

  GtkTreeSelection  *sel;
  GtkTreePath       *sel_path;

  GtkTreeViewColumn *writable_column;

  gint               num_items;
};

struct _PikaPathEditorClass
{
  GtkBoxClass  parent_class;

  void (* path_changed)     (PikaPathEditor *editor);
  void (* writable_changed) (PikaPathEditor *editor);

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


/* For information look into the C source or the html documentation */

GType       pika_path_editor_get_type          (void) G_GNUC_CONST;

GtkWidget * pika_path_editor_new               (const gchar    *title,
                                                const gchar    *path);

gchar     * pika_path_editor_get_path          (PikaPathEditor *editor);
void        pika_path_editor_set_path          (PikaPathEditor *editor,
                                                const gchar    *path);

gchar     * pika_path_editor_get_writable_path (PikaPathEditor *editor);
void        pika_path_editor_set_writable_path (PikaPathEditor *editor,
                                                const gchar    *path);

gboolean    pika_path_editor_get_dir_writable  (PikaPathEditor *editor,
                                                const gchar    *directory);
void        pika_path_editor_set_dir_writable  (PikaPathEditor *editor,
                                                const gchar    *directory,
                                                gboolean        writable);

G_END_DECLS

#endif /* __PIKA_PATH_EDITOR_H__ */
