/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_PREVIEW_AREA_H__
#define __PIKA_PREVIEW_AREA_H__

G_BEGIN_DECLS


#define PIKA_TYPE_PREVIEW_AREA            (pika_preview_area_get_type ())
#define PIKA_PREVIEW_AREA(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_PREVIEW_AREA, PikaPreviewArea))
#define PIKA_PREVIEW_AREA_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_PREVIEW_AREA, PikaPreviewAreaClass))
#define PIKA_IS_PREVIEW_AREA(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_PREVIEW_AREA))
#define PIKA_IS_PREVIEW_AREA_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_PREVIEW_AREA))
#define PIKA_PREVIEW_AREA_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_PREVIEW_AREA, PikaPreviewArea))


typedef struct _PikaPreviewAreaPrivate PikaPreviewAreaPrivate;
typedef struct _PikaPreviewAreaClass   PikaPreviewAreaClass;

struct _PikaPreviewArea
{
  GtkDrawingArea          parent_instance;

  PikaPreviewAreaPrivate *priv;
};

struct _PikaPreviewAreaClass
{
  GtkDrawingAreaClass  parent_class;

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType       pika_preview_area_get_type         (void) G_GNUC_CONST;

GtkWidget * pika_preview_area_new              (void);

void        pika_preview_area_draw             (PikaPreviewArea *area,
                                                gint             x,
                                                gint             y,
                                                gint             width,
                                                gint             height,
                                                PikaImageType    type,
                                                const guchar    *buf,
                                                gint             rowstride);
void        pika_preview_area_blend            (PikaPreviewArea *area,
                                                gint             x,
                                                gint             y,
                                                gint             width,
                                                gint             height,
                                                PikaImageType    type,
                                                const guchar    *buf1,
                                                gint             rowstride1,
                                                const guchar    *buf2,
                                                gint             rowstride2,
                                                guchar           opacity);
void        pika_preview_area_mask             (PikaPreviewArea *area,
                                                gint             x,
                                                gint             y,
                                                gint             width,
                                                gint             height,
                                                PikaImageType    type,
                                                const guchar    *buf1,
                                                gint             rowstride1,
                                                const guchar    *buf2,
                                                gint             rowstride2,
                                                const guchar    *mask,
                                                gint             rowstride_mask);
void        pika_preview_area_fill             (PikaPreviewArea *area,
                                                gint             x,
                                                gint             y,
                                                gint             width,
                                                gint             height,
                                                guchar           red,
                                                guchar           green,
                                                guchar           blue);

void        pika_preview_area_set_offsets      (PikaPreviewArea *area,
                                                gint             x,
                                                gint             y);

void        pika_preview_area_set_colormap     (PikaPreviewArea *area,
                                                const guchar    *colormap,
                                                gint             num_colors);

void        pika_preview_area_set_color_config (PikaPreviewArea *area,
                                                PikaColorConfig *config);

void        pika_preview_area_get_size         (PikaPreviewArea *area,
                                                gint            *width,
                                                gint            *height);
void        pika_preview_area_set_max_size     (PikaPreviewArea *area,
                                                gint             width,
                                                gint             height);

void        pika_preview_area_menu_popup       (PikaPreviewArea *area,
                                                GdkEventButton  *event);


G_END_DECLS

#endif /* __PIKA_PREVIEW_AREA_H__ */
