/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikascaleentry.h
 * Copyright (C) 2000 Michael Natterer <mitch@gimp.org>,
 *               2008 Bill Skaggs <weskaggs@primate.ucdavis.edu>
 *               2020 Jehan
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#include <libpikawidgets/pikalabelspin.h>

#ifndef __PIKA_SCALE_ENTRY_H__
#define __PIKA_SCALE_ENTRY_H__

G_BEGIN_DECLS

#define PIKA_TYPE_SCALE_ENTRY (pika_scale_entry_get_type ())
G_DECLARE_DERIVABLE_TYPE (PikaScaleEntry, pika_scale_entry, PIKA, SCALE_ENTRY, PikaLabelSpin)

struct _PikaScaleEntryClass
{
  PikaLabelSpinClass parent_class;

  /*  Signals        */
  void            (* value_changed)    (PikaScaleEntry *entry);

  /*  Class methods  */
  GtkWidget     * (* new_range_widget) (GtkAdjustment  *adjustment);

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};

GtkWidget     * pika_scale_entry_new             (const gchar *text,
                                                  gdouble      value,
                                                  gdouble      lower,
                                                  gdouble      upper,
                                                  guint        digits);

GtkWidget     * pika_scale_entry_get_range       (PikaScaleEntry *entry);

void            pika_scale_entry_set_bounds      (PikaScaleEntry *entry,
                                                  gdouble         lower,
                                                  gdouble         upper,
                                                  gboolean        limit_scale);

void            pika_scale_entry_set_logarithmic (PikaScaleEntry *entry,
                                                  gboolean        logarithmic);
gboolean        pika_scale_entry_get_logarithmic (PikaScaleEntry *entry);


G_END_DECLS

#endif /* __PIKA_SCALE_ENTRY_H__ */
