/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikascrolledpreview.h
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_SCROLLED_PREVIEW_H__
#define __PIKA_SCROLLED_PREVIEW_H__

#include "pikapreview.h"

G_BEGIN_DECLS


/* For information look into the C source or the html documentation */


#define PIKA_TYPE_SCROLLED_PREVIEW            (pika_scrolled_preview_get_type ())
#define PIKA_SCROLLED_PREVIEW(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_SCROLLED_PREVIEW, PikaScrolledPreview))
#define PIKA_SCROLLED_PREVIEW_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_SCROLLED_PREVIEW, PikaScrolledPreviewClass))
#define PIKA_IS_SCROLLED_PREVIEW(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_SCROLLED_PREVIEW))
#define PIKA_IS_SCROLLED_PREVIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_SCROLLED_PREVIEW))
#define PIKA_SCROLLED_PREVIEW_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_SCROLLED_PREVIEW, PikaScrolledPreviewClass))


typedef struct _PikaScrolledPreviewPrivate PikaScrolledPreviewPrivate;
typedef struct _PikaScrolledPreviewClass   PikaScrolledPreviewClass;

struct _PikaScrolledPreview
{
  PikaPreview                 parent_instance;
};

struct _PikaScrolledPreviewClass
{
  PikaPreviewClass  parent_class;

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType  pika_scrolled_preview_get_type        (void) G_GNUC_CONST;

void   pika_scrolled_preview_set_position    (PikaScrolledPreview  *preview,
                                              gint                  x,
                                              gint                  y);
void   pika_scrolled_preview_set_policy      (PikaScrolledPreview  *preview,
                                              GtkPolicyType         hscrollbar_policy,
                                              GtkPolicyType         vscrollbar_policy);

void   pika_scrolled_preview_get_adjustments (PikaScrolledPreview  *preview,
                                              GtkAdjustment       **hadj,
                                              GtkAdjustment       **vadj);

/*  only for use from derived widgets  */
void   pika_scrolled_preview_freeze          (PikaScrolledPreview  *preview);
void   pika_scrolled_preview_thaw            (PikaScrolledPreview  *preview);

/*  utility function for scrolled-window like pika widgets like the canvas  */
void   pika_scroll_adjustment_values         (GdkEventScroll       *sevent,
                                              GtkAdjustment        *hadj,
                                              GtkAdjustment        *vadj,
                                              gdouble              *hvalue,
                                              gdouble              *vvalue);


G_END_DECLS

#endif /* __PIKA_SCROLLED_PREVIEW_H__ */
