/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikasizeentry.h
 * Copyright (C) 1999-2000 Sven Neumann <sven@gimp.org>
 *                         Michael Natterer <mitch@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_SIZE_ENTRY_H__
#define __PIKA_SIZE_ENTRY_H__

G_BEGIN_DECLS

/* For information look into the C source or the html documentation */


#define PIKA_TYPE_SIZE_ENTRY            (pika_size_entry_get_type ())
#define PIKA_SIZE_ENTRY(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_SIZE_ENTRY, PikaSizeEntry))
#define PIKA_SIZE_ENTRY_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_SIZE_ENTRY, PikaSizeEntryClass))
#define PIKA_IS_SIZE_ENTRY(obj)         (G_TYPE_CHECK_INSTANCE_TYPE (obj, PIKA_TYPE_SIZE_ENTRY))
#define PIKA_IS_SIZE_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_SIZE_ENTRY))
#define PIKA_SIZE_ENTRY_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_SIZE_ENTRY, PikaSizeEntryClass))


typedef struct _PikaSizeEntryPrivate PikaSizeEntryPrivate;
typedef struct _PikaSizeEntryClass   PikaSizeEntryClass;

typedef struct _PikaSizeEntryField  PikaSizeEntryField;

struct _PikaSizeEntry
{
  GtkGrid               parent_instance;

  PikaSizeEntryPrivate *priv;
};

struct _PikaSizeEntryClass
{
  GtkGridClass  parent_class;

  void (* value_changed)  (PikaSizeEntry *gse);
  void (* refval_changed) (PikaSizeEntry *gse);
  void (* unit_changed)   (PikaSizeEntry *gse);

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


/* For information look into the C source or the html documentation */

GType       pika_size_entry_get_type (void) G_GNUC_CONST;

GtkWidget * pika_size_entry_new (gint                       number_of_fields,
                                 PikaUnit                   unit,
                                 const gchar               *unit_format,
                                 gboolean                   menu_show_pixels,
                                 gboolean                   menu_show_percent,
                                 gboolean                   show_refval,
                                 gint                       spinbutton_width,
                                 PikaSizeEntryUpdatePolicy  update_policy);

void        pika_size_entry_add_field  (PikaSizeEntry   *gse,
                                        GtkSpinButton   *value_spinbutton,
                                        GtkSpinButton   *refval_spinbutton);

PikaSizeEntryUpdatePolicy
            pika_size_entry_get_update_policy     (PikaSizeEntry *gse);
gint        pika_size_entry_get_n_fields          (PikaSizeEntry *gse);
GtkWidget * pika_size_entry_get_unit_combo        (PikaSizeEntry *gse);

GtkWidget * pika_size_entry_attach_label          (PikaSizeEntry *gse,
                                                   const gchar   *text,
                                                   gint           row,
                                                   gint           column,
                                                   gfloat         alignment);

void        pika_size_entry_set_resolution        (PikaSizeEntry *gse,
                                                   gint           field,
                                                   gdouble        resolution,
                                                   gboolean       keep_size);

void        pika_size_entry_set_size              (PikaSizeEntry *gse,
                                                   gint           field,
                                                   gdouble        lower,
                                                   gdouble        upper);

void        pika_size_entry_set_value_boundaries  (PikaSizeEntry *gse,
                                                   gint           field,
                                                   gdouble        lower,
                                                   gdouble        upper);

gdouble     pika_size_entry_get_value             (PikaSizeEntry *gse,
                                                   gint           field);
void        pika_size_entry_set_value             (PikaSizeEntry *gse,
                                                   gint           field,
                                                   gdouble        value);

void        pika_size_entry_set_refval_boundaries (PikaSizeEntry *gse,
                                                   gint           field,
                                                   gdouble        lower,
                                                   gdouble        upper);
void        pika_size_entry_set_refval_digits     (PikaSizeEntry *gse,
                                                   gint           field,
                                                   gint           digits);

gdouble     pika_size_entry_get_refval            (PikaSizeEntry *gse,
                                                   gint           field);
void        pika_size_entry_set_refval            (PikaSizeEntry *gse,
                                                   gint           field,
                                                   gdouble        refval);

PikaUnit    pika_size_entry_get_unit              (PikaSizeEntry *gse);
void        pika_size_entry_set_unit              (PikaSizeEntry *gse,
                                                   PikaUnit       unit);
void        pika_size_entry_show_unit_menu        (PikaSizeEntry *gse,
                                                   gboolean       show);

void        pika_size_entry_set_pixel_digits      (PikaSizeEntry *gse,
                                                   gint           digits);

void        pika_size_entry_grab_focus            (PikaSizeEntry *gse);
void        pika_size_entry_set_activates_default (PikaSizeEntry *gse,
                                                   gboolean       setting);
GtkWidget * pika_size_entry_get_help_widget       (PikaSizeEntry *gse,
                                                   gint           field);


G_END_DECLS

#endif /* __PIKA_SIZE_ENTRY_H__ */
