/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikaunitstore.h
 * Copyright (C) 2004  Sven Neumann <sven@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_UNIT_STORE_H__
#define __PIKA_UNIT_STORE_H__

G_BEGIN_DECLS


enum
{
  PIKA_UNIT_STORE_UNIT,
  PIKA_UNIT_STORE_UNIT_FACTOR,
  PIKA_UNIT_STORE_UNIT_DIGITS,
  PIKA_UNIT_STORE_UNIT_IDENTIFIER,
  PIKA_UNIT_STORE_UNIT_SYMBOL,
  PIKA_UNIT_STORE_UNIT_ABBREVIATION,
  PIKA_UNIT_STORE_UNIT_SINGULAR,
  PIKA_UNIT_STORE_UNIT_PLURAL,
  PIKA_UNIT_STORE_UNIT_SHORT_FORMAT,
  PIKA_UNIT_STORE_UNIT_LONG_FORMAT,
  PIKA_UNIT_STORE_UNIT_COLUMNS,
  PIKA_UNIT_STORE_FIRST_VALUE = PIKA_UNIT_STORE_UNIT_COLUMNS
};


#define PIKA_TYPE_UNIT_STORE            (pika_unit_store_get_type ())
#define PIKA_UNIT_STORE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_UNIT_STORE, PikaUnitStore))
#define PIKA_UNIT_STORE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_UNIT_STORE, PikaUnitStoreClass))
#define PIKA_IS_UNIT_STORE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_UNIT_STORE))
#define PIKA_IS_UNIT_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_UNIT_STORE))
#define PIKA_UNIT_STORE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_UNIT_STORE, PikaUnitStoreClass))


typedef struct _PikaUnitStorePrivate PikaUnitStorePrivate;
typedef struct _PikaUnitStoreClass   PikaUnitStoreClass;

struct _PikaUnitStore
{
  GObject               parent_instance;

  PikaUnitStorePrivate *priv;
};

struct _PikaUnitStoreClass
{
  GObjectClass  parent_class;

  /* Padding for future expansion */
  void (*_pika_reserved1) (void);
  void (*_pika_reserved2) (void);
  void (*_pika_reserved3) (void);
  void (*_pika_reserved4) (void);
  void (*_pika_reserved5) (void);
  void (*_pika_reserved6) (void);
  void (*_pika_reserved7) (void);
  void (*_pika_reserved8) (void);
};


GType           pika_unit_store_get_type         (void) G_GNUC_CONST;

PikaUnitStore * pika_unit_store_new              (gint           num_values);

void            pika_unit_store_set_has_pixels   (PikaUnitStore *store,
                                                  gboolean       has_pixels);
gboolean        pika_unit_store_get_has_pixels   (PikaUnitStore *store);

void            pika_unit_store_set_has_percent  (PikaUnitStore *store,
                                                  gboolean       has_percent);
gboolean        pika_unit_store_get_has_percent  (PikaUnitStore *store);

void            pika_unit_store_set_pixel_value  (PikaUnitStore *store,
                                                  gint           index,
                                                  gdouble        value);
void            pika_unit_store_set_pixel_values (PikaUnitStore *store,
                                                  gdouble        first_value,
                                                  ...);
void            pika_unit_store_set_resolution   (PikaUnitStore *store,
                                                  gint           index,
                                                  gdouble        resolution);
void            pika_unit_store_set_resolutions  (PikaUnitStore *store,
                                                  gdouble        first_resolution,
                                                  ...);
gdouble         pika_unit_store_get_nth_value    (PikaUnitStore *store,
                                                  PikaUnit       unit,
                                                  gint           index);
void            pika_unit_store_get_values       (PikaUnitStore *store,
                                                  PikaUnit       unit,
                                                  gdouble       *first_value,
                                                  ...);

void            _pika_unit_store_sync_units      (PikaUnitStore *store);


G_END_DECLS

#endif  /* __PIKA_UNIT_STORE_H__ */
