/* LIBPIKA - The PIKA Library
 * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
 *
 * pikazoommodel.h
 * Copyright (C) 2005  David Odin <dindinx@gimp.org>
 *
 * This library is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library.  If not, see
 * <https://www.gnu.org/licenses/>.
 */

#if !defined (__PIKA_WIDGETS_H_INSIDE__) && !defined (PIKA_WIDGETS_COMPILATION)
#error "Only <libpikawidgets/pikawidgets.h> can be included directly."
#endif

#ifndef __PIKA_ZOOM_MODEL_H__
#define __PIKA_ZOOM_MODEL_H__

G_BEGIN_DECLS


#define PIKA_TYPE_ZOOM_MODEL            (pika_zoom_model_get_type ())
#define PIKA_ZOOM_MODEL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_ZOOM_MODEL, PikaZoomModel))
#define PIKA_ZOOM_MODEL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_ZOOM_MODEL, PikaZoomModelClass))
#define PIKA_IS_ZOOM_MODEL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_ZOOM_MODEL))
#define PIKA_IS_ZOOM_MODEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_ZOOM_MODEL))
#define PIKA_ZOOM_MODEL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_ZOOM_MODEL, PikaZoomModel))


typedef struct _PikaZoomModelPrivate PikaZoomModelPrivate;
typedef struct _PikaZoomModelClass   PikaZoomModelClass;

struct _PikaZoomModel
{
  GObject               parent_instance;

  PikaZoomModelPrivate *priv;
};

struct _PikaZoomModelClass
{
  GObjectClass  parent_class;

  void (* zoomed) (PikaZoomModel *model,
                   gdouble        old_factor,
                   gdouble        new_factor);

  /* Padding for future expansion */
  void (* _pika_reserved1) (void);
  void (* _pika_reserved2) (void);
  void (* _pika_reserved3) (void);
  void (* _pika_reserved4) (void);
  void (* _pika_reserved5) (void);
  void (* _pika_reserved6) (void);
  void (* _pika_reserved7) (void);
  void (* _pika_reserved8) (void);
};


GType           pika_zoom_model_get_type     (void) G_GNUC_CONST;

PikaZoomModel * pika_zoom_model_new          (void);
void            pika_zoom_model_set_range    (PikaZoomModel      *model,
                                              gdouble             min,
                                              gdouble             max);
void            pika_zoom_model_zoom         (PikaZoomModel      *model,
                                              PikaZoomType        zoom_type,
                                              gdouble             scale);
gdouble         pika_zoom_model_get_factor   (PikaZoomModel      *model);
void            pika_zoom_model_get_fraction (PikaZoomModel      *model,
                                              gint               *numerator,
                                              gint               *denominator);

GtkWidget     * pika_zoom_button_new         (PikaZoomModel      *model,
                                              PikaZoomType        zoom_type,
                                              GtkIconSize         icon_size);

gdouble         pika_zoom_model_zoom_step    (PikaZoomType        zoom_type,
                                              gdouble             scale,
                                              gdouble             delta);

G_END_DECLS

#endif /* __PIKA_ZOOM_MODEL_H__ */
