/* HSV color selector for GTK+
 *
 * Copyright (C) 1999 The Free Software Foundation
 *
 * Authors: Simon Budig <Simon.Budig@unix-ag.org> (original code)
 *          Federico Mena-Quintero <federico@gimp.org> (cleanup for GTK+)
 *          Jonathan Blandford <jrb@redhat.com> (cleanup for GTK+)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/*
 * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
 */

#ifndef __PIKA_COLOR_WHEEL_H__
#define __PIKA_COLOR_WHEEL_H__

G_BEGIN_DECLS

#define PIKA_TYPE_COLOR_WHEEL            (pika_color_wheel_get_type ())
#define PIKA_COLOR_WHEEL(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_COLOR_WHEEL, PikaColorWheel))
#define PIKA_COLOR_WHEEL_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_COLOR_WHEEL, PikaColorWheelClass))
#define PIKA_IS_COLOR_WHEEL(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_COLOR_WHEEL))
#define PIKA_IS_COLOR_WHEEL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_COLOR_WHEEL))
#define PIKA_COLOR_WHEEL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_COLOR_WHEEL, PikaColorWheelClass))


typedef struct _PikaColorWheel      PikaColorWheel;
typedef struct _PikaColorWheelClass PikaColorWheelClass;

struct _PikaColorWheel
{
  GtkWidget parent_instance;
};

struct _PikaColorWheelClass
{
  GtkWidgetClass parent_class;

  /* Notification signals */
  void (* changed) (PikaColorWheel   *wheel);

  /* Keybindings */
  void (* move)    (PikaColorWheel   *wheel,
                    GtkDirectionType  type);

  /* Padding for future expansion */
  void (*_pika_reserved1) (void);
  void (*_pika_reserved2) (void);
  void (*_pika_reserved3) (void);
  void (*_pika_reserved4) (void);
};


void        color_wheel_register_type          (GTypeModule     *module);

GType       pika_color_wheel_get_type          (void) G_GNUC_CONST;
GtkWidget * pika_color_wheel_new               (void);

void        pika_color_wheel_set_color         (PikaColorWheel  *wheel,
                                                double           h,
                                                double           s,
                                                double           v);
void        pika_color_wheel_get_color         (PikaColorWheel  *wheel,
                                                gdouble         *h,
                                                gdouble         *s,
                                                gdouble         *v);

void        pika_color_wheel_set_ring_fraction (PikaColorWheel  *wheel,
                                                gdouble          fraction);
gdouble     pika_color_wheel_get_ring_fraction (PikaColorWheel  *wheel);

void        pika_color_wheel_set_color_config  (PikaColorWheel  *wheel,
                                                PikaColorConfig *config);

gboolean    pika_color_wheel_is_adjusting      (PikaColorWheel  *wheel);

G_END_DECLS

#endif /* __PIKA_COLOR_WHEEL_H__ */
