/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/*
 * dbbrowser
 * 0.08 26th sept 97  by Thomas NOEL <thomas@minet.net>
 */

/*
 * This plugin gives you the list of available procedure, with the
 * name, description and parameters for each procedure.
 * You can do regexp search (by name and by description)
 * Useful for scripts development.
 *
 * NOTE :
 * this is only a exercice for me (my first "plug-in" (extension))
 * so it's very (very) dirty.
 * Btw, hope it gives you some ideas about pika possibilities.
 *
 * The core of the plugin is not here. See dbbrowser_utils (shared
 * with script-fu-console).
 *
 * TODO
 * - bug fixes... (my method : rewrite from scratch :)
 */

#include "config.h"

#include <gtk/gtk.h>

#include <libpika/pika.h>
#include <libpika/pikaui.h>

#include "libpika/stdplugins-intl.h"


#define PLUG_IN_PROC   "plug-in-dbbrowser"
#define PLUG_IN_BINARY "procedure-browser"
#define PLUG_IN_ROLE   "pika-procedure-browser"


typedef struct _Browser      Browser;
typedef struct _BrowserClass BrowserClass;

struct _Browser
{
  PikaPlugIn parent_instance;
};

struct _BrowserClass
{
  PikaPlugInClass parent_class;
};


/* Declare local functions.
 */

#define BROWSER_TYPE  (browser_get_type ())
#define BROWSER (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BROWSER_TYPE, Browser))

GType                   browser_get_type         (void) G_GNUC_CONST;

static GList          * browser_query_procedures (PikaPlugIn           *plug_in);
static PikaProcedure  * browser_create_procedure (PikaPlugIn           *plug_in,
                                                  const gchar          *name);

static PikaValueArray * browser_run              (PikaProcedure        *procedure,
                                                  const PikaValueArray *args,
                                                  gpointer              run_data);


G_DEFINE_TYPE (Browser, browser, PIKA_TYPE_PLUG_IN)

PIKA_MAIN (BROWSER_TYPE)
DEFINE_STD_SET_I18N


static void
browser_class_init (BrowserClass *klass)
{
  PikaPlugInClass *plug_in_class = PIKA_PLUG_IN_CLASS (klass);

  plug_in_class->query_procedures = browser_query_procedures;
  plug_in_class->create_procedure = browser_create_procedure;
  plug_in_class->set_i18n         = STD_SET_I18N;
}

static void
browser_init (Browser *browser)
{
}

static GList *
browser_query_procedures (PikaPlugIn *plug_in)
{
  return g_list_append (NULL, g_strdup (PLUG_IN_PROC));
}

static PikaProcedure *
browser_create_procedure (PikaPlugIn  *plug_in,
                          const gchar *name)
{
  PikaProcedure *procedure = NULL;

  if (! strcmp (name, PLUG_IN_PROC))
    {
      procedure = pika_procedure_new (plug_in, name,
                                      PIKA_PDB_PROC_TYPE_PLUGIN,
                                      browser_run, NULL, NULL);

      pika_procedure_set_menu_label (procedure, _("Procedure _Browser"));
      pika_procedure_add_menu_path (procedure, "<Image>/Help/[Programming]");

      pika_procedure_set_documentation (procedure,
                                        _("List available procedures in the PDB"),
                                        NULL,
                                        PLUG_IN_PROC);
      pika_procedure_set_attribution (procedure,
                                      "Thomas Noel",
                                      "Thomas Noel",
                                      "23th june 1997");

      PIKA_PROC_ARG_ENUM (procedure, "run-mode",
                          "Run mode",
                          "The run mode",
                          PIKA_TYPE_RUN_MODE,
                          PIKA_RUN_INTERACTIVE,
                          G_PARAM_READWRITE);
    }

  return procedure;
}

static PikaValueArray *
browser_run (PikaProcedure        *procedure,
             const PikaValueArray *args,
             gpointer              run_data)
{
  PikaRunMode run_mode = PIKA_VALUES_GET_ENUM (args, 0);

  switch (run_mode)
    {
    case PIKA_RUN_INTERACTIVE:
      {
        GtkWidget *dialog;

        pika_ui_init (PLUG_IN_BINARY);

        dialog =
          pika_proc_browser_dialog_new (_("Procedure Browser"), PLUG_IN_BINARY,
                                        pika_standard_help_func, PLUG_IN_PROC,

                                        _("_Close"), GTK_RESPONSE_CLOSE,

                                        NULL);

        gtk_dialog_run (GTK_DIALOG (dialog));
        gtk_widget_destroy (dialog);
      }
      break;

    case PIKA_RUN_WITH_LAST_VALS:
    case PIKA_RUN_NONINTERACTIVE:
      g_printerr (PLUG_IN_PROC " allows only interactive invocation");

      return pika_procedure_new_return_values (procedure,
                                               PIKA_PDB_CALLING_ERROR,
                                               NULL);
      break;

    default:
      break;
    }

  return pika_procedure_new_return_values (procedure, PIKA_PDB_SUCCESS, NULL);
}
